------------------Rigging Toolkit----------------------

--Created byJack Bowyer
--Jackbowyer545@gmail.com

-----------------------------------------------------

--This script is free to use for any commercial or non-commercial projects (at your own risk!). I take no responsibility if anything goes wrong.


--You are free to modify and redistribute this script, just don't steal it and claim it as your own. Please leave the original credit on there.



---------------------------------------------------

--HOW TO USE---

--Just run the script by pressing CTRL+E

--Choose the name of your rig (default is 'MyRig')

--Create a bunch of points in the viewport to match your character (i.e upper arm, elbow, hand)

--Choose a rig module to create, then select the landmark points you created

--Create a layout guide, and manipulate the components to match the character.

--Edit the settings. Warning: Some settings can be very slow. Check Mirror to create an identical copy accross the X plane.

--Press create!

--https://www.youtube.com/watch?v=iX7wpOUdBS8&list=UUn-_vHoBzmWh8j9IB6sCmaA      <---    This shows some of the features in action.



--(Check the ScriptSpot page, i might make a real tutorial at some point).

------------------------------------------------------------------------------------
--ACKNOWLEDGEMENTS---
--Spine high range of motion twist system based on Felix Joleane's chain tools setup.
--------------------------------------------------------------------------------------------




---------Warning: Spaghetti code ahead-----------
--ABANDON HOPE ALL YE WHO ENTER HERE--


(
	
	
	--------------------------------------------------------------------------
	-----------------LOCAL ROLLOUT DECLARATIONS-----------------
	
	local FlexiSpineRollout, RiggingTools, IKFKSpineRollout,MainPageRollout,AnimGeoRollout,SpaceSwitchRollout,HeadGuideRollout,LegGuideRollout,MakeArmGuideRollout,CreateSpineRollout --Declare the rollouts as local
	
	
	--------------------------------------------------------------------------
	-----------------STRUCTURE DEFINITIONS-----------------
	
	struct ToolSetup
	(
		----------------------------------------------------------------------------------------
			fn DeclareNullSizes multiplier = 
		(
			local nullsizes
			nullsizes = #(2*multiplier,4*multiplier,6*multiplier,8*multiplier) --Tiny, Small, Medium, Big
			nullsizes
		),
		------------------------------------------------------------------------------------------------
		fn DeclareControllerSizes multiplier =
		(
			local controlsizes
			controllsizes = #(10*multiplier,20*multiplier,30*multiplier,40*multiplier) --Tiny, Small, Medium, Big
			controllsizes
		),
		----------------------------------------------------------------------------------------------

		NullSizes = ToolSetup.DeclareNullSizes 0.2,
		ControlSizes = ToolSetup.DeclareControllerSizes 0.2,
		GreyColor = color 120 120 120,
		NamePrefix = "MyRig_",
		TurboMode = on,
		
		LastCreatedPelvis = undefined,
		LastCreatedRibcage = undefined,
		LastPickedRoot = undefined
		
		
		
	)
	
	struct CommonFns
	(
		--------------------------------------------------------------------------------------------------------------
		fn findIncrementsBetweenTwoPoints pointA pointB divisions:3 includeEnds:false =
		(
			if classof pointA != point3 do (pointA = pointA.pos)
			if classof pointB != point3 do (pointB = pointB.pos)

			local incrementPositions = #()
			local incrementNo,difference,increment,currentincrement,currentpos

			if divisions > 0 do
		  (
			  if includeEnds == true do ( append incrementPositions pointA)
			  for incrementNo = 1 to incrementNo = divisions do
			  (
				  difference = pointA - pointB
				  increment = 1.00/(divisions+1) 
				  currentincrement = increment*incrementNo
				  currentpos = PointA - (difference*currentincrement) 
				  append incrementPositions currentpos
			  )
			 if includeEnds == true do ( append incrementPositions pointB)
		  )
		  incrementPositions
		),
		-----------------------------------------------------------------------------------------------------------------------
		fn ApplyLookat Obj Target TargetAxis TargetFlip UpNode SourceUpnodeAlign UpNodeAxisFlip UpNodeAxis:2 =
		(	
			if classof Obj == string do (obj = getnodebyname obj)
			local LookCons
			LookCons = LookAt_Constraint()
			LookCons.appendTarget Target 100
			LookCons.lookat_vector_length = 0
			LookCons.viewline_length_abs = false
			LookCons.target_axisFlip = TargetFlip
			LookCons.pickUpNode = upnode
			LookCons.upnode_world = false
			LookCons.StoUP_axis = SourceUpnodeAlign --Get/set the state of the Source/Upnode Alignment > Source Axis radiobuttons.Possible values are:  0 = X, 1 = Y, 2 = Z
			LookCons.StoUP_axisFlip = UpNodeAxisFlip
			LookCons.upnode_axis = UpNodeAxis --Get/set the stare of the Aligned To Upnode Axis radiobuttons. 
			LookCons.upnode_ctrl = 0
			Obj.rotation.controller = LookCons
			Obj.rotation.controller.target_axis = TargetAxis
			Obj.rotation.controller.StoUP_axis = SourceUpnodeAlign
		),
		-------------------------------------------------------------------------------------------------------------------------
		fn AssignPathConstraint Obj PathSpline Percentage =
		(
			if classof Obj == string do (Obj = getnodebyname Obj)
			if classof PathSpline == string do (PathSpline = getnodebyname PathSpline)
			local pc = path_constraint()
			pc.path = PathSpline
			pc.percent = Percentage
			Obj.position.controller = pc
		),
		-------------------------------------------------------------------------------------------------------------------------
		fn GetValueWithPadding InputValue  =
		(
			--Outputs the input value with padding. So 1 would become 001
			if InputValue <10 then
			(
				InputValue = "00"+(InputValue as string)
			)
			else 
			(
				if InputValue >9 and InputValue <100 then 
				(
					InputValue = "0"+(InputValue as string)
				)
			)
			InputValue
		),
		------------------------------------------------------------------------------------------------------------------------------
		fn CheckIfExists BaseName StartNo CutOutPoint =
		(
			--Input Number without padding- so 001 is just 1. 
			--works up to cutoutpoint
			local DoesNotExist = False,iteration = 0
			local CurrentNo,TryToSelect,finaliteration,Message
			while DoesNotExist == False do
			( 
				if iteration > CutOutPoint do (DoesNotExist == True)
				CurrentNo = CommonFns.GetValueWithPadding ((StartNo as integer) + iteration)
				TryToSelect =  getnodebyname (BaseName + (CurrentNo as string))
				if TryToSelect == undefined do
				(
					finaliteration = iteration
					iteration = CutOutPoint + 10
					DoesNotExist = True
				)
				iteration += 1
			)
			((StartNo as integer) + finaliteration)
			--outputs whichever number does not exist
		),
		------------------------------------------------------------------------------------------------------------------------------
		fn ChoosePointType type pointname =
		(
			if classof pointname == string do(getnodebyname pointname)
			setcenter = off
			setbox = off
			setaxis = off
			setcross = off
			if type == 1 then
			(setcenter = on)
			else if type == 2 then
			(setaxis = on)
			else if type == 3 then
			(setcross = on)
			else
			(setbox = on)
			PointName.box = setbox
			PointName.axistripod = setaxis
			PointName.centermarker = setcenter
			PointName.cross = setcross
		),
		-------------------------------------------------------------------------------------------------------------------------------
		fn CreatePathConstrainedPoints spline PointsNo alignTo size baseName type nullcolor AddLetters OveridePercents:false = 
		(
			--For Null size, use Nullsizes[i]- 1 = tiny, 2 = small, 3= medium 4= big
			--For type: 1 = centermarker, 2 = axis tripod 3= cross 4=box
			if classof spline == string do (getnodebyname spline)
			if classof alignto == string do (getnodebyname alignto)
			local UnusedNumber,createdpoints,PointName,correspondingletter,pointnamebase,unusedletternumber,pointnumber,currentpercent,loopend
			local createdpoints = #()
			UnusedNumber = CommonFns.CheckIfExists baseName 1 999
			loopend = PointsNo-1
			for i = 0 to i = loopend do
			(
				PointName = "Blank"
				if AddLetters == true then
				(
					CorrespondingLetter = CommonFns.ConvertIntegerToAlphabetNumber (i+1)
					PointNameBase = BaseName + CorrespondingLetter
					UnusedLetterNumber = CommonFns.CheckIfExists PointNameBase 1 999
					UnusedLetterNumber = CommonFns.GetValueWithPadding UnusedLetterNumber
					PointName = PointNameBase + (UnusedLetterNumber as string)
				)
				else
				(
					PointNumber = GetValueWithPadding (i+UnusedNumber)
					PointName = (baseName + (PointNumber as string))
				)
				CurrentPercent =( i as float )*(100.00/(PointsNo-1))
				Point pos:[0,0,0] isSelected:off cross: off centermarker: off axistripod: off box: off name:PointName size:size wirecolor:nullcolor
				PointName = getnodebyname PointName
				in coordsys world PointName.rotation = in coordsys world alignTo.rotation
				CommonFns.ChoosePointType type PointName
				if OveridePercents != false then (CommonFns.AssignPathConstraint PointName spline OveridePercents[i+1] )
				else (CommonFns.AssignPathConstraint PointName spline CurrentPercent )
				deletekeys PointName.position.controller
				Append createdpoints PointName
			)
			createdpoints
		),
		---------------------------------------------------------------------------------------------------------------------------------
		fn CreatePointsOnArray ArrayName alignTo size baseName type nullcolor AddLetters= 
		(
			--For Null size, use Nullsizes[i]- 1 = tiny, 2 = small, 3= medium 4= big
			--For type: 1 = centermarker, 2 = axis tripod 3= cross 4=box
			if classof alignTo == string do (getnodebyname alignTo)
			local UnusedNumber,loopend,pointname,currentposition,pointnumber,CorrespondingLetter,PointNameBase,UnusedLetterNumber
			UnusedNumber = CommonFns.CheckIfExists baseName 1 999
			loopend = ArrayName.count
			local createdpoints = #()
			for i = 1 to i = loopend do
			(
				PointName = "Blank"
				CurrentPosition = ArrayName[i]
				PointNumber = CommonFns.GetValueWithPadding ((i-1)+UnusedNumber)
				if AddLetters == true then
				(
					CorrespondingLetter = CommonFns.ConvertIntegerToAlphabetNumber i
					PointNameBase = BaseName + CorrespondingLetter
					UnusedLetterNumber = CommonFns.CheckIfExists PointNameBase 1 999
					UnusedLetterNumber = CommonFns.GetValueWithPadding UnusedLetterNumber
					PointName = PointNameBase + (UnusedLetterNumber as string)
				)
				else
				(
					PointName = (baseName + (PointNumber as string))
				)
				Point pos:CurrentPosition isSelected:off cross: off centermarker: off axistripod: off box: off name:PointName size:size wirecolor:nullcolor
				PointName = getnodebyname PointName
				in coordsys world PointName.rotation = in coordsys world alignTo.rotation
				PointName.pos = CurrentPosition
				CommonFns.ChoosePointType type PointName	
				Append createdpoints PointName
			)
			createdpoints
		),
		----------------------------------------------------------------------------------------------------------------------------------------
		fn makesphericalcontrol size ctrlcolor ctrlname = 
		(
			Circle radius:size pos:[0,0,0] isSelected:off name:"ZZZTempCircleA"
			in coordsys world rotate $ZZZTempCircleA (angleaxis -90 [0,1,0])
			Circle radius:size pos:[0,0,0] isSelected:off name:"ZZZTempCircleB"
			in coordsys world rotate $ZZZTempCircleB (angleaxis -90 [0,1,0])
			in coordsys world rotate $ZZZTempCircleB (angleaxis 90 [0,0,1])
			Circle radius:size pos:[0,0,0] isSelected:off name:"ZZZTempCircleC" wirecolor:ctrlcolor
			convertToSplineShape $ZZZTempCircleA
			convertToSplineShape $ZZZTempCircleC
			convertToSplineShape $ZZZTempCircleB
			addAndWeld $ZZZTempCircleC $ZZZTempCircleA -1
			addAndWeld $ZZZTempCircleC $ZZZTempCircleB -1
			$ZZZTempCircleC.name = ctrlname
		),
		------------------------------------------------------------------------------------------------------------------------------------------
		fn MakeTriangleController size ctrlcolor ctrlname =
		(
			--could be improved by adding orientation
			local ss
			 ss = SplineShape pos:[0,0,0] name:ctrlname wirecolor:ctrlcolor
			addNewSpline ss
			local halfsize
			halfsize = (size*0.5)
			addKnot ss 1 #corner #curve [halfsize,0,0]
			addKnot ss 1 #corner #curve [-halfsize,-size,0]
			addKnot ss 1 #corner #curve [-halfsize,size,0]
			addKnot ss 1 #corner #curve [halfsize,0,0]
			updateshape ss
			ss
		),
		---------------------------------------------------------------------------------------------------------------------------------
		fn Make2TriangleController size ctrlcolor ctrlname offset =
		(
			local ss
			ss = SplineShape pos:[0,0,0] name:ctrlname wirecolor:ctrlcolor
			addNewSpline ss
			local halfsize
			local offset
			halfsize = (size*0.5)
			offset = (size*offset)
			addKnot ss 1 #corner #curve [halfsize,-offset,0]
			addKnot ss 1 #corner #curve [-halfsize,-size-offset,0]
			addKnot ss 1 #corner #curve [-halfsize,size-offset,0]
			addKnot ss 1 #corner #curve [halfsize,-offset,0]
			addnewspline ss
			addKnot ss 2 #corner #curve [halfsize,offset,0]
			addKnot ss 2 #corner #curve [-halfsize,-size+offset,0]
			addKnot ss 2 #corner #curve [-halfsize,size+offset,0]
			addKnot ss 2 #corner #curve [halfsize,offset,0]
			updateshape ss
			ss
		),
		--------------------------------------------------------------------------------------------------------------------------
		fn MakeController Size Type AlignTo CtrlPosition Splinecolor Basename orientation posOrNeg =
		(
			local UnusedNumber,SplineName,SelectSpline,AlignToRotation
			UnusedNumber = CommonFns.CheckIfExists baseName 1 999
			UnusedNumber = CommonFns.GetValueWithPadding UnusedNumber
			SplineName = Basename + UnusedNumber
			alignToRotation = alignTo
			
			if Type == "rectangle" or Type == "square" then
			( 
			Rectangle length:Size width:Size cornerRadius:0 name:SplineName wirecolor: SplineColor
			)
			else if type == "sphere" then
			(
				CommonFns.makesphericalcontrol (Size*0.5) SplineColor SplineName
			)
			else if type == "triangle" then
			(
				CommonFns.MakeTriangleController Size Splinecolor SplineName
			)
			else if type == "circle" then
			(
				Circle radius:(size*0.5) pos:[0,0,0] isSelected:off name:SplineName wirecolor:SplineColor
			)
			else if type == "2triangle" then
			(
			CommonFns.Make2TriangleController (Size*0.5) Splinecolor SplineName 2
			)
			SelectSpline = getnodebyname SplineName
			if posOrNeg == "negative" then
			(
				rotate SelectSpline (angleaxis 180 [0,0,1])
				WorldAlignPivot SelectSpline
			)
			if orientation == "y" then
			(
			rotate SelectSpline (angleaxis 90 [0,0,1])
			WorldAlignPivot SelectSpline
			)
			else if orientation == "z" then
			(
			rotate SelectSpline (angleaxis -90 [0,1,0])
			WorldAlignPivot SelectSpline
			)
			if classof alignToRotation != quat do (AlignToRotation = (in coordsys world AlignToRotation.rotation))
			in coordsys world SelectSpline.rotation = AlignToRotation
			if Classof CtrlPosition == Point3 then (SelectSpline.pos = CtrlPosition)
			else (SelectSpline.pos = CtrlPosition.transform.pos)
			SelectSpline
		),
		----------------------------------------------------------------------------------------------------------------------------
		fn ParentObjInArraytoArray childArray ParentArray =
		(
			for i = 1 to i = ParentArray.count do
			(
				childArray[i].parent = ParentArray[i]
			)
		),
		-------------------------------------------------------------------------------------------------------------------------
		fn PosConstrainBetweenTwoObjs ObjToConstrain Obj1 Obj2 Obj1Percent =
		(
			local posCtrl
			posCtrl =Position_Constraint()
			posConstraintInterface = posCtrl.constraints
			posConstraintInterface.appendTarget Obj1 Obj1Percent
			posConstraintInterface.appendTarget Obj2 50
			ObjToConstrain.position.controller = posCtrl
		),
		--------------------------------------------------------------------------------------------------------------------
		fn drawLineBetweenTwoPoints pointA pointB divisions knotType baseName bezier =
		(
			--Draws a line between two points and adds vertices based on the number of desired divisions.
			--knotType:  1: smooth 2: corner
			--bezier true or false
			--inputs for pointA and pointB are two .pos values
			local knots = #()
			knots = #(#smooth,#corner,#bezier,#bezierCorner)
			local UnusedNumber,SplineName,ss,vertexlocations,numberofverts
			UnusedNumber = CommonFns.CheckIfExists baseName 1 999
			UnusedNumber = ((CommonFns.GetValueWithPadding UnusedNumber) as string)
			SplineName = baseName + UnusedNumber
			ss = SplineShape pos:pointA name:SplineName
			addNewSpline ss
			addKnot ss 1 knots[knotType] #curve PointA
			if divisions > 0 do
			(
			  vertexlocations = CommonFns.findIncrementsBetweenTwoPoints pointA pointB divisions:divisions includeends:false
			  for i = 1 to i = divisions do (addKnot ss 1 knots[knotType] #curve vertexlocations[i])
			)
			addKnot ss 1 knots[knotType] #curve PointB
			NumberOfVerts = numknots ss
			if bezier == true do
			(
				for k = 1 to k = NumberofVerts do (setKnotType ss 1 k #smooth)
				for k = 1 to k = NumberofVerts do (setKnotType ss 1 k #bezier)
			)
		updateShape ss
		ss
		SplineName
		),
		-----------------------------------------------------------------------------------------------------------------------
		fn AddBonesToSplineSkin spline ControllersArray =
		(
			local NewNames = #()
			local NumVerts,FinalBone,NumberPrefix,NewNameToAdd
			if classof spline == string do (Spline = getnodebyname spline)
			NumVerts = numknots Spline
			FinalBone = ControllersArray.count
			select Spline
			for c = 1 to c = ControllersArray.count do
			(
				select Spline
				skinOps.addBone $.modifiers[#Skin] ControllersArray[c] 1
				NumberPrefix = CommonFns.GetValueWithPadding c
				NewNameToAdd = ControllersArray[c].name = "ZZZ" + (NumberPrefix as string) + ControllersArray[c].name
				append NewNames NewNameToAdd
				ControllersArray[c].name = NewNameToAdd
			)
			select Spline
			ModPanel.SetCurrentObject Spline.skin
			$.modifiers[#Skin].filter_vertices = on
			skinOps.ReplaceVertexWeights $.modifiers[#Skin]  (NumVerts*3) FinalBone 1
			skinOps.ReplaceVertexWeights $.modifiers[#Skin]  ((NumVerts*3)-1) FinalBone 1
			skinOps.ReplaceVertexWeights $.modifiers[#Skin]  ((NumVerts*3)-2) FinalBone 1
			skinOps.ReplaceVertexWeights $.modifiers[#Skin] 4 2 1 
			skinOps.ReplaceVertexWeights $.modifiers[#Skin] 5 2 1
			skinOps.ReplaceVertexWeights $.modifiers[#Skin] 6 2 1
			skinOps.ReplaceVertexWeights $.modifiers[#Skin] 1 1 1 
			skinOps.ReplaceVertexWeights $.modifiers[#Skin] 2 1 1
			skinOps.ReplaceVertexWeights $.modifiers[#Skin] 3 1 1
			for n = 1 to n = ControllersArray.count do
			(
			ControllersArray[n].name = replace ControllersArray[n].name 1 6 ""
			--SelectThis = ConvertStringToName NewNames[n].name
			--SelectThis.name = replace SelectThis.name 1 6 ""
			)
		),
		-----------------------------------------------------------------------------------------------------------------
		fn LookatConstrainArraytoArray ObjsToConstrain ObjsToLookat UpNodesArray TargetAxis TargetFlip SourceUpnodeAlign UpNodeAxisFlip UpNodeAxis ArrayTrim LookArrayOffset UpnodeArrayOffset=
		(
			local end,lookatoffset,upnodeoffset
			end = ObjsToConstrain.count - ArrayTrim
			for i = 1 to i = end do
			(
				lookatoffset = (i + LookArrayOffset)
				upnodeoffset = (i + UpNodeArrayOffset)
				CommonFns.ApplyLookat ObjsToConstrain[i] ObjsToLookat [lookatoffset] TargetAxis TargetFlip UpNodesArray[upnodeoffset] SourceUpnodeAlign UpNodeAxisFlip UpNodeAxis:UpNodeAxis
			)
		),
		-------------------------------------------------------------------------------------------------------------------
		fn ApplyRotationExpression ApplyToObj Axis  ExpressionString Inputs VarNames =
		(
			local Controller
			Controller = "blank"
			if axis == "y" do 
			(
					ApplyToObj.rotation.controller.Y_Rotation.controller = Float_Expression ()
					Controller = ApplyToObj.rotation.controller.Y_Rotation.controller
			)
			if axis == "x" do 
			(
				ApplyToObj.rotation.controller.X_Rotation.controller = Float_Expression ()
				Controller = ApplyToObj.rotation.controller.X_Rotation.controller
			)
			if axis == "z" do 
			(
				ApplyToObj.rotation.controller.z_Rotation.controller = Float_Expression ()
				Controller = ApplyToObj.rotation.controller.Z_Rotation.controller
			)
			if classof Inputs == array then
			(
				for i = 1 to i = Inputs.count do
				(
				Controller.AddScalarTarget VarNames[i] Inputs[i]   --$sphere001.rotation.controller[#Y_Rotation] 
				)
			)
			else
			(
				Controller.AddScalarTarget VarNames Inputs
			)
			Controller.SetExpression ExpressionString
		),
		----------------------------------------------------------------------------------------------------------------------
		fn FreezeTransform item = 	
			(				
				for i = 1 to i = item.count do
				(
					local CurObj = item[i]
					
					if classof CurObj.rotation.controller != Rotation_Layer do
					(
						CurObj.rotation.controller = Euler_Xyz() 		
						CurObj.rotation.controller = Rotation_list() 			
						CurObj.rotation.controller.available.controller = Euler_xyz() 		
						CurObj.rotation.controller.setname 1 "Frozen Rotation" 		
						CurObj.rotation.controller.setname 2 "Zero Euler XYZ" 		
						CurObj.rotation.controller.SetActive 2 		
					)
					if classof CurObj.position.controller != Position_Layer do
					(
						CurObj.position.controller = Bezier_Position() 			
						CurObj.position.controller = position_list() 			
						CurObj.position.controller.available.controller = Position_XYZ() 	
						CurObj.position.controller.setname 1 "Frozen Position" 	
						CurObj.position.controller.setname 2 "Zero Pos XYZ" 			
						CurObj.position.controller.SetActive 2 		
						CurObj.Position.controller[2].x_Position = 0
						CurObj.Position.controller[2].y_Position = 0
						CurObj.Position.controller[2].z_Position = 0
					)
				)

			),
		--------------------------------------------------------------------------------------------------------------------------------
		fn DetectAlignment MainObj ObjectB =
		(
			if classof MainObj == string do (MainObj = (getnodebyname MainObj))
			if classof ObjectB == string do (ObjectB = (getnodebyname ObjectB))
			Point pos:MainObj.transform.pos isSelected:off name:"ZZZTempMainNull"
			in coordsys world $ZZZTempMainNull.rotation = in coordsys world MainObj.rotation
			in coordsys world $ZZZTempMainNull.pos = MainObj.transform.pos
			Point pos:ObjectB.transform.pos isSelected:off name:"ZZZExposeNull"
			$ZZZExposeNull.parent = $ZZZTempMainNull
			local Zpos,Ypos,Xpos,ZNegative,YNegative,XNegative,exposepos,outputaxis
			local finaloutput = #()
			outputaxis = "y"
			exposepos = in coordsys parent $ZZZExposeNull.pos
			ZNegative = false
			YNegative = false
			XNegative = false
			if exposepos.y < 0 do 
			(
				Ynegative = true
				exposepos.y = -(exposepos.y)
			)
			if exposepos.z < 0 do 
			(
				Znegative = true
				exposepos.z = -(exposepos.z)
			)
			if exposepos.x < 0 do 
			(
				Xnegative = true
				exposepos.x = -(exposepos.x)
			)
			if exposepos.y > exposepos.x and exposepos.y > exposepos.z do (outputaxis = "y")
			if exposepos.x > exposepos.y and exposepos.x > exposepos.z do (outputaxis = "x")
			if exposepos.z > exposepos.y and exposepos.z > exposepos.x do (outputaxis = "z")
			append finaloutput outputaxis
			if outputaxis == "y" then (append finaloutput Ynegative) else if outputaxis == "x" then (append finaloutput Xnegative) else (append finaloutput Znegative)
			delete $ZZZTempMainNull
			delete $ZZZExposeNull
			--messagebox (finaloutput as string)
			finaloutput
		),
		--------------------------------------------------------------------------------------------------------------------------------
		fn ApplyScaleExpression ApplyToObj Axis  ExpressionString Inputs VarNames =
		(
			local Controller
			Controller = "blank"
			if axis == "y" do 
			(
					ApplyToObj.scale.controller.Y_scale.controller = Float_Expression ()
					Controller = ApplyToObj.scale.controller.Y_scale.controller
			)
			if axis == "x" do 
			(
				ApplyToObj.scale.controller.X_scale.controller = Float_Expression ()
				Controller = ApplyToObj.scale.controller.X_scale.controller
			)
			if axis == "z" do 
			(
				ApplyToObj.scale.controller.Z_scale.controller = Float_Expression ()
				Controller = ApplyToObj.scale.controller.Z_scale.controller
			)
			if classof Inputs == array then
			(
				for i = 1 to i = Inputs.count do
				(
				Controller.AddScalarTarget VarNames[i] Inputs[i]   --$sphere001.rotation.controller[#Y_Rotation] 
				)
			)
			else
			(
				Controller.AddScalarTarget VarNames Inputs
			)
			Controller.SetExpression ExpressionString
		),
		----------------------------------------------------------------------------------------------------------------------------------------------------------
		fn GetSplineVertDistanceArray TheSpline =
		(
			if classof TheSpline == string do (getnodebyname TheSpline)
			local PointDistanceArray = #()
			append PointDistanceArray 0
			for v = 1 to v = (numknots TheSpline)-1 do
			(
				local FirstVert
				local NextVert
				local VertDistance
				FirstVert = getKnotPoint TheSpline 1 v
				NextVert = getKnotPoint TheSpline 1 (v+1)
				Vertdistance = (distance FirstVert NextVert)
				append PointDistanceArray (VertDistance)
			)
			PointDistanceArray
		),
		------------------------------------------------------------------------------------------------------------------------------------------------
		fn GetDistanceArrayForObjs ArrayInput =
		(
			local FirstObj
			local NextObj
			local ObjDistance
			local ObjDistanceArray = #()
			append ObjDistanceArray 0
			for o = 1 to o = ArrayInput.count-1 do
			(
				local FirstObj
				local NextObj
				local ObjDistance
				FirstObj = in coordsys world ArrayInput[o].transform.pos
				NextObj =  in coordsys world ArrayInput[o+1].transform.pos
				Objdistance = (distance FirstObj NextObj)
				append ObjDistanceArray (ObjDistance)
			)
			ObjDistanceArray
		),
		------------------------------------------------------------------------------------------------------------------------------------------------------
		fn NormaliseDistanceArray ArrayInput =
		(
			local PercentArray = #()
			append PercentArray 0
			local RunningSum = 0
			for v = 2 to v = ArrayInput.count do
			(
				append PercentArray (RunningSum + ArrayInput[v])
				RunningSum += ArrayInput[v]
			)
			local NormalisedPercent = #()
			append NormalisedPercent 0
			for v = 2 to v = PercentArray.count do
			(
				local ValToNormalise
				local NormaliseAmount
				ValToNormalise = PercentArray[v]
				NormaliseAmount = PercentArray[PercentArray.count]
				append NormalisedPercent (ValToNormalise/NormaliseAmount)
			)
			NormalisedPercent 
		),
		--------------------------------------------------------------------------------------------------------------------------------------------------------
		fn CalculateStartEndWeights PercentInputArray =
		(
			local StartWeights = #()
			local EndWeights = #()
			local OutputArray = #()
			for i = 1 to i = PercentInputArray.count do
			(
				local StartA = (1-(PercentInputArray[i]))
				StartA = StartA*StartA
				append StartWeights StartA
				local StartB = (PercentInputArray[i])
				StartB = StartB*StartB
				append EndWeights StartB
			)
			append OutputArray StartWeights
			append OutputArray EndWeights
			OutputArray
		),
		--------------------------------------------------------------------------------------------------------------------------------------------------------------------
		fn CalculateSmoothVertWeights ControlInputPercent VertInputPercent =
		(
			local OutputArray = #()
			local VposOriginal,CposOriginal,Vweight,CposNormalised,VposNormalised
			CposOriginal = ControlInputPercent
			for v = 1 to v = VertInputPercent.count do
			(
				VposOriginal = VertInputPercent [v]
				VposNormalised = "Blank"
				if VposOriginal < CposOriginal then
				(
					VposNormalised = (VposOriginal/CposOriginal)*90
				)
				else if VposOriginal > CposOriginal then
				(
					VposNormalised = ((((VposOriginal-CposOriginal)/(1-CposOriginal))+1)*90)
				)
				else if VposOriginal == 1 or VposOriginal == 0 then
				(
					VposNormalised = 0
				)
				else
				(
					VposNormalised = 90
				)
				VWeight = Sin VPosNormalised
				append OutputArray Vweight
			)
			OutputArray
		),
		------------------------------------------------------------------------------------------------------------------------------------------------------
		fn CalcMidControlsBaseWeights ControlPercents VertPercents =
		(
			local CurrentContWeights
			local OutputArray = #()
			for c = 2 to c = (ControlPercents.count)-1 do 
			(
				
				CurrentContWeights = CommonFns.CalculateSmoothVertWeights ControlPercents[c] VertPercents
				append OutputArray CurrentContWeights
			)
			--Final output for this is an array containing sub arrays for the weights of each of the controllers in the middle of the spline.
			--Example: #(#(0,  0.2,  1,  0.6,  0)) , (#(0,  0.8,  1,  0.4,  0))
			OutputArray
		),
		---------------------------------------------------------------------------------------------------------------------------------------------------------
		fn ApplySmoothWeightsToSpline spline ControllersArray WeightArray =
		(
			--Controllers Array contains $ references to all controllers in same order as weight array
			--WeightArray contains an array with sub arrays. All of Controllers 1's weights to each vertex in an array, then all of controller 2's weights etc.
			local ControlWeightArray = #()
			for c = 1 to c = ControllersArray.count do
			(
				append ControlWeightArray c
			)
			local NewNames = #()
			local NumVerts,FinalBone,NumberPrefix,NewNameToAdd
			if classof spline == string do (Spline = getnodebyname spline)
			NumVerts = numknots Spline
			FinalBone = ControllersArray.count
			select Spline
			for c = 1 to c = ControllersArray.count do
			(
				select Spline
				skinOps.addBone $.modifiers[#Skin] ControllersArray[c] 1
				NumberPrefix = CommonFns.GetValueWithPadding c
				NewNameToAdd = ControllersArray[c].name = "ZZZ" + (NumberPrefix as string) + ControllersArray[c].name
				append NewNames NewNameToAdd
				ControllersArray[c].name = NewNameToAdd
			)
			select Spline
			ModPanel.SetCurrentObject Spline.skin
			$.modifiers[#Skin].filter_vertices = on
			for v = 1 to v =NumVerts do
			(
				local WeightsToapply = #()
				local MinWeight = 0
				local MaxWeight = 0
				for c = 1 to c = ControllersArray.count do
				(
					if WeightArray[c][v] < MinWeight do (MinWeight = WeightArray[c][v])
					if WeightArray[c][v] > MaxWeight do (MaxWeight = WeightArray[c][v])
				)
				for c = 1 to c = ControllersArray.count do
				(
					append WeightsToApply (((WeightArray[c][v])-MinWeight)/(MaxWeight-MinWeight))
				)
				
				skinOps.ReplaceVertexWeights $.modifiers[#Skin]  ((v*3)-2) ControlWeightArray WeightsToApply
				skinOps.ReplaceVertexWeights $.modifiers[#Skin]  ((v*3)-1) ControlWeightArray WeightsToApply
				skinOps.ReplaceVertexWeights $.modifiers[#Skin]  (v*3) ControlWeightArray WeightsToApply
			)
					for n = 1 to n = ControllersArray.count do
					(
					ControllersArray[n].name = replace ControllersArray[n].name 1 6 ""
					)
		),
		-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
		fn GetChainStartEndArray StartObj EndObj Cycles:999 =
		(
			
			if classof StartObj == string do (StartObj = getnodebyname StartObj)
			if classof EndObj == string do (EndObj = getnodebyname EndObj)
			local ParentObj,CurrentObj,JointsBetween,ValidatedTest
			local ParentArray = #()
			local OutputArray = #()
			local PosToAdd
			local TestIfError
			ValidatedTest = 0
			CurrentObj = EndObj
			local Iteration = 1
			while Iteration < Cycles do
			(
				ParentObj = CurrentObj.parent
				if CurrentObj.Parent == undefined do
				(
					ValidatedTest = 0
					Messagebox "Error: Selected joints are not in the same hierarchy"
					Append OutputArray "Failed"
					Iteration = Cycles+1
				)
				CurrentObj = ParentObj
				if ParentObj == StartObj do 
				(
					JointsBetween = Iteration-1
					ValidatedTest = 1
					Iteration = Cycles+1
				)
				Iteration += 1
			)
			if JointsBetween == 0 do 
			(
				ValidatedTest = 0
				MessageBox "Error:Insufficient number of joints."
				append OutputArray "Failed"
			)
			if ValidatedTest == 1 do
			(
					append ParentArray EndObj
					CurrentObj = EndObj
					for i = 1 to i = JointsBetween+1 do
					(
						ParentObj = CurrentObj.parent
						CurrentObj = ParentObj
						append Parentarray ParentObj
					)
					for i = 0 to i = JointsBetween+1 do
					(
						append Outputarray Parentarray[(Parentarray.count)-i]
					)	
			)
			OutputArray
		),
		-------------------------------------------------------------------------------------------------------
		fn drawSplineOnArray VertLocationsArray KnotTypesArray BaseName  =
		(
			local knots = #()
			knots = #(#smooth,#corner,#bezier,#bezierCorner)
			local UnusedNumber,SplineName,ss,vertexlocations,numberofverts
			UnusedNumber = CommonFns.CheckIfExists baseName 1 999
			UnusedNumber = ((CommonFns.GetValueWithPadding UnusedNumber) as string)
			SplineName = baseName + UnusedNumber
			
			ss = SplineShape pos:VertLocationsArray[1] name:SplineName
			addNewSpline ss
			for i = 1 to i = VertLocationsArray.count do
			(
			addKnot ss 1 KnotTypesArray[i] #curve VertLocationsArray[i]
			)
			updateShape ss
			ss
			SplineName = (getnodebyname SplineName)
			SplineName
		),
		-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
		fn ConvertObjArrayToPosArray InputArray =
		(
			local outputarray = #()
			local posToadd
			for i = 1 to i = inputarray.count do
			(
				posToAdd = in coordsys world InputArray[i].transform.pos
				append outputarray posToAdd
			)
		OutputArray
		),
		-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
		
		fn MakeArrayEntriesUnique Inputarray =
		(
			local UniqueArray = #()
			for i = 1 to i = inputarray.count do 
			(
			appendifUnique UniqueArray Inputarray[i]
			)
			UniqueArray
		),
		---------------------------------------------------------------------------------------------------------------------------
		
		fn ConvertIntegerToAlphabetNumber InputNumber =
		(
			
			if InputNumber > 702 do ( Messagebox "Warning: Cannot convert a number this high to an alphabet letter")
			local alphabet = #("A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z")
			local MovesRemaining = InputNumber-1
			local ColumnCount = 0
			local SubCount = 0
			
			
			
			for i = 0 to i = MovesRemaining do
			(
				Subcount += 1
				if Subcount == 27 do
				(
					SubCount = 1
					ColumnCount += 1
				)
			)	
			
			if ColumnCount != 0 then
			(
				OutputString = (alphabet[columncount] + alphabet[SubCount])
			)
			else (OutputString = alphabet[SubCount])
			OutputString
		),
		-----------------------------------------------------------------------------------------------------------------------------------------------
				fn FreezeTransform item = 	
			(				
				for i = 1 to i = item.count do
				(
					local CurObj = item[i]
					
					if classof CurObj.rotation.controller != Rotation_Layer do
					(
						CurObj.rotation.controller = Euler_Xyz() 		
						CurObj.rotation.controller = Rotation_list() 			
						CurObj.rotation.controller.available.controller = Euler_xyz() 		
						CurObj.rotation.controller.setname 1 "Frozen Rotation" 		
						CurObj.rotation.controller.setname 2 "Zero Euler XYZ" 		
						CurObj.rotation.controller.SetActive 2 		
					)
					if classof CurObj.position.controller != Position_Layer do
					(
						CurObj.position.controller = Bezier_Position() 			
						CurObj.position.controller = position_list() 			
						CurObj.position.controller.available.controller = Position_XYZ() 	
						CurObj.position.controller.setname 1 "Frozen Position" 	
						CurObj.position.controller.setname 2 "Zero Pos XYZ" 			
						CurObj.position.controller.SetActive 2 		
						CurObj.Position.controller[2].x_Position = 0
						CurObj.Position.controller[2].y_Position = 0
						CurObj.Position.controller[2].z_Position = 0
					)
				)

			),
			----------------------------------------------------------------------------------------------------------------------------------------
			fn ConvertPercentArrayToSmoothedValues PeakPercent InputPercent =
			(
				local OutputArray = #()
				local VposOriginal,CposOriginal,Vweight,CposNormalised,VposNormalised
				CposOriginal = PeakPercent
				for v = 1 to v =InputPercent.count do
				(
					VposOriginal = InputPercent [v]
					VposNormalised = "Blank"
					if VposOriginal < CposOriginal then
					(
						VposNormalised = (VposOriginal/CposOriginal)*90
					)
					else if VposOriginal > CposOriginal then
					(
						VposNormalised = ((((VposOriginal-CposOriginal)/(1-CposOriginal))+1)*90)
					)
					else if VposOriginal == 1 or VposOriginal == 0 then
					(
						VposNormalised = 0
					)
					else
					(
						VposNormalised = 90
					)
					VWeight = Sin VPosNormalised
					append OutputArray Vweight
				)
				OutputArray
			),
			---------------------------------------------------------------------------------------------------
			fn CreateSystemAlignmentPoint StartObj EndObj NoLookat:Undefined =
			(
				--Creates a point which is aligned between a start and end obj. Forward position = negative y , right is Z, up is positive x

				local MainAlignTo, MainAlignPosOrNeg,Midpoint,AlignmentObject,MidPoint, LookatAlignObject, FinalAlignObject

				Alignmentobject = Point Name:"ZZZZZZZAlignmentObject" Pos:(In coordsys world StartObj.transform.pos)

				MainAlignTo = (CommonFns.DetectAlignment AlignmentObject EndObj)[1]
				MainAlignPosOrNeg = (CommonFns.DetectAlignment AlignmentObject EndObj)[2]
				Delete Alignmentobject
				print ("Detected alignment is " + (MainAlignTo as string))
				Alignmentobject = Point Name:"ZZZZZZZAlignmentObject" Pos:(In coordsys world StartObj.transform.pos)
				if MainAlignTo == "x" and MainAlignPosOrNeg == false do 
				(
					rotate Alignmentobject (angleaxis 90 [1,0,0])
				)
				if MainAlignTo == "x" and MainAlignPosOrNeg == true do 
				(
					rotate Alignmentobject (angleaxis -180 [0,0,1])
					rotate Alignmentobject (angleaxis -90 [1,0,0])
				)
				if MainAlignTo == "y" and MainAlignPosOrNeg == false do 
				(
					rotate Alignmentobject (angleaxis 90 [0,0,1])
					rotate Alignmentobject (angleaxis 90 [0,1,0])
				)
					if MainAlignTo == "y" and MainAlignPosOrNeg == true do 
				(
					rotate Alignmentobject (angleaxis -90 [0,0,1])
					rotate Alignmentobject (angleaxis -90 [0,1,0])
				)	
				if MainAlignTo == "z" and MainAlignPosOrNeg == false do (rotate Alignmentobject (angleaxis -90 [0,1,0]))	
				if MainAlignTo == "z" and MainAlignPosOrNeg == true do 
				(
				rotate Alignmentobject (angleaxis 90 [0,1,0])
				rotate Alignmentobject (angleaxis -90 [0,0,1])
				rotate Alignmentobject (angleaxis 90 [0,0,1])
				)
				
				Midpoint = CommonFns.findIncrementsBetweenTwoPoints StartObj EndObj divisions:1 IncludeEnds:False
				if NoLookat == true do 
				(
					local NoLookatPoint = Point Pos:[0,0,0] Name:"ZZZZZZNOLOOKPOINT"
					CommonFns.quickalign NoLookatPoint AlignmentObject
					tempmidpointnull = point Name:"ZZZTEMPMIDPOINT"
					tempmidpointnull.pos.controller = position_constraint()
					tempmidpointnull.pos.controller.appendtarget startobj 100
					tempmidpointnull.pos.controller.appendtarget endobj 100
					in coordsys world NoLookatPoint.pos = tempmidpointnull.transform.pos
					delete tempmidpointnull
				)
				
				LookatAlignObject = Point Name:"ZZZZZZZAlignmentObjectLookat" Pos:(In coordsys world StartObj.transform.pos)
							
				LookatAlignObject.Rotation.Controller = LookAt_Constraint()
				LookatAlignObject.Rotation.Controller.appendTarget EndObj 100
				LookatAlignObject.Rotation.Controller.target_axis = 0
				LookatAlignObject.Rotation.Controller.target_axisFlip = off
				LookatAlignObject.Rotation.Controller.upnode_ctrl = 1
				LookatAlignObject.Rotation.Controller.upnode_world = false
				LookatAlignObject.Rotation.Controller.StoUP_axis = 1
				LookatAlignObject.Rotation.Controller.upnode_axis = 1
				LookatAlignObject.Rotation.Controller.upnode_world = false	
				LookatAlignObject.Rotation.Controller.pickUpNode = Alignmentobject
							
				FinalAlignObject =  Point Name:"ZZZZZZZAlignmentObjectFinal" Pos:(In coordsys world StartObj.transform.pos) isSelected:off cross: off centermarker: off axistripod: off box: off size:0 wirecolor:Green
				FinalAlignObject.rotation.controller = Orientation_Constraint()
				FinalAlignObject.rotation.controller.appendTarget LookatalignObject 100
				CommonFns.FreezeTransform (#(FinalAlignObject))

				Delete AlignmentObject
				Delete LookatAlignObject
					
				if NoLookat == true do
				(
					Delete FinalAlignObject
					FinalAlignObject = NoLookatPoint
				)
				FinalAlignObject
				
			),
			---------------------------------------------------------------------------------------------------------------
			fn CreateLookatdistanceExpose FromLocation ToLocation BaseNames AlignTo:undefined reAssignParents:undefined justdistance:false=
			(
				if classof fromlocation != point3 do (fromlocation = (in coordsys world fromlocation.transform.pos))
				if classof tolocation != point3 do (tolocation = (in coordsys world tolocation.transform.pos)) --if inputted values are objects, extract their position
				local LookatObj, LookatLocal
				
				for i = 1 to i = 2 do --apply the number padding to the objects
				(
					local unusednumber,nametoadd
					UnusedNumber = CommonFns.CheckIfExists BaseNames[i] 1 999
					UnusedNumber = ((CommonFns.GetValueWithPadding UnusedNumber) as string)
					NameToAdd = (Basenames[i] + UnusedNumber)
					BaseNames[i] = NameToAdd
				)
				
					
				LookatObj = Point Name:BaseNames[1] Pos:(In coordsys world FromLocation) isSelected:off cross: off centermarker: off axistripod: on box: off size:ToolSetup.Nullsizes[1] wirecolor:Green
				LookatLocal = Point Name:BaseNames[2] Pos:(In coordsys world ToLocation) isSelected:off cross: on centermarker: off axistripod: off box: off size:ToolSetup.Nullsizes[1] wirecolor:Green
				if alignto != undefined do
				(
					in coordsys world LookatObj.rotation = AlignTo.transform.rotation
					in coordsys world LookatLocal.rotation = AlignTo.transform.rotation
					in coordsys world LookatObj.pos = FromLocation
					in coordsys world LookatLocal.pos = ToLocation
				)
				local LookCons = Lookat_Constraint()
				LookCons.AppendTarget LookatLocal 100
				LookCons.viewline_length_abs = off
				LookatObj.rotation.controller = LookCons
				local OutputArray = #()
				append OutputArray LookatObj
				append OutPutArray LookatLocal
				
				if reassignparents != undefined do
				(
					LookatObj.parent = reAssignParents[1]
					LookatLocal.parent = reAssignParents[2]
				)
				
				local ca
				local AttHolder
				ca=attributes StretchControls
				(
					parameters params rollout:ro_StretchExpose
					(
						paramstretch type:#float ui:spn_ScaleNoscale
						paramdistance type:#float ui:spn_distance
					)
					rollout ro_StretchExpose "Stretch_Expose" 
					(
						spinner spn_ScaleNoscale "Bone Scaling" enabled:on  range:[-99999,99999,0] scale:0.1
						spinner spn_Distance "Bone Distance" enabled:on  range:[-99999,99999,0] scale:0.1	
					)
				)
				AttHolder = (EmptyModifier ())
				AttHolder.name = "Stretch_Expose_Controls"
				addmodifier OutputArray[1] AttHolder
				custAttributes.add OutputArray[1].modifiers[#Stretch_Expose_Controls] ca
				local defaultDistance = (distance OutputArray[1] OutPutArray[2])
				
				if justdistance == false then
				(
				local ScriptToApply = float_script()
				ScriptToApply.addNode "A" OutputArray[2]
				ScriptToApply.addNode "B" OutputArray[1]	
				ScriptToApply.setexpression ("1-(((A.transform * inverse B.transform).pos.x)/" + (DefaultDistance as string)+")")
				OutputArray[1].modifiers[#Stretch_Expose_Controls].StretchControls.paramstretch.controller = ScriptToApply
				)
				else
				(
				local ScriptToApply = float_script()
				ScriptToApply.addNode "A" OutputArray[2]
				ScriptToApply.addNode "B" OutputArray[1]	
				ScriptToApply.setexpression ("((A.transform * inverse B.transform).pos.x)")
				OutputArray[1].modifiers[#Stretch_Expose_Controls].StretchControls.paramdistance.controller = ScriptToApply
				)
				--Output is: LookatObj,Local
				
				--The value has to be accessed using: (A.transform *inverse A.parent.transform).pos.x or something similar
				OutputArray
			),
			-----------------------------------------------------------------------------------------------------------------------
			fn ApplyArmLookat ApplyToObj LookatObj UpnodeObj Leg:False =
			(
			local LookCons = Lookat_Constraint()
			LookCons.appendtarget LookatObj 100
			LookCons.target_axis = 0
			LookCons.upnode_ctrl = 0
			LookCons.upnode_world = off
			LookCons.pickUpNode = UpNodeObj
			LookCons.StoUP_axis = 1
			LookCons.viewline_length_abs = off
			LookCons.lookat_vector_length = 0
			if Leg == True then (LookCons.StoUP_axisFlip = off) else (LookCons.StoUP_axisFlip = on) 
			ApplyToObj.rotation.controller = LookCons
			),
			------------------------------------------------------------------------------------------------------------------------------
			fn QuickAlign ObjToAlign AlignTo Rotonly:false =
			(
				if classof AlignTo != Matrix3 do (AlignTo = AlignTo.transform)
				
				local OrigPos = ObjToAlign.transform.pos
				ObjToAlign.rotation *= ObjToAlign.transform.rotation * (inverse AlignTo.rotation)
				if RotOnly == true then (ObjToAlign.pos = OrigPos) else (in coordsys world Objtoalign.pos = Alignto.pos)
			),
			----------------------------------------------------------------------------------------------
			fn DrawRectangleControl AlignTo StartPoint EndPoint CWidth CLength ObjName StartOffset:0.1 EndOffset:0.9  =
			(
				local BaseDistance = Distance StartPoint Endpoint
				
				BaseObj = SplineShape pos:[0,0,0] name:ObjName
				rotate BaseObj (angleaxis -90 [0,1,0])
				StartRectangle = (Rectangle pos:[0,0,(BaseDistance*StartOffset)] name:"ZZZZZSTARTRECTANGLE" length:CWidth width:CLength)
				EndRectangle = (Rectangle pos:[0,0,(BaseDistance*EndOffset)] name:"ZZZZZENDRECTANGLE" length:CWidth width:CLength)
				converttosplineshape StartRectangle
				converttosplineshape EndRectangle
				addandweld BaseObj StartRectangle 0
				addandweld BaseObj EndRectangle 0
				updateshape BaseObj
				addmodifier BaseObj (CrossSection ())
				converttosplineshape BaseObj
				CommonFns.QuickAlign BaseObj AlignTo
				BaseObj
			),
			------------------------------------------------------------------------------------------
			fn MakeExposeTransformPair AlignTo LocalParent ExposeParent BaseNames:undefined AxisOrder:1 =
			(
				local ExposeTransformBaseName
				local ExposeNodeName
				local LocalNodeName
					
					
					if BaseNames == undefined then
					(
						ExposeTransformBaseName = "helper_exptfm"
						ExposeNodeName = "helper_exposenode"
						LocalNodeName = "helper_localNode"
					)
					else
					(
						ExposeTransformBaseName = BaseNames[1]
						ExposeNodeName = BaseNames[2]
						LocalNodeName = BaseNames[3]
					)
					
					ExposeTransformNode = ExposeTm pos:[45.0585,10.7018,0] isSelected:on Name:(Uniquename ExposeTransformBaseName Numdigits:3) Cross:Off Box:on axistripod:off centermarker:off size:ToolSetup.Nullsizes[1] wirecolor:(color 20 50 20)
					ExposeNode = Point Name: (UniqueName ExposeNodeName NumDigits:3) wirecolor:(color 20 100 20) Box:on cross: off centermarker: off axistripod: off size:ToolSetup.Nullsizes[1]
					LocalNode = Point Name: (UniqueName LocalNodeName NumDigits:3) wirecolor:(color 20 20 100) Box:off cross: on centermarker: off axistripod: off size:ToolSetup.Nullsizes[1]
					
					CommonFns.QuickAlign ExposeTransformNode AlignTo
					CommonFns.QuickAlign ExposeNode AlignTo
					CommonFns.QuickAlign LocalNode AlignTo
					
					ExposeNode.parent = ExposeParent
					LocalNode.parent = LocalParent
					ExposeTransformNode.parent = LocalParent
					
					ExposeTransformNode.exposeNode = ExposeNode
					ExposeTransformNode.useParent = off
					ExposeTransformNode.localReferenceNode = LocalNode
					ExposeTransformNode.eulerXOrder = AxisOrder
					ExposeTransformNode.eulerYOrder = AxisOrder
					ExposeTransformNode.eulerZOrder = AxisOrder	
					local OutputArray = #()
					append OutputArray ExposeTransformNode
					append OutputArray ExposeNode
					append OutputArray LocalNode
			),
			------------------------------------------------------------------------------------------------------------------------------------------------------
			fn MirrorTransformsInArray ArrayToMirror =
			(
				local OutputArray =#()
				
				for i = 1 to i = ArrayToMirror.count do
				(
					local MirroredTransform = ArrayToMirror[i] * (ScaleMatrix [-1,1,1])
					local FinalTransform = (scaleMatrix [1,1,-1]) * MirroredTransform
					append OutputArray FinalTransform
				)
				OutputArray
			),
			----------------------------------------------------------------------------------------
			fn FlipZAxis Obj =
			(
				local tm1 = Obj.transform
				local finalTm=(scaleMatrix [1,1,-1])*tm1
				Obj.transform = finalTm
			),
			------------------------------------------------------------------------------------------------------------------
			fn ApplyLengthWidthProperties ObjsToApply ObjWidth ObjFrontWidth:undefined ObjOffset:undefined =

			(
				if ObjFrontWidth == undefined do (ObjFrontWidth = ObjWidth)
				if classof ObjsToApply != array do (ObjsToApply = #(ObjsToApply))
				
					for i = 1 to i = ObjsToApply.count do
					(
					
						setUserProp ObjsToApply[i] "BoneLength" ObjsToApply[i].length
						setUserProp ObjsToApply[i] "BoneWidth" ObjWidth
						setUserProp ObjsToApply[i] "BoneFrontWidth" ObjFrontWidth
						if ObjOffset!= undefined do (setUserProp ObjsToApply[i] "ObjOffset" ObjOffset)
					)
				
			),
			---------------------------------------------------------------------------------------------------
			fn TweakObjectOffset TweakType:"Pos" Positive:True Axis:"X"=
			(
				for i = 1 to i = selection.count do
				(
						CurrentObj = selection[i]
						local ObjBoundingBox = (in coordsys local nodeLocalBoundingBox CurrentObj)
						local AverageScale = Distance ObjboundingBox[1] ObjBoundingBox[2]
						local MoveAmount = AverageScale*0.02
						local RotAmount = 5
						local ScaleAmount = 0.1	
						
					
					
						
					local Xval = 0
					local Yval = 0
					local Zval = 0
					if Axis == "X" do (Xval = 1)
					if Axis == "Y" do (Yval = 1)
					if Axis == "Z" do (ZVal = 1)
					if Positive == False do
					(
						Xval = -Xval
						Yval = -Yval
						ZVal = -Zval
					)

					if TweakType == "Pos" do
					(
						CurrentObj.objectoffsetpos +=[(Xval*MoveAmount),(YVal*MoveAmount),(ZVal*MoveAmount)]
					)
					if TweakType == "Rot" do
					(
						CurrentObj.objectoffsetrot += ((angleaxis  RotAmount [Xval,Yval,ZVal]) as quat)
						--in coordsys local rotate CurrentObj  (angleaxis  RotAmount [Xval,Yval,ZVal])
					)
					if TweakType == "Scale" do
					(
						CurrentObj.objectoffsetscale += [ScaleAmount*Xval,ScaleAmount*Yval,ScaleAmount*ZVal]
					)
				)
			), --this bracket to close geo tweak function
			------------------------------------------------------------------------------------------------------
			fn ApplycontrollerTypeProperties ObjsToApply ControlType:"Big" ControlLevel:1 =
			(
				if classof ObjsToApply != array do (ObjsToApply = #(ObjsToApply))
				
				local BaseControlColor = color 158 78 177
				local FinalControlColour
				
				
					
					
					
				--SMALL 1 color 8 110 135
				--SMALL 2 color 28 149 177
				--SMALL 3 color 88 199 225
					
					
				-- MED 1 color 8 8 136	
				--MED 2 color 28 28 177
				--MED 3 color 88 88 225
					
				--BIG 1 color 57 8 136
				--BIG 2 color 85 28 177
				--BIG 3 color 140 88 225
					
				if ControlType == "Big" do 
				(
					if ControlLevel == 3 do (BaseControlColor = color 85 65 165)
					if ControlLevel == 2 do (BaseControlColor = color 105 78 177)
					if ControlLevel == 1 do (BaseControlColor = color 180 108 225)
				)
				if ControlType == "Med" or ControlType == "Medium" do 
				(
					if ControlLevel == 3 do (BaseControlColor = color 98 68 136)
					if ControlLevel == 2 do (BaseControlColor = color 158 78 177)
					if ControlLevel == 1 do (BaseControlColor = color 208 108 225)
				)
				if ControlType == "Small" do 
				(
					if ControlLevel == 3 do (BaseControlColor = color 80 110 135)
					if ControlLevel == 2 do (BaseControlColor = color 128 149 177)
					if ControlLevel == 1 do (BaseControlColor = color 188 199 225)
				)
				if ControlType == "Utility" do 
				(
					if ControlLevel == 1 do (BaseControlColor = color 177 148 27)
				)
				if ControlType == "Upnode" do 
				(
					if ControlLevel == 1 do (BaseControlColor = color 225 198 87)
				)
				
					
					
				for i = 1 to i = ObjsToApply.count do
				(
					setUserProp ObjsToApply[i] "ControlType" ControlType
					setUserProp ObjsToApply[i] "ControlLevel" ControlLevel
					ObjsToApply[i].wirecolor = BaseControlColor
				)
			
				
			),------This bracket to close apply control color function
			fn AddDeformFins ParentObj AlignToPos AlignToOri YMeasureObjects ZMeasureObjects Prefix:"" BaseNames:#("bn_fin","ctrl_fin","null_fin","null_finAUTO")  RotateAmount:0.2 Fins:#(1,1,1,1) OriConsObj:False  YWidth:2 ZWidth:2 ControlObj:False Jiggle:False PosYPrefix:"Back" PosZPrefix:"R" DefaultRot:20 DefaultExtend:0.2 =
			(
				
				local allcreated = #()
				local allbones = #()
				local allcontrols = #()
				-- 90 degrees in radians = 1.57079633
				
				local BoneBaseName = (Prefix + BaseNames[1])
				local ControlBaseName = (Prefix + BaseNames[2])
				local NullifierBaseName = (Prefix + BaseNames[3])
				local AutoNullBaseName = (Prefix + BaseNames[4])
				
				local MainAlignPointBaseName = (Prefix + "null_fin_group")
				

				local IncludePosY = fins[1]
				local IncludeNegY = fins[2]
				local IncludePosZ = fins[3]
				local IncludeNegZ = fins[4]
				
				
				if PosYPrefix == "Back" do ( NegYPrefix = "Front")
				if PosYPrefix == "L" do ( NegYPrefix = "R")
				if PosYPrefix == "R" do ( NegYPrefix = "L")
				if PosYPrefix == "Front" do ( NegYPrefix = "Back")
				
				if PosZPrefix == "Back" do (NegZPrefix = "Front")
				if PosZPrefix == "L" do (NegZPrefix = "R")	
				if PosZPrefix == "R" do (NegZPrefix = "L")	
				if PosZPrefix == "Front" do (NegZPrefix = "Back")	
				
				local PosYBoneName = (BoneBaseName + PosYPrefix)
				local NegYBoneName = (BoneBaseName + NegYPrefix)
				local PosZBoneName = (BoneBaseName + PosZPrefix)
				local NegZBoneName = (BoneBaseName + NegZPrefix)
				local PosYControlName = (ControlBaseName +PosYPrefix)
				local NegYControlName = (ControlBaseName +NegYPrefix)
				local PosZControlName = (ControlBaseName + PosZPrefix)
				local NegZControlName = (ControlBaseName + NegZPrefix)
				local PosYNullName = (NullifierBaseName + PosYPrefix)
				local NegYNullName = (NullifierBaseName + NegYPrefix)
				local PosZNullName = (NullifierBaseName + PosZPrefix)
				local NegZNullName = (NullifierBaseName + NegZPrefix)
				local PosYAUTONullName = (AutoNullBaseName + PosYPrefix)
				local NegYAUTONullName = (AutoNullBaseName + NegYPrefix)
				local PosZAUTONullName = (AutoNullBaseName + PosZPrefix)
				local NegZAUTONullName = (AutoNullBaseName + NegZPrefix)
				
				local BoneNamesArray = #(PosYBoneName,NegYBoneName,PosZBoneName,NegZBoneName)
				local ControlNamesArray =#(PosYControlName,NegYControlName,PosZControlName,NegZControlName)
				local NullNamesArray = #(PosYNullName,NegYNullName,PosZNullName,NegZNullName)
				local AutoNullNamesArray = #(PosZAUTONullName,NegZAUTONullName,PosZAUTONullName,NegZAUTONullName)
				

				
				if (classof AlignToPos != Matrix3) and (classof AlignToPos != Point3) do (AlignToPos=AlignToPos.transform)

				if (classof AlignToOri != Matrix3) and (classof AlignToOri != Point3) do (AlignToOri=AlignToOri.transform)

				local MainAlignmentPoint = Point Box:Off Cross:On AxisTripod:Off Centermarker:Off Name:(uniquename MainAlignPointBaseName Numdigits:3) Wirecolor:(color 60 180 20) Size:(YWidth*0.2)
				CommonFns.QuickAlign MainAlignmentPoint AlignToOri
				in Coordsys World MainAlignmentPoint.pos = AlignToPos.pos
				MainAlignmentPoint.parent = ParentObj
				

				
				
				append allcreated mainalignmentpoint
				
				local TempAlignMentPoint = Point Name:"ZZZZZTEMPALIGNMENTPOINT"
				CommonFns.Quickalign TempAlignmentPoint MainAlignmentPoint
				
				if OriConsObj != False do
				(
					local OriCons = Orientation_constraint()
					OriCons.appendtarget ParentObj 100
					OriCons.appendtarget OriConsObj 100
					MainAlignmentPoint.rotation.controller = OriCons
				)
				
				
				
				local TempPosYPoint = Point Name:"ZZZZZTEMPPOSYPOINT"
				local TempNegYPoint = Point Name:"ZZZZZTEMPNEGYPOINT"
				local TempPosZPoint = Point Name:"ZZZZZTEMPPOSZPOINT"
				local TempNegZPoint = Point Name:"ZZZZZTEMPNEGZPOINT"
				CommonFns.QuickAlign TempPosYPoint TempAlignmentPoint
				CommonFns.QuickAlign TempNegYPoint TempAlignmentPoint
				CommonFns.QuickAlign TempPosZPoint TempAlignmentPoint
				CommonFns.QuickAlign TempNegZPoint TempAlignmentPoint
				in coordsys TempAlignmentPoint move TempPosYPoint [0,YWidth,0]
				in coordsys TempAlignmentPoint move TempNegYPoint [0,-YWidth,0]	
				in coordsys TempAlignmentPoint move TempPosZPoint [0,0,ZWidth]
				in coordsys TempAlignmentPoint move TempNegZPoint [0,0,-ZWidth]	
				local TempPointsArray = #(TempPosYPoint,TempNegYPoint,TempPosZPoint,TempNegZPoint)
				
				local RotateZArray = #(-1,1,0,0) --used for default rotation
				local RotateYArray = #(0,0,-1,1) --used for default rotation
				
				local FixAlignArray = #([0,0,1],[0,0,-1],[0,1,0],[0,-1,0])
				
				local MeasureArray = #(YMeasureObjects[1],YMeasureObjects[2],ZMeasureObjects[1],ZMeasureObjects[2])
				local RotateAmountArray = #(-RotateAmount,RotateAmount,RotateAmount,-RotateAmount)
				
				local UserPropPosNegArray = #("-","+","-","+")
				local UserPropExprSquashArray = #("+","+","+","+")
				local UserPropRotAxisArray = #("Z","Z","Y","Y")


				

				
				
				
				
				
				
				for i = 1 to i = 4 do
				(
					if fins[i] == 0 do
					(
						append allbones false
						append allcontrols false
					)
					
					 if fins[i] != 0 do -- +Y -Y +Z -Z
					 (
						local BoneNameToAdd = BoneNamesArray[i]
						local ControlNameToAdd = ControlNamesArray[i]
						local NullNameToAdd = NullNamesArray[i]
						local AutoNullNameToAdd = AutoNullNamesArray[i]
						 
						 
						 --MAKE THE PARENT NULL
						local ParentNull = Point Box:Off Cross:On AxisTripod:Off Centermarker:Off Name:(uniquename NullNameToAdd Numdigits:3) Wirecolor:(color 20 180 60) Size:(YWidth*0.2)
						append allcreated ParentNull
						CommonFns.QuickAlign ParentNull TempAlignMentPoint
						ParentNull.parent =  MainAlignmentPoint
						ParentNull.name = (uniquename nullnamesarray[i] Numdigits:3)
						 
						 in coordsys local rotate ParentNull (angleaxis 90 FixAlignArray[i])
						 if i >2 do (in coordsys local rotate ParentNull (angleaxis DefaultRot [0,RotateYArray[i],0]))
						 if i <3 do (in coordsys local rotate ParentNull (angleaxis DefaultRot [0,0,RotateZArray[i]]))
						 
						 

						  --MAKE THE BONE
						 local FinBone = bonesys.createbone ParentNull.transform.pos TempPointsArray[i].transform.pos [0,0,1]
						FinBone.name = (uniquename bonenamesarray[i] Numdigits:3)
						 CommonFns.QuickAlign FinBone ParentNull
						 FinBone.parent = ParentNull
						 append allcreated FinBone
						 append allbones FinBone
						 
						 --MAKE THE AUTO NULL
						 local AutoNull = Point Box:Off Cross:On AxisTripod:Off Centermarker:Off Name:(uniquename AutoNullNameToAdd Numdigits:3) Wirecolor:(color 60 180 20) Size:(YWidth*0.2)
						 AutoNull.name = (uniquename autonullnamesarray[i] Numdigits:3)
							 append allcreated AutoNull
						 CommonFns.QuickAlign AutoNull ParentNull
						 AutoNull.parent = ParentNull
						 
						 --MAKE THE CONTROL
						 
						 local Controller = CommonFns.MakeController (YWidth*0.3) "circle" FinBone FinBone blue ControlNameToAdd "z" "positive"
						 Controller.objectoffsetpos = [(FinBone.Length*1.2),0,0]
						Controller.name = (uniquename controlnamesarray[i] Numdigits:3)
						 CommonFns.QuickAlign Controller ParentNull
						 Controller.parent = AutoNull
						Controller.pos.controller = position_list ()
						
						FinBone.pos.controller = position_constraint()
						FinBone.pos.controller.appendtarget controller 100
						FinBone.rotation.controller = orientation_constraint()
						FinBone.rotation.controller.appendtarget controller 100

						 
						 
						local MoveAmount = (FinBone.length)*DefaultExtend
						 
						setUserProp Controller "RigTools_UpDirection"  UserPropPosNegArray[i]
						setUserProp Controller "RigTools_SquashAmtPosNeg"  UserPropExprSquashArray[i]
						setUserProp Controller "RigTools_FinRotateAxis"  UserPropRotAxisArray[i]
						setUserProp Controller "RigTools_RotateAmount"  RotateAmount
						setUserProp Controller "RigTools_MoveAmount"  MoveAmount
						setUserProp Controller "RigTools_AutoNull"  AutoNull.name
						setUserProp Controller "RigTools_ParentNull"  ParentNull.name
						setUserProp Controller "RigTools_FinBone"  FinBone.name
						 
						 append allcreated controller
						 append allcontrols controller 
						 
						 
						 
						 --APPLY EXPRESSION
						 
						 --NEED TO WORK OUT IF POSITIVE OR NEGATIVE ROTATION 
						 
						 --POSY:NEGZ
						 --NEGT:POSZ
						 --POSZ: POSY
						 --NEGZ: NEGY
						 
						 


						local ExpressionToAdd
						ExpressionToAdd = copy expressionToAdd
						ExpressionToAdd = undefined
						ExpressionToAdd = Float_Expression()
						ExpressionToAdd.AddScalarTarget "SquashAmount" MeasureArray[i].modifiers[#Stretch_Expose_Controls].StretchControls[#paramstretch]
						ExpressionToAdd.AddScalarTarget "AutoEnabled" ControlObj.modifiers[#Spine_Controls].SpineControls[#paramFinAuto]
						ExpressionToAdd.SetExpression ("if(SquashAmount > 0,((" +(RotateAmountArray[i] as string) + ")*((SquashAmount*AutoEnabled))),0)")
						
						
						
						
						if i <3 then 
						(AutoNull.rotation.controller.Z_Rotation.controller = ExpressionToAdd)
						else
						(AutoNull.rotation.controller.Y_Rotation.controller = ExpressionToAdd)
						
						
						ExpressionToAdd = copy expressionToAdd
						ExpressionToAdd = undefined
						ExpressionToAdd = Float_Expression()

						ExpressionToAdd.AddScalarTarget "SquashAmount" MeasureArray[i].modifiers[#Stretch_Expose_Controls].StretchControls[#paramstretch]
						ExpressionToAdd.AddScalarTarget "AutoEnabled" ControlObj.modifiers[#Spine_Controls].SpineControls[#paramFinAuto]
						ExpressionToAdd.SetExpression ("if(SquashAmount > 0,((" + (MoveAmount as string) + ")*((SquashAmount*AutoEnabled))),0)")

						
						Controller.pos.controller.Position_XYZ.controller.X_Position.controller = ExpressionToAdd
					
						local ca=attributes FinAdjust
						(
							parameters params rollout:ro_AdjustFins
							(
								--paramblend type:#float ui:spn_SpineController
							)
							
							rollout ro_AdjustFins "Adjust_Fins" 
							(
								edittext edt_moveamount "Fin Extend"
								edittext edt_rotamount "Fin Rotate"
								
								
								button btn_apply "Apply" enabled:on
								button btn_delete "Delete Fin" enabled:on
								button btn_finalise "Finalise" enabled:on	
								
								local UpDirection, SquashAmtPosNeg, FinRotAxis, RotAmount, Moveamount, ParentNull, AutoNull, FinBone,finalised
								local DeleteStage = 0
								
								on ro_AdjustFins open do
								(
									UpDirection = (GetUserProp $ "RigTools_UpDirection") as string
									SquashAmtPosNeg = (GetUserProp $ "RigTools_SquashAmtPosNeg") as string
									FinRotAxis = (GetUserProp $ "RigTools_FinRotateAxis") as string
									RotAmount = (GetUserProp $ "RigTools_RotateAmount") as string
									Moveamount =  (GetUserProp $ "RigTools_MoveAmount") as string
									ParentNull = GetUserProp $ "RigTools_ParentNull"
									AutoNull =  GetUserProp $ "RigTools_AutoNull"
									FinBone =  GetUserProp $ "RigTools_FinBone"
									Finalised = (GetUserProp $ "RigTools_Finalised")
									edt_moveamount.text = (Moveamount as string)
									edt_rotamount.text = (RotAmount as string)
									if (Finalised == "true") or (Finalised == true) do
									(
										btn_apply.enabled = off
										btn_delete.enabled = off
										btn_finalise.enabled = off
										edt_moveamount.enabled = off
										edt_rotamount.enabled = off
									)
								)
									
								on btn_finalise pressed do
								(
									local FinalisedState = (GetUserProp $ "RigTools_Finalised") as string
									if FinalisedState == "1" then
									(
										setUserProp $ "RigTools_Finalised"  "true"
										btn_apply.enabled = off
										btn_delete.enabled = off
										btn_finalise.enabled = off
										edt_moveamount.enabled = off
										edt_rotamount.enabled = off
									)
									else
									(
										MessageBox "Permanently lock down the values? Press again to confirm."
										setUserProp $ "RigTools_Finalised"  "1"
									)
								)
								
									
								
								on btn_apply pressed do
								(
									
								MoveAmount = edt_moveamount.text
								RotAmount = edt_rotamount.text
								local GreaterThanLessThan	
								
								if SquashAmtPosNeg == "+" then (GreaterThanLessThan = "SquashAmount > 0") else (GreaterThanLessThan = "SquashAmount < 0")
								

								$.pos.controller.Position_XYZ.controller.X_Position.controller.setexpression ("if(" + GreaterThanLessThan + ",((" + (MoveAmount as string) + ")*((" + SquashAmtPosNeg + "SquashAmount*AutoEnabled))),0)")	
								
								if FinRotAxis == "Y" then ((getnodebyname AutoNull).rotation.controller.Y_Rotation.controller.setexpression ("if(" + GreaterThanLessThan + ",((" + Updirection + RotAmount + ")*((" + SquashAmtPosNeg + "SquashAmount*AutoEnabled))),0)"))
								else ((getnodebyname AutoNull).rotation.controller.Z_Rotation.controller.setexpression ("if(" + GreaterThanLessThan + ",((" + Updirection + RotAmount + ")*((" + SquashAmtPosNeg + "SquashAmount*AutoEnabled))),0)"))
								setUserProp $ "RigTools_RotateAmount"  RotAmount
								setUserProp $ "RigTools_MoveAmount"  MoveAmount
								
								)
								
								on btn_delete pressed do
								(
									if DeleteStage == 0 then 
									(
										messagebox "Warning: Cannot undo. Press again to confirm."
										DeleteStage += 1
									)
									else
									( 
										undo on
										(
											local objstodelete = #((Getnodebyname FinBone),(Getnodebyname ParentNull),(Getnodebyname AutoNull),$)
											delete objstodelete
										)
									)
								)
							
							)
						)
							AttHolder = (EmptyModifier())
							AttHolder.name = "Fin_Controls"
							addmodifier Controller AttHolder
							custAttributes.add Controller.modifiers[#Fin_Controls] ca
							
							
							
							
							
					)
					
					
					

					
				)--this bracket to close i = 1 to 4 loop
						
				for n in TempPointsArray do (delete n)
				delete TempAlignmentPoint

				local OutputArray = #()
				
				append OutputArray allcreated
				append OutputArray allcontrols
				append OutputArray allbones
				
				OutputArray
						

				--OutputArray Contains:
				
				--1: Allcreated
				--2: controls
				--3: bones

				
				
				
				-----COME HERE DEFORM FINS------
			),--this bracket to close deform fins function
			
			fn CreateLookatExposePair StartBone EndBone Prefix BaseName:"LookExpose" Additive:True NullifyOffset:False OffsetScalar:0.8 UpnodeAxis:1=
			(
				
				local Allcreated = #()
				
				local AverageScale = (Distance StartBone EndBone)
				
				local TinySize = AverageScale *0.2
				local MediumSize = AverageScale *0.3
				local BigSize = AverageScale * 0.4
				
				local LookatTargetName = (Prefix + "null_" + BaseName + "_LookatTarget")
				local LookatExposePointName = (Prefix +"null_" + BaseName + "_LookatExpose")
				local UpNodePointName = (Prefix + "null_" + BaseName + "_upnodePoint")
				local LocalNodeName = (Prefix + "null_" + BaseName + "_localPoint")
				local TwistNodeName =  (Prefix + "null_" + BaseName + "_TwistOutput")
				
				local TempOffsetPoint = Point Name:"ZZZZZTEMPOFFSETPOINT"
				
				local LookatTarget = Point Name:(Uniquename LookatTargetName NumDigits:3) Box:Off Cross:Off CenterMarker:On AxisTripod:Off wirecolor:(color 160 65 65) Size:TinySize
				local LookatExposePoint = Point Name:(Uniquename LookatExposePointName NumDigits:3) Box:Off Cross:Off CenterMarker:On AxisTripod:Off wirecolor:(color 160 65 65) Size:TinySize
				local UpNodePoint = Point Name:(Uniquename UpNodePointName NumDigits:3) Box:Off Cross:Off CenterMarker:On AxisTripod:Off wirecolor:(color 160 65 65) Size:TinySize
				local LocalNode = Point Name:(Uniquename LocalNodeName NumDigits:3) Box:Off Cross:Off CenterMarker:On AxisTripod:On wirecolor:(color 160 65 65) Size:TinySize
				local TwistNode = Point Name:(Uniquename TwistNodeName NumDigits:3) Box:Off Cross:Off CenterMarker:On AxisTripod:Off wirecolor:(color 160 65 65) Size:TinySize
				

				append allcreated LookatTarget
				append allcreated LookatExposePoint
				append allcreated UpNodePoint
				append allcreated LocalNode
				append allcreated TwistNode
				
				

				
				if NullifyOffset == true then
				(
					CommonFns.QuickAlign TempOffSetPoint StartBone
					in coordsys world TempOffSetPoint.pos = EndBone.transform.pos
					in coordsys StartBone move TempOffSetPoint [(AverageScale*OffsetScalar),0,0]
					CommonFns.QuickAlign UpNodePoint StartBone
				)
				else
				(
					CommonFns.QuickAlign TempOffSetPoint EndBone
					in coordsys EndBone move TempOffSetPoint [(AverageScale*OffsetScalar),0,0]
					CommonFns.QuickAlign UpNodePoint EndBone	
				)
				
				local LookatExposePointPos, LookatExposePointParent
				local LookatTargetPos, LookatTargetParent
				local UpNodeParent
				
				
				local TwistNodeOffsetPoint = Point Name:"ZZZTWISTNODEOFFSET"
				
				

				If Additive == true then
				(
					LookatExposePointPos = TempOffSetPoint
					LookatTargetPos = StartBone
					LookatTargetParent = StartBone
					LookatExposePointParent = EndBone
					UpnodeParent = EndBone
					

					TwistNodePos = StartBone
					TwistNodeParent = StartBone
					TwistNodeLookat = EndBone
					TwistNodeAxisFlip = Off
				)
				else
				(
					LookatExposePointPos = TempOffSetPoint
					LookatTargetPos = StartBone
					LookatTargetParent = StartBone
					LookatExposePointParent = EndBone
					UpNodeParent = StartBone
					
					
					CommonFns.QuickAlign TwistNodeOffsetPoint EndBone
					in CoordSys EndBone Move TwistNodeOffsetPoint [AverageScale*0.2,0,0]
					TwistNodePos = TwistNodeOffsetPoint
					TwistNodeParent = EndBone
					TwistNodeLookat = EndBone
					TwistNodeAxisFlip = On
				)
				

				
				CommonFns.QuickAlign LookatExposePoint LookatExposePointPos
				LookatExposePoint.parent = LookatExposePointParent
				CommonFns.QuickAlign LookatTarget LookatTargetPos
				LookatTarget.parent = LookatTargetparent
				UpNodePoint.parent = UpNodeParent
				
				local TempBaseLookCons = lookat_constraint()
				TempBaseLookCons.target_axis = 0
				TempBaseLookCons.upnode_ctrl = 1
				TempBaseLookCons.upnode_world = off
				TempBaseLookCons.StoUP_axis = UpNodeAxis
				TempBaseLookCons.upnode_axis = UpNodeAxis
				TempBaseLookCons.target_axisFlip = off
				
							
				local ExposePointLookCons = copy TempBaseLookCons
				ExposePointLookCons.appendtarget LookatTarget 100
				ExposePointLookCons.pickUpNode = UpNodePoint
				
				LookatExposePoint.rotation.controller = ExposePointLookCons
				
				local TwistNodeLookCons = copy TempBaseLookCons
				TwistNodeLookCons.appendtarget TwistNodeLookat 100
				TwistNodeLookCons.pickupnode = LookatExposePoint
				TwistNodeLookCons.target_axisFlip = TwistNodeAxisFlip
				TwistNode.rotation.controller = TwistNodeLookCons
				
				in coordsys world TwistNode.pos = TwistNodePos.transform.pos
				TwistNode.parent = TwistNodeParent
				CommonFns.QuickAlign LocalNode TwistNode
				LocalNode.parent = TwistNodeParent
				
				delete TempOffSetPoint
				delete TwistNodeOffsetPoint
				
				local OutputArray = #()
				append OutputArray allcreated
				
				OutputArray

			)--this bracket to close lookatexpose pair function
			

		)--this bracket to close common tools
	
	--------------------------------------------------------------------------------------------------------------------------------
	----------------------------------------IK/FK SPINE FUNCTION------------------------------------------------------------
	--------------------------------------------------------------------------------------------------------------------------------
	
	struct FinalTools
	(
	
		
		fn MakeIKFKSpine StartObj EndObj  NumFKControls AddReverse JointScaling SquashStretch EaseType NumBones EaseValue PickedOri NoScalePos SquashType squashblending:1 SpineBend:undefined ExactPercents:undefined IncludeFins:undefined TangentRotNullifiers:false Bendcontrollers:false Prefix:"" MainBonesName:Undefined MainControlObject:Undefined DisplayScale:1 TwistDisperse:0 LiveTwist:False UpNodes:True =
		(
			--AddReverse = false/true
			--JointScaling. 1 = Always on, 2= Blend, 3 = always off
			--squash stretch  1 = on, 2 = off
			--Squashtype 1 = fixed value 2 = slider
			---ease type 1 = linear 2 = ease in 3 = ease ou
			--Run the function with undefined for pickedori if you want orientation to be automatic
			--include fins:this is a hardcoded function to add deformation fins to the stomach area. Requires bone 1 to be pelvis, 2,3,4 to be stomach and 5 to be ribcage. not included if left blank.
			--include fins: fill this with an array of locations: #(PelvisWidth,PelvisFrontWidth,MidlineWidth,MidlineFrontWidth,RibcageWidth,RibcageFrontWidth).
			
			
			local AverageScale = (Distance StartObj EndObj)*DisplayScale
			local TinySize = AverageScale/16
			local MediumSize = AverageScale/12
			local BigSize = AverageScale/6
			
			
			local GlobalControllerName = (Prefix + "SubCtr_Global")
			local MainSplineName = (Prefix + "Spline_Main")
			local ReverseSplineName = (Prefix + "Spline_Reverse")
			local TangentHelperName = (Prefix + "Grp_TangentHelper")
			local StartControllerName = (Prefix + "SubCtr_Start")
			local EndControllerName = (Prefix + "SubCtr_End")
			local FKControllerName = (Prefix + "SubCtr_FK")
			local TangentControllerName = (Prefix + "SubCtr_Tangent_")
			local PositiveSplineIKName = (Prefix + "Anchor_PosSplineIK")
			local NegativeSplineIKName = (Prefix + "Anchor_NegSplineIK")
			local StretchAnchorName = (Prefix + "Anchor_Stretch")
			local BlendIKName = (Prefix + "Anchor_BlendIK")
			local MainBoneName = (Prefix + "Bn_MainSpine")
			local IKHelperName = (Prefix + "IK_Helper")
			local WorldSpaceNullName = (Prefix + "Null_World")
			local ReverseFKControllerName = (Prefix + "SubCtr_FKReverse")
			local StretchBoneHelperName = (Prefix + "Anchor_StretchHelper")
			local BendControllerName = (Prefix + "Ctrl_Bend")
			local StretchNubName = (Prefix + "nub_SpineStretch")
			local MeasureHelperLookatName = (Prefix + "FinMeasureHelper_")
			local MeasureHelperLocalName = (Prefix + "FinMeasureHelperLocal_")
			
			
			
			local PelvisTangentParentName = (Prefix + "Null_PelvisTangentParent")
			local RibcageTangentParentName = (Prefix + "Null_RibcageTangentParent")
			local PelvisExposeNullName = (Prefix + "Null_PelvisExpose")
			local RibcageExposeNullName = (Prefix + "Null_RibcageExpose")
			local PelvisNoFollowNullName = (Prefix + "Null_PelvisNoFollow")
			local RibcageNoFollowNullName = (Prefix + "Null_RibcageNoFollow")
			
			local StretchExposePointName = (Prefix + "Null_StretchExpose")
			
			
			
			
			
			if MainBonesName != undefined do MainBoneName = (Prefix + MainBonesName)
				
			local UpNodeGroupName = (Prefix + "Grp_Upnode")
			local UpNodeName =  (Prefix + "CnsTarget_Upnode")
			local ExposeTfmName = (Prefix + "ExposeTfm_")
			local NullifierName = (Prefix + "Null_")
			
			
			local allcreated = #()
			local allcontrollers =#()
			
			local DeformControlsForLayers = #()
			local DeformBonesForLayers = #()
			
			
			
			if ExactPercents != undefined do NumBones = ((ExactPercents.count)-1) --Update number of bones if the exact locations have been inputted. If not then use inputted number.
			
			-------------Detect the Alignment and Make Global Controller--------------
				
			if PickedOri == undefined then
			(
			local Midpoint,GlobalController,FinalAlignObject

			Midpoint = CommonFns.findIncrementsBetweenTwoPoints StartObj EndObj divisions:1 IncludeEnds:False
			FinalAlignObject = CommonFns.createsystemalignmentpoint StartObj EndObj
			GlobalController = CommonFns.MakeController Bigsize "2triangle" FinalalignObject Midpoint[1] blue GlobalControllerName "x" "positive"
			delete FinalAlignObject
				
			)
			else if PickedOri == "nolookat" then
			(
			local Midpoint,GlobalController,FinalAlignObject
			Midpoint = CommonFns.findIncrementsBetweenTwoPoints StartObj EndObj divisions:1 IncludeEnds:False
			FinalAlignObject = CommonFns.createsystemalignmentpoint StartObj EndObj  NoLookat:true 
			GlobalController = CommonFns.MakeController Bigsize "2triangle" FinalalignObject Midpoint[1] blue GlobalControllerName "x" "positive"
			delete FinalAlignObject	
			)
			else
			(
				local GlobalController,Midpoint
				Midpoint = CommonFns.findIncrementsBetweenTwoPoints StartObj EndObj divisions:1 IncludeEnds:False
				GlobalController = CommonFns.MakeController BigSize "2triangle" PickedOri Midpoint[1] blue GlobalControllerName "x" "positive"
			)
			
			
			append Allcreated GlobalController
			
			----------Make The Splines----------------------
			local MainSpline,ReverseSpline,UnusedNumber,ss
			
			UnusedNumber = CommonFns.CheckIfExists MainSplineName 1 999
			UnusedNumber = ((CommonFns.GetValueWithPadding UnusedNumber) as string)
			MainSplineName = MainSplineName + UnusedNumber
			
			ss = SplineShape pos:StartObj.Transform.pos name:MainSplineName
			ss.optimize = false
			ss.steps = 12
			addNewSpline ss
			addKnot ss 1 #beziercorner #curve StartObj.transform.pos StartObj.Transform.pos Midpoint[1]
			addKnot ss 1 #beziercorner #curve EndObj.transform.pos Midpoint[1] EndObj.Transform.pos
			updateShape ss
			ss
			MainSpline = (getnodebyname MainSplineName)
			
			UnusedNumber = CommonFns.CheckIfExists ReverseSplineName 1 999
			UnusedNumber = ((CommonFns.GetValueWithPadding UnusedNumber) as string)
			ReverseSplineName = ReverseSplineName + UnusedNumber
			ss = undefined
			ss = SplineShape pos:EndObj.Transform.pos name:ReverseSplineName
			ss.optimize = false
			ss.steps = 12
			addNewSpline ss
			addKnot ss 1 #beziercorner #curve EndObj.transform.pos EndObj.Transform.pos Midpoint[1]
			addKnot ss 1 #beziercorner #curve StartObj.transform.pos Midpoint[1] StartObj.Transform.pos
			updateShape ss
			ss
			ReverseSpline = (getnodebyname ReverseSplineName)
			MainSpline.parent = GlobalController
			
			ReverseSpline.parent = GlobalController
			
			append Allcreated MainSpline
			--Reverse is appended later
			
			

			---------------MAKE THE CONTROLLERS------------------------------------
			local FKControlPositions,FKControlToAdd,TangentToAdd,FirstTangent,SecondTangent,CurrentLetter
			local FKControllers = #()
			local ReverseFKControllers = #()
			

			StartController =  (CommonFns.MakeController BigSize "square" GlobalController StartObj blue (StartControllerName)  "x" "positive")
			EndController = (CommonFns.MakeController BigSize "square" GlobalController EndObj blue (EndControllerName)  "x" "positive")
			
			if NumFKControls > 0 then
			(
			FKControlPositions = CommonFns.findIncrementsBetweenTwoPoints StartObj EndObj divisions:NumFKControls IncludeEnds:False
			CurrentLetter = 0
			for i = 1 to i = NumFKControls do
			(
			CurrentLetter +=1
	
			FKControlToAdd = (CommonFns.MakeController MediumSize "triangle" GlobalController FKControlPositions[i] green (FKControllerName + "_" + (CommonFns.ConvertIntegerToAlphabetNumber CurrentLetter))  "x" "positive")
			append FKControllers FKControlToAdd
			if i == 1 then (FKControlToAdd.parent = GlobalController) else (FKControlToAdd.parent = FKControllers[i-1])
			)
			EndController.parent = FkControllers[FKControllers.count]
			
			for i = 1 to i = FKControllers.count do (append allcreated FKControllers[i])
			
			)
			else
			(
				EndController.parent = GlobalController
			)
			
			
			
			append Allcreated StartController
			append Allcreated EndController
			
			
			UnusedNumber = CommonFns.CheckIfExists (TangentControllerName + "A") 1 999
			UnusedNumber = ((CommonFns.GetValueWithPadding UnusedNumber) as string)
			TangentToAdd = (TangentControllerName + "A") + UnusedNumber
			FirstTangent = Point pos:Midpoint[1] isSelected:off cross: off centermarker: off axistripod: off box: on name:TangentToAdd size:TinySize wirecolor:Green
			UnusedNumber = CommonFns.CheckIfExists (TangentControllerName + "B") 1 999
			UnusedNumber = ((CommonFns.GetValueWithPadding UnusedNumber) as string)
			TangentToAdd = (TangentControllerName + "B") + UnusedNumber
			SecondTangent = Point pos:Midpoint[1] isSelected:off cross: off centermarker: off axistripod: off box: on name:TangentToAdd size:TinySize wirecolor:Green
			FirstTangent.parent = StartController
			SecondTangent.parent = EndController
			StartController.Parent = GlobalController
			
			append allcreated FirstTangent
			append Allcreated SecondTangent
			
			if AddReverse == true and NumFKControls >0 do 
			(
				CurrentLetter = 0
					for i = 1 to i = NumFKControls do
				(
				CurrentLetter += 1

				FKControlToAdd = (CommonFns.MakeController MediumSize "triangle" GlobalController FKControlPositions[NumFKControls-(i-1)] blue (ReverseFKControllerName + "_" + (CommonFns.ConvertIntegerToAlphabetNumber CurrentLetter))  "x" "negative")
				append ReverseFKControllers FKControlToAdd
				if i == 1 then (FKControlToAdd.parent = GlobalController) else (FKControlToAdd.parent = ReverseFKControllers[i-1])
				)
				StartController.Parent = ReverseFKControllers[ReverseFKControllers.count]
				
				for i = 1 to i = ReverseFKControllers.count do (append allcreated ReverseFKControllers[i])
			)
			
			----------------------SKIN THE SPLINES----------------------------------
			local theskin,StartControllerOrigName,EndControllerOrigName,FirstTangentOrigName,SecondTangentOrigName,theskinreverse
			Select MainSpline
			theskin = Skin()
			max modify mode
			addmodifier $ theskin
			StartControllerOrigName = StartController.Name
			Startcontroller.Name = "A"
			EndControllerOrigName = EndController.Name
			EndController.Name = "D"
			FirstTangentOrigName = FirstTangent.Name
			FirstTangent.Name = "B"
			SecondTangentOrigName = SecondTangent.Name
			SecondTangent.Name = "C"
			skinOps.addBone $.modifiers[#Skin] StartController 1
			skinOps.addBone $.modifiers[#Skin] FirstTangent 1
			skinOps.addBone $.modifiers[#Skin] SecondTangent 1
			skinOps.addBone $.modifiers[#Skin] EndController 1
			$.modifiers[#Skin].filter_vertices = on
			Select MainSpline
			skinOps.ReplaceVertexWeights $.modifiers[#Skin]  1 1 1
			skinOps.ReplaceVertexWeights $.modifiers[#Skin]  2 1 1
			skinOps.ReplaceVertexWeights $.modifiers[#Skin]  3 2 1
			skinOps.ReplaceVertexWeights $.modifiers[#Skin]  4 3 1
			skinOps.ReplaceVertexWeights $.modifiers[#Skin]  5 4 1
			skinOps.ReplaceVertexWeights $.modifiers[#Skin]  6 4 1
			Select ReverseSpline
			theskinreverse = Skin()
			max modify mode
			addmodifier $ theskinreverse
			Startcontroller.Name = "D"
			EndController.Name = "A"
			FirstTangent.Name = "C"
			SecondTangent.Name = "B"
			skinOps.addBone $.modifiers[#Skin] EndController 1
			skinOps.addBone $.modifiers[#Skin] SecondTangent 1
			skinOps.addBone $.modifiers[#Skin] FirstTangent 1
			skinOps.addBone $.modifiers[#Skin] StartController 1
			$.modifiers[#Skin].filter_vertices = on
			Select ReverseSpline
			skinOps.ReplaceVertexWeights $.modifiers[#Skin]  1 1 1
			skinOps.ReplaceVertexWeights $.modifiers[#Skin]  2 1 1
			skinOps.ReplaceVertexWeights $.modifiers[#Skin]  3 2 1
			skinOps.ReplaceVertexWeights $.modifiers[#Skin]  4 3 1
			skinOps.ReplaceVertexWeights $.modifiers[#Skin]  5 4 1
			skinOps.ReplaceVertexWeights $.modifiers[#Skin]  6 4 1
			StartController.Name = StartControllerOrigName
			EndController.Name = EndControllerOrigName
			FirstTangent.Name = FirstTangentOrigName
			SecondTangent.Name = SecondTangentOrigName

			if Kinematics == 1 or (NoScalePos == 3 and JointScaling !=1) then
			(
				delete ReverseSpline
			)
			else
			(
			append Allcreated ReverseSpline
			)
			
			
			-----------------------------CALCULATE THE BONE LENGTHS----------------------------------
			local BaseDistance,BaseBoneLength
			local BonePercArray = #()
			local NormalisedBonePercArray = #()
			local UpdatedLengthArray = #()
			local FinalPositionPercArray = #()
			local FinalBonePositionArray =#()
			
			
			if spinebend == undefined and exactpercents == undefined do --if there is no spinebend and no exact percents then perform the default straight line calculations
			(
				BaseDistance = Distance StartObj EndObj
				BaseBoneLength = BaseDistance/NumBones
				if EaseType != 1 then --If ease type is not linear then apply the easing calculation
				(
					for i = 1 to i = NumBones do (append BonePercArray ((i-1) * (1/NumBones)))
					for i = 1 to i = NumBones do 
					(
						append NormalisedBonePercArray (((BonePercArray[i]-BonePercArray[1])/(BonePercArray[BonePercArray.count]-BonePercArray[1]))*EaseValue)
					)
					for i = 1 to i = NumBones do
					(
						if EaseType == 3 then
						(
							append UpdatedLengthArray (BaseBoneLength + (BaseBoneLength * NormalisedBonePercArray[Numbones-(i-1)]))
							UpdatedLengthArray[i] = (UpdatedLengthArray[i]-(BaseBoneLength * NormalisedBonePercArray[i]))
						)
						else
						(
							append UpdatedLengthArray (BaseBoneLength - (BaseBoneLength * NormalisedBonePercArray[Numbones-(i-1)]))
							UpdatedLengthArray[i] = (UpdatedLengthArray[i]+(BaseBoneLength * NormalisedBonePercArray[i]))
						)
					)
				)
				else --Else, If it is set to linear just use the base length.
				(
					for i = 1 to i = NumBones do
					(
					append UpdatedLengthArray BaseBoneLength
					)
				)
				--Next: Change the length array into the final percent  array
				for i = 1 to i = NumBones do
				(
					local AddedValue = 0
					if i != 1 do (for b = 1 to b = (i-1) do (AddedValue += UpdatedLengthArray[b]))
					append FinalPositionPercArray (AddedValue/BaseDistance)
				)
				--Next: Change the final percent array into the final position array
				local C
				C = (StartObj.transform.pos - EndObj.transform.pos)
				
				for i = 1 to i = NumBones do
				(
					append FinalBonePositionArray (StartObj.transform.pos - (C*FinalPositionPercArray[i]))
				)
				append FinalBonePositionArray EndObj.transform.pos
			)
			
			if spinebend == undefined and exactpercents != undefined do --if there is exact percents but no spinebend
			(
				local C
				C = (StartObj.transform.pos - EndObj.transform.pos)
				for i = 1 to i = NumBones do
				(
					local CurrentPercent
					CurrentPercent = (ExactPercents[i] as float)
					append FinalBonePositionArray (StartObj.transform.pos - (C*(CurrentPercent/100)))
				)
				append FinalBonePositionArray EndObj.transform.pos
				for i = 1 to i = ExactPercents.count do 
				(
					local CurrentPercent
					CurrentPercent = ((ExactPercents[i]) as float)
					append FinalPositionPercArray (CurrentPercent/100)
				)

			)
			

			
			if spinebend != undefined and ExactPercents == undefined do --if there is spinebend, but no specified bone percents are inputted then calculate locations along the spline
			(
				local SpineBendPathConsPercArray = #()
				append SpineBendPathConsPercArray 0
				for i = 1 to i = NumBones-1 do 
				(
					append SpineBendPathConsPercArray ((100/Numbones)*i)
				)
				append SpineBendPathConsPercArray 100
				for l = 1 to l = SpineBendPathConsPercArray.count do (append FinalPositionPercArray ((SpineBendPathConsPercArray[l])*0.01))
				local CurrentLetter,NameToAdd,UnusedNumber,TempLocation
				local TempCurveNulls = #()
				for i = 1 to i = SpineBendPathConsPercArray.count do
				(
					CurrentLetter = CommonFns.ConvertIntegerToAlphabetNumber i
					UnusedNumber = CommonFns.CheckIfExists ("ZZZTempCurveNull" + CurrentLetter) 1 999
					UnusedNumber = ((CommonFns.GetValueWithPadding UnusedNumber) as string)
					NameToAdd = Point Name:("ZZZTempCurveNull" + CurrentLetter + UnusedNumber ) wirecolor:ToolSetup.Greycolor Box:Off cross: off centermarker: off axistripod: on size:TinySize
					append TempCurveNulls NameToAdd
					CommonFns.AssignPathConstraint NameToAdd MainSpline SpineBendPathConsPercArray[i]
					TempLocation = in coordsys world NameToAdd.transform.pos
					append FinalBonePositionArray Templocation
				)
				delete TempCurveNulls
			)
			
			if spinebend != undefined and ExactPercents != undefined do --if there is both spinebend and exact percents then just use the exact percents
			(
				local CurrentLetter,NameToAdd,UnusedNumber,TempLocation
				local TempCurveNulls = #()
				for i = 1 to i = ExactPercents.count do
				(
					CurrentLetter = CommonFns.ConvertIntegerToAlphabetNumber i
					UnusedNumber = CommonFns.CheckIfExists ("ZZZTempCurveNull" + CurrentLetter) 1 999
					UnusedNumber = ((CommonFns.GetValueWithPadding UnusedNumber) as string)
					NameToAdd = Point Name:("ZZZTempCurveNull" + CurrentLetter + UnusedNumber ) wirecolor:ToolSetup.Greycolor Box:Off cross: off centermarker: off axistripod: on size:TinySize
					append TempCurveNulls NameToAdd
					CommonFns.AssignPathConstraint NameToAdd MainSpline ExactPercents[i]
					TempLocation = in coordsys world NameToAdd.transform.pos
					append FinalBonePositionArray Templocation
				)
				for i = 1 to i = ExactPercents.count do 
					(
					local CurrentPercent
					CurrentPercent = (ExactPercents[i] as float)
					append FinalPositionPercArray ((CurrentPercent)/100))
				delete TempCurveNulls
				
			)
				
			
				
				
			-------------------------------------MAKE ALL OF THE BONE CHAINS-------------------------------------------------------------
			local PointName,CurrentLetter,NameToAdd
			local PositiveIKAnchors = #()
			local NegativeIKAnchors = #()
			local StretchAnchors = #()
			local BlendedIKAnchors = #()
			local MainBonesArray = #()
			
			local UpnodeGroupsArray = #()
			local UpnodeArray = #()
			
			
			--JointScaling SquashStretch EaseType NumBones EaseValue PickedOri NoScalePos SquashType

			--First: The PositiveIK Chain
			if JointScaling != 1 and NoScalePos != 1 do
			(
				for i = 1 to i = FinalBonePositionArray.count do
				(
					CurrentLetter = CommonFns.ConvertIntegerToAlphabetNumber i
					UnusedNumber = CommonFns.CheckIfExists (PositiveSplineIKName + CurrentLetter) 1 999
					UnusedNumber = ((CommonFns.GetValueWithPadding UnusedNumber) as string)
					NameToAdd = Point Name:(PositiveSplineIKName + CurrentLetter + UnusedNumber ) wirecolor:Green Box:Off cross: off centermarker: off axistripod: on size:TinySize
					in coordsys world NameToAdd.rotation = GlobalController.rotation
					NameToAdd.pos = (FinalBonePositionArray[i])
					append PositiveIKAnchors NameToAdd
					if i == 1 then PositiveIKAnchors[i].parent = GlobalController else (PositiveIKAnchors[i].parent = PositiveIKAnchors[(i-1)])
				)
				
				for i = 1 to i = PositiveIKAnchors.count do (append Allcreated PositiveIKAnchors[i])
			)
			
			--Next: TheNegativeIK Chain
			
			if JointScaling != 1 and NoScalePos !=3 do
			(
				for i = 1 to i = FinalBonePositionArray.count do
				(
					CurrentLetter = CommonFns.ConvertIntegerToAlphabetNumber i
					UnusedNumber = CommonFns.CheckIfExists (NegativeSplineIKName + CurrentLetter) 1 999
					UnusedNumber = ((CommonFns.GetValueWithPadding UnusedNumber) as string)
					NameToAdd = Point Name:(NegativeSplineIKName + CurrentLetter + UnusedNumber ) wirecolor:Blue Box:Off cross: off centermarker: off axistripod: on size:TinySize
					in coordsys world NameToAdd.rotation = GlobalController.rotation
					NameToAdd.pos = (FinalBonePositionArray[(FinalBonePositionArray.count-(i-1))])
					append NegativeIKAnchors NameToAdd
					if i == 1 then NegativeIKAnchors[i].parent = GlobalController else (NegativeIKAnchors[i].parent = NegativeIKAnchors[(i-1)])
				)
				
				for i = 1 to i = NegativeIKAnchors.count do (append Allcreated NegativeIKAnchors[i])
			)
			--Next:The Stretch Anchor Chain
			
			if JointScaling != 3 do
			(
				local PathConsPercArray = #()
				for i = 1 to i = FinalPositionPercArray.count do 
					
				
				(
					append PathConsPercArray ((FinalPositionPercArray[i])*100)
				)
				append PathConsPercArray 100
				
				
				for i = 1 to i = FinalBonePositionArray.count do
				(
					CurrentLetter = CommonFns.ConvertIntegerToAlphabetNumber i
					UnusedNumber = CommonFns.CheckIfExists (StretchAnchorName + CurrentLetter) 1 999
					UnusedNumber = ((CommonFns.GetValueWithPadding UnusedNumber) as string)
					NameToAdd = Point Name:(StretchAnchorName + CurrentLetter + UnusedNumber ) wirecolor:ToolSetup.Greycolor Box:Off cross: off centermarker: off axistripod: on size:TinySize
					in coordsys world NameToAdd.rotation = GlobalController.rotation
					NameToAdd.pos = (FinalBonePositionArray[i])
					append StretchAnchors NameToAdd
					StretchAnchors[i].parent = GlobalController
					CommonFns.AssignPathConstraint NameToAdd MainSpline PathConsPercArray[i]
					deletekeys NameToAdd.position.controller
					append Allcreated NameToAdd
					
				)
			)
			
			---Next: The IK Blend Chain
			
			local TestState = true
			if JointScaling == 1 then (TestState = false)
			if jointScaling == 3 and NoScalePos == 1 then (TestState = false)
			else if jointScaling == 3 and NoScale == 3 then (TestState = false)
			
			
			
			if TestState == true do
			(
				for i = 1 to i = FinalBonePositionArray.count do
				(
					CurrentLetter = CommonFns.ConvertIntegerToAlphabetNumber i
					UnusedNumber = CommonFns.CheckIfExists (BlendIKName + CurrentLetter) 1 999
					UnusedNumber = ((CommonFns.GetValueWithPadding UnusedNumber) as string)
					NameToAdd = Point Name:(BlendIKName + CurrentLetter + UnusedNumber ) wirecolor:Red Box:Off cross: off centermarker: off axistripod: on size:TinySize
					in coordsys world NameToAdd.rotation = GlobalController.rotation
					NameToAdd.pos = (FinalBonePositionArray[i])
					append BlendedIKAnchors NameToAdd
					if i == 1 then BlendedIKAnchors[i].parent = GlobalController else (BlendedIKAnchors[i].parent = BlendedIKAnchors[(i-1)])
					append Allcreated NameToAdd
				)
			)
			
			-- If there is rig scaling, an additional array of points will need to be added
			
			if jointscaling != 3 then --if there is joint scaling, make the extra helpers
			(
			local JointScaleHelpers =#()
			local UnusedNumber,CurrentLetter,StretchHelperToAdd
			for i = 1 to i = FinalBonePositionArray.count do
			(
				CurrentLetter = CommonFns.ConvertIntegerToAlphabetNumber i
				UnusedNumber = CommonFns.CheckIfExists (StretchBoneHelperName + CurrentLetter) 1 999
				UnusedNumber = ((CommonFns.GetValueWithPadding UnusedNumber) as string)
				StretchHelperToAdd = Point Name:(StretchBoneHelperName + CurrentLetter + UnusedNumber ) pos:FinalBonePositionArray[i] wirecolor:blue Box:off cross: off centermarker: off axistripod: on size:TinySize
				in coordsys world StretchHelperToAdd.rotation = GlobalController.transform.rotation
				in coordsys world StretchHelperToAdd.pos = FinalBonePositionArray[i]
				append JointScaleHelpers StretchHelperToAdd
				StretchHelperToAdd.parent = GlobalController
				append Allcreated StretchHelperToAdd
			)
			)
			
			
			
			
			
			
			--Finally: The main bones
			local BoneToAdd
			for i = 1 to i = FinalBonePositionArray.count-1 do
			(
				CurrentLetter = CommonFns.ConvertIntegerToAlphabetNumber i
				UnusedNumber = CommonFns.CheckIfExists (MainBoneName + CurrentLetter) 1 999
				UnusedNumber = ((CommonFns.GetValueWithPadding UnusedNumber) as string)
				BoneToAdd = boneSys.createBone FinalBonePositionArray[i] FinalBonePositionArray[(i+1)] [0,0,1]
				BoneToAdd.width = TinySize*1.2
				BoneToAdd.height = TinySize*1.2
				BoneToAdd.taper = 70
				BoneToAdd.sideFins = off
				BoneToAdd.frontFin = off
				BoneToAdd.backFin = off
				BoneToAdd.name = (MainBoneName + CurrentLetter + UnusedNumber)
				append MainBonesArray BoneToAdd
				BoneToAdd.parent = GlobalController
				append Allcreated BoneToAdd
			)
			

			local BoneToAdd
			UnusedNumber = CommonFns.CheckIfExists (MainBoneName + "_Nub") 1 999
			UnusedNumber = ((CommonFns.GetValueWithPadding UnusedNumber) as string)
			BoneToAdd = boneSys.createBone FinalBonePositionArray[i] FinalBonePositionArray[i] [0,0,1]
			BoneToAdd.name = (MainBoneName + "_Nub" + UnusedNumber)
			in coordsys world BoneToAdd.rotation = GlobalController.rotation
			in coordsys world BoneToAdd.pos = FinalBonePositionArray[FinalBonePositionArray.count]
			append MainBonesArray BoneToAdd
			BoneToAdd.parent = GlobalController
			append Allcreated BoneToAdd
			
			
			
			if jointscaling != 3 do --if there is joint scaling, then parent these bones to eachother and enable joint scaling
			(
				for i = 1 to i = ((mainBonesArray.count)-1) do 
				(
					MainBonesArray[((MainBonesArray.count)-(i-1))].parent = MainBonesArray[((MainBonesArray.count)-(i))]
				)
				for i = 1 to i = Mainbonesarray.count do
				(
					MainBonesArray[i].boneFreezeLength=false
				)
				
				if SquashStretch != 1 do --if there is squash and stretch, add the stretch nubs and reparent the hierarchy
				(
					for i = 2 to i = ((mainBonesArray.count)-1) do 
					(
						local BoneToAdd
						mainBonesArray[i].parent = globalcontroller
						BoneToAdd = boneSys.createBone [0,0,0] [0,0,TinySize*0.2] [0,0,1]
						BoneToAdd.name = (uniquename StretchNubName NumDigits:3)
						BoneToAdd.wirecolor = (color 20 20 60)
						BoneToAdd.frontfin = off
						BoneToAdd.sidefins = off
						BoneToAdd.backfin = off
						BoneToAdd.width = tinysize
						BoneToAdd.height = tinysize
						CommonFns.QuickAlign BoneToAdd mainBonesArray[i]
						BoneToAdd.parent = mainBonesArray[(i-1)]
						append allcreated BoneToAdd
						local poscons = position_constraint()
						poscons.appendtarget JointScaleHelpers[i] 100
						BoneToAdd.pos.controller = poscons
					)
				)
				
				
			)
			
			----------------------If scaling is on, Apply the position and lookat constraints to the main bones to create a stretchy chain-----------------------
			if jointscaling != 3 do
			(
				for i = 1 to i = mainbonesarray.count-1 do
				(
					local poscons = position_constraint()
					local lookcons = LookAt_constraint()
					LookCons.appendtarget JointScaleHelpers[i+1] 100
					LookCons.lookat_vector_length = 0
					LookCons.viewline_length_abs = false
					LookCons.upnode_ctrl = 1
					LookCons.upnode_world = false
					LookCons.pickUpNode = JointScaleHelpers[i]
					LookCons.StoUP_axis = 2
					LookCons.upnode_axis = 2
					mainbonesarray[i].rotation.controller = lookcons
					poscons.appendtarget JointScaleHelpers[i] 100
					mainbonesarray[i].pos.controller = poscons
				)
				local poscons = position_constraint()
				poscons.appendtarget JointScaleHelpers[jointscalehelpers.count] 100
				mainbonesarray[mainbonesarray.count].pos.controller = poscons
				poscons = copy poscons
			)
			
			
			-----------------------------------If using upnodes instead of scripts-----------------
			
			local StartControlExposeTfm,EndControlExposeTfm
			
			
			
			if Upnodes == True do
			(
				
				--------Make the Upnodes-------------------
				
				local UpNodePosCons
				
				StartControlExposeTfm = ExposeTransform Name:(Uniquename (ExposeTfmName + "Pelvis") Numdigits:3) Size:TinySize Box:On Cross:Off AxisTripod:Off color:(color 20 180 60)
				EndControlExposeTfm = ExposeTransform Name:(Uniquename (ExposeTfmName + "Ribcage") Numdigits:3) Size:TinySize Box:On Cross:Off AxisTripod:Off color:(color 20 180 60)
				StartControlExposeTfm.parent = globalcontroller
				EndControlExposeTfm.parent = globalcontroller
				
				append allcreated StartControlExposeTfm
				append allcreated EndControlExposeTfm

				StartControlExposeTfm.exposeNode = StartController
				StartControlExposeTfm.useParent = off
				StartControlExposeTfm.localReferenceNode = GlobalController
				StartControlExposeTfm.eulerXOrder = 1
				StartControlExposeTfm.eulerYOrder = 1
				StartControlExposeTfm.eulerZOrder = 1
				
				EndControlExposeTfm.exposeNode = EndController
				EndControlExposeTfm.useParent = off
				EndControlExposeTfm.localReferenceNode = GlobalController
				EndControlExposeTfm.eulerXOrder = 1
				EndControlExposeTfm.eulerYOrder = 1
				EndControlExposeTfm.eulerZOrder = 1
				
				--BoneToAdd.rotation.controller.x_rotation.controller.AddScalarTarget "LocalEulerX" AnkleExposeObjs[1].baseobject[#local_euler_x]
				
				local UpNodePosCons,UpnodeRotExpression,UpnodeRibcageExpression
				
				UpnodeRotExpression = float_expression()
				UpnodeRotExpression.AddScalarTarget "StartX" StartControlExposeTfm.baseobject[#local_euler_x]
				UpnodeRotExpression.AddScalarTarget "EndX" EndControlExposeTfm.baseobject[#local_euler_x]
				UpNodeRotExpression.setexpression ("(StartX)")
				
				UpnodeRibcageExpression = float_expression()
				UpnodeRibcageExpression.AddScalarTarget "StartX" StartControlExposeTfm.baseobject[#local_euler_x]
				UpnodeRibcageExpression.AddScalarTarget "EndX" EndControlExposeTfm.baseobject[#local_euler_x]
				UpnodeRibcageExpression.setexpression ("(EndX)")
				
				
				for i = 1 to i = FinalBonePositionArray.count do
				(
					CurrentLetter = CommonFns.ConvertIntegerToAlphabetNumber i
					UnusedNumber = CommonFns.CheckIfExists (UpNodeName + CurrentLetter) 1 999
					UnusedNumber = ((CommonFns.GetValueWithPadding UnusedNumber) as string)
					NameToAdd = Point Name:(UpNodeName + CurrentLetter + UnusedNumber ) wirecolor:(color 60 170 190) Box:Off cross: off centermarker: off axistripod: on size:TinySize
					in coordsys world NameToAdd.rotation = GlobalController.rotation
					NameToAdd.pos = (FinalBonePositionArray[i])
					append UpNodeArray NameToAdd
					NameToAdd.parent = GlobalController
					append Allcreated NameToAdd
					UpNodePosCons = copy UpNodePosCons
					UpNodePosCons = position_constraint()
					if JointScaling ==1 then (UpNodePosCons.appendtarget StretchAnchors[i] 100)
					else if (JointScaling == 2) or (JointScaling ==3 and NoScalePos == 2) then (UpNodePosCons.appendtarget BlendedIKAnchors[i] 100)
					else if (JointScaling == 3) and (NoScalePos == 1) then (UpNodePosCons.appendtarget NegativeIKAnchors[i] 100)
					else if (JointScaling == 3) and (NoScalePos == 6) then (UpNodePosCons.appendtarget PositiveIKAnchors[i] 100)
					NameToAdd.pos.controller = upnodeposcons
					
					NameToAdd.rotation.controller = Rotation_List()
					NameToAdd.rotation.controller.available.controller = Euler_XYZ()
					NameToAdd.rotation.controller.available.controller = Euler_XYZ()
					NameToAdd.rotation.controller[2].controller.X_Rotation.controller = UpnodeRotExpression
					
					NameToAdd.rotation.controller[3].controller.X_Rotation.controller = UpnodeRibcageExpression
					
					NameToAdd.rotation.controller.weight[2] = (100-((i-1 as float)*(100.00/((FinalBonePositionArray.count as float)-2))))
					NameToAdd.rotation.controller.weight[3] = ((i-1 as float)* (100.00/((FinalBonePositionArray.count as float)-2)))
						
					
				)
				
				
						--	local PointName,CurrentLetter,NameToAdd
			--local PositiveIKAnchors = #()
			--local NegativeIKAnchors = #()
			--local StretchAnchors = #()
			--local BlendedIKAnchors = #()
			--local MainBonesArray = #()
			
		--	local UpnodeGroupsArray = #()
			--local UpnodeArray = #()
							--local UpNodeGroupName = (Prefix + "Grp_Upnode")
			--local UpNodeName =  (Prefix + "CnsTarget_Upnode")
				
				
			)


					
			
			
			
			------------------------Apply The SplineIK Solvers To the Two Chains and the lookat to the stretch--------------------------------
			local IKChainPositive
			local IKChainNegative
			
			if PositiveIKAnchors.count != 0 do
			(
					IKChainPositive = ikSys.ikChain PositiveIKAnchors[1] PositiveIKAnchors[PositiveIKAnchors.count] "SplineIKSolver"
					UnusedNumber = CommonFns.CheckIfExists (IKHelperName + "Positive") 1 999
					UnusedNumber = ((CommonFns.GetValueWithPadding UnusedNumber) as string)
					IKChainPositive.Name = (IKHelperName+ "Positive" + UnusedNumber)
					IKChainPositive.parent = GlobalController
					IKChainPositive.transform.controller.goalSize = TinySize
					IKChainPositive.transform.controller.pickShape = MainSpline
					IKChainPositive.transform.controller.upnode = Globalcontroller
					CommonFns.AssignPathConstraint PositiveIKAnchors[1] MainSpline 0
					deletekeys PositiveIKAnchors[1].position.controller
					append Allcreated IKChainPositive
			)
			
			if NegativeIKAnchors.count != 0 do
			(
				IKChainNegative = ikSys.ikChain NegativeIKAnchors[1] NegativeIKAnchors[NegativeIKAnchors.count] "SplineIKSolver"
				IKChainNegative.parent = GlobalController
				UnusedNumber = CommonFns.CheckIfExists (IKHelperName + "Negative") 1 999
				UnusedNumber = ((CommonFns.GetValueWithPadding UnusedNumber) as string)
				IKChainNegative.Name = (IKHelperName+ "Negative" + UnusedNumber)
				IKChainNegative.transform.controller.goalSize = TinySize
				IKChainNegative.transform.controller.pickShape = ReverseSpline
				IKChainNegative.transform.controller.upnode = Globalcontroller
				CommonFns.AssignPathConstraint NegativeIKAnchors[1] ReverseSpline 0
				deletekeys NegativeIKAnchors[1].position.controller
				append Allcreated IKChainNegative
			)
			
			if StretchAnchors.count != 0 do
			(
				for i = 1 to i = StretchAnchors.count-1 do
				(
				local LookCons
				local Nextanchor = StretchAnchors[i+1]
				local CurrentAnchor = StretchAnchors[i]
				Currentanchor.Rotation.Controller = LookAt_Constraint()
				Currentanchor.Rotation.Controller.appendTarget Nextanchor 100
				Currentanchor.Rotation.Controller.target_axis = 0
				Currentanchor.Rotation.Controller.lookat_vector_length = 0
				Currentanchor.Rotation.Controller.viewline_length_abs = false
				Currentanchor.Rotation.Controller.target_axisFlip = off
				Currentanchor.Rotation.Controller.upnode_ctrl = 1
				Currentanchor.Rotation.Controller.upnode_world = false
				Currentanchor.Rotation.Controller.StoUP_axis = 1
				Currentanchor.Rotation.Controller.upnode_axis = 1
				Currentanchor.Rotation.Controller.upnode_world = false	
				Currentanchor.Rotation.Controller.pickUpNode = PositiveIKAnchors[i]	
			)
			)
			-------------------------------Apply The Custom Attributes to the global Controller-------------------------------------------

			local ca
			local AttHolder
			
			if MainControlObject == undefined then (MainControlObject = GlobalController)
			
			ca=attributes SpineControls
				(
					parameters params rollout:ro_SpineControls
					(
						paramscale type:#float ui:spn_ScaleNoscale
						paramposorneg type:#float ui:spn_PosOrNeg
						paramSquash type:#float ui:spn_Squash
						paramFinAuto type:#float ui:spn_FinAuto
						paramPelvisFollow type:#float ui:spn_PelvisFollow
						paramRibcageFollow type:#float ui:spn_RibcageFollow
						paramTwistDisperse type:#float ui:spn_TwistDisperse
					)
					rollout ro_SpineControls "Spine_Controls" 
					(
						spinner spn_ScaleNoscale "Bone Scaling" enabled:on  range:[0,100,0] scale:0.2
						spinner spn_PosOrNeg "Chain Up/down" enabled:on  range:[0,100,0] scale:0.2
						spinner spn_FinAuto "Fin Auto" enabled:on  range:[0,1,1] scale:0.1	
						spinner spn_PelvisFollow "Pelvis Nullify" enabled:on  range:[0,100,0] scale:0.1	
						spinner spn_RibcageFollow "Ribcage Nullifiy" enabled:on  range:[0,100,0] scale:0.1
						spinner spn_TwistDisperse "Twist Dispersion" enabled:on  range:[-1,1,0] scale:0.1
						spinner spn_Squash "Squash Enabled" enabled:on  range:[0,3,0] scale:0.01	
							
						on ro_spineControls open do
						(
							local ScaleNoScaleEnabled,PorOrNegEnabled,FinAutoEnabled,PelvisFollowEnabled,RibcageFollowEnabled,TwistDisperseEnabled,SquashSliderEnabled
							ScaleNoScaleEnabled = (GetUserProp $ "ScaleNoScaleEnabled") as string
							PorOrNegEnabled = (GetUserProp $ "PorOrNegEnabled") as string
							FinAutoEnabled = (GetUserProp $ "FinAutoEnabled") as string
							PelvisFollowEnabled = (GetUserProp $ "PelvisFollowEnabled") as string
							RibcageFollowEnabled = (GetUserProp $ "RibcageFollowEnabled") as string
							TwistDisperseEnabled = (GetUserProp $ "TwistDisperseEnabled") as string
							SquashSliderEnabled = (GetUserProp $ "SquashSliderEnabled") as string
							
							if ScaleNoScaleEnabled == "off" then (ScaleNoScaleEnabled = off) else (ScaleNoScaleEnabled = on)
							if PorOrNegEnabled == "off" then (PorOrNegEnabled = off) else (PorOrNegEnabled = on)
							if FinAutoEnabled == "off" then (FinAutoEnabled = off) else (FinAutoEnabled = on)
							if PelvisFollowEnabled == "off" then (PelvisFollowEnabled = off) else (PelvisFollowEnabled = on)
							if RibcageFollowEnabled == "off" then (RibcageFollowEnabled = off) else (RibcageFollowEnabled = on)
							if TwistDisperseEnabled == "off" then (TwistDisperseEnabled = off) else (TwistDisperseEnabled = on)
							if SquashSliderEnabled == "off" then (SquashSliderEnabled = off) else (SquashSliderEnabled = on)
							
							try
							(
							spn_ScaleNoscale.visible = ScaleNoScaleEnabled
							spn_PosOrNeg.visible = PorOrNegEnabled
							spn_FinAuto.visible = FinAutoEnabled
							spn_PelvisFollow.visible = PelvisFollowEnabled
							spn_RibcageFollow.visible = RibcageFollowEnabled
							spn_TwistDisperse.visible = TwistDisperseEnabled
							spn_Squash.visible = SquashSliderEnabled
							)
							catch()
						)
					)
				)
				AttHolder = (EmptyModifier ())
				AttHolder.name = "Spine_Controls"
				addmodifier MainControlObject AttHolder
				custAttributes.add MainControlObject.modifiers[#Spine_Controls] ca
					
					
				if JointScaling == 2 then (SetUserProp MainControlObject "ScaleNoScaleEnabled" "on") else (SetUserProp MainControlObject "ScaleNoScaleEnabled" "off") 
				if NoScalePos == 2 and JointScaling != 1 then (SetUserProp MainControlObject "PorOrNegEnabled"  "on") else (SetUserProp MainControlObject "PorOrNegEnabled"  "off")
				if includefins != undefined then (SetUserProp MainControlObject "FinAutoEnabled" "on") else (SetUserProp MainControlObject "FinAutoEnabled" "off")
				if TangentRotNullifiers == true then (SetUserProp MainControlObject "PelvisFollowEnabled" "on") else (SetUserProp MainControlObject "PelvisFollowEnabled" "off")
				if TangentRotNullifiers == true then (SetUserProp MainControlObject "RibcageFollowEnabled" "on") else (SetUserProp MainControlObject "RibcageFollowEnabled" "off")
				if LiveTwist == true then (SetUserProp MainControlObject "TwistDisperseEnabled" "on") else (SetUserProp MainControlObject "TwistDisperseEnabled" "off") 
				if SquashStretch != 1 then (SetUserProp MainControlObject "SquashSliderEnabled" "on") else (SetUserProp MainControlObject "SquashSliderEnabled" "off")
					
					
			----------------------------------------------------Apply the position switch controller to the blend bones------------------------------------
					
			if BlendedIKAnchors.count != 0 and NegativeIKAnchors.count != 0 and PositiveIKAnchors.count != 0 then
			(
				local posSwitchController = float_expression ()
				local poscons
					
				PosSwitchController.AddScalarTarget "ScaleBlend" MainControlObject.modifiers[#Spine_Controls].SpineControls[#paramscale]
				PosSwitchController.AddScalarTarget "NegativeAmount" MainControlObject.modifiers[#Spine_Controls].SpineControls[#paramposorneg]
				PosSwitchController.setexpression "(NegativeAmount * ((100 - ScaleBlend)/100))" --This is for the second position weight
				PosCons = Position_Constraint()
				PosCons.appendTarget Startcontroller 100
				PosCons.appendTarget NegativeIKAnchors[NegativeIKAnchors.count] 100
				BlendedIKAnchors[1].pos.controller = Poscons
				BlendedIKAnchors[1].pos.controller[#Position_Weight_1].controller = posSwitchController		
				posSwitchController = copy PosSwitchController
				PosSwitchController.setexpression "(100-(NegativeAmount * ((100 - ScaleBlend)/100)))"		
				BlendedIKAnchors[1].pos.controller[#Position_Weight_0].controller = posSwitchController
			)
			else if BlendedIKAnchors.count != 0 and NegativeIKAnchors.count != 0  and PositiveIKAnchors.count == 0 then
			(
				local posSwitchController = float_expression ()
				local poscons
					
				PosSwitchController.AddScalarTarget "ScaleBlend" MainControlObject.modifiers[#Spine_Controls].SpineControls[#paramscale]
				PosSwitchController.setexpression "100-ScaleBlend" --This is for the second position weight
				PosCons = Position_Constraint()
				PosCons.appendTarget Startcontroller 100
				PosCons.appendTarget NegativeIKAnchors[NegativeIKAnchors.count] 100
				BlendedIKAnchors[1].pos.controller = Poscons
				BlendedIKAnchors[1].pos.controller[#Position_Weight_1].controller = posSwitchController		
				posSwitchController = copy PosSwitchController
				PosSwitchController.setexpression "ScaleBlend"		
				BlendedIKAnchors[1].pos.controller[#Position_Weight_0].controller = posSwitchController
			)
			else if BlendedIKAnchors.count != 0 and NegativeIKAnchors.count == 0 and PositiveIKAnchors.count != 0 then
			(
				local poscons
				PosCons = Position_Constraint()
				PosCons.appendTarget Startcontroller 100
				BlendedIKAnchors[1].pos.controller = Poscons
			)
			----------------------------------------------------Local Orientation Blend Controller----------------------------------------------------------------
			
			local OriCons
			local OriSwitchController
			
			if BlendedIKAnchors.count  != 0 and PositiveIKAnchors.count != 0 and NegativeIKAnchors.count != 0 and StretchAnchors.count != 0 then
			(
				for i = 1 to i = BlendedIKAnchors.count-1 do
				(
					OriCons = Copy OriCons
					OriCons = Orientation_Constraint()
					OriCons.appendTarget StretchAnchors[i] 100
					OriCons.appendTarget PositiveIKAnchors[i] 100
					OriCons.appendTarget NegativeIKAnchors[NegativeIKAnchors.count-(i)] 100
					OriCons.local_world = 0
					BlendedIKAnchors[i].rotation.controller = rotation_list ()
					BlendedIKAnchors[i].rotation.controller.Available.controller = OriCons
					
					OriSwitchController = copy OriSwitchController
					OriSwitchController = undefined
					OriSwitchController = float_expression()
					
					OriSwitchController.AddScalarTarget "ScaleBlend" MainControlObject.modifiers[#Spine_Controls].SpineControls[#paramscale]
					OriSwitchController.AddScalarTarget "NegativeAmount" MainControlObject.modifiers[#Spine_Controls].SpineControls[#paramposorneg]
					OriSwitchController.setexpression "ScaleBlend"
					BlendedIKAnchors[i].rotation.controller.Orientation_Constraint.controller.weight[1].controller = OriSwitchController
					OriSwitchController = copy OriSwitchController
					OriSwitchController.setexpression "(100-NegativeAmount)*((100-ScaleBlend)/100)"
					BlendedIKAnchors[i].rotation.controller.Orientation_Constraint.controller.weight[2].controller = OriSwitchController
					OriSwitchController = copy OriSwitchController
					OriSwitchController.setexpression "(NegativeAmount)*((100-ScaleBlend)/100)"
					BlendedIKAnchors[i].rotation.controller.Orientation_Constraint.controller.weight[3].controller = OriSwitchController
				)
			)
			else if BlendedIKAnchors.count != 0 and StretchAnchors.count != 0 then
			(
				for i = 1 to i = BlendedIKAnchors.count-1 do
				(
					OriCons = Copy OriCons
					OriCons = Orientation_Constraint()
					OriCons.appendTarget StretchAnchors[i] 100
					if NegativeIKAnchors.count != 0 do (OriCons.appendTarget NegativeIKAnchors[NegativeIKAnchors.count-(i)] 100)
					if PositiveIKAnchors.count  != 0 do  (OriCons.appendTarget PositiveIKAnchors[i] 100)
					OriCons.local_world = 0
					BlendedIKAnchors[i].rotation.controller = rotation_list ()
					BlendedIKAnchors[i].rotation.controller.Available.controller = OriCons

					OriSwitchController = copy OriSwitchController
					OriSwitchController = undefined
					OriSwitchController = float_expression()
					
					OriSwitchController.AddScalarTarget "ScaleBlend" MainControlObject.modifiers[#Spine_Controls].SpineControls[#paramscale]
					OriSwitchController.setexpression "ScaleBlend"
					BlendedIKAnchors[i].rotation.controller.Orientation_Constraint.controller.weight[1].controller = OriSwitchController
					OriSwitchController = copy OriSwitchController
					OriSwitchController.setexpression "(100-ScaleBlend)"
					BlendedIKAnchors[i].rotation.controller.Orientation_Constraint.controller.weight[2].controller = OriSwitchController
				)
			)
			else if BlendedIKAnchors.count  != 0 and StretchAnchors.count == 0 then
			(
				for i = 1 to i = BlendedIKAnchors.count-1 do
				(
					OriCons = Copy OriCons
					OriCons = Orientation_Constraint()
					if PositiveIKAnchors.count != 0 do (OriCons.appendTarget PositiveIKAnchors[i] 100)
					if NegativeIKAnchors.count != 0 do (	OriCons.appendTarget NegativeIKAnchors[NegativeIKAnchors.count-(i)] 100)
					OriCons.local_world = 0
					BlendedIKAnchors[i].rotation.controller = rotation_list ()
					BlendedIKAnchors[i].rotation.controller.Available.controller = OriCons
					
					if PositiveIKAnchors.count != 0 and NegativeIKAnchors.count != 0 then
					(
						OriSwitchController = copy OriSwitchController
						OriSwitchController = undefined
						OriSwitchController = float_expression()
						
						OriSwitchController.AddScalarTarget "NegativeAmount" MainControlObject.modifiers[#Spine_Controls].SpineControls[#paramposorneg]
						OriSwitchController.setexpression "(100-NegativeAmount)"
						BlendedIKAnchors[i].rotation.controller.Orientation_Constraint.controller.weight[1].controller = OriSwitchController
						OriSwitchController = copy OriSwitchController
						OriSwitchController.setexpression "(NegativeAmount)"
						BlendedIKAnchors[i].rotation.controller.Orientation_Constraint.controller.weight[2].controller = OriSwitchController
					)
				)
			)
			
			
			----------------------------------------------------Scale Blend Controllers---------------------------------------------------------------------------------------
			
			
			--THIS WILL CHANGE
			
			if BlendedIKAnchors.count != 0 and StretchAnchors.count != 0 do
			(
				local PosExtendController = position_script()
				local OrigLength
				local PosExpressionString
				for i = 2 to i = BlendedIKAnchors.count do
				(
					PosExtendController = copy PosExtendController
					PosExtendController = undefined
					PosExtendController =  position_script()
					PosExtendController.AddTarget "ScaleBlend" MainControlObject.modifiers[#Spine_Controls].SpineControls[#paramscale]
					if NegativeIKAnchors.count != 0 do 
					(
						local ObjA
						local ObjB
						ObjA = NegativeIKAnchors[NegativeIKAnchors.count-(i-1)]
						ObjB = NegativeIKAnchors[NegativeIKAnchors.count-(i-2)]
						OrigLength = (distance ObjA ObjB)
					)
					if PositiveIKAnchors.count != 0 do (OrigLength = PositiveIKAnchors[i].pos.controller.X_Position as float)
					

					PosExtendController.AddNode "CurrentStretch" StretchAnchors[i]
					PosExtendController.AddNode "PreviousStretch" StretchAnchors[i-1]
					PosExpressionString = "local Difference = (CurrentStretch.transform * inverse PreviousStretch.transform).position\n" +
						"local C = " + (OrigLength as string) + " - Difference.x\n" +
						"local AmountToSub = C * (ScaleBlend/100)\n"+
						"local OutputValue = (" + (OrigLength as string) + " - AmountToSub)\n" +
						"[OutputValue,0,0]"
						
					PosExtendController.SetExpression PosExpressionString
					BlendedIKAnchors[i].pos.controller = PosExtendController
				)
			)
			
			---------------------------------------------------Set Up the final  bone position constraint--------------------------------------------
			
			local BonePosCons
			BonePosCons = Position_Constraint()
			if BlendedIKAnchors.count != 0 then 
			(
				for i = 1 to i = MainBonesArray.count do
				(
					local ApplyToThis
					if jointScaling == 3 then (ApplyToThis = MainBonesArray[i]) else (ApplyToThis = JointScaleHelpers[i])
					BonePosCons = copy BonePosCons
					BonePosCons = undefined
					BonePosCons = Position_Constraint()
					BonePosCons.appendtarget BlendedIKAnchors[i] 100
					ApplyToThis.pos.controller = Position_List()
					ApplyToThis.pos.controller[1].controller = BonePosCons
					ApplyToThis.pos.controller.available.controller =Position_XYZ()
				)
			)
			else
			(
				for i = 1 to i = MainBonesArray.count do
				(
					BonePosCons = copy BonePosCons
					BonePosCons = undefined
					BonePosCons = Position_Constraint()
					if jointScaling == 3 then (ApplyToThis = MainBonesArray[i]) else (ApplyToThis = JointScaleHelpers[i])
					if StretchAnchors.count != 0 do (BonePosCons.appendtarget StretchAnchors[i] 100)
					if PositiveIKAnchors.count  != 0 do (BonePosCons.appendtarget PositiveIKAnchors[i] 100)
					if NegativeIKAnchors.count != 0 do (BonePosCons.appendtarget NegativeIKAnchors[(NegativeIKAnchors.count-(i-1))] 100)
					ApplyToThis.pos.controller = Position_List()
					ApplyToThis.pos.controller[1].controller = BonePosCons
					ApplyToThis.pos.controller.available.controller =Position_XYZ()
				)
			)
			
			---------------------------------------------------Set Up the final Twist/Lookat Expression On the Main Bone--------------------------------------------
			
			--Note: This part of the script is based heavily on felix joleane's chain tools setup. 
			--SOURCE: Joleanes, F. 2012. Chain Tools [online] Available From: http://joleanes.com/scripts_plugins/chaintools.php Accessed: 30/05/2014
			
			if  Upnodes != True do
			(				
			
				local ExpressionString
				local ScriptControlToApply
				local EnvDrivers
				if jointScaling == 3 then (EnvDrivers = MainBonesArray) else (EnvDrivers = JointScaleHelpers)

				for i = 1 to i = envdrivers.count do 
				(
					envDrivers[i].rotation.controller=rotation_list()
					envDrivers[i].rotation.controller[2].controller=euler_xyz()
					envDrivers[i].rotation.controller.active=2
				)
				
				ExpressionString = "posThis=posThisCtrl.value*inverse nodeParent.transform\n"+
									"posTarget=posTargetCtrl.value*inverse nodeParent.transform\n"+
									"vector=posTarget-posThis\n"+
									"axis=normalize (cross vector [1,0,0])\n"+
									"angle=acos (normalize vector).x\n" +
									"(quat angle axis)*nodeParent.transform.rotation"
				ScriptcontrolToApply =rotation_script()
				EnvDrivers[1].rotation.controller[1].controller =ScriptControlToApply
				ScriptControlToApply.addNode "nodeParent" GlobalController
				ScriptControlToApply.addObject "posTargetCtrl" EnvDrivers[2].position.controller
				ScriptControlToApply.addObject "posThisCtrl" EnvDrivers[1].position.controller
				ScriptControlToApply.script = ExpressionString
				ExpressionString = "posThis=posThisCtrl.value*inverse nodeParent.transform\n"+
									"posTarget=posTargetCtrl.value*inverse nodeParent.transform\n"+
									"vector=posTarget-posThis\n"+
									"axis=normalize (cross vector [1,0,0])\n"+
									"angle=acos (normalize vector).x\n" +
									"(quat rotX [1,0,0])*(quat angle axis)*nodeParent.transform.rotation"
				for i = 2 to i = (envdrivers.count-1) do
				(
				ScriptcontrolToApply =rotation_script()	
				EnvDrivers[i].rotation.controller[1].controller = ScriptControlToApply
				ScriptControlToApply.addNode "nodeParent" EnvDrivers[i-1]
				ScriptControlToApply.addObject "posTargetCtrl" EnvDrivers[i+1].position.controller
				ScriptControlToApply.addObject "posThisCtrl" EnvDrivers[i].position.controller
				ScriptControlToApply.addTarget "rotX" envdrivers[i-1].rotation.controller[2].x_rotation.controller
				ScriptControlToApply.script = ExpressionString
				)
				local rotScriptCtrl
				rotScriptCtrl=rotation_script()
				rotScriptCtrl.addNode "nodeA" GlobalController
				rotScriptCtrl.addNode "nodeB" StartController
				rotScriptCtrl.script="rot=quatToEuler (nodeB.transform*inverse nodeA.transform).rotation order:7\n"+
													   "quat (rot.x+rot.z) [-1,0,0]"
				EnvDrivers[1].rotation.controller.available.controller=rotScriptCtrl
				rotScriptCtrl=copy rotScriptCtrl
				rotScriptCtrl.setNode "nodeA" StartController
				rotScriptCtrl.setNode "nodeB" EndController
				for i=2 to i= (EnvDrivers.count-1) do
				(
					EnvDrivers[i].rotation.controller.available.controller= copy rotScriptCtrl
					EnvDrivers[i].rotation.controller.weight[EnvDrivers[i].rotation.controller.count]=100.0/(EnvDrivers.count-2)
				)
				local WorldSpaceNull
				UnusedNumber = CommonFns.CheckIfExists (WorldSpaceNullName) 1 999
				UnusedNumber = ((CommonFns.GetValueWithPadding UnusedNumber) as string)
				WorldSpaceNull =  Point Name:(WorldSpaceNullName + UnusedNumber ) wirecolor:blue Box:On cross: off centermarker: off axistripod: off size:TinySize
				in coordsys world WorldSpaceNull.pos = GlobalController.pos
				WorldSpaceNull.parent = globalcontroller
				append Allcreated WorldSpaceNull
				
				local WorldSpaceOri = Orientation_constraint()
				WorldSpaceOri.appendtarget world 100
				WorldSpaceNull.rotation.controller = WorldSpaceOri
				
				
				for i = 1 to i = EnvDrivers.count do
				(
					EnvDrivers[i].Parent = WorldSpaceNull
					in coordsys world EnvDrivers[i].rotation = GlobalController.rotation
					--rotate EnvDrivers[i] (angleaxis 90 [0,1,0])
				)
				--------------------Twist Dispersion--------------------------------
				local BaseTwistWeight = (100.0/(EnvDrivers.count-2))
				local TotalTwistDispBones = 0
				local TwistExpression
					
				for i=2 to i= (EnvDrivers.count-1) do (TotalTwistDispBones += 1)
				
				for i=2 to i= (EnvDrivers.count-1) do 
				(
					local BoneNumber = (i-1)
					
					local BaseIncrement = (100/TotalTwistDispBones)
					
					local CurrentIncrement = (BaseIncrement*BoneNumber)
					--MessageBox (CurrentIncrement as string)
					local NormalisedCurrentIncrement = (((CurrentIncrement - BaseIncrement)as float) / ((100-BaseIncrement) as float))
					
					local NormalisedIncrement = ((NormalisedCurrentIncrement*2)-1)

					local FinalTwistWeight = (BaseTwistWeight - (BaseTwistWeight * (TwistDisperse * NormalisedIncrement)))
					EnvDrivers[i].rotation.controller.weight[EnvDrivers[i].rotation.controller.count]= FinalTwistWeight
					
						
					if LiveTwist == true do
					(
						TwistExpression = copy TwistExpression
						TwistExpression = float_expression()
						TwistExpression.AddScalarTarget "SliderValue" MainControlObject.modifiers[#Spine_Controls].SpineControls[#paramtwistdisperse]
						TwistExpression.setexpression ("(" + ((BaseTwistWeight) as string) + " - (" + ((BaseTwistWeight) as string) + " * (SliderValue * " + (NormalisedIncrement as string) + ")))/100")
						EnvDrivers[i].rotation.controller.weight[EnvDrivers[i].rotation.controller.count].controller = TwistExpression
					)
				)
			)
			
			----------------------------IF USING THE UPNODE SYSTEM INSTEAD-----------------------------
			
			if Upnodes == True do
			(
				
				local EnvDrivers
				if jointScaling == 3 then (EnvDrivers = MainBonesArray) else (EnvDrivers = JointScaleHelpers)
				local lookcons = LookAt_constraint()
			
				for i = 1 to i = ((UpnodeArray.count)-1) do
				(
					Lookcons = copy lookcons
					LookCons = Lookat_Constraint()
					LookCons.appendtarget EnvDrivers[i+1] 100
					LookCons.lookat_vector_length = 0
					LookCons.viewline_length_abs = false
					LookCons.upnode_ctrl = 1
					LookCons.upnode_world = false
					LookCons.pickUpNode = UpnodeArray[i]
					LookCons.StoUP_axis = 2
					LookCons.upnode_axis = 2
					EnvDrivers[i].rotation.controller = lookcons
				)
					
					------Twist Dispersion for upnode system-------------------------


				local TwistExpression
				
				for i=2.0 to i= (EnvDrivers.count-1) do 
				(
					
					local BaseIncrement = (100/(UpNodeArray.count-2))
					local BoneNumber = (i-1)as float
					local CurrentIncrement = (BoneNumber*BaseIncrement)
					local ReverseIncrement = (100-CurrentIncrement)
					
					
					
						
						
						
						
					UpNodeArray[i].rotation.controller.weight[3] = CurrentIncrement - (CurrentIncrement * (TwistDisperse * (ReverseIncrement*0.01)))

			
					if LiveTwist == true do
					(
						TwistExpression = copy TwistExpression
						TwistExpression = float_expression()
						TwistExpression.AddScalarTarget "SliderValue" MainControlObject.modifiers[#Spine_Controls].SpineControls[#paramtwistdisperse]
						TwistExpression.setexpression ("(" + ((CurrentIncrement) as string) + " - (" + ((CurrentIncrement) as string) + " * (SliderValue *" + ((ReverseIncrement*0.01) as string) + ")))*0.01")
						UpNodeArray[i].rotation.controller.weight[3].controller = TwistExpression
					)
				)
					
				
			)
							
				
				
			--TwistDisperse:0 LiveTwist:False =
				--EnvDrivers[i].rotation.controller.weight[EnvDrivers[i].rotation.controller.count]=100.0/(EnvDrivers.count-2)
			
			
			
			
			
			
			
			
			
			
			
			---------------------------------------Squash and Stretch------------------------------------------------
			
			
			
			
			
				if SquashStretch != 1 do --if there is squash and stretch, add the stretch nubs and reparent the hierarchy
				(
					
					--First:Create the stretch expose measure system along the main spline
					
					local StretchExposePoint = Point Name:(uniquename StretchExposePointName Numdigits:3) Size:TinySize Box:On Cross:On AxisTripod:Off WireColor:(Color 20 170 20)
					append allcreated StretchExposePoint
					StretchExposePoint.pos.controller = Path_Constraint()
					StretchExposePoint.pos.controller.path = MainSpline
					deletekeys StretchExposePoint.position.controller
					StretchExposePoint.pos.controller.percent = 0.1
					local TotalStretchbones = ((mainBonesArray.count)-1)
					local stretchlookat =  CommonFns.CreateLookatdistanceExpose StretchAnchors[1] StretchExposePoint #((MeasureHelperLookatName + "Stretch"),(MeasureHelperLocalName + "Stretch")) AlignTo:undefined reAssignParents:#(StretchAnchors[1],StretchExposePoint) justdistance:false
					for n in stretchlookat do (append allcreated n)
					
					local BoneStartIncArray = #()
					local BoneEndIncArray = #()
					local NormalisedBoneEndIncArray = #() --normalised to -1, 1
					local DefaultCurvedWeightArray = #()
					local DefaultStartToEndWeightArray = #()
					local DefaultEndToStartWeightArray = #()
					
					
					--Next: Create the scale expression and add the relevant variables for use in the expression
					
					--1:none
					--2:curved
					--3: 1 to 0 start to end
					--4: 0 to 1 end to start
					
					
					for i = 1 to i = TotalStretchbones do 
					(
						mainBonesArray[i].scale.controller = Scale_List ()
						mainBonesArray[i].scale.controller.available.controller = Scale_Expression ()
						
						mainBonesArray[i].scale.controller[2].controller.AddScalarTarget "StretchAmount" stretchlookat[1].modifiers[#Stretch_Expose_Controls].StretchControls[#paramstretch]

						mainBonesArray[i].scale.controller[2].controller.setexpression ("[1,max((sqrt(((1+(max(StretchAmount,(-0.99))))))),0.05),max((sqrt(((1+(max(StretchAmount,(-0.99))))))),0.05)]")
						
						append BoneStartIncArray (((i-1) * (1/(TotalStretchbones-1))) as float) --Create some arrays for the next part
						append BoneEndIncArray (((i as float)* (1.0/(TotalStretchbones))) as float) --Create some arrays for the next part
					)
					
					for i = 1 to i = TotalStretchBones do
					(
						append DefaultEndToStartWeightArray ((BoneEndIncArray[i])) --for 0 to 1
						append DefaultStartToEndWeightArray (1-(BoneEndIncArray[i])) --for 1 to 0
					)
					
					DefaultEndToStartWeightArray = #()
					
					for i = 1 to i = TotalStretchBones do
					(
						append DefaultEndToStartWeightArray (DefaultStartToEndWeightArray[((DefaultStartToEndWeightArray.count)-(i-1))])
					)
					

					----Next: Set up the default weighting depending on the SquashStretch input value

					for i = 1 to i = TotalStretchBones do
					(
						append NormalisedBoneEndIncArray ((((BoneEndIncArray[i]-BoneEndIncArray[1])/(BoneEndIncArray[BoneEndIncArray.count]-BoneEndIncArray[1]))*2)-1)
						append DefaultCurvedWeightArray (1-(abs NormalisedBoneEndIncArray[i])) --for default in out in
						
						


						if squashblending == 1	do	--if always on
						(							
							if SquashStretch == 2 then --if it is curved 
							(							
								mainBonesArray[i].scale.controller.weight[mainBonesArray[i].scale.controller.count] =  (DefaultCurvedWeightArray[i]*100)
							)
							else if SquashStretch == 3 then -- 1 to 0 start to end
							(
								mainBonesArray[i].scale.controller.weight[mainBonesArray[i].scale.controller.count] =  (DefaultStartToEndWeightArray[i]*100)
							)
							else if SquashStretch == 4 then -- 0 to 1 end to start
							(
								mainBonesArray[i].scale.controller.weight[mainBonesArray[i].scale.controller.count] =  (DefaultEndToStartWeightArray[i]*100)
								
							)
						)
						if squashblending == 2 do --if there is a blend, add a float expression for sliders
						(
							mainBonesArray[i].scale.controller.weight[mainBonesArray[i].scale.controller.count].controller = float_expression()
							mainBonesArray[i].scale.controller.weight[mainBonesArray[i].scale.controller.count].controller.addscalartarget  "SliderValue" MainControlObject.modifiers[#Spine_Controls].SpineControls[#paramsquash]
							if SquashStretch == 2 then --if it is curved 
							(							
							mainBonesArray[i].scale.controller.weight[mainBonesArray[i].scale.controller.count].controller.setexpression ("SliderValue * " + (DefaultCurvedWeightArray[i] as string))
							)
							else if SquashStretch == 3 then -- 1 to 0 start to end
							(
							mainBonesArray[i].scale.controller.weight[mainBonesArray[i].scale.controller.count].controller.setexpression ("SliderValue *" + (DefaultStartToEndWeightArray[i] as string))
							)
							else if SquashStretch == 4 then -- 0 to 1 end to start
							(
							mainBonesArray[i].scale.controller.weight[mainBonesArray[i].scale.controller.count].controller.setexpression ("SliderValue *" + (((DefaultEndToStartWeightArray[i])) as string))
							)
							
						)
						
						
					)
				)
				
						--------------SQUASH STRETCH DISPERSION--------
-- 					local TwistExpression
-- 				
-- 					for i=2.0 to i= (EnvDrivers.count-1) do 
-- 					(
-- 						
-- 						local BaseIncrement = (100/(UpNodeArray.count-2))
-- 						local BoneNumber = (i-1)as float
-- 						local CurrentIncrement = (BoneNumber*BaseIncrement)
-- 						local ReverseIncrement = (100-CurrentIncrement)
-- 						

-- 						UpNodeArray[i].rotation.controller.weight[3] = CurrentIncrement - (CurrentIncrement * (TwistDisperse * (ReverseIncrement*0.01)))
-- 				
-- 						if LiveTwist == true do
-- 						(
-- 							TwistExpression = copy TwistExpression
-- 							TwistExpression = float_expression()
-- 							TwistExpression.AddScalarTarget "SliderValue" MainControlObject.modifiers[#Spine_Controls].SpineControls[#paramtwistdisperse]
-- 							TwistExpression.setexpression ("(" + ((CurrentIncrement) as string) + " - (" + ((CurrentIncrement) as string) + " * (SliderValue *" + ((ReverseIncrement*0.01) as string) + ")))*0.01")
-- 							UpNodeArray[i].rotation.controller.weight[3].controller = TwistExpression
-- 						)
-- 					)	
						
						
					
					----------------------	
					
				
			
			
			
			
			
			
			
			
			
			
			
			
			
			
			----------------------------RESPOSITION THE TANGENTS IF THERE IS A DEFAULT BEND
			if spinebend != undefined do
			(
				in coordsys world FirstTangent.pos = spinebend[1]
				in coordsys world SecondTangent.pos = spinebend[2]
			)
			----------------------------------ADD THE DEFORMATION FINS-----------------------------
			

			
			local DeformFinControls = #()
			

			
			
			if includefins != undefined do 
				(
					local FrontWidths = #()
					append FrontWidths includefins[2]
					append FrontWidths includefins[4]
					append FrontWidths includefins[6]
					local SideWidths = #()
					append SideWidths includefins[1]
					append SideWidths includefins[3]
					append SideWidths includefins[5]	
					
					local TempYTopPoint = Point Name:"ZZZZZTEMPYTOP"
					local TempYBotPoint = Point Name:"ZZZZZTEMPYBOT"
					local TempZTopPoint = Point Name:"ZZZZZTEMPZTOP"
					local TempZBotPoint =  Point Name:"ZZZZZTEMPZBOT"
					local TempZTopNegPoint = Point Name:"ZZZZZTEMPZTOP"
					local TempZBotNegPoint =  Point Name:"ZZZZZTEMPZBOT"
					for n in #(TempYTopPoint,TempYBotPoint,TempZTopPoint,TempZBotPoint,TempZTopNegPoint,TempZBotNegPoint) do (CommonFns.QuickAlign n GlobalController)
					
					in coordsys world TempYTopPoint.pos = MainBonesArray[((MainBonesArray.count)-1)].transform.pos
					in coordsys world TempZTopPoint.pos = MainBonesArray[((MainBonesArray.count)-1)].transform.pos
					in coordsys world TempZTopNegPoint.pos = MainBonesArray[((MainBonesArray.count)-1)].transform.pos
					in coordsys world TempYBotPoint.pos = MainBonesArray[2].transform.pos
					in coordsys world TempZBotPoint.pos = MainBonesArray[2].transform.pos
					in coordsys world TempZBotNegPoint.pos = MainBonesArray[2].transform.pos

					in coordsys local move TempYTopPoint [0,-includefins[1],0]
					in coordsys local move TempZTopPoint [0,0,-includefins[5]]
					in coordsys local move TempZTopNegPoint [0,0,includefins[5]]	
					in coordsys local move TempYBotPoint [0,-includefins[1],0]
					in coordsys local move TempZBotPoint [0,0,-includefins[5]]	
					in coordsys local move TempZBotNegPoint [0,0,includefins[5]]	

					
					

					--NEXT: Create the measurement helpers at the temporary location nulls
					local YMeasureHelpers = #()
					local ZMeasureHelpers = #()
					local ZNegMeasureHelpers = #()


					YMeasureHelpers = CommonFns.CreateLookatdistanceExpose TempYBotPoint TempYTopPoint #((MeasureHelperLookatName + "Y"),(MeasureHelperLocalName + "Y")) AlignTo: GlobalController reassignparents:#(MainBonesArray[1],MainBonesArray[((MainBonesArray.count)-2)])
					ZMeasureHelpers = CommonFns.CreateLookatdistanceExpose TempZBotPoint TempZTopPoint #((MeasureHelperLookatName + "Z"),(MeasureHelperLocalName + "Z")) AlignTo: GlobalController reassignparents:#(MainBonesArray[1],MainBonesArray[((MainBonesArray.count)-2)])
					ZNegMeasureHelpers = CommonFns.CreateLookatdistanceExpose TempZBotNegPoint TempZTopNegPoint #((MeasureHelperLookatName + "Z"),(MeasureHelperLocalName + "Z")) AlignTo: GlobalController reassignparents:#(MainBonesArray[1],MainBonesArray[((MainBonesArray.count)-2)])
					
					
					for n in YMeasureHelpers do (append allcreated n)
					for n in ZMeasureHelpers do (append allcreated n)
					for n in ZNegMeasureHelpers do (append allcreated n)
					
					local TotalMidBones = ((mainbonesarray.count)-3)
					
					if TotalMidBones <3 do
					(
						for i = 2 to i = ((mainbonesarray.count)-2) do
						(
							(

								local CreatedFins = CommonFns.AddDeformFins MainBonesArray[i] MainBonesArray[i] GlobalController #(YMeasureHelpers[1],YMeasureHelpers[1]) #(ZMeasureHelpers[1],ZNegMeasureHelpers[1]) Prefix:Prefix BaseNames:#("bn_fin","ctrl_fin","null_fin","null_finAUTO")  RotateAmount:0.2 Fins:#(0,1,1,1) OriConsObj:MainBonesArray[(i-1)]  YWidth:includefins[2] ZWidth:includefins[1] ControlObj:MainControlObject Jiggle:False PosYPrefix:"Back" PosZPrefix:"L" DefaultRot:5 DefaultExtend:0.6
								append allcreated CreatedFins[1]
								for n in CreatedFins[1] do (FinalTools.AppendObjsToLayer n HiddenComponents:True MainBones:False Controllers:False AnimGeo:False)
								for n in CreatedFins[3] do ( if n != false do append deformbonesforlayers n)
								for n in CreatedFins[2] do ( if n != false do append DeformControlsForLayers n)

							)
						)
					)
					
					
					if TotalMidBones>2 do
					(
						for i = 2 to i = ((mainbonesarray.count)-2) do
						(
							if (i ==2) or (i == ((mainbonesarray.count)-2)) then 
							(
								local CreatedFins = CommonFns.AddDeformFins MainBonesArray[i] MainBonesArray[i] GlobalController #(YMeasureHelpers[1],YMeasureHelpers[1]) #(ZMeasureHelpers[1],ZNegMeasureHelpers[1]) Prefix:Prefix BaseNames:#("bn_fin","ctrl_fin","null_fin","null_finAUTO")  RotateAmount:0.2 Fins:#(0,1,0,0) OriConsObj:MainBonesArray[(i-1)]  YWidth:includefins[2] ZWidth:includefins[1] ControlObj:MainControlObject Jiggle:False PosYPrefix:"Back" PosZPrefix:"L" DefaultRot:5 DefaultExtend:0.4
								append allcreated CreatedFins[1]
								for n in CreatedFins[1] do (FinalTools.AppendObjsToLayer n HiddenComponents:True MainBones:False Controllers:False AnimGeo:False)
								for n in CreatedFins[3] do ( if n != false do append deformbonesforlayers n)
								for n in CreatedFins[2] do ( if n != false do append DeformControlsForLayers n)
							)
							else 
							(
								local CreatedFins = CommonFns.AddDeformFins MainBonesArray[i] MainBonesArray[i] GlobalController #(YMeasureHelpers[1],YMeasureHelpers[1]) #(ZMeasureHelpers[1],ZNegMeasureHelpers[1]) Prefix:Prefix BaseNames:#("bn_fin","ctrl_fin","null_fin","null_finAUTO")  RotateAmount:0.2 Fins:#(0,1,1,1) OriConsObj:MainBonesArray[(i-1)]  YWidth:includefins[2] ZWidth:includefins[1] ControlObj:MainControlObject Jiggle:False PosYPrefix:"Back" PosZPrefix:"L" DefaultRot:5 DefaultExtend:0.6
								append allcreated CreatedFins[1]
								for n in CreatedFins[1] do (FinalTools.AppendObjsToLayer n HiddenComponents:True MainBones:False Controllers:False AnimGeo:False)
								for n in CreatedFins[3] do ( if n != false do append deformbonesforlayers n)
								for n in CreatedFins[2] do ( if n != false do append DeformControlsForLayers n)
							)
						)
					)
					
					
					
					delete TempYTopPoint
					delete TempYBotPoint
					delete TempZTopPoint
					delete TempZBotPoint
					delete TempZTopNegPoint
					delete TempZBotNegPoint
					
					
					--AddDeformFins ParentObj AlignToPos AlignToOri YMeasureObject ZMeasureObject Prefix:"" BaseNames:#("bn_fin","ctrl_fin","null_fin","null_finAUTO")  RotateAmount:0.2 Fins:#(1,1,1,1) OriConsObj:False  YWidth:2 ZWidth:2 ControlObj:False Jiggle:False PosYPrefix:"Back" PosZPrefix:"R" DefaultRot:20 DefaultExtend:0.2
					
				)--THIS BRACKET TO CLOSE THE INCLUDE FINS LOOP
				
				
				
				---------------------------Add the rotate nullifier system if TangentRotNullifiers is true------------------------------
				if TangentRotNullifiers == true do
				(
					
					if Upnodes != True do
					(
					

						local PelvisExposeNull = Point  isSelected:off cross: off centermarker: off axistripod: off box: on name:((uniquename (PelvisExposeNullName) numdigits:3)) size:TinySize wirecolor:Blue
						local RibcageExposeNull = Point  isSelected:off cross: off centermarker: off axistripod: off box: on name:((uniquename (RibcageExposeNullName) numdigits:3)) size:TinySize wirecolor:Blue
						local PelvisTangentParent = Point isSelected:off cross: off centermarker: off axistripod: off box: on name:((uniquename (PelvisTangentParentName) numdigits:3)) size:TinySize wirecolor:Blue
						local RibcageTangentParent = Point isSelected:off cross: off centermarker: off axistripod: off box: on name:((uniquename (RibcageTangentParentName) numdigits:3)) size:TinySize wirecolor:Blue
						local PelvisNoFollowNull = Point isSelected:off cross: off centermarker: off axistripod: off box: on name:((uniquename (PelvisNoFollowNullName) numdigits:3)) size:TinySize wirecolor:Blue
						local RibcageNoFollowNull = Point isSelected:off cross: off centermarker: off axistripod: off box: on name:((uniquename (RibcageNoFollowNullName) numdigits:3)) size:TinySize wirecolor:Blue
						
						append Allcreated PelvisExposeNull
						append Allcreated RibcageExposeNull
						append Allcreated PelvisTangentParent
						append Allcreated RibcageTangentParent
						append Allcreated PelvisNoFollowNull
						append Allcreated RibcageNoFollowNull
						
						local PosCons = position_constraint()
						posCons.appendtarget StartController 100
						PelvisExposeNull.pos.controller = poscons
						local OriCons = orientation_constraint()
						oricons.appendtarget StartController 100
						PelvisExposeNull.rotation.controller = OriCons
						PelvisExposeNull.parent = GlobalController
						
						OriCons = Copy OriCons
						OriCons = undefined
						OriCons = Orientation_Constraint()
						poscons = Copy PosCons
						poscons = undefined
						PosCons = Position_Constraint()
						
						PosCons.appendtarget EndController 100
						RibcageExposeNull.pos.controller = poscons
						OriCons.appendtarget EndController 100
						RibcageExposeNull.rotation.controller = oricons
						RibcageExposeNull.parent = GlobalController
						
						poscons = Copy PosCons
						poscons = undefined
						PosCons = Position_Constraint()
						PosCons.appendtarget StartController 100
						PelvisTangentParent.pos.controller = PosCons	
						PelvisNoFollowNull.pos.controller = PosCons
						PelvisNoFollowNull.parent = GlobalController
						poscons = Copy PosCons
						poscons = undefined
						PosCons = Position_Constraint()
						PosCons.appendtarget EndController 100
						RibcageTangentParent.pos.controller = PosCons
						RibcageNoFollowNull.pos.controller = PosCons
						RibcageNoFollowNull.parent = GlobalController
						
						
						--More  felix joleanes magic here.
						local ScriptControlToApply = Rotation_Script()
						local ExpressionString = "theRot=QuatToEuler (expose.transform * Inverse expose.parent.transform).rotation order:7\n"+ 
														"theRot.x=-theRot.z\n"+
														"EulerToQuat theRot order:7"
						local PelvisControlToApply = Copy ScriptControlToApply
						PelvisControlToApply.addNode "expose" PelvisExposeNull
						PelvisControlToApply.SetExpression ExpressionString
						PelvisNoFollowNull.rotation.controller = PelvisControlToApply
						
						local RibcageControlToapply = Copy ScriptControlToApply
						RibcageControlToApply.addNode "expose" RibcageExposeNull
						RibcageControlToApply.SetExpression ExpressionString
						RibcageNoFollowNull.rotation.controller = RibcageControlToApply
						
						RibcageNoFollowNull.rotation.controller = Rotation_List()
						PelvisNoFollowNull.Rotation.controller = Rotation_List()
						
						oricons = copy oricons
						oricons = undefined
						oricons = orientation_constraint()
						oricons.appendtarget EndController 100
						RibcageNoFollowNull.rotation.controller.available.controller = oricons
						oricons = copy oricons
						oricons = undefined
						oricons = orientation_constraint()
						oricons.appendtarget StartController 100
						PelvisNoFollowNull.rotation.controller.available.controller = oricons
						
						
						
						local ExpressionToAdd = Float_Expression()
						ExpressionToAdd.AddScalarTarget "NoFollow" MainControlObject.modifiers[#Spine_Controls].SpineControls[#paramPelvisFollow]
						ExpressionToAdd.setExpression "NoFollow"
						PelvisNoFollowNull.rotation.controller.weight[2].controller = ExpressionToAdd
						
						ExpressionToAdd = copy ExpressionToAdd
						ExpressionToAdd = undefined
						ExpressionToAdd = Float_Expression()
						ExpressionToAdd.AddScalarTarget "NoFollow" MainControlObject.modifiers[#Spine_Controls].SpineControls[#paramRibcageFollow]
						ExpressionToAdd.setExpression "NoFollow"
						RibcageNoFollowNull.rotation.controller.weight[2].controller = ExpressionToAdd
						
						PelvisTangentParent.parent = globalController
						RibcageTangentParent.parent = globalcontroller
						
						FirstTangent.Parent = PelvisNoFollowNull
						SecondTangent.Parent = RibcageNoFollowNull
					)
					if Upnodes == True do
					(
						
						local PelvisNoFollowNull = Point isSelected:off cross: off centermarker: off axistripod: off box: on name:((uniquename (PelvisNoFollowNullName) numdigits:3)) size:TinySize wirecolor:Blue
						local RibcageNoFollowNull = Point isSelected:off cross: off centermarker: off axistripod: off box: on name:((uniquename (RibcageNoFollowNullName) numdigits:3)) size:TinySize wirecolor:Blue
						append allcreated PelvisNoFollowNull
						append allcreated RibcageNoFollowNull
						
						
						local PelvisExpression = float_expression()
						local RibcageExpression = float_expression()
						
						PelvisExpression.AddScalarTarget "StartX" StartControlExposeTfm.baseobject[#local_euler_X]
						PelvisExpression.AddScalarTarget "NoFollow" MainControlObject.modifiers[#Spine_Controls].SpineControls[#paramPelvisFollow]
						PelvisExpression.setexpression ("((StartX)*(NoFollow*0.01))")
						RibcageExpression.AddScalarTarget "EndX" EndControlExposeTfm.baseobject[#local_euler_X]
						RibcageExpression.AddScalarTarget "NoFollow" MainControlObject.modifiers[#Spine_Controls].SpineControls[#paramRibcageFollow]
						RibcageExpression.setexpression ("((EndX)*(NoFollow*0.01))")
						
						
						CommonFns.QuickAlign PelvisNoFollowNull Startcontroller
						CommonFns.QuickAlign RibcageNoFollowNull EndController
						PelvisNoFollowNull.parent = Startcontroller
						RibcageNoFollowNull.parent = EndController
							
						PelvisNoFollowNull.rotation.controller.X_Rotation.controller = PelvisExpression
						RibcageNoFollowNull.rotation.controller.X_Rotation.controller = RibcageExpression
						
						FirstTangent.Parent = PelvisNoFollowNull
						SecondTangent.Parent = RibcageNoFollowNull
						
						FinalTools.AppendObjsToLayer PelvisNoFollowNull HiddenComponents:True MainBones:False Controllers:False AnimGeo:False
						FinalTools.AppendObjsToLayer RibcageNoFollowNull HiddenComponents:True MainBones:False Controllers:False AnimGeo:False

						

					)
					
					)--This bracket to close tangent rot nullifiers loop
			
			-------------------------------------Add the bend controllers-----------------------------
			local StartBendController
			local EndBendController
			local StartBendNull
			local EndBendNull
			local BendControllersArray = #()
			if BendControllers == true do
			(
				StartBendController = CommonFns.MakeController (MediumSize) "sphere" GlobalController GlobalController (color 40 40 255) (BendControllerName + "_start") "z" "positive"
				CommonFns.QuickAlign StartBendController FirstTangent
				StartBendNull = Point isSelected:off cross: off centermarker: off axistripod: off box: on name:((uniquename (ToolSetup.nameprefix + "Null_StartBend") numdigits:3)) size:TinySize wirecolor:Blue
				CommonFns.QuickAlign StartBendNull StartBendController
				StartBendNull.parent = (FirstTangent.parent)
				StartBendController.parent = (StartBendNull)
				FirstTangent.parent = StartBendController
				
				EndBendNull = Point isSelected:off cross: off centermarker: off axistripod: off box: on name:((uniquename (ToolSetup.nameprefix + "Null_EndBend") numdigits:3)) size:TinySize wirecolor:Blue
				EndBendController = CommonFns.MakeController (Bigsize*4.5) "circle" GlobalController GlobalController (color 40 40 255) (BendControllerName + "_spine") "x" "positive"
				converttosplineshape EndBendController
				
				
				for i = 1 to i = 4 do
				(
				local TempSphere = CommonFns.MakeController (Mediumsize*0.8) "sphere" GlobalController GlobalController (color 40 40 255) ("BlahSphere") "x" "positive"
				local TempPath = path_constraint()
				TempPath.path = EndBendController
				TempPath.percent = ((i-1)*25)
				TempSphere.pos.controller = copy TempPath
				addandweld EndBendController TempSphere 0
				)
				deleteSpline EndBendController 1
				

				
				CommonFns.QuickAlign EndBendController SecondTangent
				CommonFns.QuickAlign EndBendNull EndBendController
				EndBendController.parent = (EndBendNull)
				EndBendNull.parent = (SecondTangent.parent)
				SecondTangent.parent = EndBendController
				CommonFns.ApplycontrollerTypeProperties EndBendController ControlType:"Medium" ControlLevel:2
				--EndBendController.pos.controller = position_constraint()
				--EndBendController.pos.controller.appendtarget StartBendController
				paramWire.connect EndBendController.pos.controller[#Z_Position] StartBendController.pos.controller[#Z_Position] "Z_Position"
				paramWire.connect EndBendController.pos.controller[#Y_Position] StartBendController.pos.controller[#Y_Position] "Y_Position"
				paramWire.connect EndBendController.pos.controller[#X_Position] StartBendController.pos.controller[#X_Position] "X_Position"
				
				append allcreated StartBendController
				append allcreated EndBendController
				append allcreated StartBendNull
				append allcreated EndBendNull
				
				FinalTools.AppendObjsToLayer StartBendNull HiddenComponents:True MainBones:False Controllers:False AnimGeo:False
				FinalTools.AppendObjsToLayer EndBendNull HiddenComponents:True MainBones:False Controllers:False AnimGeo:False
				append BendControllersArray  StartBendController
				append BendControllersArray  EndBendController
			)
					
					
					
			--------------------------------Final Bug Fixes (These should be fixed properly at some point!)----------------------------------
			
			--Bug where if you have Joint Scaling always off and No-Scale Always top, the bottom bone goes out of place)
			if PositiveIKAnchors.count == 0 and StretchAnchors.count == 0 and BlendedIKAnchors.count == 0 do
			(
				in CoordSys World NegativeIKAnchors[NegativeIKAnchors.count].pos = StartController.transform.pos
			)
			
			---Make the end nub follow the final bone if joint scaling is always off
			MainBonesArray[MainBonesArray.count].rotation.controller = orientation_constraint()
			MainBonesArray[MainBonesArray.count].rotation.controller.appendtarget MainBonesArray[((MainBonesArray.count)-1)] 100
			
			
			
			
			------------------------------------------------------------------------
			
			local FinalOutputArray = #()
			local TangentsArray = #()
			local MainControlsArray = #()
			append MainControlsArray GlobalController
			append MainControlsArray StartController
			append MainControlsArray EndController
			
			

			
			append TangentsArray FirstTangent
			append TangentsArray SecondTangent
			if BendControllers == true do
			(
				append TangentsArray StartBendController
				append TangentsArray EndBendController
			)
			
			append FinalOutputArray allcreated
			append FinalOutputArray MainControlsArray
			append FinalOutputArray FKControllers
			append FinalOutputArray ReverseFKControllers
			append FinalOutputArray DeformControlsForLayers
			append FinalOutputArray TangentsArray
			append FinalOutputArray MainBonesArray
			append FinalOutputArray BendControllersArray
			
			--FinalOutputArray:
			
			--1: Allcreated
			--2: GlobalController, Start Controller, EndController
			--3: FK Controllers (or empty array)
			--4:Reverse FK Controllers (or empty array)
			--5: Deformation fin controllers (or empty array )
			--6: StartTangent, EndTangent, StartBendController, EndBendController
			--7:MainBones
			--8:BendControllers
			
			try 
			(
				MainBonesArray[MainBonesArray.count].width = TinySize
				MainBonesArray[MainBonesArray.count].sidefins = On
				MainBonesArray[MainBonesArray.count].wirecolor = color 160 250 180
				MainBonesArray[MainBonesArray.count].sidefinssize = MediumSize
				MainBonesArray[MainBonesArray.count].sidefinsendtaper = -100
			)
			catch()
				
			
			FinalTools.AppendObjsToLayer FinalOutputArray[1] HiddenComponents:True MainBones:False Controllers:False AnimGeo:False
			FinalTools.AppendObjsToLayer FinalOutputArray[3] HiddenComponents:False MainBones:False Controllers:True AnimGeo:False
			FinalTools.AppendObjsToLayer FinalOutputArray[4] HiddenComponents:False MainBones:False Controllers:True AnimGeo:False
			FinalTools.AppendObjsToLayer FinalOutputArray[5] HiddenComponents:False MainBones:False Controllers:True AnimGeo:False
			FinalTools.AppendObjsToLayer FinalOutputArray[7] HiddenComponents:False MainBones:True Controllers:False AnimGeo:False
			FinalTools.AppendObjsToLayer FinalOutputArray[8] HiddenComponents:False MainBones:False Controllers:True AnimGeo:False
			if StartBendController != undefined do (FinalTools.AppendObjsToLayer StartBendController HiddenComponents:True MainBones:False Controllers:False AnimGeo:False)
			
			if DeformBonesForLayers.count != 0 do 
			(
				
				for n in DeformBonesForLayers do 
				(
					n.wirecolor = color 180 60 60
					n.sidefins= off
					n.frontfin = off
					n.backfin = off
					n.width = tinysize
					n.height = tinysize
				)
				
				FinalTools.AppendObjsToLayer DeformBonesForLayers  HiddenComponents:False MainBones:False Controllers:False AnimGeo:False DeformerBones:True
			)
			if DeformControlsForLayers.count != 0 do 
			(
				
				FinalTools.AppendObjsToLayer DeformControlsForLayers  HiddenComponents:False MainBones:False Controllers:True AnimGeo:False DeformerBones:False
			)
			
			
			
			FinalOutputArray
			
		), --THIS BRACKET TO CLOSE IK/FK SPINE FUNCTION
		
		----------------------------------------------------------------------------------------------------------------------------------------------
		------------------------------------------  SPINE  GUIDE FUNCTION-----------------------------------------------------------------
		----------------------------------------------------------------------------------------------------------------------------------------------
		----------------------------------------------------------------------------------------------------------------------------------------------
		
		fn MakeSpineGuide StartObj EndObj  =
		(
			undo on 
		(
			if classof StartObj == Point3 do (StartObj = Point Name:"ZZZZ_SplineGuideStartObj" Pos: (in coordsys world StartObj))
			if classof EndObj == Point3 do (EndObj = Point Name:"ZZZZ_SplineGuideEndObj" Pos: (in coordsys world EndObj))	
			
			local AverageScale = (Distance StartObj EndObj)
			local BigSize = AverageScale/3
			local MediumSize = AverageScale/12
			local TinySize = AverageScale/6
		----MAKE THE GUIDE SPLINES------
			
			local GuideSpline,UnusedNumber,ss,GuideSplineName,Midpoint,GuideSpline
			
			Midpoint = CommonFns.findIncrementsBetweenTwoPoints StartObj EndObj divisions:1 IncludeEnds:False
			
			UnusedNumber = CommonFns.CheckIfExists "SpineGuideSpline" 1 999
			UnusedNumber = ((CommonFns.GetValueWithPadding UnusedNumber) as string)
			GuideSplineName = "SpineGuideSpline" + UnusedNumber
			ss = SplineShape pos:StartObj.Transform.pos name:GuideSplineName color:(color 20 140 70)
			ss.optimize = false
			ss.steps = 12
			addNewSpline ss
			addKnot ss 1 #beziercorner #curve StartObj.transform.pos StartObj.Transform.pos Midpoint[1]
			addKnot ss 1 #beziercorner #curve EndObj.transform.pos Midpoint[1] EndObj.Transform.pos
			updateShape ss
			ss
			GuideSpline = (getnodebyname GuideSplineName)
			
			------------MAKE THE GUIDE SPLINE TANGENT HELPERS AND LANDMARKS-------------------
			
			local PelvisTangent, RibcageTangent,UnusedNumber,PelvisMidpoint,RibcageMidpoint,TangentToAdd,PelvisBendControl,RibcageBendControl,TempWorldPoint
			local PelvisWidthGuide,RibcageWidthguide,MidlineWidthGuide,FloatExpr
			local allcreated = #()

			TempWorldPoint = CommonFns.createsystemalignmentpoint StartObj EndObj
			
			
			GuideSpline.wirecolor = (color 10 150 80)
			PelvisMidpoint = CommonFns.findIncrementsBetweenTwoPoints StartObj Midpoint[1] divisions:1 IncludeEnds:False
			RibcageMidpoint = CommonFns.findIncrementsBetweenTwoPoints Midpoint[1] EndObj divisions:1 IncludeEnds:False
			BendControl = (CommonFns.MakeController ToolSetup.ControlSizes[2] "sphere" TempWorldPoint MidPoint[1] (color 20 200 100)  "Spinebend_Pelvis"  "y" "negative")

			settransformlockflags BendControl #{4,5,6,7,8,9}

			
			
			
			UnusedNumber = CommonFns.CheckIfExists ("PelvisTangent") 1 999
			UnusedNumber = ((CommonFns.GetValueWithPadding UnusedNumber) as string)
			TangentToAdd = ("PelvisTangent") + UnusedNumber

			PelvisTangent = Point pos:Midpoint[1] isSelected:off cross: off centermarker: off axistripod: off box: off name:TangentToAdd size:0 wirecolor:Green
			
			UnusedNumber = CommonFns.CheckIfExists ("RibcageTangent") 1 999
			UnusedNumber = ((CommonFns.GetValueWithPadding UnusedNumber) as string)
			TangentToAdd = ("RibcageTangent") + UnusedNumber
			RibcageTangent = Point pos:Midpoint[1] isSelected:off cross: off centermarker: off axistripod: off box: off name:TangentToAdd size:0 wirecolor:Green
			PelvisTangent.parent = BendControl
			RibcageTangent.parent = BendControl
			
			PelvisWidthGuide = (CommonFns.MakeController (Bigsize) "circle" TempWorldPoint PelvisMidPoint[1] blue "Pelvis_Width"  "z" "positive")
			CommonFns.AssignPathConstraint PelvisWidthGuide GuideSpline 30
			RibcageWidthGuide = (CommonFns.MakeController (BigSize) "circle" TempWorldPoint RibcageMidPoint[1] blue "Ribcage_Width"  "z" "positive")
			CommonFns.AssignPathConstraint RibcageWidthGuide GuideSpline 60
			
			MidlineWidthGuide = (CommonFns.MakeController ((BigSize)*0.8) "circle" TempWorldPoint RibcageMidPoint[1] (color 40 40 255) "Midline_Width"  "z" "positive")
			CommonFns.AssignPathConstraint MidlineWidthGuide GuideSpline 50
			
			FloatExpr = Float_Expression()
			FloatExpr.AddscalarTarget "A" PelvisWidthGuide.pos.controller[#percent]
			FloatExpr.AddscalarTarget "B" RibcageWidthGuide.pos.controller[#percent]
			FloatExpr.SetExpression "((A+B)*0.5)"
			MidlineWidthGuide.pos.controller.percent.controller = FloatExpr
			
			settransformlockflags MidLineWidthGuide #{4,5,6}
			settransformlockflags PelvisWidthGuide #{4,5,6}
			settransformlockflags RibcageWidthGuide #{4,5,6}
			
			--------------------------MAKE THE PATH CONSTRAINED POINTS ON THE WIDTH SPLINES--------------
			
			local PelvisWidthPoint,PelvisFrontWidthPoint,MidlineWidthPoint,MidlineFrontWidthPoint,RibcageWidthpoint,RibcageFrontWidthPoint
			local MeasurementPairs = #()
			PelvisWidthPoint =  Point pos:PelvisMidpoint[1] isSelected:off cross: off centermarker: off axistripod: off box: off name:"SpineGuide_PelvisWidth" size:0 wirecolor:Green
			PelvisFrontWidthPoint =  Point pos:PelvisMidpoint[1] isSelected:off cross: off centermarker: off axistripod: off box: off name:"SpineGuide_PelvisFrontWidth" size:0 wirecolor:Green
			MidlineWidthPoint =  Point pos:PelvisMidpoint[1] isSelected:off cross: off centermarker: off axistripod: off box: off name:"SpineGuide_MidlineFrontWidth" size:0 wirecolor:Green
			MidlineFrontWidthPoint =  Point pos:PelvisMidpoint[1] isSelected:off cross: off centermarker: off axistripod: off box: off name:"SpineGuide_MidlineFrontWidth" size:0 wirecolor:Green
			RibcageWidthPoint =  Point pos:PelvisMidpoint[1] isSelected:off cross: off centermarker: off axistripod: off box: off name:"SpineGuide_RibcageWidth" size:0 wirecolor:Green
			RibcageFrontWidthPoint =  Point pos:PelvisMidpoint[1] isSelected:off cross: off centermarker: off axistripod: off box: off name:"SpineGuide_RibcageFrontWidth" size:0 wirecolor:Green
			CommonFns.AssignPathConstraint PelvisWidthPoint PelvisWidthGuide 0
			CommonFns.AssignPathConstraint PelvisFrontWidthPoint PelvisWidthGuide 75
			CommonFns.AssignPathConstraint MidlineWidthPoint MidlineWidthGuide 0
			CommonFns.AssignPathConstraint MidlineFrontWidthPoint MidlineWidthGuide 75
			CommonFns.AssignPathConstraint RibcageWidthPoint RibcageWidthGuide 0
			CommonFns.AssignPathConstraint RibcageFrontWidthPoint RibcageWidthGuide 75
			
			allcreated = #(BendControl,PelvisTangent,RibcageTangent,PelvisWidthGuide,RibcageWidthguide,MidlineWidthGuide,GuideSpline,PelvisWidthPoint,RibcageWidthPoint,MidlineWidthPoint,MidlineFrontWidthPoint,PelvisFrontWidthPoint,RibcageFrontWidthPoint,StartObj,EndObj)
			measurementPairs = #(PelvisWidthGuide,PelvisWidthPoint,PelvisFrontWidthPoint,MidlineWidthGuide,MidlineWidthPoint,MidlineFrontWidthPoint,RibcageWidthGuide,RibCageWidthPoint,RibcageFrontWidthPoint)

			
			----------------------SKIN THE SPLINES----------------------------------
			local theskin,StartControllerOrigName,EndControllerOrigName,FirstTangentOrigName,SecondTangentOrigName,theskinreverse
			Select GuideSpline
			theskin = Skin()
			addmodifier $ theskin
			max modify mode
			
			EndObj.Name = "Z2_RibcageTop"
			StartObj.Name = "Z1_RibcageBottom"

			skinOps.addBone $.modifiers[#Skin] PelvisTangent 1
			skinOps.addBone $.modifiers[#Skin] RibcageTangent 1
			skinOps.addBone $.modifiers[#Skin] StartObj 1
			skinOps.addBone $.modifiers[#Skin] EndObj 1
			$.modifiers[#Skin].filter_vertices = on
			Select GuideSpline
			skinOps.ReplaceVertexWeights $.modifiers[#Skin]  1 3 1
			skinOps.ReplaceVertexWeights $.modifiers[#Skin]  2 3 1
			skinOps.ReplaceVertexWeights $.modifiers[#Skin]  3 1 1
			skinOps.ReplaceVertexWeights $.modifiers[#Skin]  4 2 1
			skinOps.ReplaceVertexWeights $.modifiers[#Skin]  5 4 1
			skinOps.ReplaceVertexWeights $.modifiers[#Skin]  6 4 1
			
			$Z2_RibcageTop.Name = "Ribcage_Top"
			$Z1_RibcageBottom.Name = "Ribcage_Bottom"
			
					for n in allcreated do 
			(
				if classof n == bonegeometry do
				(
					n.showFrozenInGray = off
					n.sidefins = off
					n.frontfin = off
					n.backfin = off
					n.width = tinysize
					freeze n
				)
			)
			
			
			
			select allcreated
			local BothBendControls = #(BendControl,BendControl)
			
			local OutputArray =#()
			append OutputArray allcreated
			append OutputArray measurementPairs
			append OutputArray BothBendControls
			
			-------------------------------------------------------------
			--OutputArray contains:
			--1: All created
			--2: Measurement Pairs (Pelvis Guide, Pelvis Width, Pelvis Front),(Midline Guide, Width, Front), (Ribcage Guide, Width, Front)
			--3:Bend Locators (point3)
			-------------------------------------------------------------

			
			delete TempWorldPoint
			OutputArray
		) --THIS BRACKET TO CLOSE THE UNDO-ON FOR SPINEGUIDE 
		
		

		--1 draw spline between points
		--skin the spline to tangent helpers
		--make controllers for the tangents
		--add the pelvis/ribcage markers
		--add the front/side splines
		--
		),--THIS BRACKET TO CLOSE SPINEGUIDE FUNCTION
		
	-----------------------------------------------------------------------------------------------------------
	fn MakeArmGuide PickedUpArm PickedElbow PickedHand NumFingers FingerJoints IncludeThumb SplitShoulder ExtraShoulder ForceAlignment:False =
	(
	
		local UparmLocator,ElbowLocator,HandLocator
		
		local DetectedAlignment
		if PickedUpArm.transform.pos.x <0 then (DetectedAlignment = "R") else (DetectedAlignment = "L")
		if ForceAlignment == "R" or ForceAlignment == "r" or ForceAlignment == "right" then (DetectedAlignment = "R")
		if ForceAlignment == "L" or ForceAlignment == "l" or ForceAlignment == "left" then (DetectedAlignment = "L")
		
		
		local AverageScale = ((Distance PickedUpArm PickedElbow) + (Distance PickedElbow PickedHand))/2
		local TinySize = AverageScale/16
		local MediumSize = AverageScale/10
		local BigSize = AverageScale/6
		
				--OutputArray:
		--1:Allcreated
		--2: #(Clavicle,UpArmBone,LowArmBone,HandBone,HandNub). #(Upper Pec,Lower Pec, Deltoid,Scapula)
		--4: #(Number Of Fingers, JointsPerFinger,Bool:IncludeThumb,Bool:IncludeShoulderGirdle) 
		--:5 #(Finger1Control,Finger1Joint1,Finger1Joint2,Finger1Joint3,Finger1Nub),(Finger2Control,Finger2Joint1,Joint2 ETC 
		--6:Any Extra stuff
		
		local allCreated = #()
		local mainBones = #()
		local shoulderBones = #()
		local SettingsArray = #()
		local FingerOutputArray = #()
		local WidthHelpersArray = #()
		
		append SettingsArray NumFingers
		append SettingsArray FingerJoints
		append SettingsArray IncludeThumb
		append SettingsArray ShoulderDeformers
		append SettingsArray DetectedAlignment
		
		
		
		------------------Replace the points with controls----------------------------------------
		UparmLocator = CommonFns.MakeController (BigSize) "sphere" PickedUpArm PickedUpArm (color 40 40 255) (uniquename "UpperArm_Locator" numdigits:3)  "z" "positive"
		delete PickedUpArm
		ElbowLocator = CommonFns.MakeController (BigSize) "sphere" PickedElbow PickedElbow (color 40 40 255) (uniquename "Elbow_Locator" numdigits:3)  "z" "positive"
		delete PickedElbow
		HandLocator = CommonFns.MakeController (BigSize) "square" PickedHand PickedHand (color 40 40 255) (uniquename "Hand_Locator" numdigits:3)  "z" "positive"
		delete PickedHand
		
		append AllCreated UpArmLocator
		append AllCreated ElbowLocator
		append AllCreated HandLocator
		
		-------------------Draw the bones and nubs--------------------------------------------------------------------------------------------------------	
			
		local UpArmGuideBone,LowArmGuideBone,upArmGuideNub,lowArmGuideNub
		
		UpArmGuideBone  = boneSys.createBone uparmlocator.transform.pos elbowLocator.transform.pos [0,0,1]
		UpArmGuideBone.name = (uniquename "Upperarm_Guide" numdigits:3)
		LowArmGuideBone = boneSys.createBone elbowLocator.transform.pos handLocator.transform.pos [0,0,1]
		LowArmGuideBone.name = (uniquename "Lowerarm_Guide" numdigits:3)
		UpArmGuideNub = boneSys.createBone ElbowLocator.transform.pos ElbowLocator.transform.pos [0,0,1]
		UpArmGuideNub.name = (uniquename "Upperarm_Guide_Nub" numdigits:3)	
		lowArmGuideNub = boneSys.createBone Handlocator.transform.pos HandLocator.transform.pos [0,0,1]
		LowArmGuideNub.name = (uniquename "Lowerarm_Guide_Nub" numdigits:3)
		
		UpArmGuideNub.parent = UpArmGuideBone
		LowArmGuideNub.parent = LowArmGuideBone
		LowArmGuideBone.parent = ElbowLocator
		UpArmGuideBone.boneFreezeLength=false
		LowArmGuidEBone.boneFreezeLength=false
		LowArmGuideBone.boneScaleType=#scale
		UpArmGuideBone.boneScaleType=#scale
		UpArmGuideBone.parent = UpArmLocator
		
		append AllCreated UpArmGuideBone
		append AllCreated LowArmGuideBone
		append AllCreated UpArmGuideNub
		append AllCreated LowArmGuideNub
		
		append MainBones UpArmGuideBone
		append MainBones LowArmGuideBone
		
		
		
		-----------------Apply The Constraints----------------------------------
		CommonFns.ApplyArmLookat UpArmGuideBone ElbowLocator HandLocator Leg:False
		CommonFns.ApplyArmLookat LowArmGuideBone HandLocator UpArmLocator Leg:False
		local UpArmNubPosCons = position_constraint()
		local LowArmNubPosCons = position_constraint()
		UpArmNubPosCons.appendtarget ElbowLocator 100
		LowArmNubPosCons.appendtarget HandLocator 100
		UpArmGuideNub.pos.controller = UpArmNubPosCons
		LowArmGuideNub.pos.controller = LowArmNubPosCons
		-----------------Match the hand locator rotation to the LowArm Bone--------------------
		local TempHandLocatorPos = HandLocator.transform.pos


		HandLocator.rotation *= HandLocator.transform.rotation * (inverse LowArmGuideBone.transform.rotation)

		in Coordsys world HandLocator.pos = TempHandLocatorPos
		-----------------Create the controller for the end of the hand----------------------------------
		local HandEndLocator
		
		HandEndLocator = CommonFns.MakeController (BigSize) "triangle" HandLocator HandLocator (color 40 40 255) ("HandEnd_Locator")  "x" "positive"
		HandEndLocator.parent = HandLocator
		in CoordSys parent HandEndLocator.pos.x += ((distance HandLocator ElbowLocator)*0.4)
		settransformlockflags HandEndLocator #{2,3,4,5,6,7,8,9}
		-----------------Create The Hand Bone-------------------------------------
		local HandGuideBone,HandGuideNub
		HandGuideBone  = boneSys.createBone HandLocator.transform.pos HandEndLocator.transform.pos [0,0,1]
		HandGuideBone.name = (uniquename "Hand_Guide" numdigits:3)
		HandGuideNub  = boneSys.createBone HandEndLocator.transform.pos HandEndLocator.transform.pos [0,0,1]
		HandGuideNub.name = (uniquename "Hand_GuideNub" numdigits:3)
		HandGuideNub.parent = HandGuideBone
		HandGuideBone.rotation *= HandGuideBone.transform.rotation * (inverse HandLocator.transform.rotation)
		in coordsys world HandGuideBone.pos = HandLocator.Transform.pos
		HandGuideBone.boneFreezeLength=false
		HandGuideBone.boneScaleType=#scale
		local HandGuideNubPosCons = position_constraint()
		HandGuideNubPosCons.appendtarget HandEndLocator 100
		HandGuideNub.pos.controller = HandGuideNubPosCons
		HandEndLocator.parent = HandLocator
		HandGuidEBone.Parent = HandLocator
		
		select #(UpArmGuideBone,UpArmGuideNub,LowArmGuideBone,LowArmGuideNub,HandGuidEBone,HandGuideNub)
		max freeze selection
		
		append AllCreated HandEndLocator
		append AllCreated HandGuideBone
		append AllCreated HandGuideNub
		
		append MainBones HandGuideBone
		append MainBones HandGuideNub
		--------------------Create the width controls--------------------------------------------------
		
		local WristWidthGuide = CommonFns.MakeController (BigSize*1.2) "circle" LowArmGuideBone HandLocator  (color 40 190 40) ("Lowarm_Width")  "z" "positive"
		local HandEndWidthGuide = CommonFns.MakeController (1) "circle" HandLocator HandEndLocator  (color 40 190 40) ("Hand_Width")  "z" "positive"
		local UpArmEndWidthGuide = CommonFns.MakeController (BigSize*1.2) "circle" UpArmGuideBone LowArmGuideBone  (color 40 190 40) ("UpArm_EndWidth")  "z" "positive"
		local UpArmStartWidthGuide = CommonFns.MakeController (BigSize*1.2) "circle" UpArmGuideBone UpArmGuideBone  (color 40 190 40) ("UpArm_StartWidth")  "z" "positive"
		in coordsys local scale HandEndWidthGuide [1,1,TinySize]
		local HandScaleAmount = (1/(2/(distance HandGuideBone HandGuideNub))*1.2)
		in coordsys local scale HandEndWidthGuide [1,HandScaleAmount,1]
		WristWidthGuide.rotation.controller = orientation_constraint()
		WristWidthGuide.rotation.controller.appendtarget LowArmGuideBone 100
		HandEndWidthGuide.rotation.controller = orientation_constraint()
		HandEndWidthGuide.rotation.controller.appendtarget HandGuideBone 100
		UpArmEndWidthGuide.rotation.controller = orientation_constraint()
		UpArmEndWidthGuide.rotation.controller.appendtarget  UpArmGuideBone 100
		UpArmStartWidthGuide.rotation.controller = orientation_constraint()
		UpArmStartWidthGuide.rotation.controller.appendtarget UpArmGuideBone 100 

		WristWidthGuide.pos.controller = position_constraint()
		WristWidthGuide.pos.controller.appendtarget HandGuideBone 90
		WristWidthGuide.pos.controller.appendtarget LowArmGuideBone 10
		
		HandEndWidthGuide.pos.controller = position_constraint()
		HandEndWidthGuide.pos.controller.appendtarget HandGuideNub 90
		HandEndWidthGuide.pos.controller.appendtarget HandGuideBone 10
		
		UpArmEndWidthGuide.pos.controller = position_constraint()
		UpArmEndWidthGuide.pos.controller.appendtarget LowArmGuideBone 90
		UpArmEndWidthGuide.pos.controller.appendtarget UpArmGuideBone 10
		
		UpArmStartWidthGuide.pos.controller = position_constraint()
		UpArmStartWidthGuide.pos.controller.appendtarget UpArmGuideBone 90
		UpArmStartWidthGuide.pos.controller.appendtarget LowArmGuideBone 10
		
		append AllCreated WristWidthGuide
		append AllCreated HandEndWidthGuide
		append AllCreated UpArmEndWidthGuide
		append AllCreated UpArmStartWidthGuide
		

		
		
		-------------create path constrained points on width controls----------------------
		
		local HandFrontWidthPoint =  Point  isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename "HandFrontWidthPoint" Numdigits:3) size:0 wirecolor:Green
		local HandWidthPoint =  Point  isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename "HandWidthPoint" Numdigits:3) size:0 wirecolor:Green
		local HandBackWidthPoint = Point  isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename "HandBackWidthPoint" Numdigits:3) size:0 wirecolor:Green
		
		HandFrontWidthPoint.pos.controller = path_constraint()
		HandFrontWidthPoint.pos.controller.path = HandEndWidthGuide
		HandFrontWidthPoint.pos.controller.percent = -25
		HandWidthPoint.pos.controller = path_constraint()
		HandWidthPoint.pos.controller.path = HandEndWidthGuide
		HandWidthPoint.pos.controller.percent = 0
		HandBackWidthPoint.pos.controller = path_constraint()
		HandBackWidthPoint.pos.controller.path = HandEndWidthGuide
		HandBackWidthPoint.pos.controller.percent = 25
		
		
		local WristFrontWidthPoint =  Point  isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename "WristFrontWidthPoint" Numdigits:3) size:0 wirecolor:Green
		local WristWidthPoint = Point  isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename "WristWidthPoint" Numdigits:3) size:0 wirecolor:Green
		
		WristFrontWidthPoint.pos.controller = path_constraint()
		WristFrontWidthPoint.pos.controller.path = WristWidthGuide
		WristFrontWidthPoint.pos.controller.percent = 25
		WristWidthPoint.pos.controller = path_constraint()
		WristWidthPoint.pos.controller.path = WristWidthGuide
		WristWidthPoint.pos.controller.percent = 0
		
		local UpArmEndFrontWidthPoint = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename "UparmEndFrontWidthPoint" Numdigits:3) size:0 wirecolor:Green
		local UpArmEndWidthPoint =  Point  isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename "UpArmEndWidthPoint" Numdigits:3) size:0 wirecolor:Green
		
		UpArmEndFrontWidthPoint.pos.controller = path_constraint()
		UpArmEndFrontWidthPoint.pos.controller.path = UpArmEndWidthGuide
		UpArmEndFrontWidthPoint.pos.controller.percent = 25
		UpArmEndWidthPoint.pos.controller = path_constraint()
		UpArmEndWidthPoint.pos.controller.path =  UpArmEndWidthGuide
		UpArmEndWidthPoint.pos.controller.percent = 0
		
		local UpArmStartFrontWidthPoint = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename "UpArmStartFrontWidthPoint" Numdigits:3) size:0 wirecolor:Green
		local UpArmStartWidthPoint = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename "UpArmStartWidthPoint" Numdigits:3) size:0 wirecolor:Green
		
		UpArmStartFrontWidthPoint.pos.controller = path_constraint()
		UpArmStartFrontWidthPoint.pos.controller.path =UpArmStartWidthGuide
		UpArmStartFrontWidthPoint.pos.controller.percent = 25
		UpArmStartWidthPoint.pos.controller = path_constraint()
		UpArmStartWidthPoint.pos.controller.path =  UpArmStartWidthGuide
		UpArmStartWidthPoint.pos.controller.percent = 0
		
		append AllCreated HandFrontWidthPoint
		append AllCreated HandWidthPoint
		append AllCreated HandBackWidthPoint
		append AllCreated WristFrontWidthPoint
		append AllCreated WristWidthPoint
		append AllCreated UpArmEndFrontWidthPoint
		append AllCreated UpArmEndWidthPoint
		append AllCreated UpArmStartFrontWidthPoint
		append AllCreated UpArmStartWidthPoint
		
		
		local UpArmStartWidthHelpers = #()
		append UpArmStartWidthHelpers UpArmStartWidthGuide
		append UpArmStartWidthHelpers UpArmStartFrontWidthPoint
		append UpArmStartWidthHelpers UpArmStartWidthPoint
		
		local UpArmEndWidthHelpers = #()
		append UpArmEndWidthHelpers UpArmEndWidthGuide
		append UpArmEndWidthHelpers UpArmEndFrontWidthPoint
		append UpArmEndWidthHelpers UpArmEndWidthPoint
		
		local WristWidthHelpers = #()
		append WristWidthHelpers WristWidthGuide
		append WristWidthHelpers WristFrontWidthPoint
		append WristWidthHelpers WristWidthPoint
		
		local HandWidthHelpers = #()
		append HandWidthHelpers HandEndWidthGuide
		append HandWidthHelpers HandFrontWidthPoint
		append HandWidthHelpers HandWidthPoint
		
		append WidthHelpersArray UpArmStartWidthHelpers
		append WidthHelpersArray UpArmEndWidthHelpers
		append WidthHelpersArray WristWidthHelpers
		append WidthHelpersArray HandWidthHelpers
		
	-------------------------create the finger parent nulls, pos constrained on hand width--------------------------------------------
		
		local fingerMainControlParents = #()
		local fingerMainControls = #()
		local ThumbMainControlParent
		local ThumbMainControl
		local BaseHandLength = (distance handendlocator handlocator)
		if IncludeThumb == true then (IncludeThumb = 1) else (IncludeThumb = 0)
		if (NumFingers+IncludeThumb) != 0 do
		(
			for i = 1 to i = (numfingers+IncludeThumb) do
			(
				if i <= numfingers do
				(
					local PointToAdd = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename ("Finger" + ((i as integer) as string) + "_parent") Numdigits:3) size:0 wirecolor:Green
					PointToAdd.rotation *= PointToAdd.transform.rotation * (inverse HandGuideBone.transform.rotation)

					PointToAdd.pos.controller = Position_Constraint()
					local FrontPercent
					FrontPercent= (100/((NumFingers as integer)+1)*i)
					PointToAdd.pos.controller.appendtarget HandFrontWidthPoint (100-FrontPercent)
					PointToAdd.pos.controller.appendtarget HandBackWidthPoint (FrontPercent)
					append FingerMainControlParents PointToAdd
					pointToadd.parent = HandLocator
					local controlToAdd = CommonFns.MakeController (TinySize) "square" PointToAdd PointToAdd  (color 65 130 65) (("Finger" + i as string) + "_control")  "x" "positive"
					append fingerMainControls ControlToAdd
					controlToAdd.parent = pointToAdd
					append allcreated pointToAdd
					append allcreated ControlToAdd
					controlToAdd.objectoffsetpos = [0,0,-(BaseHandLength*0.5)]
					if DetectedAlignment == "L" do (controlToAdd.objectoffsetpos = [0,0,(BaseHandLength*0.5)])
				)
				if i > numfingers do
				(
					local PointToAdd = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename ("Finger" + ((i as integer) as string) + "_parent") Numdigits:3) size:0 wirecolor:Green
					PointToAdd.rotation *= PointToAdd.transform.rotation * (inverse HandGuideBone.transform.rotation)
					PointToAdd.pos.controller = Position_Constraint()
					local FrontPercent
					FrontPercent= (90)
					PointToAdd.pos.controller.appendtarget HandFrontWidthPoint FrontPercent
					PointToAdd.pos.controller.appendtarget HandBackWidthPoint (100-FrontPercent)
					PointToAdd.parent = HandLocator
					ThumbMainControlParent = PointToAdd
					local controlToAdd = CommonFns.MakeController (TinySize) "square" PointToAdd PointToAdd  (color 65 130 65) ("Thumb_control")  "x" "positive"
					in coordsys local move ControlToAdd [(-(BaseHandLength*0.8)),0,0]
					in coordsys local rotate ControlToAdd (angleaxis -45 [0,0,1])
					in coordsys local rotate ControlToAdd (angleaxis -45 [1,0,0])
					controlToAdd.objectoffsetpos = [0,0,-(BaseHandLength*0.5)]
					if DetectedAlignment == "L" do (controlToAdd.objectoffsetpos = [0,0,(BaseHandLength*0.5)])
					
					controlToAdd.parent = pointToAdd
					ThumbMainControl = ControlToAdd
					append Allcreated pointtoAdd
					append Allcreated ControlToAdd
					
				)
				

----------------------Add the custom attributes for the finger guide controls ---------------------------------------------------------
				
				local ca
				local AttHolder
				
				ca=attributes FingerControls
				(
					parameters params rollout:ro_FingerControls
					(
							paramjoint1length type:#float ui:spn_joint1length
							paramjoint1bend type:#float ui:spn_joint1bend
							paramfingerlength type:#float ui:spn_fingerlength
							paramfingerbend type:#float ui:spn_fingerbend
							paramjoint2length type:#float ui:spn_joint2length
							paramjoint2bend type:#float ui:spn_joint2bend
							paramjoint3length type:#float ui:spn_joint3length
							paramjoint3bend type:#float ui:spn_joint3bend
					)
					rollout ro_FingerControls "Finger_Controls" 
					(
						spinner spn_joint1length "Joint1 Length" enabled:on  range:[0,9999,0] scale:0.1
						spinner spn_joint1bend "Joint1 Bend" enabled:on  range:[-170,170,15] scale:0.1
						spinner spn_joint2length "Joint2 Length" enabled:on  range:[0,9999,0] scale:0.1
						spinner spn_joint2bend "Joint2 Bend" enabled:on  range:[-170,170,15] scale:0.1
						spinner spn_joint3length "Joint3 Length" enabled:on  range:[0,9999,0] scale:0.1
						spinner spn_joint3bend "Joint3 Bend" enabled:on  range:[-170,170,15] scale:0.1
						spinner spn_fingerlength "Finger Scale" enabled:on  range:[0,5,1] scale:0.05
						spinner spn_fingerbend "Finger Bend" enabled:on  range:[-170,170,0] scale:0.1
					)
				)
					
				AttHolder = (EmptyModifier ())
				AttHolder.name = "Finger_Controls"
				addmodifier ControlToAdd AttHolder
				custAttributes.add ControlToAdd.modifiers[#Finger_Controls] ca
				ControlToAdd.modifiers[#Finger_Controls].ro_FingerControls.spn_fingerlength.value = 1

				if fingerjoints <3 do
				(
					ControlToAdd.modifiers[#Finger_Controls].ro_FingerControls.spn_joint3length.enabled = false
					ControlToAdd.modifiers[#Finger_Controls].ro_FingerControls.spn_joint3bend.enabled = false
					ControlToAdd.modifiers[#Finger_Controls].ro_FingerControls.spn_joint3bend.enabled = false
					ControlToAdd.modifiers[#Finger_Controls].ro_FingerControls.spn_joint1length.value = (BaseHandLength*0.6)
					ControlToAdd.modifiers[#Finger_Controls].ro_FingerControls.spn_joint2length.value = (BaseHandLength*0.4)
					ControlToAdd.modifiers[#Finger_Controls].ro_FingerControls.spn_joint3length.value = (0)
				)
				if fingerjoints <2 do
				(
					ControlToAdd.modifiers[#Finger_Controls].ro_FingerControls.spn_joint2length.enabled = false
					ControlToAdd.modifiers[#Finger_Controls].ro_FingerControls.spn_joint2bend.enabled = false
					ControlToAdd.modifiers[#Finger_Controls].ro_FingerControls.spn_fingerlength.enabled = false
					ControlToAdd.modifiers[#Finger_Controls].ro_FingerControls.spn_fingerbend.enabled = false
					ControlToAdd.modifiers[#Finger_Controls].ro_FingerControls.spn_joint1length.value = (BaseHandLength)
					ControlToAdd.modifiers[#Finger_Controls].ro_FingerControls.spn_joint2length.value = (0)	
					ControlToAdd.modifiers[#Finger_Controls].ro_FingerControls.spn_joint3length.value = (0)	
				)
				if fingerjoints == 3 do
				(
				ControlToAdd.modifiers[#Finger_Controls].ro_FingerControls.spn_joint1length.value = (BaseHandLength*0.4)
				ControlToAdd.modifiers[#Finger_Controls].ro_FingerControls.spn_joint2length.value = (BaseHandLength*0.35)	
				ControlToAdd.modifiers[#Finger_Controls].ro_FingerControls.spn_joint3length.value = (BaseHandLength*0.25)	
				)
				
------------------------------------------------------Create the finger guide bones------------------------------------------------------				
				
				local CurrentFingerGuideJoints = #() 
				local CurrentFingerControl = ControlToAdd
				local CurrentFingerForOutput = #()
				
				
				
				for j = 1 to j = fingerjoints+1 do
				(
					local BoneToAdd
					if j != 4 then (BoneToAdd  = boneSys.createBone [(j*10),i*2,0] [((j+1)*10),i*2,0] [0,0,1]) else (BoneToAdd  = boneSys.createBone [((j)*10),i*2,0] [((j+0.005)*10),i*2,0] [0,0,1])
					if j <= fingerjoints then 
					(
						BoneToAdd.Name = (uniquename ("Finger" + (i as string) +"_Bone" + (j as string)) numdigits:3)
						append CurrentFingerForOutput BoneToAdd
					)
					else 
					(
						BoneToAdd.Name = (uniquename ("Finger" + (i as string) +"_Nub") numdigits:3)
						append CurrentFingerForOutput BoneToAdd
					)
						
					append CurrentFingerGuideJoints BoneToAdd
					BoneToAdd.boneFreezeLength=false
					BoneToAdd.boneScaleType=#scale
					BoneToAdd.frontfin = off
					BoneToAdd.BackFin = off
					BoneToAdd.Width = (ToolSetup.Nullsizes[1]*0.5)
					append Allcreated BoneToAdd
					
				)
				append FingerOutputArray CurrentFingerForOutput
				
				for j = 2 to j = fingerjoints+1 do (CurrentFingerGuideJoints[j].parent = CurrentFingerGuideJoints[j-1])
				
				CurrentFingerGuideJoints[1].rotation *= CurrentFingerGuideJoints[1].transform.rotation * (inverse CurrentFingerControl.transform.rotation)
				in coordsys world CurrentFingerGuideJoints[1].pos = CurrentFingerControl.transform.pos
				CurrentFingerGuideJoints[1].parent = CurrentFingerControl
				
---------------------------------------------Apply the position and rotation expressions to the finger guide bones------------------------------------------
					
				for j = 1 to j = fingerjoints+1 do
				(					
					if j == 1 do
					(
						local RotExpressionToAdd
						RotExpressionToAdd = Float_Expression()
						RotExpressionToAdd.AddScalarTarget "SliderValue" CurrentFingerControl.modifiers[#Finger_Controls].FingerControls[#paramjoint1bend]
						RotExpressionToAdd.setexpression "SliderValue"		
						if FingerJoints > 1 do
						(
						RotExpressionToAdd.AddscalarTarget "GlobalValue" CurrentFingerControl.modifiers[#Finger_Controls].FingerControls[#paramfingerbend]	
						RotExpressionToAdd.setexpression "SliderValue+GlobalValue"	
						)
						CurrentFingerGuideJoints[j].rotation.controller.Y_Rotation.controller = RotExpressionToAdd
					)
					
					
					if j ==2 do
					(
						local PosExpressionToAdd
						local RotExpressionToAdd
						PosExpressionToAdd = Float_Expression()
						PosExpressionToAdd.AddScalarTarget "SliderValue" CurrentFingerControl.modifiers[#Finger_Controls].FingerControls[#paramjoint1length]
						PosExpressionToAdd.AddscalarTarget "GlobalValue" CurrentFingerControl.modifiers[#Finger_Controls].FingerControls[#paramfingerlength]
						PosExpressionToAdd.setexpression "SliderValue*GlobalValue"
						CurrentFingerGuideJoints[j].pos.controller.X_Position.controller = PosExpressionToAdd
						
						if FingerJoints > 1 do
						(
						RotExpressionToAdd = Float_Expression()
						RotExpressionToAdd.AddScalarTarget "SliderValue" CurrentFingerControl.modifiers[#Finger_Controls].FingerControls[#paramjoint2bend]
						RotExpressionToAdd.AddscalarTarget "GlobalValue" CurrentFingerControl.modifiers[#Finger_Controls].FingerControls[#paramfingerbend]		
						RotExpressionToAdd.setexpression "SliderValue+GlobalValue"
						CurrentFingerGuideJoints[j].rotation.controller.Y_Rotation.controller = RotExpressionToAdd
						)								
					)
					
					if j ==3 do
					(
						local PosExpressionToAdd
						local RotExpressionToAdd
						PosExpressionToAdd = Float_Expression()
						PosExpressionToAdd.AddScalarTarget "SliderValue" CurrentFingerControl.modifiers[#Finger_Controls].FingerControls[#paramjoint2length]
						PosExpressionToAdd.AddscalarTarget "GlobalValue" CurrentFingerControl.modifiers[#Finger_Controls].FingerControls[#paramfingerlength]	
						PosExpressionToAdd.setexpression "SliderValue*GlobalValue"
						CurrentFingerGuideJoints[j].pos.controller.X_Position.controller = PosExpressionToAdd
						
						if FingerJoints > 2 do
						(
							RotExpressionToAdd = Float_Expression()
							RotExpressionToAdd.AddScalarTarget "SliderValue" CurrentFingerControl.modifiers[#Finger_Controls].FingerControls[#paramjoint3bend]
							RotExpressionToAdd.AddscalarTarget "GlobalValue" CurrentFingerControl.modifiers[#Finger_Controls].FingerControls[#paramfingerbend]	
							RotExpressionToAdd.setexpression "SliderValue+GlobalValue"	
							CurrentFingerGuideJoints[j].rotation.controller.Y_Rotation.controller = RotExpressionToAdd
						)
					)
					
					if j == 4 do
					(
						local PosExpressionToAdd
						PosExpressionToAdd = Float_Expression()
						PosExpressionToAdd.AddScalarTarget "SliderValue" CurrentFingerControl.modifiers[#Finger_Controls].FingerControls[#paramjoint3length]
						PosExpressionToAdd.AddscalarTarget "GlobalValue" CurrentFingerControl.modifiers[#Finger_Controls].FingerControls[#paramfingerlength]	
						PosExpressionToAdd.setexpression "SliderValue*GlobalValue"
						CurrentFingerGuideJoints[j].pos.controller.X_Position.controller = PosExpressionToAdd
					)
				)	
			)--this bracket to close finger making loop
		)
		
		------------NEXT: Make the shoulder and extra deformation guide bones-------------------------------
		
		--Start By Making the ribcage control--------------------------------
		
		local ribcageLocator = Ngon Name: (Uniquename "Ribcage_Guide" numdigits:3) radius:((Distance UpArmLocator.transform.pos [0,UpArmLocator.transform.pos.y,UpArmLocator.transform.pos.z])) cornerRadius:0 nsides:8 circular:off scribe:1 pos:UpArmLocator.transform.pos isSelected:off
		converttosplineshape ribcagelocator
		local ribcageSplineBottom = Ngon Name:(Uniquename "Ribcage_BottomSpline" numdigits:3) radius:((Distance UpArmLocator.transform.pos [0,UpArmLocator.transform.pos.y,UpArmLocator.transform.pos.z])) cornerRadius:0 nsides:8 circular:off scribe:1 pos:UpArmLocator.transform.pos isSelected:off
		
		in coordsys local scale ribcageLocator [1,0.6,1]
		in coordsys local scale ribcageSplineBottom [1,0.6,1]
		
		local ribcageSplineMid = Ngon Name:(Uniquename "Ribcage_MidSpline" numdigits:3) radius:((Distance UpArmLocator.transform.pos [0,UpArmLocator.transform.pos.y,UpArmLocator.transform.pos.z])) cornerRadius:0 nsides:8 circular:off scribe:1 pos:(UpArmLocator.transform.pos) isSelected:off
		in coordsys local move ribcageSplineMid [0,0,((Distance UparmLocator ElbowLocator)*0.3)]
		in coordsys local scale ribcageSplineMid [1,0.6,1]
		converttosplineshape ribcageSplineMid
		addandweld ribcageLocator RibcageSplineMid 0
		local ribcageSplineMid = Ngon Name:(Uniquename "Ribcage_MidSpline" numdigits:3) radius:((Distance UpArmLocator.transform.pos [0,UpArmLocator.transform.pos.y,UpArmLocator.transform.pos.z])) cornerRadius:0 nsides:8 circular:off scribe:1 pos:(UpArmLocator.transform.pos) isSelected:off
		in coordsys local move ribcageSplineMid [0,0,((Distance UparmLocator ElbowLocator)*0.3)]
		in coordsys local scale ribcageSplineMid [1,0.6,1]
		
		local ribcageSplineTop = Ngon Name:(Uniquename "Ribcage_TopSpline" numdigits:3) radius:((Distance UpArmLocator.transform.pos [0,UpArmLocator.transform.pos.y,UpArmLocator.transform.pos.z])) cornerRadius:0 nsides:8 circular:off scribe:1 pos:(ribcageSplineMid.transform.pos) isSelected:off
		converttosplineshape ribcageSplineTop
		in coordsys local move ribcageSplineTop [0,0,((Distance UparmLocator ElbowLocator)*0.05)]
		in coordsys local scale ribcageSplineTop [0.7,0.4,1]
		addandweld ribcageLocator ribcageSplineTop 0
		local ribcageSplineTop = Ngon Name:(Uniquename "Ribcage_TopSpline" numdigits:3) radius:((Distance UpArmLocator.transform.pos [0,UpArmLocator.transform.pos.y,UpArmLocator.transform.pos.z])) cornerRadius:0 nsides:8 circular:off scribe:1 pos:(ribcageSplineMid.transform.pos) isSelected:off
		in coordsys local move ribcageSplineTop [0,0,((Distance UparmLocator ElbowLocator)*0.05)]
		in coordsys local scale ribcageSplineTop [0.7,0.4,1]
			
		addmodifier ribcagelocator (CrossSection ())
			
		ribcagesplinebottom.parent = ribcagelocator
		ribcagesplinemid.parent = ribcagelocator
		ribcagesplinetop.parent = ribcagelocator
		ribcagesplinebottom.ishidden = true
		ribcagesplinemid.ishidden = true
		ribcagesplinetop.ishidden = true		
		ribcagelocator.ishidden = true			
			
		local WidthPercent
		if DetectedAlignment == "R" then (WidthPercent = -50) else (WidthPercent = 0)
			
		local RibcageBotFront = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename "RibcageBotFrontPoint" Numdigits:3) size:0 wirecolor:Green
		ribcageBotFront.pos.controller = path_constraint()
		ribcageBotFront.pos.controller.path =  ribcagesplinebottom
		ribcageBotFront.pos.controller.percent = -25
			
		
		local RibcageBotWidth = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename "RibcageBotWidthPoint" Numdigits:3) size:0 wirecolor:Green
		ribcagebotwidth.pos.controller = path_constraint()
		ribcagebotwidth.pos.controller.path =  ribcagesplinebottom
		ribcagebotwidth.pos.controller.percent = WidthPercent
		
		local RibcageMidFront = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename "RibcageMidFrontPoint" Numdigits:3) size:0 wirecolor:Green
		ribcagemidfront.pos.controller = path_constraint()
		ribcagemidfront.pos.controller.path =  ribcagesplinemid
		ribcagemidfront.pos.controller	.percent = -25
			
		local RibcageMidWidth = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename "RibcageMidWidthPoint" Numdigits:3) size:0 wirecolor:Green
		ribcagemidwidth.pos.controller = path_constraint()
		ribcagemidwidth.pos.controller.path =  ribcagesplinemid
		ribcagemidwidth.pos.controller.percent = WidthPercent
			
		local RibcageTopFront = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename "RibcageTopFrontPoint" Numdigits:3) size:0 wirecolor:Green
		ribcagetopfront.pos.controller = path_constraint()
		ribcagetopfront.pos.controller.path =  ribcagesplinetop
		ribcagetopfront.pos.controller.percent = -25
			
		local RibcageTopWidth = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename "RibcageTopWidthPoint" Numdigits:3) size:0 wirecolor:Green
		ribcagetopwidth.pos.controller = path_constraint()
		ribcagetopwidth.pos.controller.path =  ribcagesplinetop
		ribcagetopwidth.pos.controller.percent = WidthPercent
		
		in coordsys world RibcageLocator.pos.x = 0
		in coordsys world move RibcageLocator[0,0,-((Distance UparmLocator ElbowLocator)*0.25)]
		
		
		append AllCreated RibcageLocator
		append AllCreated RibcageSplineBottom
		append AllCreated RibCageSplineMid
		append AllCreated RibcageSplineTop
		append AllCreated RibcageBotFront
		append AllCreated RibcageBotWidth
		append AllCreated RibcageMidFront
		append AllCreated RibcageMidWidth
		append AllCreated RibcageTopFront
		append AllCreated RibcageTopWidth
		
		local RibcageWidthHelpersArray = #()
		append RibcageWidthHelpersArray RibcageSplineMid
		append RibcageWidthHelpersArray RibcageMidFront
		append RibcageWidthHelpersArray RibcageMidWidth
		append WidthHelpersArray RibcageWidthHelpersArray
		
		------------------Setup the basic lookcons settings to be used in guidebone stuff--------------------------------------
		local BaseLookcons = Lookat_Constraint()
		BaseLookcons.target_axis = 0
		BaseLookcons.upnode_ctrl = 1
		BaseLookcons.upnode_world = off
		BaseLookcons.pickUpNode = RibcageLocator
		BaseLookcons.StoUP_axis = 2
		BaseLookcons.viewline_length_abs = off
		BaseLookcons.lookat_vector_length = 0
		BaseLookcons.upnode_axis = 2
		if DetectedAlignment == "R" then (BaseLookCons.StoUP_axisFlip = on) else (BaseLookCons.StoUP_axisFlip = off)
		
		------------------------------------------Make the clavicle------------------------------------------------------------------
		
		local ClavLocParent = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename "ClavicleParent" Numdigits:3) size:0 wirecolor:Green
		ClavLocParent.pos.controller = position_constraint()
		ClavLocParent.pos.controller.appendtarget RibcageMidFront 85
		ClavLocParent.pos.controller.appendtarget RibcageBotFront 15
		local ClavLocator = CommonFns.MakeController (MediumSize) "sphere" ClavLocParent ClavLocParent (color 40 40 255) (uniquename "Clavicle_Locator" numdigits:3)  "z" "positive"
		ClavLocator.parent = ClavLocParent
		if detectedAlignment == "R" then (in coordsys world move ClavLocator -[((distance [0,0,0] [UpArmLocator.transform.pos.x,0,0])*0.1),0,0])
		else (in coordsys world move ClavLocator [((distance [0,0,0] [UpArmLocator.transform.pos.x,0,0])*0.1),0,0])
		
		setInheritanceFlags ClavLocParent #{1,2,3,4,5,6}
		local ClavGuideBone = (boneSys.createBone [0,0,0] [10,0,0] [0,0,1])
		ClavGuideBone.name = (Uniquename ("ClavGuideBone") Numdigits:3)
		ClavGuideBone.boneFreezeLength=false
		ClavGuideBone.boneScaleType=#scale
		ClavGuideBone.frontfin = off
		local ClavGuideNub = (boneSys.createBone [10,0,0] [1.1,0,0] [0,0,1])
		ClavGuideNub.parent = ClavGuideBone
		ClavGuideBone.pos.controller = position_constraint()
		ClavGuideBone.pos.controller.appendtarget ClavLocator 100
		ClavGuideBone.parent = ClavLocator

		
		if SplitShoulder != true do
		(
		ClavGuideBone.rotation.controller = (copy BaseLookCons)
		ClavGuideBone.rotation.controller.appendtarget UpArmLocator 100
		ClavGuideNub.pos.controller = position_constraint()
		ClavGuideNub.pos.controller.appendtarget UpArmLocator 100
		)
		
		append AllCreated ClavLocParent
		append AllCreated ClavLocator
		append AllCreated ClavGuideBone
		append AllCreated ClavGuideNub
		append ShoulderBones ClavGuideBone
		append ShoulderBones ClavGuideNub
		
		
		
		local ClavSplitLocator
		local ClavSplitGuideBone
		local ClavSplitGuideNub
		

		if SplitShoulder == true do
		(
			ClavSplitLocator = CommonFns.MakeController (MediumSize) "sphere" ClavLocParent ClavLocParent (color 40 40 255) ("Clavicle_Split_Locator")  "z" "positive"
			local TempPoint = point Name:"ZZZZZTEMPPOINT"
			temppoint.pos.controller = position_constraint()
			temppoint.pos.controller.appendtarget ClavLocator 75
			temppoint.pos.controller.appendtarget UpArmLocator 100
			in coordsys world ClavSplitLocator.pos = temppoint.transform.pos
			ClavGuideBone.rotation.controller = (copy BaseLookCons)
			ClavGuideBone.rotation.controller.appendtarget ClavSplitLocator 100
			ClavGuideNub.pos.controller = position_constraint()
			ClavGuideNub.pos.controller.appendtarget ClavSplitLocator 100
			ClavSplitLocator.parent = ClavLocParent
			
			ClavSplitGuideBone = (boneSys.createBone [0,0,0] [10,0,0] [0,0,1])
			ClavSplitGuideNub = (boneSys.createBone [10,0,0] [10.1,0,0] [0,0,1])
			ClavSplitGuideNub.parent = ClavSplitGuideBone
			ClavSplitGuideBone.name = (Uniquename ("ClavSplitGuideBone") Numdigits:3)
			ClavSplitGuideNub.name = (Uniquename ("ClavSplitGuideNub") Numdigits:3)
			ClavSplitGuideNub.width = 0
			ClavSplitGuideNub.height = 0
	
			ClavSplitGuideBone.boneFreezeLength=false
			ClavSplitGuideBone.boneScaleType=#scale
			ClavSplitGuideBone.frontfin = off
			ClavSplitGuideBone.rotation.controller = (copy BaseLookCons)
			ClavSplitGuideBone.rotation.controller.appendtarget UpArmLocator 100
			ClavSplitGuideBone.pos.controller = position_constraint()
			ClavSplitGuideBone.pos.controller.appendtarget ClavSplitLocator 100
			
			ClavSplitGuideNub.pos.controller = position_constraint()
			ClavSplitGuideNub.pos.controller.appendtarget UpArmLocator 100
			
			append allcreated ClavSplitGuideBone
			append allcreated ClavSplitGuideNub
			append allcreated ClavSplitLocator
			delete temppoint

			ClavSplitGuideNub.ishidden = true
			append ShoulderBones ClavSplitGuideBone
			append ShoulderBones ClavSplitGuideNub

		)
		

		--------------------------------Make the Upper Pectoral Fin-------------------------------
		
		local PectoralComponents = #()
		
		local TempPecStartParent = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename "PecParent" Numdigits:3) size:0 wirecolor:Green
		TempPecStartParent.pos.controller = position_constraint()
		TempPecStartParent.pos.controller.appendtarget RibcageMidFront 30
		TempPecStartParent.pos.controller.appendtarget RibcageBotFront 70
		
		
		local TempPecEndPoint = Point Name:"ZZZZUpperPecEnd"
		TempPecEndPoint.pos.controller = position_constraint()
		TempPecEndpoint.pos.controller.appendtarget UpArmGuideBone 80
		TempPecEndpoint.pos.controller.appendtarget LowArmGuideBone 20
		TempPecEndPoint.pos.controller = Position_XYZ()
		in coordsys ClavGuideBone Move TempPecEndPoint [0,-TinySize,0]
		
		local TempLookatPoint = Point Name:"ZZZZTEMPPECLOOKATPOINT"
		TempLookatPoint.pos.controller = position_constraint()
		TempLookatPoint.pos.controller.appendtarget RibcageSplineMid 30
		TempLookatPoint.pos.controller.appendtarget RibcageSplineBottom 70
		
		
		local TempMuscleUpNode = CommonFns.createsystemalignmentpoint RibcageSplineBottom RibcageSplineMid
		TempMuscleUpNode.ishidden = true
		append Allcreated TempMuscleUpNode
		
		
		local PecGuideOutput = FinalTools.CreateMuscleGuide TempPecStartParent TempPecEndPoint TempLookatPoint TempMuscleUpNode BoneDirection:1 MuscleName:"Pec"  UpNodeAxis:1
		
		
		
		local PecStartLocator = PecGuideOutput[2][1]
		local PecEndLocator = PecGuideOutput[2][2]
		local PecMidPointLocator = PecGuideOutput[2][3]
		local PecLookatLocator = PecGuideOutput[2][4]
		local PecPosConsPointA = PecGuideOutput[4][1]
		local PecPosConsPointB =  PecGuideOutput[4][1]
		local PecStartBone = PecGuideOutput[3][1]
		local PecEndBone = PecGuideOutput[3][2]
		
		for n in  PecGuideOutput[1] do (append allcreated n)
		for n in #(PecStartLocator,PecEndLocator,PecMidPointLocator,PecLookatLocator,PecPosConsPointA,PecPosConsPointB,PecStartBone,PecEndBone) do (append PectoralComponents n)

		
		
		
		
		
		delete TempPecEndPoint
		delete TempPecStartParent
		delete TempLookatPoint

		
						--------------------------------Make the scapula fin-------------------------------
						
		local ScapulaComponents = #()
		
		local ScapulaLocParent = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename "ScapulaParent" Numdigits:3) size:0 wirecolor:Green
		ScapulaLocParent.pos.controller = position_constraint()
		ScapulaLocParent.pos.controller.appendtarget RibcageMidFront 70
		ScapulaLocParent.pos.controller.appendtarget RibcageBotFront 30
		local ScapulaLocator = CommonFns.MakeController (MediumSize) "sphere" ScapulaLocParent ScapulaLocParent (color 40 40 255) ("Scapula_Locator")  "z" "positive"
		ScapulaLocator.parent = ScapulaLocParent
		setInheritanceFlags ScapulaLocParent #{1,2,3,4,5,6}
		in coordsys world move ScapulaLocator -[((distance [0,0,0] [UpArmLocator.transform.pos.x,0,0])*0.2),-((distance RibcageLocator RibcageBotFront)*2),0]  
		
		local ScapulaEndLocator = CommonFns.MakeController (MediumSize) "sphere" ScapulaLocator ScapulaLocator (color 40 200 40) ("Scapula_End_Locator")  "x" "positive"
		if DetectedAlignment == "R" then 
		(
			in coordsys world move ScapulaEndLocator -[((distance [0,0,0] [UpArmLocator.transform.pos.x,0,0])*0.6),0,0]  
		)
		else
		(
			in coordsys world move ScapulaEndLocator [((distance [0,0,0] [UpArmLocator.transform.pos.x,0,0])*0.6),0,0]  
		)
		ScapulaEndLocator.parent = ScapulaLocParent
		
		local ScapulaGuideBone = (boneSys.createBone [0,0,0] [10,0,0] [0,0,1])
		ScapulaGuideBone.boneFreezeLength=false
		ScapulaGuideBone.boneScaleType=#scale
		ScapulaGuideBone.frontfin = off
		local ScapulaGuideNub = (boneSys.createBone [10,0,0] [10.1,0,0] [0,0,1])
		ScapulaGuideNub.parent = ScapulaGuideBone
		ScapulaGuideBone.pos.controller = position_constraint()
		ScapulaGuideBone.pos.controller.appendtarget ScapulaLocator 100
		ScapulaGuideBone.parent = ScapulaLocator
		ScapulaGuideBone.rotation.controller = (copy BaseLookCons)
		ScapulaGuideBone.rotation.controller.appendtarget ScapulaEndLocator 100
		ScapulaGuideNub.pos.controller = position_constraint()
		ScapulaGuideNub.pos.controller.appendtarget ScapulaEndLocator 100
		
		
		
		append AllCreated ScapulaLocParent
		append AllCreated ScapulaLocator
		append AllCreated ScapulaEndLocator
		append AllCreated ScapulaGuideBone
		append AllCreated ScapulaGuideNub
		append ScapulaComponents ScapulaGuideBone
		append ScapulaComponents ScapulaGuideNub
		
						--------------------------------Make the teres components------------------------------
						
		local TeresComponents = #()
		
		local TempTeresStartParent = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename "TeresParent" Numdigits:3) size:0 wirecolor:Green
		TempTeresStartParent.pos.controller = position_constraint()
		TempTeresStartParent.pos.controller.appendtarget ScapulaLocator 40
		TempTeresStartParent.pos.controller.appendtarget ScapulaEndLocator 60
		
		local TempTeresStartPoint = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename "TeresPoint" Numdigits:3) size:0 wirecolor:Green
		CommonFns.quickalign TempTeresStartPoint TempTeresStartParent
		in CoordSys RibcageLocator move TempTeresStartPoint [0,-TinySize,-MediumSize]
		

		
		
		local TempTeresEndPoint = Point Name:"ZZZZUTeresEnd"
		TempTeresEndPoint.pos.controller = position_constraint()
		TempTeresEndPoint.pos.controller.appendtarget UpArmGuideBone 70
		TempTeresEndPoint.pos.controller.appendtarget LowArmGuideBone 30
		TempTeresEndPoint.pos.controller = Position_XYZ()
		in coordsys ClavGuideBone Move TempTeresEndPoint [0,TinySize,0]
		
		local TempTeresLookatPointParent = Point Name:"ZZZZTEMPTeresLOOKATPOINTPARENT"
		TempTeresLookatPointParent.pos.controller = position_constraint()
		TempTeresLookatPointParent.pos.controller.appendtarget RibcageSplineMid 80
		TempTeresLookatPointParent.pos.controller.appendtarget RibcageSplineBottom 20
		local TempTeresLookatPoint = Point Name:"ZZZZTEMPTeresLOOKATPOINT"
		CommonFns.QuickAlign TempTeresLookatPoint TempTeresLookatPointParent
		
		
		local TeresSideOffset = (Distance RibcageSplineMid RibcageMidWidth)*0.4
		if DetectedAlignment == "R" do (TeresSideOffset = (-TeresSideOffset))

		in coordsys RibcageLocator move TempTeresLookatPoint [TeresSideOffset,0,0]
		
		
		local TeresGuideOutput = FinalTools.CreateMuscleGuide TempTeresStartPoint TempTeresEndPoint TempTeresLookatPoint TempMuscleUpNode BoneDirection:3 MuscleName:"Teres"  UpNodeAxis:1 MidPointOffset:1.0
		
		
		
		local TeresStartLocator = TeresGuideOutput[2][1]
		local TeresEndLocator = TeresGuideOutput[2][2]
		local TeresMidPointLocator = TeresGuideOutput[2][3]
		local TeresLookatLocator = TeresGuideOutput[2][4]
		local TeresPosConsPointA = TeresGuideOutput[4][1]
		local TeresPosConsPointB =  TeresGuideOutput[4][1]
		local TeresStartBone = TeresGuideOutput[3][1]
		local TeresEndBone = TeresGuideOutput[3][2]
		
		for n in  TeresGuideOutput[1] do (append allcreated n)
		for n in #(TeresStartLocator,TeresEndLocator,TeresMidPointLocator,TeresLookatLocator,TeresPosConsPointA,TeresPosConsPointB,TeresStartBone,TeresEndBone) do (append TeresComponents n)
		
		delete TempTeresStartParent
		delete TempTeresStartPoint
		delete TempTeresEndPoint
		delete TempTeresLookatPoint
		delete TempTeresLookatPointParent


		
		
		

		
		-----------------------------------Neaten up all the bones-----------------------------------------
		

		
		
		for i =1 to i = MainBones.count do
		(
			MainBones[i].frontfin = off
			MainBones[i].sidefins = off
			MainBones[i].backfin = off
			MainBones[i].width = (tinysize*1.2)
			MainBones[i].height = (tinysize*1.2)
			MainBones[i].resetbonestretch
		)
		
		if FingerOutputArray != #() do
		(
			for i = 1 to i = FingerOutputArray.count do
			(
				for j = 1 to j = FingerOutputArray[i].count do
				(
					FingerOutputArray[i][j].frontfin = off
					FingerOutputArray[i][j].sidefins = off
					FingerOutputArray[i][j].backfin = off
					FingerOutputArray[i][j].width = (tinysize*0.5)
					FingerOutputArray[i][j].height = (tinysize*0.5)
					FingerOutputArray[i][j].resetbonestretch
				)
			)
		)
		
		for i =1 to i = shoulderbones.count do 
		(
			shoulderbones[i].frontfin = off
			shoulderbones[i].sidefins = off
			shoulderbones[i].backfin = off
			shoulderbones[i].width = (TinySize)
			shoulderbones[i].height = (TinySize)
			shoulderbones[i].resetbonestretch
		)
		
		ClavGuidebone.width = (Tinysize)
		ClavGuidebone.height = (Tinysize)
		ClavGuideBone.resetbonestretch


		
		HandGuideNub.width = 0
		HandGuideNub.height = 0
		UpArmGuideNub.width = 0
		UpArmGuideNub.height = 0
		LowarmGuideNub.height = 0
		LowarmGuideNub.width = 0
		
		if ExtraShoulder == false do
		(
			for n in PectoralComponents do (n.ishidden = true)
			for n in TeresComponents do (n.ishidden = true)
			for n in ScapulaComponents do (n.ishidden = true)
			ScapulaLocator.ishidden = true
			ScapulaEndLocator.ishidden = true
		)
		
		------------------Finally: Assemble all of the armguide data into an output. -------------------------------------------
		
		
		local FinalOutputArray = #()
		
		append SettingsArray RibcageLocator
		
		append FinalOutputArray allcreated
		append FinalOutputArray mainbones
		append FinalOutputArray shoulderbones
		append FinalOutputArray FingerOutputArray
		append FinalOutputArray SettingsArray
		append FinalOutPutArray WidthHelpersArray
		append FinalOutputArray ScapulaComponents
		append FinalOutputArray PectoralComponents 
		append FinalOutputArray TeresComponents

				for n in allcreated do 
		(
			if classof n == bonegeometry do
			(
				n.showFrozenInGray = off
				freeze n
			)
		)
		

		--OutputArray:
		--1:Allcreated
		--2: Main Bones (UpArmBone,LowArmBone,HandBone,HandNub)
		--3: Shoulder Bones: Clavicle, Clavicle Nub,  --Possibly: ClavSplitBone, Nub
		--4: Finger Bones: FINGER 1: base mid end nub, FINGER 2: etc final is THUMB
		--5: Settings Array
		--6: Width Helpers (UpArmStart Guide, UparmStartFrontWidth, UpArmStartWidth) (UpaRm end) (Wrist) (Hand)
		--7: Scapula GuideBone, ScapulaGuideNub
		--8:Pectoral Components --PecStartLocator,PecEndLocator,PecMidPointLocator,PecLookatLocator,PecPosConsPointA,PecPosConsPointB,PecStartBone,PecEndBone
		--9: Teres Components

		
		--SETTINGS ARRAY
		--NumFingers (0 to 4)
		--FingerJoints (1 to 3)
		--IncludeThumb (True of False)
		--ShoulderDeformers (True or False)
		--DetectedAlignment (L or R)
		--RibcageLocator
		
		

		-- if 4 then a split with no extras
		--if 2 then no split no extras
		
		
		FinalOutputArray
		
	),--THIS BRACKET TO CLOSE ARMGUIDE FUNCTION
	
	--------------------------------------------------------------------------------------------------------------------------------------------------
	fn MakeArm MainBones Finger1 Finger2 Finger3 Finger4 Thumb ShoulderBones SideAlignment RibcageAlignment Kinematics Stretchy Bendy Prefix:"" ArmWidths:undefined ArmParent:Undefined Root:Undefined TwistBones:3 ExtraShoulderBones:false Split:False ExtraBonesInput:undefined Squash:false ElbowType:1 BendyRom:1 =
	(
		----------------------------INITIAL MAKE ARM SETUP----------------------------------------
		
		
			local AverageScale = ((distance MainBones[1].pos MainBones[2].pos) + (distance mainBones[2].pos mainbones[3].pos))/2
		
			local TinySize = AverageScale/18
			local MediumSize = AverageScale/12
			local BigSize = AverageScale/6
		

			local FlipControllers
			if SideAlignment == "L" then 
			(
				FlipControllers = false

			) else 
			(
				FlipControllers = true
			)

		
		
			local UparmBoneName = Prefix + "cnstarget_" + SideAlignment + "_Arm_Upper"
			local UparmIKBoneName = Prefix + "cnstarget_" + SideAlignment + "_IKArm_Upper"
			local UparmFKBoneName = Prefix + "cnsTarget_" + SideAlignment + "_FKArm_Upper"
			local LowarmIKBoneName = Prefix + "cnstarget_" + SideAlignment + "_IKArm_Lower"
			local LowArmFKBoneName = Prefix + "cnstarget_" + SideAlignment + "_FKArm_Lower"
		
			local UparmTwistBoneName =	"bn_uparmtwist" + SideAlignment
			local LowarmTwistBoneName =	"bn_lowarmtwist" + SideAlignment
		
		
			local UparmBoneNameForSkin = Prefix + "bn_" + SideAlignment + "_Arm_Upper"
			local LowarmBoneNameForSkin = Prefix + "bn_" + SideAlignment + "_Arm_Lower"
		
			local IKHandBoneName = Prefix + "cnstarget_" + SideAlignment + "_IKHand"
			local FKHandBoneName = Prefix + "cnstarget_" + SideAlignment + "_FKHand"
			local LowarmBoneName = Prefix + "cnstarget_" + SideAlignment + "_Arm_Lower"
			local HandBoneName = Prefix + "bn_" + SideAlignment + "_hand"
			local ClavicleBoneName = Prefix + "bn_" + SideAlignment + "_clavicle"
			local ScapulaBoneName = Prefix + "bn_" + SideAlignment + "_Scapula"
			local PecBoneName = Prefix + "bn_" + SideAlignment + "_Pec"
			local DeltoidBoneName = Prefix + "bn_" + SideAlignment + "_Deltoid"
			local FingerBoneName = Prefix + "bn_" + SideAlignment + "_finger"
			local ClavSplitName = Prefix + "bn_" + SideAlignment + "_Shoulder_Split"
		
			local PectoralNubName = Prefix + "nub_" + SideAlignment + "_pectoral"
			local DeltoidNubName = Prefix + "nub_" + SideAlignment + "_deltoid"
			local SwitchControllerName = Prefix + "ctrl_" + SideAlignment + "_Arm_Controls"
		
			local Shoulder50BoneName = Prefix + "bn_" + SideAlignment + "_shoulder"
			local Shoulder50NullName = Prefix + "null_" + SideAlignment + "_shoulder"
			local Shoulder50ParentNullName = Prefix + "null_" + SideAlignment + "deltoid_raise"
		
			local Elbow50NullName = Prefix + "null_" + SideAlignment + "_elbow50blend"
			local Elbow50ParentName = Prefix + "null_" + SideAlignment + "_elbowdeform"
			
			local Elbow50BoneName = Prefix + "bn_" + SideAlignment + "_elbowdeform"
			
			
			
			local ElbowControlName = Prefix + "ctrl_" + SideAlignment + "elbow"
			local ScapulaControlName = Prefix + "ctrl_" + SideAlignment + "scapula"
			
			local Elbow50BoneFrontName = Prefix + "bn_" + SideAlignment + "_elbow_back"
			local Elbow50BoneBackName = Prefix + "bn_" + SideAlignment + "_elbow_front"
			
			
			local ElbowBendyControlName = Prefix + "ctrl_" + SideAlignment + "_elbowbend"
			local UparmBendyControlName = Prefix + "ctrl_" + SideAlignment + "_uparmbend"
			local LowArmBendyControlName = Prefix + "ctrl_" +SideAlignment + "_lowarmbend"
			
			local MeasureHelperLookatName = (Prefix + "null_" + SideAlignment + "_ElbowBendMeasure")
			local MeasureHelperLocalName = (Prefix + "null_" + SideAlignment + "_ElbowBendLocal")
			
		
		
		
		
			local UpArmFKControlName = Prefix + "ctrl_" + SideAlignment + "_UpArmFK"
			local LowArmFKControlName = Prefix + "ctrl_" + SideAlignment + "_LowArmFK"
			local HandFKControlName = Prefix + "ctrl_" + SideAlignment + "_HandFK"
			local HandIKControlName = Prefix + "ctrl_" + SideAlignment + "_HandIK"
			local FingerControlName = Prefix + "ctrl_" + SideAlignment + "_finger"
			local ArmIKSolverName = Prefix + "IKsolver_" +SideAlignment
			local ElbowControlName = Prefix + "ctrl_elbow" + SideAlignment
			local ShoulderControlName = Prefix + "ctrl_shoulder" + SideAlignment
			local ClavicleControlName = Prefix + "ctrl" + SideAlignment + "_clavicle"
			local ClavSplitControlName = Prefix + "ctrl" + SideAlignment + "_clavicle_split"
			local ExposeTransformBaseName = Prefix + "helper_exptfm_" + Sidealignment
			local ExposeNodeName = Prefix + "helper_exposenode_" + Sidealignment
			local LocalNodeName = Prefix + "helper_localNode_" + sidealignment
			local ShoulderNoTwistNullName = Prefix + "Null_NoTwist_" + sidealignment + "shoulder"
			local WristNoTwistNullName = Prefix + "Null_NoTwist_" + sidealignment + "wrist"
			local TeresUpNodePointName = Prefix + "null_TeresUpNodePoint_" + SideAlignment 
			

			
			if TwistBones == 1 or TwistBones == 2 do (TwistBones = 3)
			
			local MeasureHelperBaseName = Prefix + "helper_MeasureLookat_" + SideAlignment
			
			local NullifierName = Prefix + "null_" + SideAlignment
			local CnsTargetName = Prefix + "cnstarget_" + Sidealignment
			
			local allcreated = #()
			local mainbonesarray = #()
			local ExtraBones =#()
			local ControlsArray = #()
			local NubsArray = #()
			local CnsTargetBones = #()
			
		
		
			local NumFingers = 0
			local IncludeThumb = 0
			local FingerJoints
			if Finger1.count != 0 then (NumFingers +=1)
			if Finger2.count != 0  then (NumFingers +=1)
			if Finger3.count != 0  then (NumFingers +=1)
			if Finger4.count != 0 then (NumFingers +=1)
			if Thumb.count != 0 then (IncludeThumb = 1)
			
			if Numfingers != 0 do (Fingerjoints = (Finger1.count-1))
			if IncludeThumb != 0 do (Fingerjoints = (Thumb.count-1))
			
				
			local UpArmWidth
			local UpArmFrontWidth
			local LowArmWidth
			local LowArmFrontWidth
			local HandWidth
			local HandFrontWidth
			local FingerWidth
			local RibcageFrontWidth
			local RibcageWidth
			if ArmWidths != undefined do
			(
				UpArmWidth = ArmWidths[1] *1.2
				UpArmFrontWidth = ArmWidths[2] *1.2
				LowArmWidth = ArmWidths[5] *1.2
				LowArmFrontWidth = ArmWidths[6] *1.2
				HandWidth = ArmWidths[7] *1.2
				HandFrontWidth = ArmWidths[8] *1.2
				RibcageFrontWidth = ArmWidths[9]
				RibcageWidth = ArmWidths[10]
				if NumFingers > 2 then (FingerWidth = (HandWidth/NumFingers)) else ((FingerWidth = (HandWidth/3)))
			)
			if ArmWidths == undefined do
			(
				UpArmWidth = 10
				UpArmFrontWidth = 10
				LowArmWidth = 10
				LowArmFrontWidth = 10
				HandWidth = 10
				HandFrontWidth = 10
				FingerWidth = 2
				RibcageFrontWidth = 20
				RibcageWidth = 20
			)
			
			
				
			
		------------------------------Create Main Arm Bones----------------------------------------
			
			local UpArmBone = boneSys.createBone MainBones[1].pos MainBones[2].pos [0,0,1]
			CommonFns.QuickAlign UpArmBone MainBones[1]
			UpArmBone.Name = uniquename (UparmBoneName) Numdigits:3
			append allcreated UpArmBone
			
			

			
			local LowArmBone = boneSys.createBone MainBones[2].pos MainBones[3].pos [0,0,1]
			CommonFns.QuickAlign LowArmBone MainBones[2]
			LowArmBone.Name = uniquename (LowArmBoneName) Numdigits:3
			LowArmBone.parent = UpArmBone
			append allcreated LowArmBone
			
			
			if stretchy == true or bendy == true then
			(
				append CnsTargetBones UpArmBone
				append CnsTargetBones LowArmBone
			)
			else if twistbones > 0 then
			(	
				append CnsTargetBones UpArmBone
				append CnsTargetBones LowArmBone
			)
			else --if no bendy stretchy or twist then they become the main bones
			(
				UpArmBone.name = 	(uniquename UparmBoneNameForSkin numdigits:3)
				LowArmBone.name = (uniquename LowarmBoneNameForSkin numdigits:3)
				CommonFns.ApplyLengthWidthProperties UpArmBone (UpArmWidth*0.8) ObjFrontWidth:(UpArmFrontWidth*0.8) ObjOffset:undefined
				CommonFns.ApplyLengthWidthProperties LowArmBone (LowArmWidth*0.8) ObjFrontWidth:(LowArmFrontWidth*0.8) ObjOffset:undefined
				append mainbonesarray UpArmBone
				append mainbonesarray LowArmBone
			)
			
			
			
			local HandBone = boneSys.createBone MainBones[3].pos MainBones[4].pos [0,0,1]
			CommonFns.QuickAlign HandBone MainBones[3]
			HandBone.Name = uniquename (HandBoneName) Numdigits:3
			HandBone.parent = LowArmBone
			append allcreated HandBone
			append mainbonesarray HandBone
			
			CommonFns.ApplyLengthWidthProperties HandBone (HandWidth*0.8) ObjFrontWidth:(HandFrontWidth*0.8) ObjOffset:undefined
			--------------------Create Fingers-------------------------------------------------------
			
				local CreatedFingersArray = #()
				local CreatedFingerControlsArray = #()
				local CreatedFingerNullsArray = #()
			
			if (NumFingers + IncludeThumb) != 0 do
			(
				for f = 1 to f = (NumFingers + IncludeThumb) do
				(
					local ArrayToAdd = #()
					local ControlsArrayToAdd = #()
					local CurrentFingerToAdd
					local FingerPrefix
					if f >Numfingers then (CurrentFingerToAdd = Thumb) 
					else if f ==1 then (CurrentFingerToAdd = Finger1)
					else if f ==2 then (CurrentFingerToAdd = Finger2)
					else if f==3 then (currentFingerToAdd = Finger3)
					else if f ==4 then (CurrentFingerToAdd = Finger4)
					
					if f > NumFingers then (currentFingerToAdd = Thumb)
					FingerPrefix = (f as string)
					
					for j = 1 to j = Fingerjoints do 
					(
						local JointName
						if j == 1 then jointName = ("Base")
						else if j == 2 then jointName = ("Mid")
						else if j == 3 then jointname = ("End")
						
						local BoneToAdd = boneSys.createBone CurrentFingerToAdd[j].pos CurrentFingerToAdd[j+1].pos [0,0,1]
						CommonFns.QuickAlign boneToadd CurrentFingerToAdd[j]
						CommonFns.ApplyLengthWidthProperties boneToadd (FingerWidth*0.55) ObjFrontWidth:(FingerWidth*0.55) ObjOffset:undefined
						BoneToAdd.Name = (uniquename (FingerBoneName + (FingerPrefix) + "_" + JointName) NumDigits:3)
						append ArrayToAdd BoneToAdd
						local ControlToAdd = CommonFns.DrawRectangleControl BoneToAdd CurrentFingerToAdd[j].pos CurrentFingerToAdd[j+1].pos FingerWidth FingerWidth (uniquename (FingerControlName + (FingerPrefix) + "_" + JointName)  numdigits:3)
						CommonFns.ApplycontrollerTypeProperties ControlToAdd ControlType:"Small" ControlLevel:j
						append ControlsArrayToAdd ControlToAdd
						
						append allcreated BoneToAdd
						append MainBonesarray BoneToAdd
						append allcreated ControlToAdd
						append ControlsArray ControlToAdd
						
						boneToAdd.rotation.controller = orientation_constraint()
						boneToAdd.rotation.controller.appendtarget ControlToAdd 100
						if j != 1 do 
						(
							BoneToAdd.parent = ArrayToAdd[j-1]
							ControlToAdd.parent = ControlsArrayToAdd[j-1]
						)
					
					
						if j ==1 do 
						(
							local NullToAdd = Point Name:(uniquename (Nullifiername + FingerBoneName + (FingerPrefix)) NumDigits:3) Size:Tinysize wireColor: (color 20 80 20)
							CommonFns.quickalign NullToAdd ArrayToAdd[1]
							ControlsArrayToAdd[1].parent = NullToAdd
							ArrayToAdd[1].parent = HandBone
							NullToAdd.parent = HandBone
								
							append allcreated NullToAdd
						)
						
						if j ==1 and fingerjoints >1 do
						(
							local ca
							local AttHolder
								
								ca=attributes FingerBendControls
								(
									parameters params rollout:ro_FingerBendControls
									(
											paramFingerBend type:#float ui:spn_FingerBend
									)
									rollout ro_FingerBendControls "FingerBend_Controls" 
									(
										spinner spn_FingerBend "FingerBend" enabled:on  range:[0,1,0] scale:0.1
									)
								)
									
								AttHolder = (EmptyModifier ())
								AttHolder.name = "FingerBend_Controls"
								addmodifier ControlsArrayToAdd[1] AttHolder
								custAttributes.add ControlsArrayToAdd[1].modifiers[#FingerBend_Controls] ca
								ControlsArrayToAdd[1].modifiers[#FingerBend_Controls].ro_FingerBendControls.spn_FingerBend.value = 1
								--RotExpressionToAdd.AddScalarTarget "SliderValue" switchcontroller.modifiers[#FingerBend_Controls].FingerBendControls[#paramFingerBend]
						)
						
						if j>1 do
						(
							local NullToAdd = Point Name:(uniquename (Nullifiername + FingerPrefix + JointName) NumDigits:3) Size:Tinysize wireColor: (color 20 80 20)
							CommonFns.quickalign NullToAdd BoneToAdd
							local AutoNullToAdd = Point Name:(uniquename (Nullifiername +"AUTO" + (FingerPrefix) + JointName) NumDigits:3) Size:Tinysize wireColor: (color 20 80 20)
							CommonFns.quickalign AutoNullToAdd NullToAdd
							AutoNullToAdd.parent = NullToAdd
							NullToAdd.parent = ArrayToAdd[j-1]
							ControlToAdd.parent = AutoNullToAdd
							
							append allcreated NullToAdd
							append allcreated AutoNullToAdd
							
							local AutoNullScript = Rotation_Script()
							AutoNullScript.addNode "MainControl" ControlsArrayToAdd[1]
							AutoNullScript.AddTarget "Slider" ControlsArrayToAdd[1].modifiers[#FingerBend_Controls].FingerBendControls[#paramFingerBend]
							AutoNullScript.setexpression ("C= quattoeuler (MainControl.transform * inverse MainControl.parent.transform).rotation\n"+
								"C.x=0\n"+
								"C.z =0\n"+
								"C.y = C.y*slider\n"+
								"eulertoquat C")
							AutoNullToAdd.rotation.controller = AutoNullScript
						)
					)
							
						
						append CreatedFingersArray ArrayToAdd
						append CreatedFingerControlsArray ControlsArrayToAdd
				)
			)
			------------------Create Shoulder and deform Bones---------------------------
			
			local ClavicleBone
			local DeltoidBone
			local PectoralBone
			local ScapulaBone
			local ClavicleController
			
			local DeltoidNull
			local ScapulaNull
			local PectoralNull
			
			local ClavContNull
			local ClavSplitContNull
			
			local ClavSplitBone
			local ClavSplitController
			
			local PectoralNub
			local DeltoidNub
			
			local ShoulderConnectParent
			
			local ClavContCnsTarget
			local ClavSplitContCnsTarget
			

				ClavContCnsTarget = Point Name:(UniqueName (CnsTargetName + "_ClavControl") numdigits:3) Size:Tinysize wireColor: (color 20 80 20)
				ClavSplitContCnsTarget = Point Name:(UniqueName (CnsTargetName + "_ClavSplitControl") numdigits:3) Size:Tinysize wireColor: (color 20 80 20)
				append allcreated ClavContCnsTarget
				append allcreated ClavSplitContCnsTarget

			
			
			ClavicleBone = boneSys.createBone ShoulderBones[1].pos ShoulderBones[2].pos [0,0,1]
			CommonFns.QuickAlign ClavicleBone ShoulderBones[1]
			ClavicleBone.name = uniquename ClavicleBoneName numdigits:3
			CommonFns.ApplyLengthWidthProperties ClavicleBone (UpArmWidth*0.8) ObjFrontWidth:(UpArmFrontWidth*0.8) ObjOffset:undefined
			
			
			append allcreated ClavicleBone
			append mainbonesarray ClavicleBone
			
			local TempClavEnd = point name:"ZZZZTEMPCLAVEND" pos:ShoulderBones[2].pos
			ClavicleController = CommonFns.DrawRectangleControl ClavicleBone ClavicleBone TempClavEnd UpArmWidth UpArmWidth (uniquename ClavicleControlName numdigits:3)
			CommonFns.ApplycontrollerTypeProperties ClavicleController ControlType:"Medium" ControlLevel:3
			delete TempClavEnd
			
			CommonFns.QuickAlign ClavContCnsTarget ClavicleController
			
			if FlipControllers == true do 
			(CommonFns.FlipZAxis ClavicleController)
			
			ClavContCnsTarget.parent = ClavicleController
			
			ClavicleBone.rotation.controller = Orientation_Constraint()
			ClavicleBone.rotation.controller.appendtarget ClavContCnsTarget 100

			ClavContNull = Point Name:(UniqueName (NullifierName + "ClavicleControl") numdigits:3) Size:Tinysize wireColor: (color 20 80 20)
			CommonFns.quickalign ClavContNull ClavicleBone
			if FlipControllers == true then (CommonFns.FlipZAxis ClavContNull)
			ClavicleController.parent = ClavContNull
			append allcreated ClavicleController
			append controlsarray ClavicleController
			append allcreated ClavContNull
			
			if ArmParent != undefined do 
			(
				ClavContNull.parent = ArmParent
				ClavicleBone.parent = ArmParent
			)
			
			ShoulderConnectParent = ClavicleBone
			
			if ShoulderBones.count == 4 do --if the split is included
			(
				local SplitStart = ShoulderBones[((ShoulderBones.count)-1)]
				local SplitEnd = ShoulderBones[(ShoulderBones.count)]
				
				
				ClavSplitContNull = Point Name:(UniqueName (NullifierName + "ClavicleSplitControl") numdigits:3) Size:Tinysize wireColor: (color 20 80 20)
				ClavSplitBone = boneSys.createBone SplitStart.pos SplitEnd.pos [0,0,1]
				CommonFns.QuickAlign ClavSplitBone SplitStart
				ClavSplitBone.name = uniquename ClavSplitName Numdigits:3
				ClavSplitController = CommonFns.DrawRectangleControl ClavSplitbone ClavSplitBone UpArmBone UpArmWidth UpArmWidth (uniquename ClavSplitControlName numdigits:3)
				CommonFns.ApplycontrollerTypeProperties ClavSplitController ControlType:"Medium" ControlLevel:2
				
				CommonFns.QuickAlign ClavSplitContCnsTarget ClavSplitController
			if FlipControllers == true do 
			(

				CommonFns.FlipZAxis ClavSplitController
				CommonFns.FlipZAxis ClavSplitContNull
				
			)
				ClavSplitContCnsTarget.parent = ClavSplitController
				
				ClavSplitBone.rotation.controller = Orientation_Constraint()
			
				ClavSplitBone.rotation.controller.appendtarget ClavSplitContCnsTarget 100
				CommonFns.quickalign ClavSplitContNull ClavSplitController
				ClavSplitContNull.parent = ClavicleBone
				ClavSplitController.parent = ClavSplitContNull
				ShoulderConnectParent = ClavSplitBone
				
				ClavSplitBone.parent = ClavicleBone
				
				CommonFns.ApplyLengthWidthProperties ClavSplitBone (UpArmWidth*0.8) ObjFrontWidth:(UpArmWidth*0.8) ObjOffset:undefined
				
				append Allcreated ClavSplitContNull
				append Allcreated ClavSplitController
				append Allcreated ClavSplitBone
				append mainbonesarray ClavSplitBone
			)
			
			UpArmBone.parent = ShoulderConnectParent
			
	
			-----------------------------------Create the No-Twist nulls and expose transforms--------------------------
			local WristNoTwistNull
			local ShoulderNoTwistNull
			
			
							--OutputArrayContains:
				--1:LookatTarget,LookatExposePoint,UpNodePoint,LocalNode,TwistNode

				--CreateLookatExposePair StartBone EndBone Prefix BaseName:"LookExpose" Additive:True NullifyOffset:False OffsetScalar:0.8 UpnodeAxis:1=
			
			WristNoTwistNull = CommonFns.CreateLookatExposePair LowArmBone HandBone Prefix BaseName:("Wrist_" + SideAlignment) Additive:True NullifyOffset:True OffsetScalar:0.8 UpnodeAxis:1
			for n in WristNoTwistNull[1] do (append allcreated n)
			
			local WristLocalNull = WristNoTwistNull[1][4]
			WristNoTwistNull = WristNoTwistNull[1][5]
			
			
			
			ShoulderNoTwistNull = CommonFns.CreateLookatExposePair ShoulderConnectParent UpArmBone Prefix BaseName:("Shoulder_" + SideAlignment) Additive:False NullifyOffset:False OffsetScalar:1.5 UpnodeAxis:1
			for n in ShoulderNoTwistNull[1] do (append allcreated n)
			local ShoulderLocalNull = ShoulderNoTwistNull[1][4]
			
		
			ShoulderNoTwistNull = ShoulderNoTwistNull[1][5]
			
			local TempShoulder = Point Name:(UniqueName ShoulderNoTwistNullName NumDigits:3) Box:Off AxisTripod:On CenterMarker:Off Color:(Color 20 90 20) Size:TinySize

			CommonFns.QuickAlign TempShoulder ShoulderNoTwistNull

			
			TempShoulder.rotation.controller = Orientation_Constraint()
			TempShoulder.rotation.controller.appendtarget ShoulderNoTwistNull 80
			TempShoulder.rotation.controller.appendtarget UpArmBone 20
			TempShoulder.parent = UpArmBone
			

			ShoulderNoTwistNull = TempShoulder
			
			append allcreated ShoulderNoTwistNull

			

			
			---------------------Create the shoulder 50% Null--------------------------------------------------------------
			
			--local Shoulder50Null
			local Shoulder50Bone
			Shoulder50Bone = boneSys.createBone [0,0,0] [(distance UpArmBone LowArmBone *0.1),0,0] [0,0,1]
			Shoulder50Bone.Name = (uniquename (Shoulder50BoneName) Numdigits:3)

			commonFns.quickAlign Shoulder50Bone UpArmBone
			
			Shoulder50Bone.rotation.controller = Orientation_constraint()
			Shoulder50Bone.rotation.controller.appendtarget ShoulderConnectParent 30
			Shoulder50Bone.rotation.controller.appendtarget ShoulderNoTwistNull 70
			Shoulder50Null = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename Shoulder50NullName Numdigits:3) size:Tinysize wirecolor:Green
			CommonFns.QuickAlign Shoulder50Null Shoulder50Bone
			Shoulder50Null.parent = ShoulderConnectParent
			Shoulder50Bone.parent = Shoulder50Null
			
			
			local Shoulder50ParentNull


			If ElbowType != 1 do  --if elbow type is not basic, add the expression to shoulder
			(
			Shoulder50ParentNull = Point isSelected:off cross: on centermarker: off axistripod: off box: off name:(uniquename Shoulder50ParentNullName Numdigits:3) size:Tinysize wirecolor:Green
			append allcreated Shoulder50ParentNull
			CommonFns.quickalign Shoulder50ParentNull ShoulderConnectPArent
			in coordsys world Shoulder50ParentNull.pos = UpArmBone.transform.pos
			Shoulder50ParentNull.parent = ShoulderConnectParent
			Shoulder50Null.parent = Shoulder50ParentNull
			Shoulder50RevealNull = Point isSelected:off cross: on centermarker: off axistripod: off box: off name:(uniquename (Shoulder50NullName + "_expose") Numdigits:3) size:Tinysize wirecolor:Green
			append allcreated Shoulder50RevealNull
			CommonFns.QuickAlign Shoulder50RevealNull LowArmBone
			Shoulder50RevealNull.parent = UpArmBone
			Shoulder50Null.pos.controller = position_list()
			Shoulder50Null.pos.controller.available.controller =  position_xyz()
			Shoulder50Null.pos.controller[2].Z_position.controller = Float_Script()
			Shoulder50Null.pos.controller[2].Z_position.controller.addNode "A" Shoulder50RevealNull	
			Shoulder50Null.pos.controller[2].Z_position.controller.addNode "B" Shoulder50ParentNull	
			local DefaultLength = UpArmBone.length
			
			
			Shoulder50Null.pos.controller[2].Z_position.controller.setexpression ("BasePos = ((A.transform * inverse B.transform).pos.z)\n" + "if BasePos<0 do (BasePos = 0)\n" + "((BasePos - " + (DefaultLength as string) + ")/(" + (DefaultLength as string) + "))*" + ((DefaultLength*0.1) as string) )
			if SideAlignment == "L" do (Shoulder50Null.pos.controller[2].Z_position.controller.setexpression ("BasePos = ((A.transform * inverse B.transform).pos.z)\n" + "if BasePos>0 do (BasePos = 0)\n" + "((BasePos + " + (DefaultLength as string) + ")/(" + (DefaultLength as string) + "))*" + ((DefaultLength*0.1) as string) ))

			local ShoulderController = CommonFns.MakeController (TinySize) "circle" Shoulder50ParentNull Shoulder50ParentNull blue ShoulderControlName "y" "positive"
			ShoulderController.objectoffsetpos = [0,0,-BigSize]	
			if SideAlignment == "L" do (ShoulderController.objectoffsetpos = [0,0,BigSize])
			CommonFns.QuickAlign ShoulderController Shoulder50ParentNull
			ShoulderController.parent = Shoulder50ParentNull
			Shoulder50Null.parent = ShoulderController
			setUserProp ShoulderController "RigTools_MoveAmount"  ((DefaultLength*0.1) as string)
			setUserProp ShoulderController "RigTools_ShoulderBoneName"  Shoulder50Null.name
			setUserProp ShoulderController "RigTools_SideAlignment"  (Sidealignment as string)
			setUserProp ShoulderController "RigTools_DefaultLength"  (DefaultLength as string)
			
			local ca=attributes ShoulderAdjust
			(
				parameters params rollout:ro_AdjustShoulder
				(
				paramblend type:#float ui:spn_ShoulderBlend
				)
				
				rollout ro_AdjustShoulder "Adjust_Shoulder" 
				(
					edittext edt_moveamount "ExtendAmount"
					spinner spn_ShoulderBlend "Auto Off/On" Range:[0,1,1] scale:0.1
					button btn_apply "Apply" enabled:on
					local MoveAmount
					
					on ro_AdjustShoulder open do
					(
							moveamount = (GetUserProp $ "RigTools_MoveAmount")
							edt_moveamount.text = (Moveamount as string)
					)
						
					on btn_apply pressed do
					(
						local NewMoveAmount = edt_moveamount.text
						local DefaultLength = GetUserProp $ "RigTools_DefaultLength"
						local SideAlignment = GetUserProp $ "RigTools_SideAlignment"
						setUserProp $ "RigTools_MoveAmount"  NewMoveAmount	
						local ShoulderBone = getnodebyname (GetUserProp $ "RigTools_ShoulderBoneName")
						ShoulderBone.pos.controller[2].Z_position.controller.setexpression ("BasePos = ((A.transform * inverse B.transform).pos.z)\n" + "if BasePos<0 do (BasePos = 0)\n" + "((BasePos - " + (DefaultLength as string) + ")/(" + (DefaultLength as string) + "))*" + ((NewMoveAmount) as string) )
						if SideAlignment == "L" do 
						(
							ShoulderBone.pos.controller[2].Z_position.controller.setexpression ("BasePos = ((A.transform * inverse B.transform).pos.z)\n" + "if BasePos>0 do (BasePos = 0)\n" + "((BasePos + " + (DefaultLength as string) + ")/(" + (DefaultLength as string) + "))*" + ((NewMoveAmount) as string) )
						)
					)
				)
			)
			local AttHolder = (EmptyModifier())
			AttHolder.name = "Shoulder_Controls"
			addmodifier ShoulderController AttHolder
			custAttributes.add ShoulderController.modifiers[#Shoulder_Controls] ca

			Shoulder50Null.pos.controller.weight[2].controller = float_expression()
			Shoulder50Null.pos.controller.weight[2].controller.addscalartarget "SliderValue" ShoulderController.modifiers[#Shoulder_Controls].ShoulderAdjust[#paramblend]
			Shoulder50Null.pos.controller.weight[2].controller.setexpression "SliderValue"
			

			
				--.pos.controller.Y_Position.controller = Float_Expression ()   -Z (minus Z) is up
			) 
			
			
			


			
			append Allcreated Shoulder50Bone
			append allcreated SHoulder50Null
			append ExtraBones Shoulder50Bone
			

			if ExtraShoulderBones == true or ExtraShoulderBones == on do --If the shoulder girlde is included add the bones
			(
				--ExtraBonesInput: Scapula, Pec, Teres. ( PecStartLocator,PecEndLocator,PecMidPointLocator,PecLookatLocator,PecPosConsPointA,PecPosConsPointB,PecStartBone,PecEndBone)
				
				--append extrabones PectoralBone
				--append nubsarray PectoralNub
				
				local PecStartLocator = ExtraBonesInput[2][1]
				local PecEndLocator = ExtraBonesInput[2][2]
				local PecMidPointLocator = ExtraBonesInput[2][3]
				local PecLookatLocator = ExtraBonesInput[2][4]
				local PecPosConsPointA = ExtraBonesInput[2][5]
				local PecPosConsPointB =  ExtraBonesInput[2][6]
				local PecStartBone = ExtraBonesInput[2][7]
				local PecEndBone = ExtraBonesInput[2][8]
				
				local TeresStartLocator = ExtraBonesInput[3][1]
				local TeresEndLocator = ExtraBonesInput[3][2]
				local TeresMidPointLocator = ExtraBonesInput[3][3]
				local TeresLookatLocator = ExtraBonesInput[3][4]
				local TeresPosConsPointA = ExtraBonesInput[3][5]
				local TeresPosConsPointB =  ExtraBonesInput[3][6]
				local TeresStartBone = ExtraBonesInput[3][7]
				local TeresEndBone = ExtraBonesInput[3][8]
				
				local ScapulaBoneLocation = ExtraBonesInput[1][1]
				local ScapulaNubLocation = ExtraBonesInput[1][2]
				

				
				ScapulaBone = boneSys.createBone ScapulaBoneLocation.pos ScapulaNubLocation.pos [0,0,1]
				CommonFns.QuickAlign ScapulaBone ScapulaBoneLocation
				ScapulaBone.name = uniquename ScapulaBoneName Numdigits:3
				ScapulaNull = Point Name:(UniqueName (NullifierName + "ScapParent") numdigits:3) Size:Tinysize wireColor: (color 20 80 20)
				CommonFns.quickAlign ScapulaNull ScapulaBone
				ScapulaBone.parent = ScapulaNull
				ScapulaNull.parent = ClavicleBone
	
				
				append allcreated ScapulaBone
				append extrabones ScapulaBone
				append Allcreated ScapulaNull
				
				local PecMuscleOutput = (finaltools.CreateMuscle  PecStartLocator PecEndLocator PecMidPointLocator PecLookatLocator PecPosConsPointA PecPosConsPointB PecStartBone PecEndBone ArmParent ShoulderNoTwistNull Prefix:Prefix MuscleName:("Pec_" + SideAlignment) BoneDirection:1 Scaling:true UpNodeAxis:1)
				FinalTools.appendobjstolayer PecMuscleOutput[1] HiddenComponents:True
				for n in PecMuscleOutput[1] do (append allcreated n)
				append extrabones PecMuscleOutput[3][1]
				append extrabones PecMuscleOutput[3][3]
				append nubsarray PecMuscleOutput[3][2]
				append nubsarray PecMuscleOutput[3][4]

				local TeresUpNodePoint = Point Name:(Uniquename TeresUpNodePointName NumDigits:3) Size:TinySize Color:(color 40 80 100) Box:Off Cross:Off AxisTripod:On
				if ArmParent != undefined do (CommonFns.QuickAlign TeresUpNodePoint ArmParent)
				In coordsys world TeresUpNodePoint.pos = ScapulaBone.transform.pos
				TeresUpNodePoint.parent = ScapulaBone
				append allcreated TeresUpNodePoint

				
				local TeresMuscleOutput = (finaltools.CreateMuscle  TeresStartLocator TeresEndLocator TeresMidPointLocator TeresLookatLocator TeresPosConsPointA TeresPosConsPointB TeresStartBone TeresEndBone TeresUpNodePoint ShoulderNoTwistNull Prefix:Prefix MuscleName:("Teres_" + SideAlignment) BoneDirection:3 Scaling:true UpNodeAxis:1)				for n in TeresMuscleOutput[1] do (append allcreated n)
				for n in TeresMuscleOutput[1] do (append allcreated n)
				FinalTools.appendobjstolayer TeresMuscleOutput[1] HiddenComponents:True
				append extrabones TeresMuscleOutput[3][1]
				append extrabones TeresMuscleOutput[3][3]
				append nubsarray TeresMuscleOutput[3][2]
				append nubsarray TeresMuscleOutput[3][4]
				

				

				
				--Muscle Output Array Contains:
				
				--1: Allcreated
				--2: (StartLocator,EndLocator,MidPointLocator,LookatLocator)
				--3: (StartBone,StartBoneNub,EndBone,EndBoneNub)
				--4: (PosConsPointA,PosConsPointB,MidPointParent)

			)
			
			

			
			-------------------------------------------------------------------------
			
			local UparmFKBone
			local LowarmFKBone
			local UpArmIKBone
			local LowArmIKBone
			local IKHandBone
			local FKHandBone
			
			
			local UpArmFKController
			local LowArmFKController
			local HandFKController
			local HandIKController
			local ArmIKChain
			local ElbowControl
			local UpArmFKContNull
			local LowArmFKContNull
			local HandIKControlNull
			local HandFKControlNull
			local ElbowControlNull
			
			if Kinematics == 1 or kinematics == 3 do --if there is IK
			(
				
				ElbowControl = CommonFns.MakeController (BigSize) "sphere" LowArmBone LowArmBone (color 40 200 40) (ElbowControlName)  "x" "positive"
				CommonFns.ApplycontrollerTypeProperties ElbowControl ControlType:"Upnode" ControlLevel:1
				CommonFns.quickalign ElbowControl LowArmBone
				
				
				local ElbowCnsTarget =  Point Name:(UniqueName (CnsTargetName + "_ElbowControl") numdigits:3) Size:Tinysize wireColor: (color 20 80 20)
				CommonFns.QuickAlign ElbowCnsTarget ElbowControl

				if FlipControllers == true do 
				(
					CommonFns.FlipZAxis ElbowControl
				)
				
				ElbowCnsTarget.parent = ElbowControl
				append allcreated ElbowCnsTarget
				
				
				ElbowControlNull = Point Name:(UniqueName (NullifierName + "ElbowControl") numdigits:3) Size:Tinysize wireColor: (color 20 80 20)
				CommonFns.QuickAlign ElbowControlNull Elbowcontrol
				if FlipControllers == true do (CommonFns.FlipZAxis ElbowControlNull)
				ElbowControl.parent = ElbowControlNull
				
				append allcreated ElbowControl
				append controlsarray ElbowControl
				append allcreated ElbowControlNull
				

				
				
				local TempHandEndPoint = Point Pos: MainBones[4].pos Name:"ZZZZZZTEMPHANDENDPOINT"
				HandIKController = CommonFns.DrawRectangleControl HandBone HandBone TempHandEndPoint (HandWidth*1.1) (HandFrontWidth*1.1) (uniquename HandIKControlName numdigits:3)
				CommonFns.ApplycontrollerTypeProperties HandIKController ControlType:"Medium" ControlLevel:1
				delete TempHandEndPoint
				
				local HandIKCnsTarget =  Point Name:(UniqueName (CnsTargetName + "_HandIKControl") numdigits:3) Size:Tinysize wireColor: (color 20 80 20)
				CommonFns.QuickAlign HandIKCnsTarget HandIKController
				
				if FlipControllers == true do (CommonFns.FlipZAxis HandIKController)
				
				HandIKCnsTarget.parent = HandIKController
				append allcreated HandIKCnsTarget
				
				HandIKControlNull = Point Name:(UniqueName (NullifierName + "HandIKControl") numdigits:3) Size:Tinysize wireColor: (color 20 80 20)
				if FlipControllers == true do (CommonFns.FlipZAxis HandIKControlNull)
				CommonFns.quickalign HandIKControlNull HandIKController
				HandIKController.parent = HandIKControlNull
				
				append allcreated HandIKController
				append controlsarray HandIKController
				append allcreated HandIKControlNull
				
				if Root != undefined do
				(
					ElbowcontrolNull.parent = Root
					HandIKControlNull.parent = Root
				)
			)
			
			if Kinematics == 3 do --If there is IK/FK blend
			(
				UpArmIKBone = boneSys.createBone MainBones[1].pos MainBones[2].pos [0,0,1]
				CommonFns.QuickAlign UpArmIKBone MainBones[1]
				UpArmIKBone.Name = uniquename (UparmIKBoneName) Numdigits:3
				UpArmIKBone.wirecolor = (color 60 60 180)
				UpArmIKBone.parent = ShoulderConnectParent
				
				
				append allcreated UpArmIKBone
				append cnstargetbones UpArmIKBone
	
				LowArmIKBone = boneSys.createBone MainBones[2].pos MainBones[3].pos [0,0,1]
				CommonFns.QuickAlign LowArmIKBone MainBones[2]
				LowArmIKBone.Name = uniquename (LowArmIKBoneName) Numdigits:3
				LowArmIKBone.wirecolor = (color 60 60 180)
				LowArmIKBone.parent =  UpArmIKBone
				
				append allcreated LowArmIKBone
				append cnstargetbones LowArmIKBone
				
				IKHandBone = boneSys.createBone MainBones[3].pos MainBones[4].pos [0,0,1]
				CommonFns.QuickAlign IKhandBone MainBones[3]
				IKHandBone.Name = uniquename (IKHandBoneName) Numdigits:3
				IKHandBone.wirecolor = (color 60 60 180)
				IKHandBone.Parent = LowArmIKBone
				append allcreated IKHandBone
				append cnstargetbones IKHandBone

				UpArmFKBone = boneSys.createBone MainBones[1].pos MainBones[2].pos [0,0,1]
				CommonFns.QuickAlign UpArmFKBone MainBones[1]
				UpArmFKBone.Name = uniquename (UparmFKBoneName) Numdigits:3
				UpArmFKBone.wirecolor = (color 60 180 60)
				UpArmFKBone.parent = ShoulderConnectParent
				append allcreated UpArmFKBone
				append cnstargetbones UpArmFKBone
				
				LowArmFKBone = boneSys.createBone MainBones[2].pos MainBones[3].pos [0,0,1]
				CommonFns.QuickAlign LowArmFKBone MainBones[2]
				LowArmFKBone.Name = uniquename (LowArmFKBoneName) Numdigits:3
				LowArmFKBone.wirecolor = (color 60 180 60)
				LowArmFKBone.parent = UpArmFKBone
				append allcreated LowArmFKBone
				append cnstargetbones LowArmFKBone
				
				ArmIKChain = IKSys.ikChain UpArmIKBone IKHandBone "IKHISolver"
				ArmIKChain.name = UniqueName ArmIKSolverName
				ArmIKChain.transform.controller.VHTarget = ElbowControl
				ArmIKChain.transform.controller.goalSize = MediumSize
				append allcreated ArmIKChain
				
				
				local TempHandEndPoint = Point Pos: MainBones[4].pos Name:"ZZZZZZTEMPHANDENDPOINT"
				HandFKController = CommonFns.DrawRectangleControl HandBone HandBone TempHandEndPoint HandWidth HandFrontWidth (uniquename HandFKControlName numdigits:3)
				CommonFns.ApplycontrollerTypeProperties HandFKController ControlType:"Medium" ControlLevel:1
				delete TempHandEndPoint
				
				local HandFKCnsTarget =  Point Name:(UniqueName (CnsTargetName + "_HandFKControl") numdigits:3) Size:Tinysize wireColor: (color 20 80 20)
				CommonFns.QuickAlign HandFKCnsTarget HandFKController

				if FlipControllers == true do 
				(
					CommonFns.FlipZAxis HandFKController
				)
				HandFKCnsTarget.parent = HandFKController
				append allcreated HandFKCnsTarget
				
				
				
				
				HandFKControlNull = Point Name:(UniqueName (NullifierName + "HandFKControl") NumDigits:3) Size:Tinysize wireColor: (color 20 80 20)
				if FlipControllers == true do (CommonFns.FlipZAxis HandFKControlNull)
				CommonFns.QuickAlign HandFKControlNull HandFKController
				HandFKController.parent = HandFKControlNull
				HandFKControlNull.parent = LowArmFKBone
				
				append allcreated HandFKController
				append controlsarray HandFKController
				append allcreated HandFKControlNull
				
				FKHandBone = boneSys.createBone MainBones[3].pos MainBones[4].pos [0,0,1]
				CommonFns.QuickAlign FKhandBone MainBones[3]
				FKHandBone.Name = uniquename (FKHandBoneName) Numdigits:3
				FKHandBone.wirecolor = (color 60 180 60)
				FKHandBone.Parent = LowArmFKBone
				
				append allcreated FKHandBone
				append cnstargetbones FKHandBone
				
				UpArmFKController = CommonFns.DrawRectangleControl UpArmBone UpArmBone LowArmBone LowArmWidth LowArmFrontWidth (uniquename UpArmFKControlName numdigits:3)
				CommonFns.ApplycontrollerTypeProperties UpArmFKController ControlType:"Medium" ControlLevel:1
				LowArmFKController = CommonFns.DrawRectangleControl LowArmBone LowArmBone HandBone UpArmWidth UpArmFrontWidth (uniquename LowArmFKControlName numdigits:3)
				CommonFns.ApplycontrollerTypeProperties LowArmFKController ControlType:"Medium" ControlLevel:2
				

					local LowArmFKCnsTarget =  Point Name:(UniqueName (CnsTargetName + "_LowArmFKControl") numdigits:3) Size:Tinysize wireColor: (color 20 80 20)
					local UpArmFKCnsTarget =  Point Name:(UniqueName (CnsTargetName + "_UpArmFKControl") numdigits:3) Size:Tinysize wireColor: (color 20 80 20)
					CommonFns.QuickAlign LowArmFKCnsTarget LowArmFKController
					CommonFns.QuickAlign UpArmFKCnsTarget UpArmFKController
				if FlipControllers == true do 
				(
					CommonFns.FlipZAxis LowArmFKController
					CommonFns.FlipZAxis UpArmFKController
				)
					LowArmFKCnsTarget.parent = LowArmFKController
					UpArmFKCnsTarget.parent = UpArmFKController
					append allcreated LowArmFKCnsTarget
					append allcreated UpArmFKCnsTarget
				
				
				
				LowArmFKContNull = Point Name:(UniqueName NullifierName + "LowArmFKControl") Size:Tinysize wireColor: (color 20 80 20)
				UpArmFKContNull = Point Name:(UniqueName NullifierName + "UpArmFKControl") Size:Tinysize wireColor: (color 20 80 20)
				if FlipControllers == true do 
				(
					CommonFns.FlipZAxis UpArmFKContNull
					CommonFns.FlipZAxis LowArmFKContNull
				)
				CommonFns.QuickAlign LowArmFKContNull LowArmFKController
				CommonFns.QuickAlign UpArmFKContNull UpArmFKController
				
				LowArmFKController.parent = LowArmFKContNull
				LowArmFKContNull.parent = UpArmFKBone
				UpArmFKContNull.parent = ShoulderConnectParent
				UpArmFKController.parent = UpArmFKContNull
				
				append allcreated UpArmFKController
				append controlsarray UpArmFKController
				append allcreated LowArmFKController
				append controlsarray LowArmFKController
				append allcreated LowArmFKContNull
				append allcreated UpArmFKContNull
				
				UpArmFKBone.rotation.controller = Orientation_Constraint()
				UpArmFKBone.rotation.controller.appendtarget UpArmFKCnsTarget 100
				LowArmFKBone.rotation.controller = Orientation_Constraint()
				LowArmFKBone.rotation.controller.appendtarget LowArmFKCnstarget 100
				FKHandBone.rotation.controller = Orientation_Constraint()
				FKHandBone.rotation.controller.appendtarget HandFKCnstarget 100
				IKHandBone.rotation.controller = Orientation_Constraint()
				IKHandBone.rotation.controller.appendtarget HandIKCnstarget 100
				
				
				ArmIKChain.parent = HandIKController
				
				
			)
			
			
			
			if Kinematics == 2 do --if there is only FK
			(
				UpArmFKController = CommonFns.DrawRectangleControl UpArmBone UpArmBone LowArmBone LowArmWidth LowArmFrontWidth (uniquename UpArmFKControlName numdigits:3)
				CommonFns.ApplycontrollerTypeProperties UpArmFKController ControlType:"Medium" ControlLevel:1
				LowArmFKController = CommonFns.DrawRectangleControl LowArmBone LowArmBone HandBone UpArmWidth UpArmFrontWidth (uniquename LowArmFKControlName numdigits:3)
				CommonFns.ApplycontrollerTypeProperties LowArmFKController ControlType:"Medium" ControlLevel:2
				

				local LowArmFKCnsTarget =  Point Name:(UniqueName (CnsTargetName + "_LowArmFKControl") numdigits:3) Size:Tinysize wireColor: (color 20 80 20)
				local UpArmFKCnsTarget =  Point Name:(UniqueName (CnsTargetName + "_UpArmFKControl") numdigits:3) Size:Tinysize wireColor: (color 20 80 20)
				CommonFns.QuickAlign LowArmFKCnsTarget LowArmFKController
				CommonFns.QuickAlign UpArmFKCnsTarget UpArmFKController
				if FlipControllers == true do 
				(
					CommonFns.FlipZAxis LowArmFKController
					CommonFns.FlipZAxis UpArmFKController
				)
				LowArmFKCnsTarget.parent = LowArmFKController
				UpArmFKCnsTarget.parent = UpArmFKController
				append allcreated LowArmFKCnsTarget
				append allcreated UpArmFKCnsTarget
				
				
				
				
				
				LowArmFKContNull = Point Name:(UniqueName NullifierName + "LowArmFKControl") Size:Tinysize wireColor: (color 20 80 20)

				UpArmFKContNull = Point Name:(UniqueName NullifierName + "UpArmFKControl") Size:Tinysize wireColor: (color 20 80 20)

				if FlipControllers == true do 
				(
					CommonFns.FlipZAxis LowArmFKContNull
					CommonFns.FlipZAxis UpArmFKContNull
				)
				CommonFns.QuickAlign LowArmFKContNull LowArmFKController
				CommonFns.QuickAlign UpArmFKContNull UpArmFKController
				LowArmFKController.parent = LowArmFKContNull
				LowArmFKContNull.parent = UpArmBone
				UpArmFKContNull.parent = ShoulderConnectParent
				UpArmFKController.parent = UpArmFKContNull
				local TempHandEndPoint = Point Pos: MainBones[4].pos Name:"ZZZZZZTEMPHANDENDPOINT"
				HandFKController = CommonFns.DrawRectangleControl HandBone HandBone TempHandEndPoint HandWidth HandFrontWidth (uniquename HandFKControlName numdigits:3)
				CommonFns.ApplycontrollerTypeProperties HandFKController ControlType:"Medium" ControlLevel:3
				delete TempHandEndPoint
				

				local HandFKCnsTarget =  Point Name:(UniqueName (CnsTargetName + "_HandFKControl") numdigits:3) Size:Tinysize wireColor: (color 20 80 20)
				CommonFns.QuickAlign HandFKCnsTarget HandFKController
				if FlipControllers == true do (CommonFns.FlipZAxis HandFKController)
				HandFKCnsTarget.parent = HandFKController
				append allcreated HandFKCnsTarget
				
				
				
				HandFKControlNull = Point Name:(UniqueName (NullifierName + "HandFKControl") NumDigits:3) Size:Tinysize wireColor: (color 20 80 20)
				if FlipControllers == true do (CommonFns.FlipZAxis HandFKControlNull)
				CommonFns.QuickAlign HandFKControlNull HandFKController
				
				HandFKController.parent = HandFKControlNull
				HandFKControlNull.parent = LowArmBone
				UpArmBone.rotation.controller = Orientation_Constraint()
				UpArmBone.rotation.controller.appendtarget UpArmFKCnstarget 100
				LowArmBone.rotation.controller = Orientation_Constraint()
				LowArmBone.rotation.controller.appendtarget LowArmFKCnstarget 100
				HandBone.rotation.controller = Orientation_Constraint()
				HandBone.rotation.controller.appendtarget HandFKCnstarget 100
				
				append allcreated UpArmFKController
				append controlsarray UpArmFKController
				append allcreated LowArmFKController
				append controlsarray LowArmFKController
				append allcreated LowArmFKContNull
				append allcreated UpArmFKContNull
				append allcreated HandFKcontroller
				append controlsarray HandFKcontroller
				append allcreated HandFKControlNull

			)
			
			if Kinematics == 1 do --if there is only IK
			(
				ArmIKChain = IKSys.ikChain UpArmBone HandBone "IKHISolver"
				ArmIKChain.name = UniqueName ArmIKSolverName NumDigits:3
				ArmIKChain.transform.controller.VHTarget = ElbowControl
				ArmIKChain.transform.controller.goalSize = MediumSize
				ArmIKChain.parent = HandIKController
				HandBone.rotation.controller = Orientation_Constraint()
				HandBone.rotation.controller.appendtarget HandIKCnstarget 100
				
				append allcreated ArmIKChain
			)
			
			
			
			
			
			--------------------------------------------Add the blend controls to the clavicle controller------------------------------------------------------------------
			
				local ca
				local AttHolder
			

			
			
				local SwitchController = text size:Mediumsize kerning:0 leading:0 pos:[0,0,0] isSelected:on text:"IK/FK" name:SwitchControllerName
				--CommonFns.QuickAlign SwitchController LowArmBone
				CommonFns.ApplycontrollerTypeProperties SwitchController ControlType:"Utility" ControlLevel:3
				in coordsys local rotate SwitchController (angleaxis 90 [1,0,0])
				
				in coordsys world SwitchController.pos = LowArmBone.transform.pos
				in CoordSys Local Move SwitchController [0,0,-AverageScale*0.7]
				SwitchController.parent = UpArmBone
				
				append Allcreated SwitchController
				append controlsarray SwitchController
					
					ca=attributes ArmBlendControls
					(
						parameters params rollout:ro_ArmBlendControls
						(
								paramIKFKBlend type:#float ui:spn_IKFKblend
								paramStretchBlend type:#float ui:spn_stretchblend
								paramBendyBlend type:#float ui:spn_BendyBlend
								paramUpArmFKCont type:#string ui:edt_FKUpArm
								paramLowArmFKCont type:#string ui:edt_FKLowArm
								paramHandFKCont type:#string ui:edt_FKHand
								paramHandIKCont type:#string ui:edt_IKHand
								paramElbowCont type:#string ui:edt_Elbow
								paramSquash type:#float ui:spn_squash
						)
						rollout ro_ArmBlendControls "ArmBlend_Controls" 
						(
							spinner spn_IKFKblend "IK/FK Blend" enabled:on  range:[0,1,0] scale:0.1
							spinner spn_stretchBlend "Stretch Blend" enabled:on  range:[0,1,0] scale:0.1	
							spinner spn_BendyBlend "Bendy Blend" enabled:on  range:[0,100,0] scale:1
							spinner spn_squash "Squash Enabled" enabled:on  range:[0,3,0] scale:0.1	
							button btn_showIK "Hide/Show IK" enabled:true visible:on
							button btn_showFK "Hide/Show FK" enabled:true visible:on
							edittext edt_Divide enabled:On text:"J" visible:off
							button btn_IKtoFK "IK to FK" enabled:true visible:off
							button btn_FKtoIK "FK to IK" enabled:true visible:off
							edittext edt_FKUpArm enabled:On text:"J" visible:off
							edittext edt_FKLowArm enabled:On text:"J" visible:off
							edittext edt_FKHand enabled:On text:"J" visible:off
							edittext edt_IKHand enabled:On text:"J" visible:off
							edittext edt_Elbow enabled:On text:"J" visible:off
							
							local UpArmFKControlName,LowArmFKControlName,HandFKControlName,HandIKControlName,ElbowControlName
							
							local IKHandControl,FKHandControl,ElbowControl,FKUpArmControl,FKLowArmControl,IKUpArmBone,IKLowArmBone,ArmSideAlignment
							
							on btn_IKtoFK pressed do 
							(
								IKHandControl = GetUserProp $ "RigTools_IKHandControl"
								FKHandControl = GetUserProp $ "RigTools_FKHandControl"
								ElbowControl = GetUserProp $ "RigTools_ElbowControl"
								FKUpArmControl = GetUserProp $ "RigTools_FKUpArmControl"
								FKLowArmControl = GetUserProp $ "RigTools_FKLowArmControl"
								IKUpArmBone = GetUserProp $ "RigTools_IKUpArmBone"
								IKLowArmBone = GetUserProp $ "RigTools_IKLowArmBone"
								ArmSideAlignment = GetUserProp $ "RigTools_SideAlignment"
							)
							
							on btn_FKtoIK pressed do 
							(
								IKHandControl = GetUserProp $ "RigTools_IKHandControl"
								FKHandControl = GetUserProp $ "RigTools_FKHandControl"
								ElbowControl = GetUserProp $ "RigTools_ElbowControl"
								FKUpArmControl = GetUserProp $ "RigTools_FKUpArmControl"
								FKLowArmControl = GetUserProp $ "RigTools_FKLowArmControl"
								IKUpArmBone = GetUserProp $ "RigTools_IKUpArmBone"
								IKLowArmBone = GetUserProp $ "RigTools_IKLowArmBone"
								ArmSideAlignment = GetUserProp $ "RigTools_SideAlignment"
							)
							
							
							on btn_showFK pressed do 
							(
								try
								(
								local UpArmFkControl = (getnodebyname  ($.modifiers[#ArmBlend_Controls].ArmBlendControls.paramUpArmFKCont))
								if UpArmFKControl.ishidden == false then (UparmFKControl.ishidden = true) else (UpArmFKControl.ishidden = false) 
								local LowArmFkControl = (getnodebyname  ($.modifiers[#ArmBlend_Controls].ArmBlendControls.paramLowArmFKCont))
								if LowArmFKControl.ishidden == false then (LowarmFKControl.ishidden = true) else (LowArmFKControl.ishidden = false) 
								local HandFkControl = (getnodebyname  ($.modifiers[#ArmBlend_Controls].ArmBlendControls.paramHandFKCont))
								if HandFKControl.ishidden == false then (HandFKControl.ishidden = true) else (HandFKControl.ishidden = false) 
								)
								catch()
							)
							
							on btn_showIK pressed do 
							(
								try
								(
								local HandIKControl = (getnodebyname  ($.modifiers[#ArmBlend_Controls].ArmBlendControls.paramHandIKcont))
								if HandIKControl.ishidden == false then (HandIKControl.ishidden = true) else (HandIKControl.ishidden = false) 
								local ElbowControl = (getnodebyname  ($.modifiers[#ArmBlend_Controls].ArmBlendControls.paramElbowCont))
								if ElbowControl.ishidden == false then (ElbowControl.ishidden = true) else (ElbowControl.ishidden = false) 
								)
								catch()
							)
							
							
							
						)
					)
					
					
					
					AttHolder = (EmptyModifier ())
					AttHolder.name = "ArmBlend_Controls"
					addmodifier SwitchController AttHolder
					custAttributes.add switchcontroller.modifiers[#ArmBlend_Controls] ca
					try(SetUserProp SwitchController "RigTools_IKHandControl" HandIKController.name) catch()
					try (SetUserProp SwitchController "RigTools_FKHandControl" HandFKController.name) catch()
					try (SetUserProp SwitchController "RigTools_ElbowControl" ElbowControl) catch()
					try (SetUserProp SwitchController "RigTools_FKUpArmControl" UparmFKController.name) catch()
					try (SetUserProp SwitchController "RigTools_FKLowArmControl" LowArmFKController.name) catch()
					try (SetUserProp SwitchController "RigTools_IKUpArmBone" UparmIKBone.name) catch()
					try (SetUserProp SwitchController "RigTools_IKLowArmBone" LowArmIKBone.name) catch()
					try (SetUserProp SwitchController "RigTools_SideAlignment" SideAlignment) catch()
						
						
					
					--switchcontroller.modifiers[#ArmBlend_Controls].ro_ArmBlendControls.spn_fingerlength.value = 1 - SET A SLIDER DEFAULT VALUE
					--RotExpressionToAdd.AddScalarTarget "SliderValue" switchcontroller.modifiers[#ArmBlend_Controls].ArmBlendControls[#paramIKFKBlend]
					
					if kinematics == 3 do
					(
						
					UpArmBone.rotation.controller = Orientation_Constraint()
					UpArmBone.rotation.controller.appendtarget UpArmIKBone 100
					UpArmBone.rotation.controller.appendtarget UpArmFKBone 100
					LowArmBone.rotation.controller = Orientation_Constraint()
					LowArmBone.rotation.controller.appendtarget LowArmIKBone 100	
					LowArmBone.rotation.controller.appendtarget LowArmFKBone 100	
						
					HandBone.rotation.controller = Orientation_Constraint()
					HandBone.rotation.controller.appendtarget IKHandBone 100	
					HandBone.rotation.controller.appendtarget FKHandBone 100		
						
					local ExpressionToAdd = Float_Expression()
					ExpressionToAdd.AddScalarTarget "SliderValue" switchcontroller.modifiers[#ArmBlend_Controls].ArmBlendControls[#paramIKFKBlend]
					ExpressionToAdd.setexpression "(100-(SliderValue*100))"
					
					LowArmBone.rotation.controller.weight[1].controller = ExpressionToAdd
					UpArmBone.rotation.controller.weight[1].controller = ExpressionToAdd
					HandBone.rotation.controller.weight[1].controller = ExpressionToAdd
					ExpressionToAdd = copy ExpressionToAdd
					ExpressionToAdd.setexpression "SliderValue*100"
					LowArmBone.rotation.controller.weight[2].controller = ExpressionToAdd
					UpArmBone.rotation.controller.weight[2].controller = ExpressionToAdd
					HandBone.rotation.controller.weight[2].controller = ExpressionToAdd
					
					
					switchcontroller.modifiers[#ArmBlend_Controls].ArmBlendControls.paramUpArmFKCont = UpArmFKController.name
					switchcontroller.modifiers[#ArmBlend_Controls].ArmBlendControls.paramLowArmFKCont = LowArmFKController.name
					switchcontroller.modifiers[#ArmBlend_Controls].ArmBlendControls.paramHandFKCont = HandFKController.name
					switchcontroller.modifiers[#ArmBlend_Controls].ArmBlendControls.paramHandIKCont = HandIKController.name
					switchcontroller.modifiers[#ArmBlend_Controls].ArmBlendControls.paramElbowCont = ElbowControl.name
					
					switchcontroller.modifiers[#ArmBlend_Controls].ro_ArmBlendControls.btn_showIK.visible = on
					switchcontroller.modifiers[#ArmBlend_Controls].ro_ArmBlendControls.btn_showFK.visible = on	
					)
					

			--------------------------------------Set up the strechy expression------------------------------------------------------------------------------------------
			
			if Stretchy == true and Kinematics != 2 do
			(
				local DefaultLowArmLength = ((Distance HandBone LowArmBone) as string)
				local DefaultUpArmLength = ((Distance LowArmBone UpArmBone) as string)
				local DefaultBentDistance = ((Distance UpArmBone HandBone) as string)
				local DefaultTotal = ((DefaultLowArmLength as float) + (DefaultUpArmLength as float)) as string
				local ArmDistanceExpose = CommonFns.CreateLookatdistanceExpose UpArmBone ArmIKChain #(MeasureHelperBaseName,MeasurEHelperBaseName) AlignTo:undefined reAssignParents:#(ClavicleBone,ArmIKChain) justdistance:true
				append Allcreated ArmDistanceExpose[1]
				append Allcreated ArmDistanceExpose[2]
				
				local StretchExpressionToAdd = Float_Expression()
				StretchExpressionToAdd.AddScalarTarget "Distance" ArmDistanceExpose[1].modifiers[#Stretch_Expose_Controls].StretchControls[#paramdistance]
				StretchExpressionToAdd.AddScalarTarget "StretchValue" switchcontroller.modifiers[#ArmBlend_Controls].ArmBlendControls[#paramstretchBlend]
				
				local HandExpression = Copy StretchExpressionToAdd
				HandExpression.setexpression ("if(Distance>" + DefaultTotal + "," + DefaultLowArmLength + "+(((Distance-" + DefaultTotal +")*0.5)*StretchValue)," + DefaultLowArmLength +")")

				local LowArmExpression = Copy StretchExpressionToAdd
				LowArmExpression.setexpression ("if(Distance>" + DefaultTotal + "," + DefaultUpArmLength + "+(((Distance-" + DefaultTotal +")*0.5)*StretchValue)," + DefaultUpArmLength +")")
				HandBone.pos.controller.X_Position.controller = HandExpression
				LowArmBone.pos.controller.X_Position.controller = LowArmExpression
			)
			

			
			
			-----------------------------------Next: Draw the twist chains ---------------------------------
			
			if bendy == true or stretchy == true then --if it requires the curve based ik/fk spine to be added
			(
				
				if BendyRom == 1 then (BendyRom = true) else (BendyRom = false)
				local SquashTypeInput
				
				
				if (Squash == true) or (Squash == on) then (SquashTypeInput = 4) else (SquashTypeInput = 1)
				

				local UpArmTwistChain = FinalTools.MakeIKFKSpine UpArmBone LowArmBone  0 false 1 SquashTypeInput 1 TwistBones 0.5 undefined 1 2 Prefix:Prefix MainBonesName:UparmTwistBoneName squashblending:2 upnodes:bendyrom
				
				if (Squash == true) or (Squash == on) then (SquashTypeInput = 3) else (SquashTypeInput = 1)
				local LowArmTwistChain = FinalTools.MakeIKFKSpine LowArmBone HandBone  0 false 1 SquashTypeInput 1 TwistBones 0.5 undefined 1 2 Prefix:Prefix MainBonesName:LowarmTwistBoneNames squashblending:2 upnodes:bendyrom
				
				for i = 1 to i = ((UpArmTwistChain[7].count)-1) do
				(
				local ExtraShrink = ((i as float)/(UpArmTwistChain[7].count))
				CommonFns.ApplyLengthWidthProperties UpArmTwistChain[7][i] (UpArmWidth*(0.8-(ExtraShrink*0.2))) ObjFrontWidth:(UpArmFrontWidth*(0.8-(ExtraShrink*0.2))) ObjOffset:undefined
				if Squash == true do (UpArmTwistChain[7][i].scale.controller.weight[UpArmTwistChain[7][i].scale.controller.count].controller.addscalartarget  "SliderValue"  switchcontroller.modifiers[#ArmBlend_Controls].ArmBlendControls[#paramsquash])
				)	

				
				for i = 1 to i = ((LowArmTwistChain[7].count)-1) do
				(
				CommonFns.ApplyLengthWidthProperties LowArmTwistChain[7][i] (LowArmWidth*0.8) ObjFrontWidth:(LowArmFrontWidth*0.8) ObjOffset:undefined
				if Squash == true do (LowArmTwistChain[7][i].scale.controller.weight[LowArmTwistChain[7][i].scale.controller.count].controller.addscalartarget  "SliderValue"  switchcontroller.modifiers[#ArmBlend_Controls].ArmBlendControls[#paramsquash])
				)	
				
				
				CommonFns.QuickAlign UpArmTwistChain[2][1] UpArmBone Rotonly:true
				UpArmTwistChain[2][1].parent = UpArmBone
				
				CommonFns.QuickAlign LowArmTwistChain[2][1] LowArmBone Rotonly:true
				LowArmTwistChain[2][1].parent = LowArmBone
				
				local LowArmTwistCnsTarget = Point Pos:[0,0,0] Name:(uniquename (NullifierName + "LowArmTwistCnsTarget") Numdigits:3) Size:TinySize Box:On AxisTripod:Off Cross:Off
				CommonFns.QuickAlign LowArmTwistCnsTarget WristNoTwistNull
				CommonFns.QuickAlign LowArmTwistCnsTarget LowArmTwistChain[2][3] RotOnly:true
				LowArmTwistCnsTarget.parent = WristNoTwistNull
				
				append allcreated LowArmTwistCnsTarget

				LowArmTwistChain[2][3].pos.controller = position_constraint()
				LowArmTwistChain[2][3].pos.controller.appendtarget HandBone 100	
				UpArmTwistChain[2][3].pos.controller = position_constraint()
				UpArmTwistChain[2][3].pos.controller.appendtarget LowarmBone 100	
				
				UpArmTwistChain[2][2].rotation.controller = orientation_constraint()
				UpArmTwistChain[2][2].rotation.controller.appendtarget ShoulderNoTwistNull 100
				LowArmTwistChain[2][3].rotation.controller = orientation_constraint()
				LowArmTwistChain[2][3].rotation.controller.appendtarget LowArmTwistCnsTarget 100
			)
		else if TwistBones>0 do --if no bendy or sretchy then add the standard twist setup
		(

			local UpArmTwistChain = #()
			local LowArmTwistChain = #()

			
			
			local UparmTwistLocations = CommonFns.findIncrementsBetweenTwoPoints UpArmBone LowArmBone divisions:(TwistBones-1) IncludeEnds:True
			local LowarmTwistLocations = CommonFns.findIncrementsBetweenTwoPoints LowArmBone HandBone divisions:(TwistBones-1) IncludeEnds:True
		
			for i = 1 to i = (UpArmTwistLocations.count-1) do
			(
				local BoneToAdd
				BoneToAdd = boneSys.createBone UpArmTwistLocations[i] UpArmTwistLocations[(i+1)] [0,0,1]
				BoneToAdd.sidefins = off
				BoneToAdd.frontfin = off
				BoneToAdd.backfin = off
				BoneToAdd.width = tinysize
				
				CommonFns.QuickAlign BoneToAdd UpArmBone RotOnly:True
				BoneToAdd.parent = UpArmBone
				BoneToAdd.Name = (uniquename (Prefix + UpArmTwistBoneName) Numdigits:3)
				CommonFns.ApplyLengthWidthProperties BoneToAdd (UpArmWidth*0.8) ObjFrontWidth:(UpArmFrontWidth*0.8) ObjOffset:undefined
				append MainBonesArray BoneToAdd
				
				if i < ((UpArmTwistLocations.count)-1) do
				(
					local CurrentPercent = ((1.0-((i-1.0)*(1.0/(TwistBones-1.0)))))
					BoneToAdd.rotation.controller = orientation_Constraint() 
					BoneToAdd.rotation.controller.appendtarget ShoulderNoTwistNull (CurrentPercent)
					BoneToAdd.rotation.controller.appendtarget ShoulderLocalNull (1-CurrentPercent)	

				)					
			)
			
			for i = 1 to i = (LowArmTwistLocations.count-1) do
			(
				local BoneToAdd

				BoneToAdd = boneSys.createBone LowArmTwistLocations[i] LowArmTwistLocations[(i+1)] [0,0,1]
				BoneToAdd.sidefins = off
				BoneToAdd.frontfin = off
				BoneToAdd.backfin = off
				BoneToAdd.width = tinysize
				CommonFns.QuickAlign BoneToAdd LowArmBone RotOnly:True
				BoneToAdd.parent = LowArmBone
				BoneToAdd.Name = (uniquename (Prefix + LowArmTwistBoneName) Numdigits:3)
				CommonFns.ApplyLengthWidthProperties BoneToAdd (LowArmWidth*0.8) ObjFrontWidth:(LowArmFrontWidth*0.8) ObjOffset:undefined
				append MainBonesArray BoneToAdd
				
				if i > 1 do
				(
					local CurrentPercent = (((i-1.0)*(1.0/(TwistBones-1.0))))
					BoneToAdd.rotation.controller = orientation_Constraint() 
					BoneToAdd.rotation.controller.appendtarget WristNoTwistNull CurrentPercent
					BoneToAdd.rotation.controller.appendtarget WristLocalNull (1-CurrentPercent)
				)			



				
			)
			
			
			
			
			
			
		)
		

			--LowArmTwistChain[2][1] = Global
			--LowArmTwistChain[2][2]  = Start
			--LowArmTwistChain[2][3] = End
			
			--LowArmTwistChain[6][1] = StartTangent
			--LowArmTwistChain[6][2] = EndTangent
			
			--------------------------Next Draw the elbow 50% system---------------------------------------------
			
			local Elbow50Null
			local UparmBendController
			local LowArmBendController
			local Elbow50Bone
			local Elbow50Parent


			
			Elbow50Null = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename Elbow50NullName Numdigits:3) size:Tinysize wirecolor:Green
			CommonFns.QuickAlign Elbow50Null LowArmBone
			
			Elbow50Parent = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename Elbow50ParentName Numdigits:3) size:Tinysize wirecolor:Green
			CommonFns.QuickAlign Elbow50Parent LowArmBone
			Elbow50Parent.parent = Elbow50Null
			
			Elbow50Bone = boneSys.createBone [0,0,0] [(distance UpArmBone LowArmBone *0.05),0,0] [0,0,1]
			
			Elbow50Bone.Name = (uniquename (Elbow50BoneName) Numdigits:3)
			CommonFns.QuickAlign Elbow50Bone Elbow50Null
			Elbow50Bone.parent = Elbow50Parent
			
			Elbow50Null.parent = LowArmBone
			Elbow50Null.rotation.controller = Orientation_Constraint()
			Elbow50Null.rotation.controller.appendtarget UpArmBone 100
			Elbow50Null.rotation.controller.appendtarget LowArmBone 100
			
			append allcreated Elbow50Null
			append allcreated Elbow50Parent
			append allcreated Elbow50Bone
			append ExtraBones Elbow50Bone
			
			if Bendy == true or Stretchy == true do
			(
				if Squash == true do (Elbow50parent.scale.controller = copy LowArmTwistChain[7][1].scale.controller)
			)
			
			local ElbowDistExpose 
			local Elbow50BoneFront
			local ca
			
			If ElbowType != 1 do  --Create the measure helper/expose transform system.
			(
				local TempStartPoint = Point Name:"ZZZZZZTEMPSTART"
				local TempEndPoint = Point Name:"ZZZZZTEMPBLAH"
				TempStartPoint.pos.controller = position_constraint()
				TempEndPoint.pos.controller = position_constraint()
				TempStartPoint.pos.controller.appendtarget UpArmBone 25
				TempStartPoint.pos.controller.appendtarget LowArmBone 75
				TempEndPoint.pos.controller.appendtarget LowArmBone 75
				TempEndPoint.pos.controller.appendtarget HandBone 25
				ElbowDistExpose =  CommonFns.CreateLookatdistanceExpose TempStartPoint TempEndPoint #((MeasureHelperLookatName),(MeasureHelperLocalName)) AlignTo:undefined reAssignParents:#(UpArmBone,LowArmBone) justdistance:false
				delete TempStartPoint
				delete TempEndPoint
				for n in ElbowDistExpose do (append allcreated n)
					
				 local ElbowController = CommonFns.MakeController (TinySize) "circle" Elbow50Null Elbow50Null blue ElbowControlName "y" "positive"
				ElbowController.objectoffsetpos = [0,(MediumSize),0]
				CommonFns.QuickAlign ElbowController Elbow50Null
				ElbowController.parent = Elbow50Null
				Elbow50Parent.parent = ElbowController
				append allcreated ElbowController
				append ControlsArray ElbowController
			 
			
				ca=attributes ElbowAdjust
				(
					parameters params rollout:ro_AdjustElbow
					(
					paramblend type:#float ui:spn_ElbowBlend
					)
					
					rollout ro_AdjustElbow "Adjust_Elbow" 
					(
						edittext edt_moveamount "ExtendAmount"
						spinner spn_elbowblend "Auto Off/On" Range:[0,1,1] scale:0.1
						button btn_apply "Apply" enabled:on

						local ElbowType,MoveAmount,FrontBoneName,BackBoneName
						
						on ro_AdjustElbow open do
						(
							elbowtype = 	(GetUserProp $ "RigTools_ElbowType") as string
							if Elbowtype == "Scale" then 
							(
								edt_moveamount.visible = false
								btn_apply.visible = false
							)
							else
							(
								moveamount = (GetUserProp $ "RigTools_MoveAmount")
								frontbonename = (GetUserProp $ "RigTools_FrontBoneName")
								backbonename = (GetUserProp $ "RigTools_BackBoneName")
								edt_moveamount.text = (Moveamount as string)
							)
						)
							
						on btn_apply pressed do
						(
							local NewMoveAmount = edt_moveamount.text
							local frontbone = getnodebyname frontbonename
							local backbone = getnodebyname backbonename
							setUserProp $ "RigTools_MoveAmount"  NewMoveAmount
							backbone.pos.controller[2].Y_Position.controller.setexpression ("(max(Distance*" + (NewMoveAmount as string) + ",0))")
							frontbone.pos.controller[2].Y_Position.controller.setexpression ("-(max(Distance*" + (NewMoveAmount as string) + ",0))")				
						)
					)
				)
				local AttHolder = (EmptyModifier())
				AttHolder.name = "Elbow_Controls"
				addmodifier ElbowController AttHolder
				custAttributes.add ElbowController.modifiers[#Elbow_Controls] ca
			
			)
			
			
			
			
			
			
			
			
			If ElbowType == 2 do  -- Scale Bone
			(

			Elbow50Bone.scale.controller = Scale_List()
			Elbow50Bone.scale.controller.available.controller = Scale_Expression ()
			Elbow50Bone.scale.controller[2].controller.addscalartarget  "Distance" ElbowDistExpose[1].modifiers[#Stretch_Expose_Controls].StretchControls[#paramstretch]
			Elbow50Bone.scale.controller[2].controller.setexpression ("[1,(max((Distance*0.9),0))+1,1]")
			setUserProp ElbowController "RigTools_ElbowType"  "Scale"
			Elbow50Bone.scale.controller.weight[2].controller = Float_Expression()
			Elbow50Bone.scale.controller.weight[2].controller.addscalartarget "SliderValue" ElbowController.modifiers[#Elbow_Controls].ElbowAdjust[#paramblend]
			Elbow50Bone.scale.controller.weight[2].controller.setexpression "SliderValue"
			
			)
			
			If ElbowType == 3 do 
			(
			local Elbow50FrontNull = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename (Elbow50NullName + "_front") Numdigits:3) size:Tinysize wirecolor:Green
			local Elbow50BackNull = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename (Elbow50NullName + "_back") Numdigits:3) size:Tinysize wirecolor:Green
			CommonFns.quickalign Elbow50FrontNull Elbow50Null
			CommonFns.quickalign Elbow50BackNull Elbow50Null
			Elbow50FrontNull.parent = Elbow50Null
			Elbow50BackNull.parent = Elbow50Null
			in coordsys Local Move Elbow50FrontNull [0,-(TinySize*0.2),0]
			in coordsys local Move Elbow50BackNull [0,TinySize*0.2,0]
			Elbow50Bone.parent = Elbow50BackNull
			CommonFns.QuickAlign Elbow50Bone Elbow50BackNull
			
			append allcreated Elbow50FrontNull
			append allcreated Elbow50BackNull
			
			Elbow50Bone.pos.controller = Position_List()
			Elbow50Bone.pos.controller.available.controller = Position_XYZ()
				
			Elbow50Bone.pos.controller[2].Y_Position.controller = Float_Expression ()	
			Elbow50Bone.name = (uniquename (Elbow50BoneName + "_back") numdigits:3)
			
			Elbow50BoneFront = boneSys.createBone [0,0,0] [(distance UpArmBone LowArmBone *0.05),0,0] [0,0,1]
			Elbow50BoneFront.Name = (uniquename (Elbow50BoneName+ "_back") Numdigits:3)
			CommonFns.QuickAlign Elbow50BoneFront Elbow50FrontNull
			Elbow50BoneFront.parent = Elbow50FrontNull
			append allcreated elbow50BoneFront
			append ExtraBones elbow50bonefront
				
			Elbow50BoneFront.pos.controller = position_list()
			Elbow50BoneFront.pos.controller.available.controller = position_xyz()
			Elbow50BoneFront.pos.controller[2].Y_Position.controller = Float_Expression ()
				
				
			Elbow50BoneFront.pos.controller[2].Y_Position.controller.addscalartarget  "Distance" ElbowDistExpose[1].modifiers[#Stretch_Expose_Controls].StretchControls[#paramstretch]
			Elbow50BoneFront.pos.controller[2].Y_Position.controller.setexpression ("-(max(Distance*" + (Bigsize as string) + ",0))")				
			Elbow50Bone.pos.controller[2].Y_Position.controller.addscalartarget  "Distance" ElbowDistExpose[1].modifiers[#Stretch_Expose_Controls].StretchControls[#paramstretch]
			Elbow50Bone.pos.controller[2].Y_Position.controller.setexpression ("(max(Distance*" + (Bigsize as string) + ",0))")

			Elbow50Bone.pos.controller.weight[2].controller = float_expression()
			Elbow50Bone.pos.controller.weight[2].controller.addscalartarget "SliderValue" ElbowController.modifiers[#Elbow_Controls].ElbowAdjust[#paramblend]
			Elbow50Bone.pos.controller.weight[2].controller.setexpression "SliderValue"
			Elbow50BoneFront.pos.controller.weight[2].controller = float_expression()
			Elbow50BoneFront.pos.controller.weight[2].controller.addscalartarget "SliderValue" ElbowController.modifiers[#Elbow_Controls].ElbowAdjust[#paramblend]
			Elbow50BoneFront.pos.controller.weight[2].controller.setexpression "SliderValue"
			
			setUserProp ElbowController "RigTools_MoveAmount"  (BigSize as string)
			setUserProp ElbowController "RigTools_ElbowType"  "Game"
			setUserProp ElbowController "RigTools_FrontBoneName"  (Elbow50BoneFront.name)
			setUserProp ElbowController "RigTools_BackBoneName"  (Elbow50Bone.name)		
			) --Game Compatible

				

			
			-----------------------------Add the bendy system-------------------------------------------------------------------------
			

			
			if Bendy == true do
			(
				

			
				UpArmBendController = CommonFns.MakeController (MediumSize) "triangle" LowArmBone LowArmBone (color 40 200 40) (UparmBendyControlName)  "x" "negative"	
				
				CommonFns.quickalign UpArmBendController Elbow50Null
				UpArmBendController.objectoffsetpos = [-((Distance UparmBone LowArmBone)*0.4),0,0]
				UpArmBendController.parent = Elbow50Null
				
				append allcreated UpArmBendController
				
				LowArmBendController = CommonFns.MakeController (MediumSize) "triangle" LowArmBone LowArmBone (color 40 200 40) (LowarmBendyControlName)  "x" "positive"
				CommonFns.quickalign LowArmBendController Elbow50Null
				LowArmBendController.objectoffsetpos = [((Distance UparmBone LowArmBone)*0.4),0,0]
				LowArmBendController.parent = Elbow50Null
				
				append Allcreated LowArmBendController
				
				UpArmTwistChain[2][3].rotation.controller = orientation_constraint()
				UpArmTwistChain[2][3].rotation.controller.appendtarget UpArmBone 100
				UpArmTwistChain[2][3].rotation.controller.appendtarget UpArmBendController 100
				
				
				LowArmTwistChain[2][2].rotation.controller = orientation_constraint()
				LowArmTwistChain[2][2].rotation.controller.appendtarget LowArmBone 100
				LowArmTwistChain[2][2].rotation.controller.appendtarget LowArmBendController 100
				
				local BaseBendExpression = float_expression()
				BaseBendExpression.AddScalarTarget "SliderValue" switchcontroller.modifiers[#ArmBlend_Controls].ArmBlendControls[#parambendyBlend]
				
				local ArmBendExpression = copy BaseBendExpression
				ArmBendExpression.setExpression "100-SliderValue"
				LowArmTwistChain[2][2].rotation.controller.weight[1].controller = ArmBendExpression
				UpArmTwistChain[2][3].rotation.controller.weight[1].controller = ArmBendExpression
				ArmBendExpression = copy ArmBendExpression
				ArmBendExpression.setexpression "SliderValue"
				LowArmTwistChain[2][2].rotation.controller.weight[2].controller = ArmBendExpression
				UpArmTwistChain[2][3].rotation.controller.weight[2].controller = ArmBendExpression
				
				local TempTangentPoint = Point Name:"ZZZZZTEMPTANGENT" Pos:[0,0,0]
				TempTangentPoint.pos.controller = position_constraint()
				TempTangentPoint.pos.controller.appendtarget LowArmBone 20
				TempTangentPoint.pos.controller.appendtarget UpArmBone 80
				CommonFns.quickalign UpArmTwistChain[6][1] TempTangentPoint
				TempTangentPoint.pos.controller = position_xyz()
				TempTangentPoint.pos.controller = position_constraint()
				TempTangentPoint.pos.controller.appendtarget LowArmBone 20
				TempTangentPoint.pos.controller.appendtarget Handbone 80
				CommonFns.quickalign LowArmTwistChain[6][2] TempTangentPoint
				delete TempTangentPoint
				
			)

			------------------------------------------------------------------------------------------
			
			for i = 1 to i = mainbonesarray.count do
			(
				mainbonesarray[i].width = TinySize
				mainbonesarray[i].height = TinySize
				mainbonesarray[i].sidefins = off
				mainbonesarray[i].frontfin = off
				mainbonesarray[i].backfin = off
			)
			
			for i = 1 to i = cnstargetbones.count do
			(
				cnstargetbones[i].width = TinySize
				cnstargetbones[i].height = TinySize
				cnstargetbones[i].sidefins = off
				cnstargetbones[i].frontfin = off
				cnstargetbones[i].backfin = off
			)
			
			for i = 1 to i = createdfingersarray.count do 
			(
				createdfingersarray[i].width = (TinySize*0.6)
				createdfingersarray[i].height = (TinySize*0.6)
				createdfingersarray[i].sidefins = off
				createdfingersarray[i].frontfin = off
				createdfingersarray[i].backfin = off
			)
			
			for i = 1 to i = extrabones.count do
			(
				extrabones[i].width = (TinySize*0.6)
				extrabones[i].height = (TinySize*0.6)
				extrabones[i].sidefins = off
				extrabones[i].frontfin = off
				extrabones[i].backfin = off
				extrabones[i].wirecolor = (color 160 80 80)
			)
			
			for i = 1 to i = nubsarray.count do
			(
				nubsarray[i].width = (TinySize*0.1)
				nubsarray[i].height = (TinySize*0.1)
				nubsarray[i].sidefins = off
				nubsarray[i].frontfin = off
				nubsarray[i].backfin = off
				nubsarray[i].wirecolor = (color 20 20 20)
			)
			
			
			local OutputArray = #()
			


			FinalTools.appendobjstolayer Allcreated HiddenComponents:True
			FinalTools.appendobjstolayer nubsarray HiddenComponents:True
			FinalTools.appendobjstolayer MainBonesArray HiddenComponents:False MainBones:True
			FinalTools.appendobjstolayer ControlsArray HiddenComponents:False Controllers:True
			FinalTools.appendobjstolayer ExtraBones HiddenComponents:False DeformerBones:True
			
			try
			(
				FinalTools.appendobjstolayer UpArmTwistChain[7][(UpArmTwistChain[7].count)] HiddenComponents:True
				FinalTools.appendobjstolayer LowArmTwistChain[7][(LowArmTwistChain[7].count)] HiddenComponents:True
			)
			catch()
			
			if bendy == true or stretchy == true do
			(
				for n in UpArmTwistChain[1] do (append allcreated n)
				for n in LowArmTwistChain[1] do (append allcreated n)
			)
			
			
			
			Append OutputArray Allcreated
			append OutputArray ControlsArray
			append OutputArray MainBonesArray
			append OutputArray ExtraBones

						--UpArmTwistChain = FinalTools.MakeIKFKSpine UpArmBone LowArmBone  0 false 1 false 1 5 0.5 undefined 1 1 Prefix:Prefix
			--local LowArmTwistChain 
			
			
			
			--OutputArray Contains
			--1:Allcreated
			--2: MainControls
			--3: MainBones
			--4: Deform Bones

			
	),--THIS BRACKET TO CLOSE MAKE ARM ROLLOUT
	-----------------------------------------------------------------------------------------------------------
	fn MakeLegGuide UpperLegPoint KneePoint AnklePoint ForceAlignment:Undefined LegType:1 =
	(
	
		local UpLegLocator,KneeLocator,AnkleLocator
		
		if LegType == 1 or LegType == 3 then ( LegType = "Human") else (LegType = "Quadruped")
		
		
		local DetectedAlignment
		if UpperLegPoint.transform.pos.x <0 then (DetectedAlignment = "R") else (DetectedAlignment = "L")
		if ForceAlignment == "R" or ForceAlignment == "r" or ForceAlignment == "right" then (DetectedAlignment = "R")
		if ForceAlignment == "L" or ForceAlignment == "l" or ForceAlignment == "left" then (DetectedAlignment = "L")
		
		local AverageSize = (((distance UpperLegPoint KneePoint) + (distance KneePoint AnklePoint)) / 2)
		local TinySize = AverageSize/16
		local MediumSize = AverageSize/12
		local BigSize = AverageSize/8
		
			

		
		local allCreated = #()
		local mainBones = #()
		local WidthHelpersArray = #()

		
		------------------Replace the points with controls----------------------------------------
		UpLegLocator = CommonFns.MakeController (BigSize) "sphere" UpperLegPoint UpperLegPoint (color 40 40 255) ("UpperLeg_Locator")  "z" "positive"
		delete UpperLegPoint
		KneeLocator = CommonFns.MakeController (BigSize) "sphere" KneePoint KneePoint (color 40 40 255) ("Knee_Locator")  "z" "positive"
		delete KneePoint
		AnkleLocator = CommonFns.MakeController (BigSize) "square" AnklePoint AnklePoint (color 40 40 255) ("Ankle_Locator" )  "z" "positive"
		delete AnklePoint
		
		append AllCreated UpLegLocator
		append AllCreated KneeLocator
		append AllCreated AnkleLocator
		
		-------------------Draw the bones and nubs--------------------------------------------------------------------------------------------------------	
			
		local UpLegGuideBone,LowLegGuideBone,upLegGuideNub,lowLegGuideNub
		
		UpLegGuideBone  = boneSys.createBone upLeglocator.transform.pos kneeLocator.transform.pos [0,0,1]
		UpLegGuideBone.name = (uniquename "UpperLeg_Guide" numdigits:3)
		LowLegGuideBone = boneSys.createBone KneeLocator.transform.pos AnkleLocator.transform.pos [0,0,1]
		LowLegGuideBone.name = (uniquename "LowerLeg_Guide" numdigits:3)
		UpLegGuideNub = boneSys.createBone KneeLocator.transform.pos KneeLocator.transform.pos [0,0,1]
		UpLegGuideNub.name = (uniquename "UpperLeg_Guide_Nub" numdigits:3)	
		lowLegGuideNub = boneSys.createBone Anklelocator.transform.pos AnkleLocator.transform.pos [0,0,1]
		LowLegGuideNub.name = (uniquename "LowerLeg_Guide_Nub" numdigits:3)
		
		UpLegGuideNub.parent = UpLegGuideBone
		lowLegGuideNub.parent = LowLegGuideBone
		LowLegGuideBone.parent = KneeLocator
		UpLegGuideBone.boneFreezeLength=false
		LowLegGuidEBone.boneFreezeLength=false
		LowLegGuideBone.boneScaleType=#scale
		UpLegGuideBone.boneScaleType=#scale
		UpLegGuideBone.parent = UpLegLocator
		
		local ScapulaGuideBone
		local ScapulaGuideNub
		local ScapulaLocator
		
		if LegType == "Quadruped" do 
		(
			ScapulaLocator = CommonFns.MakeController (BigSize) "sphere" UpLegGuideBone UpLegGuideBone (color 40 40 255) ("Scapula_Locator")  "z" "positive"

			ScapulaGuideBone = boneSys.createBone [0,0,0] [0,0,Bigsize*1.2] [0,0,1]
			ScapulaGuideBone.name = (uniquename "Scapula_Guide" numdigits:3)
			ScapulaGuideNub = boneSys.createBone [0,0,Bigsize*1.2] [0,0,Bigsize*1.2] [0,0,1]
			ScapulaGuideNub.name = (uniquename "Scapula_Guide_Nub" numdigits:3)
			ScapulaGuideNub.parent = ScapulaGuideBone
			
			append AllCreated ScapulaGuideBone
			append AllCreated ScapulaGuideNub
			append AllCreated ScapulaLocator

			CommonFns.ApplyArmLookat ScapulaGuideBone UpLegLocator KneeLocator Leg:false
			CommonFns.QuickAlign ScapulaGuideBone UpLegLocator
			ScapulaGuideBone.boneFreezeLength=false
			ScapulaGuideBone.boneScaleType=#scale
			In coordsys world Move ScapulaGuideBone [0,0,Bigsize*1.2]
			ScapulaGuideNub.pos.controller = position_constraint()
			ScapulaGuideNub.pos.controller.appendtarget UpLegLocator 100
			ScapulaGuideNub.ishidden = true
			
			ScapulaGuideBone.height = tinysize
			ScapulaGuideBone.width = tinysize
			ScapulaGuideBone.frontfin = off
			ScapulaGuideBone.sidefins = off
			ScapulaGuideBone.backfin = off
			CommonFns.QuickAlign ScapulaLocator ScapulaGuidEBone
			ScapulaGuideBone.parent = ScapulaLocator
			
			ScapulaGuideBone.showFrozenInGray = off
			freeze ScapulaGuideBone
		)
			
		
		append AllCreated UpLegGuideBone
		append AllCreated LowLegGuideBone
		append AllCreated UpLegGuideNub
		append AllCreated LowLegGuideNub
		
		append MainBones UpLegGuideBone
		append MainBones LowLegGuideBone
		append MainBones LowLegGuideNub
		
		
		
		-----------------Apply The Constraints----------------------------------
		local LegLookat = true
		if LegType == "Quadruped" do (LegLookat = false)
		
		CommonFns.ApplyArmLookat UpLegGuideBone KneeLocator AnkleLocator Leg:LegLookat
		CommonFns.ApplyArmLookat LowLegGuideBone AnkleLocator UpLegLocator Leg:LegLookat
		local UpLegNubPosCons = position_constraint()
		local LowLegNubPosCons = position_constraint()
		UpLegNubPosCons.appendtarget KneeLocator 100
		LowLegNubPosCons.appendtarget AnkleLocator 100
		UpLegGuideNub.pos.controller = UpLegNubPosCons
		LowLegGuideNub.pos.controller = LowLegNubPosCons
		-----------------Match the hand locator rotation to the LowArm Bone--------------------
		local TempFootLocatorPos = AnkleLocator.transform.pos


		AnkleLocator.rotation *= AnkleLocator.transform.rotation * (inverse LowLegGuideBone.transform.rotation)

		in Coordsys world AnkleLocator.pos = TempFootLocatorPos
		-----------------Create the controller for the foot and toe ----------------------------------
		local ToeLocator
		
		ToeLocator = CommonFns.MakeController (BigSize) "sphere" AnkleLocator AnkleLocator (color 40 40 255) ("Toe_Locator")  "x" "positive"
		ToeLocator.parent = AnkleLocator
		in CoordSys parent ToeLocator.pos.x += ((distance AnkleLocator KneeLocator)*0.3)
		--settransformlockflags HandEndLocator #{2,3,4,5,6,7,8,9}
		
		local TempLocation = ToeLocator.transform.pos --(used to place heel later)
		
		local ToeEndLocator
		
		
		ToeEndLocator = CommonFns.MakeController (BigSize) "sphere" ToeLocator ToeLocator (color 40 40 255) ("Toe_End_Locator")  "x" "positive"
		ToeEndLocator.parent = AnkleLocator
		in CoordSys parent ToeEndLocator.pos.x += ((distance AnkleLocator KneeLocator)*0.1)
		
		append AllCreated ToeLocator
		append AllCreated ToeEndLocator
		
		
		
		
		-----------------Create The Foot Bones-------------------------------------
		local FootGuideBone,FootGuideNub,ToeGuideBone,ToeGuideNub
		FootGuideBone  = boneSys.createBone ToeLocator.transform.pos LowLegGuideNub.transform.pos [0,0,1]
		FootGuideBone.name = (uniquename "Foot_Guide" numdigits:3)
		
		
		FootGuideBone.boneFreezeLength=false
		FootGuideBone.boneScaleType=#scale
		CommonFns.applyArmLookat FootGuideBone AnkleLocator KneeLocator Leg:False
		FootGuideBone.pos.controller = Position_constraint()
		FootGuideBone.pos.controller.appendtarget ToeLocator 100
		FootGuideNub  = boneSys.createBone AnkleLocator.transform.pos AnkleLocator.transform.pos [0,0,1]
		FootGuideNub.name = (uniquename "Foot_GuideNub" numdigits:3)
		FootGuideNub.pos.controller = Position_constraint()
		FootGuideNub.pos.controller.appendtarget AnkleLocator 100
		FootGuideNub.parent = FootGuideBone
		
		ToeGuideBone  = boneSys.createBone ToeEndLocator.transform.pos ToeLocator.transform.pos [0,0,1]
		ToeGuideBone.name = (uniquename "Toe_Guide" numdigits:3)
		ToeGuideNub  = boneSys.createBone AnkleLocator.transform.pos AnkleLocator.transform.pos [0,0,1]
		ToeGuideNub.name = (uniquename "Toe_GuideNub" numdigits:3)
		
		ToeGuideBone.boneFreezeLength=false
		ToeGuideBone.boneScaleType=#scale
		
		CommonFns.applyArmLookat ToeGuideBone ToeLocator KneeLocator Leg:False
		CommonFns.applyArmLookat ToeLocator ankleLocator KneeLocator Leg:False
		ToeGuideNub.pos.controller = Position_constraint()
		ToeGuideNub.pos.controller.appendtarget ToeLocator 100
		ToeGuideBone.pos.controller = Position_constraint()
		ToeGuideBone.pos.controller.appendtarget ToeEndLocator 100
		ToeGuideNub.parent = ToeGuideBone

		
		ToeEndLocator.parent = ToeLocator
		if LegType != "Quadruped" then 
		(
			in coordsys local rotate AnkleLocator (angleaxis -75 [0,0,1])
		)
		else
		(
			in coordsys local rotate AnkleLocator (angleaxis -20 [0,0,1])
		)
		
		ToeEndLocator.parent = AnkleLocator
		ToeLocator.parent = AnkleLocator
		
		append AllCreated FootGuideBone
		append AllCreated FootGuideNub
		append AllCreated ToeGuideBone
		append AllCreated ToeGuideNub
		append MainBones FootGuideBone
		append MainBones ToeGuideBone
		append MainBones ToeGuideNub
		
		for i = 1 to i = MainBones.count do
		(
			MainBones[i].height = tinysize
			MainBones[i].width = tinysize
			MainBones[i].frontfin = off
			MainBones[i].sidefins = off
			MainBones[i].backfin = off
		)
		
		----------------Create the extra rig points-------------------------------------
		
		local HeelPoint, FootTipPoint, FootOuterPoint, FootInnerPoint
		
		HeelPoint = CommonFns.MakeController (MediumSize) "sphere" AnkleLocator TempLocation (color 40 180 40) ("Heel_Locator")  "x" "positive"
		FootTipPoint = CommonFns.MakeController (MediumSize) "sphere" AnkleLocator ToeEndLocator (color 40 180 40) ("FootTip_Locator")  "x" "positive"
		FootInnerPoint = CommonFns.MakeController (MediumSize) "sphere" AnkleLocator TempLocation (color 40 180 40) ("InnerFoot_Locator")  "x" "positive"
		FootOuterPoint = CommonFns.MakeController (MediumSize) "sphere" AnkleLocator TempLocation (color 40 180 40) ("OuterFoot_Locator")  "x" "positive"
		if detectedalignment == "R" or detectedalignment == "r" then		
		(
			in coordsys world move FootInnerPoint [((distance AnkleLocator ToeLocator)*0.2),0,0]
			in coordsys world move FootOuterPoint [-((distance AnkleLocator ToeLocator)*0.2),0,0]
		)
		else
		(
			in coordsys world move FootInnerPoint [-((distance AnkleLocator ToeLocator)*0.2),0,0]
			in coordsys world move FootOuterPoint [((distance AnkleLocator ToeLocator)*0.2),0,0]
		)
		
		----------------Create the width guides---------------------------------
		
		local ThighWidthGuide,KneeWidthGuide,AnkleWidthGuide,ToeWidthGuide
		
		ThighWidthGuide = CommonFns.MakeController (BigSize*3) "circle" AnkleLocator AnkleLocator (color 40 120 80) ("ThighWidth")  "z" "positive"
		KneeWidthGuide = CommonFns.MakeController (BigSize*2) "circle" AnkleLocator AnkleLocator (color 40 120 80) ("KneeWidth")  "z" "positive"
		AnkleWidthGuide = CommonFns.MakeController (BigSize*2) "circle" AnkleLocator AnkleLocator (color 40 120 80) ("AnkleWidth")  "z" "positive"
		ToeWidthGuide = CommonFns.MakeController (BigSize*2) "circle" AnkleLocator AnkleLocator (color 40 120 80) ("ToeWidth")  "z" "positive"
		
		ThighWidthGuide.pos.controller = position_constraint()
		ThighWidthGuide.pos.controller.appendtarget UpLegLocator 80
		ThighWidthGuide.pos.controller.appendtarget KneeLocator 20
		ThighWidthGuide.Rotation.controller = Orientation_constraint()
		ThighWidthGuide.Rotation.controller.appendtarget UpLegGuideBone 100
		
		
		KneeWidthGuide.pos.controller = position_constraint()
		KneeWidthGuide.pos.controller.appendtarget KneeLocator 80
		KneeWidthGuide.pos.controller.appendtarget AnkleLocator 20
		KneeWidthGuide.Rotation.controller = Orientation_constraint()
		KneeWidthGuide.Rotation.controller.appendtarget LowLegGuideBone 100
		
		AnkleWidthGuide.pos.controller = position_constraint()
		AnkleWidthGuide.pos.controller.appendtarget KneeLocator 20
		AnkleWidthGuide.pos.controller.appendtarget AnkleLocator 80
		AnkleWidthGuide.Rotation.controller = Orientation_constraint()
		AnkleWidthGuide.Rotation.controller.appendtarget LowLegGuideBone 100
		
		
		ToeWidthGuide.pos.controller = position_constraint()
		ToeWidthGuide.pos.controller.appendtarget ToeLocator 20
		ToeWidthGuide.pos.controller.appendtarget AnkleLocator 80
		ToeWidthGuide.Rotation.controller = Orientation_constraint()
		ToeWidthGuide.Rotation.controller.appendtarget FootGuideBone 100
		
		append allcreated KneeWidthGuide
		append allcreated ToeWidthGuide
		append allcreated ThighWidthGuide
		append allcreated AnkleWidthGuide
		
		local ThighWidthPoint,ThighFrontWidthPoint,KneeWidthPoint,KneeFrontWidthPoint,AnkleWidthPoint,AnkleFrontWidthPoint,ToeWidthPoint,ToeFrontWidthPoint
		
		ThighWidthPoint = Point Pos:[0,0,0] Name:"ZZZZZThighWidth" Box:off Cross:off axistripod:off centermarker:off
		ThighFrontWidthPoint = Point Pos:[0,0,0] Name:"ZZZZZThighFronWidth" Box:off Cross:off axistripod:off centermarker:off
		KneeWidthPoint = Point Pos:[0,0,0] Name:"ZZZZZKneeWidth" Box:off Cross:off axistripod:off centermarker:off
		KneeFrontWidthPoint = Point Pos:[0,0,0] Name:"ZZZZZKneeFrontWidth" Box:off Cross:off axistripod:off centermarker:off
		AnkleWidthPoint = Point Pos:[0,0,0] Name:"ZZZZZAnkleWidth" Box:off Cross:off axistripod:off centermarker:off
		AnkleFrontWidthPoint = Point Pos:[0,0,0] Name:"ZZZZZAnkleFrontWidth" Box:off Cross:off axistripod:off centermarker:off
		ToeWidthPoint = Point Pos:[0,0,0] Name:"ZZZZZToeWidth" Box:off Cross:off axistripod:off centermarker:off
		ToeFrontWidthPoint = Point Pos:[0,0,0] Name:"ZZZZZToeFrontWidth" Box:off Cross:off axistripod:off centermarker:off
		
		AnkleWidthPoint.pos.controller = path_constraint()
		AnkleWidthPoint.pos.controller.path = AnkleWidthGuide
		AnkleWidthPoint.pos.controller.percent = 0
		AnkleFrontWidthPoint.pos.controller = path_constraint()
		AnkleFrontWidthPoint.pos.controller.path = AnkleWidthGuide
		AnkleFrontWidthPoint.pos.controller.percent = 25
		
		ToeWidthPoint.pos.controller = path_constraint()
		ToeWidthPoint.pos.controller.path = ToeWidthGuide
		ToeWidthPoint.pos.controller.percent = 0
		ToeFrontWidthPoint.pos.controller = path_constraint()
		ToeFrontWidthPoint.pos.controller.path = ToeWidthGuide
		ToeFrontWidthPoint.pos.controller.percent = 25
		
		ThighWidthPoint.pos.controller = path_constraint()
		ThighWidthPoint.pos.controller.path = ThighWidthGuide
		ThighWidthPoint.pos.controller.percent = 0
		ThighFrontWidthPoint.pos.controller = path_constraint()
		ThighFrontWidthPoint.pos.controller.path = ThighWidthGuide
		ThighFrontWidthPoint.pos.controller.percent = 25
		
		KneeWidthPoint.pos.controller = path_constraint()
		KneeWidthPoint.pos.controller.path = KneeWidthGuide
		KneeWidthPoint.pos.controller.percent = 0
		KneeFrontWidthPoint.pos.controller = path_constraint()
		KneeFrontWidthPoint.pos.controller.path = KneeWidthGuide
		KneeFrontWidthPoint.pos.controller.percent = 25
		
		local widthsarray = #()
		
		append WidthsArray ThighWidthGuide
		append WidthsArray ThighWidthPoint
		append WidthsArray ThighFrontWidthPoint
		append WidthsArray KneeWidthGuide
		append WidthsArray KneeWidthPoint
		append WidthsArray KneeFrontWidthPoint
		append WidthsArray AnkleWidthGuide
		append WidthsArray AnkleWidthPoint
		append WidthsArray AnkleFrontWidthPoint
		append WidthsArray ToeWidthGuide
		append WidthsArray ToeWidthPoint
		append WidthsArray ToeFrontWidthPoint
		
		append allcreated ThighWidthPoint
		append allcreated ThighFrontWidthPoint
		append allcreated KneeWidthPoint
		append allcreated KneeFrontWidthPoint
		append allcreated AnkleWidthPoint
		append allcreated AnkleFrontWidthPoint
		append allcreated ToeWidthPoint
		append allcreated ToeFrontWidthPoint
		
		--------------Final output stuff---------------------------------
		
		local FootRigLocators = #()
		append FootRigLocators HeelPoint
		append FootRigLocators FootTipPoint
		append FootRigLocators FootInnerPoint
		append FootRigLocators FootOuterPoint
		
		for n in FootRigLocators do (n.parent = AnkleLocator)
		
		append allcreated HeelPoint
		append allcreated FootTipPoint
		append allcreated FootInnerPoint
		append allcreated FootOuterPoint
		
		
		local OutputArray = #()
		append OutputArray allcreated
		append OutputArray MainBones
		append OutputArray FootRigLocators
		append OutputArray WidthsArray
		append OutputArray DetectedAlignment
		
		if LegType == "Quadruped" do  (append OutputArray #(ScapulaGuideBone,ScapulaGuideNub))
		
				for n in allcreated do 
		(
			if classof n == bonegeometry do
			(
				n.showFrozenInGray = off
				freeze n
			)
		)
		
		
		----------OutputArray Contains---------------
		--1: all created
		--2: Main Bones: Upper Leg, Lower Leg,LowerLegNub, Foot, Toe
		--3: Foot Rig Locators: Heel, Foot Tip, Foot Inner, Foot Outer
		--4: Widths: ThighGuide, Thigh Width, Thigh Front >>>> Knee, Ankle, Toe
		--5: DetectedAlignment
		--6:Scapula Bone,ScapulaNub
		
		OutputArray
		
		
	), --This bracket to close make leg guide
	
	
	fn MakeLeg MainLegBones FootRigPoints Prefix:"" InputWidths:undefined Kinematics:1 Stretchy:false bendy:false ConnectTo:undefined Root:undefined DetectedAlignment:"R" TwistBones:3 FootRig:True IncludeSquash:false BendyRom:True KneeType:1 LegType:1 Scapula:false =
	(
		
		local AverageScale = ((distance MainLegBones[1].pos MainLegBones[2].pos) + (distance MainLegBones[2].pos MainLegBones[3].pos))/2
		
		
			
		
			local TinySize = AverageScale/18
			local MediumSize = AverageScale/12
			local BigSize = AverageScale/6
		
			local SideAlignment = DetectedAlignment
			local ScapulaLocation
			if LegType == 1 or LegType == 3 then (LegType = "Human") else (LegType = "Quadruped")
			if Scapula != false do (ScapulaLocation = Scapula[1])
			
		
			local FlipControllers = true
			if SideAlignment == "L" do (FlipControllers = false)
		
			local UpLegBoneName = Prefix + "cnstarget_" + SideAlignment + "_Leg_Upper"
			local UpLegIKBoneName = Prefix + "cnstarget_" + SideAlignment + "_IKLeg_Upper"
			local UpLegFKBoneName = Prefix + "cnsTarget_" + SideAlignment + "_FKLeg_Upper"
			local LowLegIKBoneName = Prefix + "cnstarget_" + SideAlignment + "_IKLeg_Lower"
			local LowLegFKBoneName = Prefix + "cnstarget_" + SideAlignment + "_FKLeg_Lower"
			local IKFootBoneName = Prefix + "cnstarget_" + SideAlignment + "_IKFoot"
			local FKFootBoneName = Prefix + "cnstarget_" + SideAlignment + "_FKFoot"
			local FKToeBoneName = Prefix + "cnstarget_" + SideAlignment + "_FKToe"
			local IKToeBoneName = Prefix + "cnstarget_" + SideAlignment + "_IKToe"
			local LowLegBoneName = Prefix + "cnstarget_" + SideAlignment + "_Leg_Lower"
			

		
			local LowLegBoneNameForSkin = Prefix + "bn_" + SideAlignment + "_Leg_Lower"
			local UpLegBoneNameForSkin = Prefix + "bn_" + SideAlignment + "_Leg_Upper"
			local UpLegTwistBoneName = "bn_UpLegTwist" + SideAlignment
			local LowLegTwistBoneName = "bn_LowLegTwist" + SideAlignment
		
		
			local FootBoneName = Prefix + "bn_" + SideAlignment + "_Foot"
			local ToeBoneName = Prefix + "bn_" + SideAlignment + "_Toe"
			local Leg50BoneName = Prefix + "bn_" + SideAlignment + "_leg50"
			local Hip50BoneName = Prefix + "bn_" + SideAlignment + "_hip50"
			
			local ButtDeformControlName = Prefix + "ctrl_" + SideAlignment + "_hip50"
			local HipSideDeformControlName = Prefix + "ctrl_" + SideAlignment + "_hipSide"
			local HipFrontDeformControlName = Prefix + "ctrl_" + SideAlignment + "_hipFront"
			
			local ButtBoneName = Prefix + "bn_" + SideAlignment + "_butt"
			local HipSideBoneName = Prefix + "bn_" + SideAlignment + "_hip_Side"
			local HipFrontBoneName = Prefix + "bn_" + SideAlignment + "_hip_front"
			
			
			local Knee50NullName = Prefix + "null_" + SideAlignment + "_knee50blend"
			local Knee50ParentName = Prefix + "null_" + SideAlignment + "_kneedeform"
			local MeasureHelperLookatName = (Prefix + "null_" + SideAlignment + "_KneeBendMeasure")
			local MeasureHelperLocalName = (Prefix + "null_" + SideAlignment + "_KneeBendLocal")
			local KneeDeformControlName =  (Prefix + "ctrl_" + SideAlignment + "_KneeDeform")
			
			local ScapulaBoneName =  (Prefix + "bn_" + SideAlignment + "_scapula")
			local ScapulaControlName =  (Prefix + "ctrl_" + SideAlignment + "_scapula")
			local ScapulaCnsTargetName =  (Prefix + "cnstarget_" + SideAlignment + "_scapula")

			local Knee50BoneName = Prefix + "bn_" + SideAlignment + "_kneedeform"
			local KneeBendyControlName = Prefix + "ctrl_" + SideAlignment + "_kneebend"
			local UpLegBendyControlName = Prefix + "ctrl_" + SideAlignment + "_upLegbend"
			local LowLegBendyControlName = Prefix + "ctrl_" +SideAlignment + "_lowLegbend"
			local UplegFKControlName = Prefix + "ctrl_" + SideAlignment + "_UplegFK"
			local LowlegFKControlName = Prefix + "ctrl_" + SideAlignment + "_LowlegFK"
			local footFKControlName = Prefix + "ctrl_" + SideAlignment + "_footFK"
			local toeFKControlName = Prefix + "ctrl_" + SideAlignment + "_toeFK"
			local footIKControlName = Prefix + "ctrl_" + SideAlignment + "_footIK"
			local FingerControlName = Prefix + "ctrl_" + SideAlignment + "_finger"
			local legIKSolverName = Prefix + "IKsolver_" +SideAlignment
			local ToeIKSolverName = Prefix + "IKsolver_" +SideAlignment + "_Toe"
			local FootIKSolverName = Prefix + "IKsolver_" +SideAlignment + "_Foot"
			local kneeControlName = Prefix + "ctrl_knee" + SideAlignment
			local ClavicleControlName = Prefix + "ctrl" + SideAlignment + "_clavicle"
			local ClavSplitControlName = Prefix + "ctrl" + SideAlignment + "_clavicle_split"
			local ExposeTransformBaseName = Prefix + "helper_exptfm_" + Sidealignment
			local ExposeNodeName = Prefix + "helper_exposenode_" + Sidealignment
			local LocalNodeName = Prefix + "helper_localNode_" + sidealignment
			local MeasureHelperBaseName = Prefix + "helper_MeasureLookat_" + SideAlignment
			local NullifierName = Prefix + "null_" + SideAlignment
			local NubName = Prefix + "nub_" + Sidealignment
			local CnsTargetName = Prefix + "cnstarget_" + Sidealignment
			
			local HeelControllerName = Prefix + "ctrl_" + SideAlignment + "HeelRoll"
			local ToeRollControllerName = Prefix + "ctrl_" + SideAlignment + "ToeRoll"
			local FootInnerControllerName = Prefix + "ctrl_" + SideAlignment + " FootInnerBank"
			local FootOutercontrollerName = Prefix + "ctrl_" + SideAlignment + "FootOuterBank"
			local FootRollControllerName = Prefix + "ctrl_" + SideAlignment + "FootRoll"
			local ToeFlapControllerName = Prefix + "ctrl_" + SideAlignment + "ToeFlap"
			local SwitchControllerName = Prefix + "ctrl_" + SideAlignment + "Switch"
			
			local HipNoTwistNullName = Prefix + "Null_NoTwist_" + sidealignment + "Hip"
			local AnkleNoTwistNullName = Prefix + "Null_NoTwist_" + sidealignment + "Ankle"
			
			
			if TwistBones == 1 or TwistBones == 2 do (TwistBones = 3)
			
			local allcreated = #()
			local allbones = #()
			local ExtraBones =#()
			local AllControls = #()
			local MainBonesArray = #()
			local NubsArray = #()
			local CnsTargetBones = #()

			
			

			if InputWidths != undefined do
			(
				UpLegWidth = InputWidths[1] *1.2
				UpLegFrontWidth = InputWidths[2] *1.2
				KneeWidth = InputWidths[3] *1.2
				KneeFrontWidth = InputWidths[4] *1.2
				LowLegWidth = InputWidths[5] *1.2
				LowLegFrontWidth = InputWidths[6] *1.2
				FootWidth = InputWidths[7]
				FootFrontWidth = InputWidths[8]
			)
			if InputWidths == undefined do
			(
				UpLegWidth = BigSize
				UpLegFrontWidth = BigSize
				KneeWidth = BigSize
				KneeFrontWidth = BigSize
				LowLegWidth = BigSize
				LowLegFrontWidth = BigSize
				FootWidth = BigSize
				FootFrontWidth = BigSize
			)
			
			-----------------------------------Calculate the foot controller's rotation---------------------------
			local TempFootControlAlignmentPoint = Point Pos:[0,0,0] Name:"ZZZZFOOTCONTROLALIGNMENT"
			in coordsys world TempFootControlAlignmentPoint.pos = MainLegBones[3].pos
			local TempFootBonePoint = Point Pos:[0,0,0]
			CommonFns.quickalign TempFootBonePoint MainLegBones[4]
			local TempFootControlAlignmentLookat = Point Pos:[0,0,0] Name:"TEMPFOOTCONTROLLOOKAT"
			in coordsys world TempFootControlAlignmentLookat.pos = MainLegBones[4].pos
			in coordsys world TempFootControlAlignmentLookat.pos.z = MainLegBones[3].pos.z
			LookCons = Lookat_Constraint()
			LookCons.appendtarget TempFootControlAlignmentLookat 100
			LookCons.target_axis = 0
			LookCons.upnode_ctrl = 1
			LookCons.upnode_world = off
			LookCons.pickUpNode = TempFootBonePoint
			LookCons.StoUP_axis = 2
			LookCons.upnode_axis = 2
			LookCons.viewline_length_abs = off
			LookCons.lookat_vector_length = 0
			TempFootControlAlignmentPoint.rotation.controller = LookCons
			LookCons = Copy LookCons
			local ForwardFootTemp = Point Pos:[0,0,0] Name:"ZZZZZFORWARDFOOTTEMP"
			CommonFns.quickalign ForwardFootTemp MainLegBones[3]
			local LookCons = Lookat_Constraint()
			LookCons.appendtarget TempFootBonePoint 100
			LookCons.target_axis = 0
			LookCons.upnode_ctrl = 1
			LookCons.upnode_world = off
			LookCons.pickUpNode = TempFootBonePoint
			LookCons.StoUP_axis = 1
			LookCons.upnode_axis = 1
			LookCons.viewline_length_abs = off
			LookCons.lookat_vector_length = 0
			ForwardFootTemp.rotation.controller = LookCons
			local ForwardToeTemp = Point Pos:[0,0,0] Name:"ZZZZZFORWARDTOETEMP"
			local TempToeBonePoint = Point Pos:[0,0,0] Name:"ZZZZTEMPTOEBONE"
			CommonFns.QuickAlign TempToeBonePoint MainLegBones[5]
			CommonFns.quickalign ForwardToeTemp MainLegBones[4]
			LookCons = copy Lookcons
			LookCons = Lookat_Constraint()
			LookCons.appendtarget TempToeBonePoint 100
			LookCons.target_axis = 0
			LookCons.upnode_ctrl = 1
			LookCons.upnode_world = off
			LookCons.pickUpNode = TempToeBonePoint
			LookCons.StoUP_axis = 1
			LookCons.upnode_axis = 1
			LookCons.viewline_length_abs = off
			LookCons.lookat_vector_length = 0
			ForwardToeTemp.rotation.controller = LookCons
			LookCons = Copy LookCons

			local FootControlLoc = TempFootControlAlignmentPoint.transform
			local UpperLegLoc = MainLegBones[1]
			local LowerLegLoc = MainLegBones[2]
			local ReverseFootLoc = MainLegBones[4]
			local ReverseToeLoc = MainLegBones[5]
			local ForwardFootLoc = ForwardFootTemp.transform
			local ForwardToeLoc = ForwardToeTemp.transform
			
			delete TempFootControlAlignmentPoint
			delete TempFootBonePoint
			delete TempFootControlAlignmentLookat
			delete ForwardFootTemp
			delete ForwardToeTemp
			delete TempToeBonePoint
		------------------------------Create Main Leg Bones----------------------------------------
			
			local ScapulaBone
			local ScapulaController
			local ScapulaControlNull
			local ScapulaCnsTarget
			
			local UpperLegParent = ConnectTo
			
			if LegType == "Quadruped" do
			(
				
				ScapulaBone = boneSys.createBone ScapulaLocation.pos UpperLegLoc.pos [0,0,1]
				CommonFns.QuickAlign ScapulaBone ScapulaLocation
				
				ScapulaBone.name = (uniquename ScapulaBoneName Numdigits:3)
				local TempPoint = Point Name:"ZZZZTEMPPOINT"
				CommonFns.QuickAlign TempPoint UpperLegLoc
				UpperLegParent = ScapulaBone
				ScapulaController = CommonFns.DrawRectangleControl ScapulaBone ScapulaBone TempPoint UpLegFrontWidth UpLegFrontWidth (uniquename ScapulaControlName numdigits:3)
				ScapulaControlNull = point Name:(uniquename (nullifiername + "ScapulaControl") Numdigits:3) Pos:[0,0,0] Box:Off Cross:On AxisTripod:off
				CommonFns.QuickAlign ScapulaControlNull ScapulaController
				ScapulaCnsTarget = point Name:(uniquename (ScapulaCnsTargetName) Numdigits:3) Pos:[0,0,0] Box:Off Cross:On AxisTripod:off
				CommonFns.QuickAlign ScapulaCnsTarget ScapulaController
				ScapulaControlNull.parent = ConnectTo

				if FlipControllers == true do 
				(
					CommonFns.FlipZAxis ScapulaController
					CommonFns.FlipZAxis ScapulaControlNull
				)
				
				ScapulaCnsTarget.parent = ScapulaController
				ScapulaBone.parent = ScapulaControlNull
				ScapulaBone.pos.controller = position_constraint()
				ScapulaBone.pos.controller.appendtarget ScapulaCnsTarget 100
				ScapulaBone.rotation.controller = orientation_constraint()
				ScapulaBone.rotation.controller.appendtarget ScapulaCnsTarget 100
				ScapulaController.parent = ScapulaControlNull
				CommonFns.ApplycontrollerTypeProperties ScapulaController ControlType:"Medium" ControlLevel:1
				CommonFns.ApplyLengthWidthProperties ScapulaBone UpLegFrontWidth ObjFrontWidth:UpLegWidth ObjOffset:undefined



				append allcreated ScapulaBone
				append allcreated ScapulaController
				append allcreated ScapulaControlNull
				append allcreated ScapulaCnsTarget
				append allbones ScapulaBone
				append allcontrols ScapulaController
				append MainBonesArray ScapulaBone
				UpperLegParent = ScapulaBone
				delete TempPoint
			)
			
			
			
			local UpLegBone = boneSys.createBone UpperLegLoc.pos LowerLegLoc.pos [0,0,1]
			CommonFns.QuickAlign UpLegBone UpperLegLoc
			UpLegBone.Name = uniquename (UpLegBoneName) Numdigits:3
			append allcreated UpLegBone
			append allbones UpLegBone
			-----
			UpLegBone.parent = UpperLegParent
			
			local LowLegBone = boneSys.createBone LowerLegLoc.pos ForwardFootLoc.pos [0,0,1]
			CommonFns.QuickAlign LowLegBone LowerLegLoc
			LowLegBone.Name = uniquename (LowLegBoneName) Numdigits:3
			LowLegBone.parent = UpLegBone
			append allbones LowLegBone
			append allcreated LowLegBone
			
			if Stretchy != True and Bendy!= True and TwistBones == 0 do
			(
				UpLegBone.name = 	(uniquename UpLegBoneNameForSkin numdigits:3)
				LowLegBone.name = (uniquename LowLegBoneNameForSkin numdigits:3)
				CommonFns.ApplyLengthWidthProperties UpLegBone (UpLegWidth*0.8) ObjFrontWidth:(UpLegFrontWidth*0.8) ObjOffset:undefined
				CommonFns.ApplyLengthWidthProperties LowLegBone (LowLegWidth*0.8) ObjFrontWidth:(LowLegFrontWidth*0.8) ObjOffset:undefined
				append mainbonesarray UpLegBone
				append mainbonesarray LowLegBone
			)
			
			
			local FootBone = boneSys.createBone ForwardFootLoc.pos ForwardToeLoc.pos [0,0,1]
			CommonFns.QuickAlign FootBone ForwardFootLoc
			FootBone.Name = uniquename (FootBoneName) Numdigits:3
			FootBone.parent = LowLegBone
			append allcreated FootBone
			append allbones FootBone
			append MainBonesArray FootBone
			
			CommonFns.ApplyLengthWidthProperties FootBone (FootFrontWidth*0.8) ObjFrontWidth:(FootWidth*0.8) ObjOffset:undefined
			
			local ToeBone = boneSys.createBone ForwardToeLoc.pos ReverseToeLoc.pos [0,0,1]
			CommonFns.QuickAlign ToeBone ForwardToeLoc
			ToeBone.Name = uniquename (ToeBoneName) Numdigits:3
			ToeBone.parent = FootBone
			append allcreated ToeBone
			append allbones ToeBone
			append MainBonesArray ToeBone
			CommonFns.ApplyLengthWidthProperties ToeBone (FootFrontWidth*0.8) ObjFrontWidth:(FootWidth*0.8) ObjOffset:undefined
			--------------------------------------------------------------------------------------------------
			
			If kinematics == 3 or Kinematics == 1 do -- if there is ik
			(
				local FootIKBone = boneSys.createBone ForwardFootLoc.pos ForwardToeLoc.pos [0,0,1]
				CommonFns.QuickAlign FootIKBone ForwardFootLoc
				FootIKBone.Name = uniquename (IKFootBoneName) Numdigits:3
				

				
				
				append allcreated FootIKBone
				append allbones FootIKBone
					
				local FootIKNub = boneSys.createBone ReverseFootLoc.pos ReverseFootLoc.pos [0,0,1]
				CommonFns.QuickAlign FootIKNub ReverseFootLoc
				FootIKNub.Name = uniquename (NubName + "ik_foot") Numdigits:3
				FootIKNub.parent = FootIKBone
				append allcreated FootIKNub
				append allbones FootIKNub
					
				local ToeIKBone = boneSys.createBone ForwardToeLoc.pos ReverseToeLoc.pos [0,0,1]
				CommonFns.QuickAlign ToeIKBone ForwardToeLoc
				ToeIKBone.Name = uniquename (IKToeBoneName) Numdigits:3
				ToeIKBone.parent = FootIKBone
				append allcreated ToeIKBone
				append allbones ToeIKBone
					
				local ToeIKNub = boneSys.createBone ReverseToeLoc.pos ReverseToeLoc.pos [0,0,1]
				CommonFns.QuickAlign ToeIKNub ReverseToeLoc
				ToeIKNub.Name = uniquename (NubName + "toe_ik") Numdigits:3
				ToeIKNub.parent = ToeIKBone
				append allcreated ToeIKNub
				append allbones ToeIKNub
						
				
				local KneeController = CommonFns.MakeController BigSize "sphere" LowLegBone LowLegBone (color 180 40 40)  KneeControlName  "y" "negative"
				CommonFns.ApplycontrollerTypeProperties KneeController ControlType:"Upnode" ControlLevel:1
				CommonFns.quickAlign KneeController FootControlLoc
				in coordsys world KneeController.pos = LowLegBone.transform.pos
				if FlipControllers == true do (CommonFns.FlipZAxis KneeController)
				--CommonFns.quickAlign KneeController FootControlLoc
				in coordsys world KneeController.pos = LowLegBone.transform.pos
				
				
				
				
				
				
				
				local KneeControlNull = point Name:(uniquename (nullifiername + "KneeControl") Numdigits:3) Pos:[0,0,0] Box:Off Cross:On AxisTripod:off Size:Tinysize wirecolor:(color 40 100 60)
				
				
				if FlipControllers == true do (CommonFns.FlipZAxis KneeControlNull)
				CommonFns.quickAlign KneeControlNull KneeController
				KneeControlNull.parent = root
				KneeController.parent = root
				
				append allcreated KneeController
				append allcontrols KneeController
				append allcreated KneeControlNull
				
				local tempshape = CommonFns.DrawRectangleControl FootBone FootBone ToeBone LowLegWidth LowLegFrontWidth (uniquename "ZZZZTEMPSHAPE" numdigits:3)
				local FootIKController = SplineShape pos:[0,0,0] name:(uniquename FootIKControlName Numdigits:3)
				CommonFns.QuickAlign FootIKController FootControlLoc
				addandweld FootIKController tempshape 0
				updateshape FootIKController
				
				local FootIKCnsTarget =  Point Name:(UniqueName (CnsTargetName + "_FootIKControl") numdigits:3) Size:Tinysize wireColor: (color 20 80 20)
				
				CommonFns.QuickAlign FootIKCnsTarget FootIKController
				if FlipControllers == true do (CommonFns.FlipZAxis FootIKController) --this one is the wrong way round for some reason!
				FootIKCnsTarget.parent = FootIKController
				
				append allcreated FootIKCnsTarget
				append allcreated FootIKController
				append allcontrols FootIKController
				
				CommonFns.ApplycontrollerTypeProperties FootIKController ControlType:"Medium" ControlLevel:3

				
				local FootIKContNull = Point Size:Tinysize Pos:[0,0,0] Name:(uniquename (nullifiername + "lowLegIKCont") numdigits:3) Box:off Cross:on axistripod:off centermarker:off color:(color 60 180 60)
				if FlipControllers == true do (CommonFns.FlipZAxis FootIKContNull) --this one is also the wrong way round for some reason
				CommonFns.QuickAlign FootIKContNull FootIKController
				FootIKController.parent = FootIKContNull
				FootIKContNull.parent = Root
				
				append allcreated FootIKContNull

				
				
				local ToeIKChain = IKSys.ikChain ToeIKBone ToeIKNub "IKHISolver"
				ToeIKChain.name = UniqueName ToeIKSolverName NumDigits:3
				ToeIKChain.transform.controller.goalSize = MediumSize
				CommonFns.QuickAlign ToeIKChain ToeIKBone Rotonly:True
				ToeIKChain.transform.controller.SAParent = 0
				local ToeIKSwivel = Point Size:Tinysize Pos:[0,0,0] Name:(uniquename (nullifiername + "toeIKSwivel") numdigits:3) Box:off Cross:on axistripod:off centermarker:off color:(color 60 180 60)
				CommonFns.QuickAlign ToeIKSwivel ToeIKChain
				in coordsys local Move ToeIKSwivel [0,(AverageScale*0.2),0]
				ToeIKChain.transform.controller.VHTarget = ToeIKSwivel
				
				append allcreated ToeIKChain
				append allcreated ToeIKSwivel
				
				
				local FootIKChain = IKSys.ikChain FootIKBone FootIKNub "IKHISolver"
				FootIKChain.name = UniqueName FootIKSolverName NumDigits:3
				FootIKChain.transform.controller.goalSize = MediumSize
				CommonFns.QuickAlign FootIKChain FootIKBone Rotonly:True
				FootIKChain.transform.controller.SAParent = 0
				local FootIKSwivel = Point Size:Tinysize Pos:[0,0,0] Name:(uniquename (nullifiername + "toeIKSwivel") numdigits:3) Box:off Cross:on axistripod:off centermarker:off color:(color 60 180 60)
				CommonFns.QuickAlign FootIKSwivel FootIKChain
				in coordsys local Move FootIKSwivel [0,(AverageScale*0.2),0]
				FootIKChain.transform.controller.VHTarget = FootIKSwivel
				
				append allcreated FootIKChain
				append allcreated FootIKSwivel
				
			)
			
			if kinematics == 1 do --if there is only ik
			(
				local LowLegNub = boneSys.createBone ForwardFootLoc.pos ForwardFootLoc.pos [0,0,1]
				CommonFns.QuickAlign LowLegNub ForwardFootLoc
				LowLegNub.Name = uniquename (NubName + "low_leg") Numdigits:3
				LowLegNub.parent = LowLegBone
				append allcreated LowLegNub
				append allbones LowLegNub
				lowlegnub.parent = LowLegBone
				
				local LegIKChain = IKSys.ikChain UpLegBone LowLegNub "IKHISolver"
				LegIKChain.name = UniqueName LegIKSolverName NumDigits:3
				LegIKChain.transform.controller.VHTarget = KneeController
				LegIKChain.transform.controller.goalSize = MediumSize
				LegIKChain.parent = FootIKController
				
				append allcreated LegIKChain

				
				FootBone.rotation.controller = orientation_constraint()
				FootBone.rotation.controller.appendtarget FootIKBone 100
				ToeBone.rotation.controller = orientation_constraint()
				ToeBone.rotation.controller.appendtarget ToeIKBone 100
				
				FootIKBone.parent = LowLegBone
			)
			
			
			if Kinematics == 3 do --if there is IK/FK Blend
			
			(
				local UpLegFKBone = boneSys.createBone UpperLegLoc.pos LowerLegLoc.pos [0,0,1]
				CommonFns.QuickAlign UpLegFKBone UpperLegLoc
				UpLegFKBone.Name = uniquename (UpLegFKBoneName) Numdigits:3
				append allcreated UpLegFKBone
				append allbones UpLegFKBone
				UpLegFKBone.parent = UpperLegParent
				
				local LowLegFKBone = boneSys.createBone LowerLegLoc.pos ForwardFootLoc.pos [0,0,1]
				CommonFns.QuickAlign LowLegFKBone LowerLegLoc
				LowLegFKBone.Name = uniquename (LowLegFKBoneName) Numdigits:3
				LowLegFKBone.parent = UpLegFKBone
				append allbones LowLegFKBone
				append allcreated LowLegFKBone
				
				local FKFootBone = boneSys.createBone ForwardFootLoc.pos ForwardToeLoc.pos [0,0,1]
				CommonFns.QuickAlign FKFootBone ForwardFootLoc
				FKFootBone.Name = uniquename (FKFootBoneName) Numdigits:3
				FKFootBone.parent = LowLegFKBone
				append allcreated FKFootBone
				append allbones FKFootBone
				
				local FKToeBone = boneSys.createBone ForwardToeLoc.pos ReverseToeLoc.pos [0,0,1]
				CommonFns.QuickAlign FKToeBone ForwardToeLoc
				FKToeBone.Name = uniquename (FKToeBoneName) Numdigits:3
				FKToeBone.parent = FKFootBone
				append allcreated FKToeBone
				append allbones FKToeBone
				
				local UpLegIKBone = boneSys.createBone UpperLegLoc.pos LowerLegLoc.pos [0,0,1]
				CommonFns.QuickAlign UpLegIKBone UpperLegLoc
				UpLegIKBone.Name = uniquename (UpLegFKBoneName) Numdigits:3
				append allcreated UpLegIKBone
				append allbones UpLegIKBone
				UpLegIKBone.parent = UpperLegParent
				
				local LowLegIKBone = boneSys.createBone LowerLegLoc.pos ForwardFootLoc.pos [0,0,1]
				CommonFns.QuickAlign LowLegIKBone LowerLegLoc
				LowLegIKBone.Name = uniquename (LowLegIKBoneName) Numdigits:3
				LowLegIKBone.parent = UpLegIKBone
				append allbones LowLegIKBone
				append allcreated LowLegIKBone
				
				local LowLegIKNub = boneSys.createBone ForwardFootLoc.pos ForwardFootLoc.pos [0,0,1]
				CommonFns.QuickAlign LowLegIKNub ForwardFootLoc
				LowLegIKNub.Name = uniquename (NubName + "low_legIK") Numdigits:3
				LowLegIKNub.parent = LowLegBone
				append allcreated LowLegIKNub
				append allbones LowLegIKNub
				LowLegIKNub.parent = LowLegIKBone
				
				local LegIKChain = IKSys.ikChain UpLegIKBone LowLegIKNub "IKHISolver"
				LegIKChain.name = UniqueName LegIKSolverName NumDigits:3
				LegIKChain.transform.controller.VHTarget = KneeController
				LegIKChain.transform.controller.goalSize = MediumSize
				LegIKChain.parent = FootIKController
				
				FootIKBone.parent = LowLegIKBone
				
				append allcreated LegIKChain
			)
			
			if Kinematics == 2 or kinematics == 3 do --if there is FK
			(
				local TempToeEndPoint = point Name:"ZZZZZTEMPTOEEND" Pos:ReverseToeLoc.pos
				local LowLegFKController = CommonFns.DrawRectangleControl LowLegBone LowLegBone FootBone KneeWidth KneeFrontWidth (uniquename LowLegFKControlName numdigits:3)
				CommonFns.ApplycontrollerTypeProperties LowLegFKController ControlType:"Medium" ControlLevel:2
				
				local UpLegFKController = CommonFns.DrawRectangleControl UpLegBone UpLegBone LowLegBone UpLegWidth UpLegFrontWidth (uniquename UpLegFKControlName numdigits:3)
				CommonFns.ApplycontrollerTypeProperties UpLegFKController ControlType:"Medium" ControlLevel:1

				local FootFKController = CommonFns.DrawRectangleControl FootBone FootBone ToeBone LowLegWidth LowLegFrontWidth (uniquename FootFKControlName numdigits:3)
				CommonFns.ApplycontrollerTypeProperties FootFKController ControlType:"Medium" ControlLevel:3
				
				local ToeFKController = CommonFns.DrawRectangleControl ToeBone ToeBone TempToeEndPoint FootWidth FootFrontWidth (uniquename ToeFKControlName numdigits:3)
				CommonFns.ApplycontrollerTypeProperties ToeFKController ControlType:"Medium" ControlLevel:1
				
				local LowLegFKCnsTarget =  Point Name:(UniqueName (CnsTargetName + "_LowLegFKControl") numdigits:3) Size:Tinysize wireColor: (color 20 80 20)
				CommonFns.QuickAlign LowLegFKCnsTarget LowLegFKController
				local UpLegFKCnsTarget =  Point Name:(UniqueName (CnsTargetName + "_UpLegFKControl") numdigits:3) Size:Tinysize wireColor: (color 20 80 20)
				CommonFns.QuickAlign UpLegFKCnsTarget UpLegFKController
				local FootFKCnsTarget =  Point Name:(UniqueName (CnsTargetName + "_FootFKControl") numdigits:3) Size:Tinysize wireColor: (color 20 80 20)
				CommonFns.QuickAlign FootFKCnsTarget FootFKController
				local  ToeFKCnsTarget =  Point Name:(UniqueName (CnsTargetName + "_ToeFKControl") numdigits:3) Size:Tinysize wireColor: (color 20 80 20)
				CommonFns.QuickAlign ToeFKCnsTarget ToeFKController
				
				append allcreated LowLegFKCnsTarget
				append allcreated UpLegFKCnsTarget
				append allcreated FootFKCnsTarget
				append allcreated ToeFKCnsTarget
								
				if FlipControllers == true do 
				(
					CommonFns.FlipZAxis LowLegFKController
					CommonFns.FlipZAxis UpLegFKController
					CommonFns.FlipZAxis FootFKController
					CommonFns.FlipZAxis ToeFKController
				)
				
				LowLegFKCnsTarget.parent = LowLegFKController
				UpLegFKCnsTarget.parent = UpLegFKController
				FootFKCnsTarget.parent = FootFKController
				ToeFKCnsTarget.parent = ToeFKController
				
				
				
				delete TempToeEndPoint
				
				append allcreated LowLegFKController
				append allcreated UpLegFKController
				append allcreated FootFKController
				append allcreated ToeFKController
				append allcontrols LowLegFKController
				append allcontrols UpLegFKController
				append allcontrols FootFKController
				append allcontrols ToeFKController
					
				local LowLegFKContNull = Point Size:Tinysize Pos:[0,0,0] Name:(uniquename (nullifiername + "lowLegFKCont") numdigits:3) Box:off Cross:on axistripod:off centermarker:off color:(color 60 180 60)
				local UpLegFKContNull = Point Size:Tinysize Pos:[0,0,0] Name:(uniquename (nullifiername + "upLegFKCont") numdigits:3) Box:off Cross:on axistripod:off centermarker:off color:(color 60 180 60)
				local FootFKContNull = Point Size:Tinysize Pos:[0,0,0] Name:(uniquename (nullifiername + "footFKCont") numdigits:3) Box:off Cross:on axistripod:off centermarker:off color:(color 60 180 60)
				local ToeFKContNull	 = Point Size:Tinysize Pos:[0,0,0] Name:(uniquename (nullifiername + "toeFKCont") numdigits:3) Box:off Cross:on axistripod:off centermarker:off color:(color 60 180 60)
				
				if FlipControllers == true do 
				(
					CommonFns.FlipZAxis LowLegFKContNull
					CommonFns.FlipZAxis UpLegFKContNull
					CommonFns.FlipZAxis FootFKContNull
					CommonFns.FlipZAxis ToeFKContNull
				)
				
				
				
				CommonFns.quickalign LowLegFKContNull LowLegFKController
				CommonFns.quickalign UpLegFKContNull UpLegFKController
				CommonFns.quickalign FootFKContNull FootFKController
				CommonFns.quickalign ToeFKContNull ToeFKController
				UpLegFkContNull.parent = UpperLegParent
				UpLegFKController.parent = UpLegFKContNull
				LowLegFKController.parent = LowLegFKContNull
				FootFKController.parent = FootFKContNull
				ToeFKController.parent = ToeFKContNull
				
				
				append allcreated LowLegFKContNull
				append allcreated UpLegFKContNull
				append allcreated FootFKContNull
				append allcreated ToeFKContNull
				
				
			)
			
			if Kinematics == 2 do --if there is only FK
			(
				LowLegFKContNull.parent = UpLegBone
				FootFKContNull.parent = LowLegBone
				ToeFKContNull.parent = FootBone
				LowLegBone.rotation.controller = orientation_constraint()
				LowLegBone.rotation.controller.appendtarget LowLegFKCnstarget 100
				UpLegBone.rotation.controller = orientation_constraint()
				UpLegBone.rotation.controller.appendtarget UpLegFKCnstarget 100
				FootBone.rotation.controller = orientation_constraint()
				FootBone.rotation.controller.appendtarget FootFKCnstarget 100
				ToeBone.rotation.controller = orientation_constraint()
				ToeBone.rotation.controller.appendtarget ToeFKCnstarget 100
			)
			
			
			if Kinematics == 3 or Stretchy == True or Bendy == True do
			(
				local SwitchController = text size:Mediumsize kerning:0 leading:0 pos:[0,0,0] isSelected:on text:"IK/FK" name:SwitchControllerName
				CommonFns.QuickAlign SwitchController FootControlLoc
				CommonFns.ApplycontrollerTypeProperties SwitchController ControlType:"Utility" ControlLevel:3
				
				in coordsys world SwitchController.pos = LowLegBone.transform.pos
				in CoordSys Local Move SwitchController [-AverageScale*0.7,0,0]
				SwitchController.parent = UpLegBone
					
				local ca
				local AttHolder
						
				ca=attributes LegBlendControls
				(
					parameters params rollout:ro_LegBlendControls
					(
							paramIKFKblend type:#float ui:spn_Ikfkblend
							paramstretchy type:#float ui:spn_stretchy
							parambendy type:#float ui:spn_bendy
							paramUpLegFKCont type:#string ui:edt_FKUpLeg
							paramLowLegFKCont type:#string ui:edt_FKLowLeg
							paramFootFKCont type:#string ui:edt_FKFoot
							paramToeFKCont type:#string ui:edt_FKToe
							paramFootIKCont type:#string ui:edt_IKFoot
							paramKneeCont type:#string ui:edt_Knee
							paramsquash type:#float ui:spn_squash
					)
					rollout ro_LegBlendControls "LegBlend_Controls" 
					(
						spinner spn_ikfkblend "IK/FK Blend" enabled:on  range:[0,100,0] scale:1
						spinner spn_stretchy "Stretchy" enabled:on  range:[0,1,0] scale:1
						spinner spn_bendy "Bendy" enabled:on  range:[0,100,0] scale:1	
						spinner spn_squash "Squash Enabled" enabled:on  range:[0,3,0] scale:0.1	
						button btn_showIK "Hide/Show IK" enabled:true visible:on
						button btn_showFK "Hide/Show FK" enabled:true visible:on
						edittext edt_Divide enabled:On text:"J" visible:off
						button btn_IKtoFK "IK to FK" enabled:true visible:off
						button btn_FKtoIK "FK to IK" enabled:true visible:off
						edittext edt_FKUpLeg enabled:On text:"J" visible:off
						edittext edt_FKLowLeg enabled:On text:"J" visible:off
						edittext edt_FKFoot enabled:On text:"J" visible:off
						edittext edt_IKFoot enabled:On text:"J" visible:off
						edittext edt_Knee enabled:On text:"J" visible:off
						edittext edt_FKToe enabled:On text:"J" visible:off
						
						
						local IKFootControl,FKFootControl,KneeControl,FKUpLegControl,FKLowLegControl,IKUpLegBone,IKLowLegBone,LegSideAlignment
						
							on btn_IKtoFK pressed do 
							(
								IKFootControl = GetUserProp $ "RigTools_IKFootControl" 
								FKFootControl = GetUserProp $ "RigTools_FKFootControl"
								KneeControl = GetUserProp $ "RigTools_KneeControl"
								FKUpLegControl = GetUserProp $ "RigTools_FKUpLegControl"
								FKLowLegControl = GetUserProp $ "RigTools_FKLowLegControl"
								IKUpLegBone = GetUserProp $ "RigTools_IKUpLegBone"
								IKLowLegBone = GetUserProp $ "RigTools_IKLowLegBone"
								LegSideAlignment = GetUserProp $ "RigTools_SideAlignment"
								
								--if classof AlignTo != Matrix3 do (AlignTo = AlignTo.transform)
								--local OrigPos = ObjToAlign.transform.pos
								--ObjToAlign.rotation *= ObjToAlign.transform.rotation * (inverse AlignTo.rotation)
								--if RotOnly == true then (ObjToAlign.pos = OrigPos) else (in coordsys world Objtoalign.pos = Alignto.pos)
								
								
							)
							
							
							on btn_FKtoIK pressed do 
							(
								IKFootControl = GetUserProp $ "RigTools_IKFootControl" 
								FKFootControl = GetUserProp $ "RigTools_FKFootControl"
								KneeControl = GetUserProp $ "RigTools_KneeControl"
								FKUpLegControl = GetUserProp $ "RigTools_FKUpLegControl"
								FKLowLegControl = GetUserProp $ "RigTools_FKLowLegControl"
								IKUpLegBone = GetUserProp $ "RigTools_IKUpLegBone"
								IKLowLegBone = GetUserProp $ "RigTools_IKLowLegBone"
								LegSideAlignment = GetUserProp $ "RigTools_SideAlignment"
							)
						
						
							on btn_showFK pressed do 
							(
								try
								(
									local UpLegFkControl = (getnodebyname  ($.modifiers[#LegBlend_Controls].LegBlendControls.paramUpLegFKCont))
									if UpLegFKControl.ishidden == false then (UpLegFKControl.ishidden = true) else (UpLegFKControl.ishidden = false) 
									local LowLegFkControl = (getnodebyname  ($.modifiers[#LegBlend_Controls].LegBlendControls.paramLowLegFKCont))
									if LowLegFkControl.ishidden == false then (LowLegFkControl.ishidden = true) else (LowLegFkControl.ishidden = false) 
									local FootFkControl = (getnodebyname  ($.modifiers[#LegBlend_Controls].LegBlendControls.paramFootFKCont))
									if FootFkControl.ishidden == false then (FootFkControl.ishidden = true) else (FootFkControl.ishidden = false) 
									
									local ToeFkControl = (getnodebyname  ($.modifiers[#LegBlend_Controls].LegBlendControls.paramToeFKCont))
									if ToeFkControl.ishidden == false then (ToeFkControl.ishidden = true) else (ToeFkControl.ishidden = false) 
								)
								
								catch()
							)
							
							on btn_showIK pressed do 
							(
								try
								(
									local FootIkControl = (getnodebyname  ($.modifiers[#LegBlend_Controls].LegBlendControls.paramFootIKCont))
									if FootIkControl.ishidden == false then (FootIkControl.ishidden = true) else (FootIkControl.ishidden = false) 
									local KneeControl = (getnodebyname  ($.modifiers[#LegBlend_Controls].LegBlendControls.paramKneeCont))
									if KneeControl.ishidden == false then (KneeControl.ishidden = true) else (KneeControl.ishidden = false) 
								)
								catch()
							)
					)
				)
				
					
				AttHolder = (EmptyModifier ())
				AttHolder.name = "LegBlend_Controls"
				addmodifier SwitchController AttHolder
				custAttributes.add SwitchController.modifiers[#LegBlend_Controls] ca
				try(SetUserProp SwitchController "RigTools_IKFootControl" FootIKController.name) catch()
				try(SetUserProp SwitchController "RigTools_FKFootControl" FootFKController.name) catch()
				try(SetUserProp SwitchController "RigTools_KneeControl" KneeController.name) catch()
				try(SetUserProp SwitchController "RigTools_FKUpLegControl" UpLegFKController.name) catch()
				try(SetUserProp SwitchController "RigTools_FKLowLegControl" LowLegFKController.name) catch()
				try(SetUserProp SwitchController "RigTools_IKUpLegBone" UpLegIKBone.name	) catch()
				try(SetUserProp SwitchController "RigTools_IKLowLegBone" LowLegIKBone.name) catch()
				try(SetUserProp SwitchController "RigTools_SideAlignment" SideAlignment) catch()
					

				append allcreated SwitchController
				append allcontrols SwitchController

			)
			
			
			
			if Kinematics == 3 do --if there is IK/FK Blend
			(
				LowLegFKContNull.parent = UpLegFKBone
				FootFKContNull.parent = LowLegFKBone
				ToeFKContNull.parent = FKFootBone
				UpLegFKBone.rotation.controller = Orientation_constraint()
				UpLegFKBone.rotation.controller.appendtarget UpLegFKCnstarget 100
				LowLegFKBone.rotation.controller = Orientation_constraint()
				LowLegFKBone.rotation.controller.appendtarget LowLegFKCnstarget 100
				FKFootBone.rotation.controller = Orientation_constraint()
				FKFootBone.rotation.controller.appendtarget FootFKCnstarget 100
				FKToeBone.rotation.controller = Orientation_constraint()
				FKToeBone.rotation.controller.appendtarget ToeFKCnstarget 100
				
				LowLegBone.rotation.controller = orientation_constraint()
				LowLegBone.rotation.controller.appendtarget LowLegFKBone 100
				LowLegBone.rotation.controller.appendtarget LowLegIKBone 100
				
				UpLegBone.rotation.controller = orientation_constraint()
				UpLegBone.rotation.controller.appendtarget UpLegFKBone 100
				UpLegBone.rotation.controller.appendtarget UpLegIKBone 100
				
				FootBone.rotation.controller = orientation_constraint()
				FootBone.rotation.controller.appendtarget FKFootBone 100
				FootBone.rotation.controller.appendtarget FootIKBone  100
				
				ToeBone.rotation.controller = orientation_constraint()
				ToeBone.rotation.controller.appendtarget FKToeBone 100
				ToeBone.rotation.controller.appendtarget ToeIKBone  100
				


				local ExpressionToAdd = Float_Expression()
				ExpressionToAdd.AddScalarTarget "SliderValue" SwitchController.modifiers[#LegBlend_Controls].LegBlendControls[#paramIKFKBlend]
				ExpressionToAdd.setexpression "SliderValue"
					
				UpLegBone.rotation.controller.weight[1].controller = ExpressionToAdd
				LowLegBone.rotation.controller.weight[1].controller = ExpressionToAdd
				FootBone.rotation.controller.weight[1].controller = ExpressionToAdd
				ToeBone.rotation.controller.weight[1].controller = ExpressionToAdd
				ExpressionToAdd = copy ExpressionToAdd
				ExpressionToAdd.setexpression "(100-(SliderValue))"
				UpLegBone.rotation.controller.weight[2].controller = ExpressionToAdd
				LowLegBone.rotation.controller.weight[2].controller = ExpressionToAdd
				FootBone.rotation.controller.weight[2].controller = ExpressionToAdd
				ToeBone.rotation.controller.weight[2].controller = ExpressionToAdd
				
				SwitchController.modifiers[#LegBlend_Controls].LegBlendControls.paramUpLegFKCont = UpLegFKController.name
				SwitchController.modifiers[#LegBlend_Controls].LegBlendControls.paramLowLegFKCont = LowLegFKController.name
				SwitchController.modifiers[#LegBlend_Controls].LegBlendControls.paramFootFKCont = FootFKController.name
				SwitchController.modifiers[#LegBlend_Controls].LegBlendControls.paramToeFKCont = ToeFKController.name
				SwitchController.modifiers[#LegBlend_Controls].LegBlendControls.paramFootIKCont = FootIKController.name
				SwitchController.modifiers[#LegBlend_Controls].LegBlendControls.paramKneeCont = KneeController.name
				
				--$.modifiers[#LegBlend_Controls].ro_LegBlendControls.btn_showIK.visible = on
				--$.modifiers[#LegBlend_Controls].ro_LegBlendControls.btn_showFK.visible = on	
				
			)
			
			---------------------------Next: Create the foot rig-------------------------------------------
			
			if FootRig != false and Kinematics !=2 do
			(
			--Foot Rig Locators: Heel, Foot Tip, Foot Inner, Foot Outer
			local HeelPoint =  Point Size:Tinysize Pos:[0,0,0] Name:(uniquename (nullifiername + "HeelRoll") numdigits:3) Box:off Cross:on axistripod:off centermarker:off color:(color 60 180 60)
			local FootTipPoint = Point Size:Tinysize Pos:[0,0,0] Name:(uniquename (nullifiername + "ToeRoll") numdigits:3) Box:off Cross:on axistripod:off centermarker:off color:(color 60 180 60)
			local FootInnerPoint = Point Size:Tinysize Pos:[0,0,0] Name:(uniquename (nullifiername + "FootBankInner") numdigits:3) Box:off Cross:on axistripod:off centermarker:off color:(color 60 180 60)
			local FootOuterPoint = Point Size:Tinysize Pos:[0,0,0] Name:(uniquename (nullifiername + "FootBankOuter") numdigits:3) Box:off Cross:on axistripod:off centermarker:off color:(color 60 180 60)
			local FootRollPoint = Point Size:Tinysize Pos:[0,0,0] Name:(uniquename (nullifiername + "FootRoll") numdigits:3) Box:off Cross:on axistripod:off centermarker:off color:(color 60 180 60)
			local ToeFlapPoint = Point Size:Tinysize Pos:[0,0,0] Name:(uniquename (nullifiername + "ToeFlap") numdigits:3) Box:off Cross:on axistripod:off centermarker:off color:(color 60 180 60)
				
			append allcreated HeelPoint
			append allcreated FootTipPoint
			append allcreated FootInnerPoint
			append allcreated FootOuterPoint
			append allcreated FootRollPoint
			append allcreated ToeFlapPoint	

				
			CommonFns.QuickAlign FootRollPoint FootIKController	
			CommonFns.QuickAlign HeelPoint FootIKController
			CommonFns.QuickAlign FootTipPoint FootIKController
			CommonFns.QuickAlign FootInnerPoint FootIKController
			CommonFns.QuickAlign FootOuterPoint FootIKController
			CommonFns.QuickAlign ToeFlapPoint FootIKController	
			in coordsys world FootRollPoint.pos = ToeBone.transform.pos
			in coordsys world HeelPoint.pos = FootRigPoints[1].pos
			in coordsys world FootTipPoint.pos = FootRigPoints[2].pos	
			in coordsys world FootInnerPoint.pos = FootRigPoints[3].pos
			in coordsys world FootOuterPoint.pos = FootRigPoints[4].pos
			in coordsys world ToeFlapPoint.pos = ToeBone.transform.pos	
			
			ToeFlapPoint.parent = FootTipPoint
			LegIKChain.parent = FootRollPoint
			FootRollPoint.parent = FootTipPoint
			FootTipPoint.parent = FootInnerPoint
			FootInnerPoint.parent = FootOuterPoint
			FootOuterPoint.parent = HeelPoint
			HeelPoint.parent = FootIKController
			

			
			local HeelAutoPoint =  Point Size:Tinysize Pos:[0,0,0] Name:(uniquename (nullifiername + "HeelRollAuto") numdigits:3) Box:off Cross:on axistripod:off centermarker:off color:(color 60 180 60)
			local FootTipAutoPoint = Point Size:Tinysize Pos:[0,0,0] Name:(uniquename (nullifiername + "ToeRollAuto") numdigits:3) Box:off Cross:on axistripod:off centermarker:off color:(color 60 180 60)
			local FootInnerAutoPoint = Point Size:Tinysize Pos:[0,0,0] Name:(uniquename (nullifiername + "FootBankInnerAuto") numdigits:3) Box:off Cross:on axistripod:off centermarker:off color:(color 60 180 60)
			local FootOuterAutoPoint = Point Size:Tinysize Pos:[0,0,0] Name:(uniquename (nullifiername + "FootBankOuterAuto") numdigits:3) Box:off Cross:on axistripod:off centermarker:off color:(color 60 180 60)
			local FootRollAutoPoint = Point Size:Tinysize Pos:[0,0,0] Name:(uniquename (nullifiername + "FootRollAuto") numdigits:3) Box:off Cross:on axistripod:off centermarker:off color:(color 60 180 60)
			local ToeFlapAutoPoint = Point Size:Tinysize Pos:[0,0,0] Name:(uniquename (nullifiername + "ToeFlapAuto") numdigits:3) Box:off Cross:on axistripod:off centermarker:off color:(color 60 180 60)
			CommonFns.QuickAlign FootRollAutoPoint FootRollPoint	
			CommonFns.QuickAlign HeelAutoPoint HeelPoint
			CommonFns.QuickAlign FootTipAutoPoint FootTipPoint
			CommonFns.QuickAlign FootInnerAutoPoint FootInnerPoint
			CommonFns.QuickAlign FootOuterAutoPoint FootOuterPoint
			CommonFns.QuickAlign ToeFlapAutoPoint ToeFlapPoint	
			FootRollAutoPoint.parent = FootRollPoint	
			HeelAutoPoint.parent = HeelPoint
			FootTipAutoPoint.parent = FootTipPoint
			FootInnerAutoPoint.parent = FootInnerPoint
			FootOuterAutoPoint.parent = FootOuterPoint
			ToeFlapAutoPoint.parent = ToeFlapPoint	
			
			
			append allcreated HeelAutoPoint
			append allcreated FootTipAutoPoint
			append allcreated FootInnerAutoPoint
			append allcreated FootOuterAutoPoint
			append allcreated FootRollAutoPoint
			append allcreated ToeFlapAutoPoint	
			
			
			local HeelController = CommonFns.MakeController MediumSize "sphere" HeelAutoPoint HeelAutoPoint (color 60 60 160)  HeelControllerName  "y" "negative"
			local ToeRollController =  CommonFns.MakeController MediumSize "sphere" FootTipAutoPoint FootTipAutoPoint (color 60 160 60)  ToeRollControllerName  "y" "negative"
			local FootInnerController =  CommonFns.MakeController MediumSize "sphere" FootInnerAutoPoint FootInnerAutoPoint (color 60 160 60)  FootInnerControllerName  "y" "negative"
			local FootOutercontroller = CommonFns.MakeController MediumSize "sphere" FootOuterAutoPoint FootOuterAutoPoint (color 60 160 60)  FootOutercontrollerName  "y" "negative"
			local FootRollController = CommonFns.MakeController MediumSize "triangle" FootRollAutoPoint FootRollAutoPoint (color 60 160 60)  FootRollControllerName  "y" "negative"
			local ToeFlapController =  CommonFns.MakeController MediumSize "sphere" ToeFlapAutoPoint ToeFlapAutoPoint (color 60 160 60) ToeFlapControllerName  "y" "negative"
			
			CommonFns.ApplycontrollerTypeProperties HeelController ControlType:"Small" ControlLevel:3
			CommonFns.ApplycontrollerTypeProperties ToeRollController ControlType:"Small" ControlLevel:3
			CommonFns.ApplycontrollerTypeProperties FootInnerController ControlType:"Small" ControlLevel:3
			CommonFns.ApplycontrollerTypeProperties FootOutercontroller ControlType:"Small" ControlLevel:3
			CommonFns.ApplycontrollerTypeProperties FootRollController ControlType:"Small" ControlLevel:1
			CommonFns.ApplycontrollerTypeProperties ToeFlapController ControlType:"Small" ControlLevel:2

			
			HeelController.parent = HeelAutoPoint
			ToeRollController.parent = FootTipAutoPoint
			FootInnerController.parent = FootInnerAutoPoint
			FootOutercontroller.parent = FootOuterAutoPoint
			FootRollController.parent = FootRollAutoPoint
			ToeFlapController.parent = ToeFlapAutoPoint
			
			ToeFlapPoint.parent = ToeRollController

			FootRollPoint.parent = ToeRollController
			FootTipPoint.parent = FootInnerController
			FootInnerPoint.parent = FootOutercontroller
			FootOuterPoint.parent = HeelController
			HeelPoint.parent = FootIKController
			
			
			LegIKChain.parent = FootRollController
			FootIKChain.parent = FootRollController
			FootIKSwivel.parent = FootRollController
			ToeIKChain.parent = ToeFlapController
			ToeIKSwivel.parent = ToeFlapController
			
			append allcreated HeelController
			append allcreated ToeRollController
			append allcreated FootInnerController
			append allcreated FootOuterController
			append allcreated FootRollController
			append allcreated ToeFlapController
			append allcontrols HeelController
			append allcontrols ToeRollController
			append allcontrols FootInnerController
			append allcontrols FootOuterController
			append allcontrols FootRollController
			append allcontrols ToeFlapController
			
			local ca
			local AttHolder
					
			ca=attributes FootRigControls
			(
				parameters params rollout:ro_FootRigControls
				(
						paramHeelRoll type:#float ui:spn_HeelRoll
						paramHeelPivot type:#float ui:spn_HeelPivot
						paramToeRoll type:#float ui:spn_ToeRoll
						paramToePivot type:#float ui:spn_ToePivot
					
						paramInnerBank type:#float ui:spn_InnerBank
						paramOuterBank type:#float ui:spn_OuterBank
						paramFootRoll type:#float ui:spn_FootRoll
					
						paramToeFlap type:#float ui:spn_ToeFlap
				)
				rollout ro_FootRigControls "FootRig_Controls" 
				(
					spinner spn_HeelRoll "Heel Roll" enabled:on  range:[-2,2,0] scale:0.05
					spinner spn_HeelPivot "Heel Pivot" enabled:on  range:[-2,2,0] scale:0.05	
					spinner spn_ToeRoll "Toe Roll" enabled:on  range:[-2,2,0] scale:0.05
					spinner spn_ToePivot "Toe Pivot" enabled:on  range:[-2,2,0] scale:0.05
					spinner spn_InnerBank "Inner Bank" enabled:on  range:[-2,2,0] scale:0.05
					spinner spn_OuterBank "Outer Bank" enabled:on  range:[-2,2,0] scale:0.05
					spinner spn_ToeFlap "Toe Flap" enabled:on  range:[-2,2,0] scale:0.05
					spinner spn_FootRoll "Foot Roll" enabled:on  range:[-2,2,0] scale:0.05	
				)
			)
				
			AttHolder = (EmptyModifier ())
			AttHolder.name = "FootRig_Controls"
			addmodifier FootIKController AttHolder
			custAttributes.add FootIKController.modifiers[#FootRig_Controls] ca
			--switchcontroller.modifiers[#ArmBlend_Controls].ro_ArmBlendControls.spn_fingerlength.value = 1 - SET A SLIDER DEFAULT VALUE
			--RotExpressionToAdd.AddScalarTarget "SliderValue" switchcontroller.modifiers[#ArmBlend_Controls].ArmBlendControls[#paramIKFKBlend]
				
				
			HeelAutoPoint.rotation.controller.Z_Rotation.controller = Float_Expression ()
			HeelAutoPoint.rotation.controller.Z_Rotation.controller.AddScalarTarget "HeelRoll" FootIKController.modifiers[#FootRig_Controls].FootRigControls[#paramHeelRoll]
			HeelAutoPoint.rotation.controller.Z_Rotation.controller.setexpression "HeelRoll"
			HeelAutoPoint.rotation.controller.Y_Rotation.controller = Float_Expression ()
			HeelAutoPoint.rotation.controller.Y_Rotation.controller.AddScalarTarget "HeelPivot" FootIKController.modifiers[#FootRig_Controls].FootRigControls[#paramHeelPivot]
			HeelAutoPoint.rotation.controller.Y_Rotation.controller.setexpression "HeelPivot"
			FootInnerAutoPoint.rotation.controller.X_Rotation.controller = Float_Expression ()
			FootInnerAutoPoint.rotation.controller.X_Rotation.controller.AddScalarTarget "InnerBank" FootIKController.modifiers[#FootRig_Controls].FootRigControls[#paramInnerBank]
			FootInnerAutoPoint.rotation.controller.X_Rotation.controller.setexpression "InnerBank"
			FootOuterAutoPoint.rotation.controller.X_Rotation.controller = Float_Expression ()
			FootOuterAutoPoint.rotation.controller.X_Rotation.controller.AddScalarTarget "OuterBank" FootIKController.modifiers[#FootRig_Controls].FootRigControls[#paramOuterBank]
			FootOuterAutoPoint.rotation.controller.X_Rotation.controller.setexpression "OuterBank"	
			FootTipAutoPoint.rotation.controller.Z_Rotation.controller = Float_Expression ()
			FootTipAutoPoint.rotation.controller.Z_Rotation.controller.AddScalarTarget "ToeRoll" FootIKController.modifiers[#FootRig_Controls].FootRigControls[#paramToeRoll]
			FootTipAutoPoint.rotation.controller.Z_Rotation.controller.setexpression "ToeRoll"		
			FootTipAutoPoint.rotation.controller.Y_Rotation.controller = Float_Expression ()
			FootTipAutoPoint.rotation.controller.Y_Rotation.controller.AddScalarTarget "ToePivot" FootIKController.modifiers[#FootRig_Controls].FootRigControls[#paramToePivot]
			FootTipAutoPoint.rotation.controller.Y_Rotation.controller.setexpression "ToePivot"
			ToeFlapAutoPoint.rotation.controller.Z_Rotation.controller = Float_Expression ()
			ToeFlapAutoPoint.rotation.controller.Z_Rotation.controller.AddScalarTarget "ToeFlap" FootIKController.modifiers[#FootRig_Controls].FootRigControls[#paramToeFlap]
			ToeFlapAutoPoint.rotation.controller.Z_Rotation.controller.setexpression "ToeFlap"
			FootRollAutoPoint.rotation.controller.Z_Rotation.controller = Float_Expression ()
			FootRollAutoPoint.rotation.controller.Z_Rotation.controller.AddScalarTarget "FootRoll" FootIKController.modifiers[#FootRig_Controls].FootRigControls[#paramFootRoll]
			FootRollAutoPoint.rotation.controller.Z_Rotation.controller.setexpression "FootRoll"	
			)
			
			if Kinematics == 1 or Kinematics == 3 do
			(
			if FootRig == false do
			(
			LegIKChain.parent = FootIKController
			FootIKChain.parent = FootIKController
			ToeIKChain.parent = FootIKController	
			)
			)
			
			---------------------Stretchy expression-----------------------
				if Stretchy == true and Kinematics != 2 do
			(
				local DefaultLowLegLength = ((Distance FootBone LowLegBone) as string)
				local DefaultUpLegLength = ((Distance LowLegBone UpLegBone) as string)
				local DefaultBentDistance = ((Distance UpLegBone FootBone) as string)
				local DefaultTotal = ((DefaultLowLegLength as float) + (DefaultUpLegLength as float)) as string
				local LegDistanceExpose = CommonFns.CreateLookatdistanceExpose UpLegBone LegIKChain #(MeasureHelperBaseName,MeasurEHelperBaseName) AlignTo:undefined reAssignParents:#(UpperLegParent,LegIKChain) justdistance:true

				for i = 1 to i = LegDistanceExpose.count do (append allcreated LegDistanceExpose[i])

				
				local StretchExpressionToAdd = Float_Expression()
				StretchExpressionToAdd.AddScalarTarget "Distance" LegDistanceExpose[1].modifiers[#Stretch_Expose_Controls].StretchControls[#paramdistance]
				StretchExpressionToAdd.AddScalarTarget "StretchValue" SwitchController.modifiers[#LegBlend_Controls].LegBlendControls[#paramstretchy]
				
				local FootExpression = Copy StretchExpressionToAdd
				FootExpression.setexpression ("if(Distance>" + DefaultTotal + "," + DefaultLowLegLength + "+(((Distance-" + DefaultTotal +")*0.5)*StretchValue)," + DefaultLowLegLength +")")

				local LowLegExpression = Copy StretchExpressionToAdd
				LowLegExpression.setexpression ("if(Distance>" + DefaultTotal + "," + DefaultUpLegLength + "+(((Distance-" + DefaultTotal +")*0.5)*StretchValue)," + DefaultUpLegLength +")")
				FootBone.pos.controller.X_Position.controller = FootExpression
				LowLegBone.pos.controller.X_Position.controller = LowLegExpression
				FootIKBone.pos.controller.X_Position.controller = FootExpression
				if kinematics == 3 do
				(
					LowLegIKBone.pos.controller.X_Position.controller = LowLegExpression
				)
				
			)
			
			-------------------------------------Create the expose transform systems-------------------------------------------------------------------
			
			
			local HipExposeObjs = #()
			local AnkleExposeObjs = #()
			local HipNoTwistSystem
			local AnkleNoTwistSystem
			--
			
-- 			HipNoTwistSystem = CommonFns.CreateLookatExposePair ShoulderConnectParent UpArmBone Prefix BaseName:("Hip_" + SideAlignment) Additive:False NullifyOffset:False OffsetScalar:1.5 UpnodeAxis:1
-- 			
-- 			AnkleNoTwistSystem = CommonFns.CreateLookatExposePair LowArmBone HandBone Prefix BaseName:("Wrist_" + SideAlignment) Additive:True NullifyOffset:True OffsetScalar:0.8 UpnodeAxis:1
-- 	
-- 			
-- 			
-- 			
-- 			for n in HipNoTwistSystem[1] do (append allcreated n)
-- 			for n in AnkleNoTwistSystem[1] do (append allcreated n)
-- 			
-- 			local ShoulderLocalNull = ShoulderNoTwistNull[1][4]
-- 			ShoulderNoTwistNull = ShoulderNoTwistNull[1][5]

			
			--CreateLookatExposePair StartBone EndBone Prefix BaseName:"LookExpose" Additive:True NullifyOffset:False OffsetScalar:0.8 UpnodeAxis:1=
			
			HipExposeObjs = CommonFns.MakeExposeTransformPair UpLegBone UpperLegParent UpLegBone BaseNames:#(ExposeTransformBaseName+"_Hip",ExposeNodeName+"_Hip",LocalNodeName+"_Hip") AxisOrder:1
			AnkleExposeObjs = CommonFns.MakeExposeTransformPair LowLegBone LowLegBone FootBone BaseNames:#(ExposeTransformBaseName+"_Ankle",ExposeNodeName+"_Ankle",LocalNodeName+"_Ankle") AxisOrder:1
			for i = 1 to i = 3 do
			(
			in coordsys world AnkleExposeObjs.pos = FootBone.transform.pos
			)
			in coordsys world HipExposeObjs[2].pos = LowLegBone.transform.pos
			
			local HipLocalPointNull = Point Name:(uniquename (Nullifiername + "_Hiplocalpoint") Numdigits:3) Size:TinySize
			local AnkleLocalPointNull = Point Name:(uniquename (Nullifiername + "_Anklelocalpoint") Numdigits:3) Size:TinySize
			CommonFns.quickalign HipLocalPointNull HipExposeObjs[3]
			HipLocalPointNull.parent = HipExposeObjs[3].parent
			CommonFns.quickalign AnkleLocalPointNull AnkleExposeObjs[3]
			AnkleLocalPointNull.parent = AnkleExposeObjs[3].parent
			append allcreated HipLocalPointNull
			append allcreated AnkleLocalPointNull
			
			
			
			

			for i = 1 to i = 3 do
			(
				append allcreated HipExposeObjs[i]
				append allcreated AnkleExposeObjs[i]
			)
			

			-----------------------------------Create the No-Twist nulls and expose transforms--------------------------
			local AnkleNoTwistNull
			local HipNoTwistNull
			
			HipNoTwistNull = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename HipNoTwistNullName Numdigits:3) size:Tinysize wirecolor:Green
			AnkleNoTwistNull = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename AnkleNoTwistNullName Numdigits:3) size:Tinysize wirecolor:Green
			CommonFns.quickAlign HipNoTwistNull UpLegBone
			CommonFns.quickAlign AnkleNoTwistNull AnkleExposeObjs[1]
			HipNoTwistNull.parent = UpLegbone
			AnkleNoTwistNull.parent = LowLegBone
			
			
			append allcreated HipNoTwistNull
			append allcreated AnkleNoTwistNull
			
			local HipNoTwistExpression = Float_Expression()
			HipNoTwistExpression.AddScalarTarget "LocalEulerX" HipExposeObjs[1].baseobject[#local_euler_x]
			HipNoTwistExpression.setexpression "-(LocalEulerX*0.9)"
			HipNoTwistNull.rotation.controller.X_rotation.controller = HipNoTwistExpression
			
			local AnkleNoTwistExpression = Float_Expression()
			AnkleNoTwistExpression.AddScalarTarget "LocalEulerX" AnkleExposeObjs[1].baseobject[#local_euler_x]
			AnkleNoTwistExpression.setexpression "LocalEulerX"
			AnkleNoTwistNull.rotation.controller.x_rotation.controller = AnkleNoTwistExpression
			

			-----------------------------------Next: Draw the twist chains ---------------------------------
			
			if Stretchy == True or Bendy == True then
			(
				local UpLegSquashType
				local LowLegSquashType
				
				if IncludeSquash == true then 
				(
					UpLegSquashType = 4
					LowLegSquashType = 3
				) 
				else 
				(
					UpLegSquashType = 1
					LowLegSquashType = 1
				)
				
				if BendyRom == 1 then (BendyRom = true) else (BendyRom = false)
				
				local UpLegTwistChain = FinalTools.MakeIKFKSpine UpLegBone LowLegBone  0 false 1 UpLegSquashType 1 Twistbones 0.5 undefined 1 1 Prefix:ToolSetup.nameprefix SquashBlending:2 Upnodes:BendyRom
				local LowLegTwistChain = FinalTools.MakeIKFKSpine LowLegBone FootBone  0 false 1 LowLegSquashType 1 Twistbones 0.5 undefined 1 1  Prefix:ToolSetup.nameprefix SquashBlending:2 UpNodes:BendyRom
				

				for i = 1 to i = ((LowLegTwistChain[7].count)-1) do
				(
				CommonFns.ApplyLengthWidthProperties LowLegTwistChain[7][i] (LowLegFrontWidth*0.8) ObjFrontWidth:(LowLegWidth*0.8) ObjOffset:undefined
				if IncludeSquash == true do (LowLegTwistChain[7][i].scale.controller.weight[LowLegTwistChain[7][i].scale.controller.count].controller.addscalartarget  "SliderValue" SwitchController.modifiers[#LegBlend_Controls].LegBlendControls[#paramsquash])
				)	
				for i = 1 to i = ((UpLegTwistChain[7].count)-1) do
				(
				local ExtraShrink = ((i as float)/(UpLegTwistChain[7].count))
				CommonFns.ApplyLengthWidthProperties UpLegTwistChain[7][i] (UpLegFrontWidth*(0.8-(ExtraShrink*0.2))) ObjFrontWidth:(UpLegWidth*(0.8-(ExtraShrink*0.2))) ObjOffset:undefined
				if IncludeSquash == true do (UpLegTwistChain[7][i].scale.controller.weight[UpLegTwistChain[7][i].scale.controller.count].controller.addscalartarget  "SliderValue" SwitchController.modifiers[#LegBlend_Controls].LegBlendControls[#paramsquash])
				)	
				
				CommonFns.QuickAlign UpLegTwistChain[2][1] UpLegBone Rotonly:true
				UpLegTwistChain[2][1].parent = UpLegBone
				
				CommonFns.QuickAlign LowLegTwistChain[2][1] LowLegBone Rotonly:true
				LowLegTwistChain[2][1].parent = LowLegBone
				
				local LowLegTwistCnsTarget = Point Pos:[0,0,0] Name:(uniquename (NullifierName + "LowLegTwistCnsTarget") Numdigits:3) Size:TinySize Box:On AxisTripod:Off Cross:Off
				CommonFns.QuickAlign LowLegTwistCnsTarget AnkleNoTwistNull
				CommonFns.QuickAlign LowLegTwistCnsTarget LowLegTwistChain[2][3] RotOnly:true
				LowLegTwistCnsTarget.parent = AnkleNoTwistNull
				
				append allcreated LowLegTwistCnsTarget
				

				LowLegTwistChain[2][3].pos.controller = position_constraint()
				LowLegTwistChain[2][3].pos.controller.appendtarget FootBone 100	
				UpLegTwistChain[2][3].pos.controller = position_constraint()
				UpLegTwistChain[2][3].pos.controller.appendtarget LowLegBone 100	
				
				UpLegTwistChain[2][2].rotation.controller = orientation_constraint()
				UpLegTwistChain[2][2].rotation.controller.appendtarget HipNoTwistNull 100
				LowLegTwistChain[2][3].rotation.controller = orientation_constraint()
				LowLegTwistChain[2][3].rotation.controller.appendtarget LowLegTwistCnsTarget 100
			)
			
			else if twistbones>0 do -- make standard twist setup
			(
				local UpLegTwistChain = #()
				local LowLegTwistChain = #()

				
				
				local UpLegTwistLocations = CommonFns.findIncrementsBetweenTwoPoints UpLegBone LowLegBone divisions:(TwistBones-1) IncludeEnds:True
				local LowLegTwistLocations = CommonFns.findIncrementsBetweenTwoPoints LowLegBone FootBone divisions:(TwistBones-1) IncludeEnds:True
			
				for i = 1 to i = (UpLegTwistLocations.count-1) do
				(
					local BoneToAdd
					BoneToAdd = boneSys.createBone UpLegTwistLocations[i] UpLegTwistLocations[(i+1)] [0,0,1]
					BoneToAdd.sidefins = off
					BoneToAdd.frontfin = off
					BoneToAdd.backfin = off
					BoneToAdd.width = tinysize
					CommonFns.QuickAlign BoneToAdd UpLegBone RotOnly:True
					BoneToAdd.parent = UpLegBone
					BoneToAdd.Name = (uniquename (Prefix + UpLegTwistBoneName) Numdigits:3)
					CommonFns.ApplyLengthWidthProperties BoneToAdd (UpLegFrontWidth*0.8) ObjFrontWidth:(UpLegWidth*0.8) ObjOffset:undefined
					append MainBonesArray BoneToAdd
					
					if i < ((UpLegTwistLocations.count)-1) do
					(
						local CurrentPercent = ((1.0-((i-1.0)*(1.0/(TwistBones-1.0))))as string)
						BoneToAdd.rotation.controller.x_rotation.controller = float_expression()
						BoneToAdd.rotation.controller.x_rotation.controller.AddScalarTarget "LocalEulerX" HipExposeObjs[1].baseobject[#local_euler_x]
						BoneToAdd.rotation.controller.x_rotation.controller.setexpression ("-(LocalEulerX*"+CurrentPercent + ")")
					)					
				)
				
				for i = 1 to i = (LowLegTwistLocations.count-1) do
				(
					local BoneToAdd
					BoneToAdd = boneSys.createBone LowLegTwistLocations[i] LowLegTwistLocations[(i+1)] [0,0,1]
					BoneToAdd.sidefins = off
					BoneToAdd.frontfin = off
					BoneToAdd.backfin = off
					BoneToAdd.width = tinysize
					CommonFns.QuickAlign BoneToAdd LowLegBone RotOnly:True
					BoneToAdd.parent = LowLegBone
					BoneToAdd.Name = (uniquename (Prefix + LowLegTwistBoneName) Numdigits:3)
					CommonFns.ApplyLengthWidthProperties BoneToAdd (LowLegFrontWidth*0.8) ObjFrontWidth:(LowLegWidth*0.8) ObjOffset:undefined
					append MainBonesArray BoneToAdd
					
					if i > 1 do
					(
						local CurrentPercent = (((i-1.0)*(1.0/(TwistBones-1.0))) as string)
						BoneToAdd.rotation.controller.x_rotation.controller = float_expression()
						BoneToAdd.rotation.controller.x_rotation.controller.AddScalarTarget "LocalEulerX" AnkleExposeObjs[1].baseobject[#local_euler_x]
						BoneToAdd.rotation.controller.x_rotation.controller.setexpression ("(LocalEulerX*"+CurrentPercent + ")")
					)
				)
			)				
					


			--LowLegTwistChain[2][1] = Global
			--LowLegTwistChain[2][2]  = Start
			--LowLegTwistChain[2][3] = End
			
			--LowLegTwistChain[6][1] = StartTangent
			--LowLegTwistChain[6][2] = EndTangent
			
						---------------------Create the Hip 50% Null--------------------------------------------------------------
			
			local Hip50Null
			local Hip50Bone

			Hip50Bone = boneSys.createBone [0,0,0] [(distance UpLegBone LowLegBone *0.15),0,0] [0,0,1]
			Hip50Bone.Name = (uniquename (Hip50BoneName) Numdigits:3)
			commonFns.quickAlign Hip50Bone HipExposeObjs[3]
			
			Hip50Bone.wirecolor = (color 180 60 60)
			
			Hip50Null = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename (nullifiername + "Hip50") numdigits:3) size:Tinysize wirecolor:Green
			CommonFns.QuickAlign Hip50Null Hip50Bone
			Hip50Null.parent = HipLocalPointNull
			
			Hip50Null.rotation.controller = orientation_constraint()
			Hip50Null.rotation.controller.appendtarget HipNoTwistNull 50
			Hip50Null.rotation.controller.appendtarget HipLocalPointNull 50
			Hip50Bone.parent = Hip50Null
			in coordsys local rotate Hip50Bone (angleaxis 90 [0,0,1])
				append allcreated Hip50Null
				append allcreated Hip50Bone
				append allbones Hip50Bone
				append extrabones Hip50Bone

			local ButtBone
			local HipSideBone
			local HipFrontBone
			local ButtDeformController
			
			If KneeType != 1 do  --if deformation bones are not basic, create the full hip system
			(
				Hip50Null.name = (uniquename (nullifiername + "Butt50") numdigits:3)
				Hip50Bone.name = (uniquename (buttbonename) numdigits:3)
				local Hip50ParentNull = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename (nullifiername + "Hip50_Parent") numdigits:3) size:Tinysize wirecolor:Green
				commonFns.quickAlign Hip50ParentNull Hip50Null
				in coordsys local rotate Hip50ParentNull (angleaxis 90 [0,0,1])
				Hip50ParentNull.parent = Hip50Null
				
				commonFns.quickAlign Hip50Bone Hip50ParentNull	

				ButtDeformController = CommonFns.MakeController (TinySize) "circle" Hip50Bone Hip50Bone blue ButtDeformControlName "y" "positive"
				ButtDeformController.parent = Hip50Null
				Hip50ParentNull.rotation.controller = orientation_constraint()
				Hip50ParentNull.rotation.controller.appendtarget ButtDeformController 100
				ButtDeformController.objectoffsetpos = [BigSize,0,0]
				
				Hip50ParentNull.pos.controller = position_constraint()
				Hip50ParentNull.pos.controller.appendtarget ButtDeformController 100
				Hip50Bone.parent = Hip50ParentNull
				local StartDistance = (UpLegBone.length as string)
				local ButtBoneExpression = Float_Expression()
				ButtBoneExpression.AddScalarTarget "LocalPositionY" HipExposeObjs[1].baseobject[#local_position_y]
				ButtBoneExpression.setexpression ("(((max (0,LocalPositionY))/(" + StartDistance + "))*" + (TinySize as string) + ")")
				Hip50Bone.pos.controller.x_position.controller = ButtBoneExpression
				
				
				append allcontrols ButtDeformController
				append allcreated Hip50ParentNull

				HipSideBone = boneSys.createBone [0,0,0] [(distance UpLegBone LowLegBone *0.15),0,0] [0,0,1]
				HipSideBone.Name = (uniquename (HipSideBoneName) Numdigits:3)
				commonFns.quickAlign HipSideBone HipExposeObjs[3]
				HipSideBone.wirecolor = (color 180 60 60)
				local Hip50SideNull = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename (nullifiername + "Hip50_Side") numdigits:3) size:Tinysize wirecolor:Green
				local Hip50SideParentNull = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename (nullifiername + "Hip50_Side_Parent") numdigits:3) size:Tinysize wirecolor:Green

				
				
				CommonFns.QuickAlign Hip50SideParentNull Hip50Null	
				CommonFns.QuickAlign Hip50SideNull Hip50Null
				Hip50SideNull.parent = HipLocalPointNull
				Hip50SideParentNull.parent = Hip50SideNull
			
				local HipSideDeformController = CommonFns.MakeController (TinySize) "circle" HipSideBone HipSideBone blue HipSideDeformControlName "y" "positive"	
				HipSideDeformController.parent = Hip50SideNull
				Hip50SideParentNull.rotation.controller = orientation_constraint()
				Hip50SideParentNull.rotation.controller.appendtarget HipSideDeformController 100
				HipSideDeformController.objectoffsetpos = [0,0,BigSize]
				
				
				Hip50SideParentNull.pos.controller = position_constraint()
				Hip50SideParentNull.pos.controller.appendtarget HipSideDeformController 100
				
				Hip50SideNull.rotation.controller = orientation_constraint()
				Hip50SideNull.rotation.controller.appendtarget HipNoTwistNull 50
				Hip50SideNull.rotation.controller.appendtarget HipLocalPointNull 50
				
				if SideAlignment == "R" then 
				(
					in coordsys local rotate HipSideBone (angleaxis 90 [0,1,0])
					HipSideDeformController.objectoffsetpos = [0,0,-BigSize]
				)
					else
				(
					in coordsys local rotate HipSideBone (angleaxis -90 [0,1,0])
				)
				HipSideBone.parent = Hip50SideParentNull
			
				local SideBoneExpression = Float_expression()
				SideBoneExpression.AddScalarTarget "LocalPositionZ" HipExposeObjs[1].baseobject[#local_position_z]
				if SideAlignment == "R" then 
				(
					SideBoneExpression.setexpression ("-(((max (0,-LocalPositionZ))/(" + StartDistance + "))*" + (TinySize as string) + ")")
				)
				else
				(
					SideBoneExpression.setexpression ("(((max (0,LocalPositionZ))/(" + StartDistance + "))*" + (TinySize as string) + ")")
				)

				HipSideBone.pos.controller.z_position.controller = SideBoneExpression
				
				
				
				append allcreated Hip50SideParentNull
				append allcreated Hip50SideNull
				append allcreated HipSideBone
				append allbones HipSideBone
				append extrabones HipSideBone
				append allcontrols HipSideDeformController

				HipFrontBone = boneSys.createBone [0,0,0] [(distance UpLegBone LowLegBone *0.15),0,0] [0,0,1]
				HipFrontBone.Name = (uniquename (HipFrontBoneName) Numdigits:3)
				commonFns.quickAlign HipFrontBone HipExposeObjs[3]	
				HipFrontBone.wirecolor = (color 180 60 60)
				local HipFrontNull = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename (nullifiername + "Hip_Front") numdigits:3) size:Tinysize wirecolor:Green
				local HipFrontParentNull = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename (nullifiername + "Hip_Front_Parent") numdigits:3) size:Tinysize wirecolor:Green

				
				CommonFns.QuickAlign HipFrontNull Hip50Null
				CommonFns.QuickAlign HipFrontParentNull Hip50Null
				HipFrontParentNull.parent = HipFrontNull
					
				HipFrontNull.parent = HipLocalPointNull
				HipFrontNull.rotation.controller = orientation_constraint()
				HipFrontNull.rotation.controller.appendtarget HipNoTwistNull 100
				HipFrontNull.rotation.controller.appendtarget HipLocalPointNull 0
				HipFrontBone.parent = HipFrontParentNull	
				local HipFrontDeformController = CommonFns.MakeController (TinySize) "circle" HipFrontBone HipFrontBone blue HipFrontDeformControlName "y" "positive"
				HipFrontDeformController.parent = HipFrontNull
				HipFrontParentNull.rotation.controller = orientation_constraint()
				HipFrontParentNull.rotation.controller.appendtarget HipFrontDeformController 100
				HipFrontDeformController.objectoffsetpos = [BigSize,0,0]
				HipFrontParentNull.pos.controller = position_constraint()
				HipFrontParentNull.pos.controller.appendtarget HipFrontDeformController 100
				
				local FrontBoneExpression = Float_Expression()
				FrontBoneExpression.AddScalarTarget "LocalPositionY" HipExposeObjs[1].baseobject[#local_position_y]
				
				HipFrontBone.pos.controller.x_position.controller = FrontBoneExpression
				
				
				
				append allcreated HipFrontParentNull
				append allcreated HipFrontNull
				append allcreated HipFrontBone
				append allbones HipFrontBone
				append extrabones HipFrontBone
				append allcontrols HipFrontDeformController
				
				SetUserProp ButtDeformController "RigTools_ButtBone" (Hip50Bone.name as string)
				SetUserProp ButtDeformController "RigTools_SideBone" (HipSideBone.name as string)
				SetUserProp ButtDeformController "RigTools_FrontBone" (HipFrontBone.name as string)
				SetUserProp ButtDeformController "RigTools_ButtMove" (TinySize as string)
				SetUserProp ButtDeformController "RigTools_SideMove" (TinySize as string)
				SetUserProp ButtDeformController "RigTools_FrontMove" (TinySize as string)
				SetUserProp ButtDeformController "RigTools_SideAlignment" (SideAlignment as string)	
				SetUserProp ButtDeformController "RigTools_StartDistance" (StartDistance as string)	
				
				
				
				local ca=attributes HipAdjust
				(
					parameters params rollout:ro_AdjustHip
					(
						parambuttblend type:#float ui:spn_buttBlend
						paramsideblend type:#float ui:spn_sideBlend	
						paramfrontblend type:#float ui:spn_frontBlend		
					)
					
					rollout ro_AdjustHip "Adjust_Hip" 
					(
						
						spinner spn_buttBlend "Butt Auto" Range:[0,1,1] scale:0.1
						spinner spn_sideBlend "Side Auto" Range:[0,1,1] scale:0.1
						spinner spn_frontBlend "Front Auto" Range:[0,1,1] scale:0.1
						
						edittext edt_ButtMoveamount "Butt Extend"
						edittext edt_SideMoveamount "Side Extend"
						edittext edt_FrontMoveamount "Front Extend"
						button btn_apply "Apply" enabled:on

						local ButtBoneName, SideBoneName, FrontBoneName, ButtMove, SideMove, FrontMove, HipSideAlignment
						
						on ro_AdjustHip open do
						(
							ButtBoneName = 	(GetUserProp $ "RigTools_ButtBone") as string
							SideBoneName = 	(GetUserProp $ "RigTools_SideBone") as string
							FrontBoneName = (GetUserProp $ "RigTools_FrontBone") as string
							ButtMove = 	(GetUserProp $ "RigTools_ButtMove") as string
							SideMove = 	(GetUserProp $ "RigTools_SideMove") as string
							FrontMove = 	(GetUserProp $ "RigTools_FrontMove") as string
							HipSideAlignment = 	(GetUserProp $ "RigTools_SideAlignment") as string
							
							edt_ButtMoveamount.text = (ButtMove as string)
							edt_SideMoveamount.text = (SideMove as string)
							edt_FrontMoveamount.text = (FrontMove as string)
						)
							
						on btn_apply pressed do
						(
							local NewButtMoveAmount = edt_ButtMoveamount.text
							local NewSideMoveAmount = edt_SideMoveamount.text
							local NewFrontMoveAmount = edt_FrontMoveamount.text
							
							local buttbone = getnodebyname ButtBoneName
							local sidebone = getnodebyname SideBoneName
							local frontbone = getnodebyname FrontBoneName
							
							local StartDistance = (GetUserProp $ "RigTools_StartDistance") as string
							
							SetUserProp $ "RigTools_ButtMove" (NewButtMoveAmount as string)
							SetUserProp $ "RigTools_SideMove" (NewSideMoveAmount as string)
							SetUserProp $ "RigTools_FrontMove" (NewFrontMoveAmount as string)
							
							
							buttbone.pos.controller.x_position.controller.setexpression ("(((max (0,LocalPositionY))/(" + StartDistance + "))*" + (NewButtMoveAmount as string)  + ")*SliderValue")
							if HipSideAlignment == "R" then (SideBone.pos.controller.z_position.controller.setexpression ("-(((max (0,-LocalPositionZ))/(" + StartDistance + "))*" + (NewSideMoveAmount as string) + ")*SliderValue"))
								else (SideBone.pos.controller.z_position.controller.setexpression ("(((max (0,LocalPositionZ))/(" + StartDistance + "))*" + (NewSideMoveAmount as string) + ")*SliderValue"))
							FrontBone.pos.controller.x_position.controller.setexpression ("(((max (0,-LocalPositionY))/(" + StartDistance + "))*" + (NewFrontMoveAmount as string) + ")*SliderValue")
			
						)
						
						
						
					)
				)
				local AttHolder = (EmptyModifier())
				AttHolder.name = "Hip_Controls"
				addmodifier ButtDeformController AttHolder
				custAttributes.add ButtDeformController.modifiers[#Hip_Controls] ca
					
				Hip50Bone.pos.controller.x_position.controller.addscalartarget "SliderValue" ButtDeformController.modifiers[#Hip_Controls].HipAdjust[#parambuttblend]
				HipFrontBone.pos.controller.x_position.controller.addscalartarget "SliderValue" ButtDeformController.modifiers[#Hip_Controls].HipAdjust[#paramfrontblend]
				HipSideBone.pos.controller.z_position.controller.addscalartarget "SliderValue" ButtDeformController.modifiers[#Hip_Controls].HipAdjust[#paramsideblend]	
				
				Hip50Bone.pos.controller.x_position.controller.setexpression ("(((max (0,LocalPositionY))/(" + StartDistance + "))*" + (TinySize as string)  + ")*SliderValue")
				HipFrontBone.pos.controller.x_position.controller.setexpression ("(((max (0,-LocalPositionY))/(" + StartDistance + "))*" + (TinySize as string) + ")*SliderValue")
				if SideAlignment == "R" then 
				(
				HipSideBone.pos.controller.z_position.controller.setexpression ("-(((max (0,-LocalPositionZ))/(" + StartDistance + "))*" + (TinySize as string) + ")*SliderValue")
				)
				else
				(
				HipSideBone.pos.controller.z_position.controller.setexpression ("(((max (0,LocalPositionZ))/(" + StartDistance + "))*" + (TinySize as string) + ")*SliderValue")	
				)
				
				ButtDeformController.modifiers[#Hip_Controls].HipAdjust.parambuttblend = 1
				ButtDeformController.modifiers[#Hip_Controls].HipAdjust.paramsideblend = 1
				ButtDeformController.modifiers[#Hip_Controls].HipAdjust.paramfrontblend = 1
				
				

					
			) --this bracket to close advanced hip deform loop
			
			
			
			--------------------------Next Draw the Knee 50% system---------------------------------------------
			
			local Knee50Null
			local UpLegBendController
			local LowLegBendController
			local Knee50Bone

			Knee50Bone = boneSys.createBone [0,0,0] [((distance uplegbone lowlegbone) *0.15),0,0] [0,0,1]
			Knee50Bone.name = (uniquename Knee50BoneName Numdigits:3)
			Knee50Bone.wirecolor = (color 180 60 60)

			Knee50Null = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename Knee50NullName Numdigits:3) size:Tinysize wirecolor:Green
			CommonFns.QuickAlign Knee50Null LowLegBone
			
			Knee50Parent = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename Knee50ParentName Numdigits:3) size:Tinysize wirecolor:Green
			CommonFns.QuickAlign Knee50Parent LowLegBone
			Knee50Parent.parent = Knee50Null
			
			
			Knee50Null.parent = LowLegBone
			Knee50Null.rotation.controller = Orientation_Constraint()
			Knee50Null.rotation.controller.appendtarget UpLegBone 100
			Knee50Null.rotation.controller.appendtarget LowLegBone 100
			
			CommonFns.QuickAlign Knee50Bone Knee50Null
			Knee50Bone.parent = Knee50Parent
			in coordsys local rotate Knee50Bone (angleaxis -90 [0,0,1])
			
			if (Bendy == true) or (Stretchy == true) do
			(
				if IncludeSquash == true do 
				(
					Knee50parent.scale.controller = copy LowLegTwistChain[7][1].scale.controller
				)
			)
			
			local KneeDistExpose 
			local Knee50BoneFront
			local ca
			
			
			If KneeType != 1 do  --Create the measure helper/expose transform system.
			(
				local TempStartPoint = Point Name:"ZZZZZZTEMPSTART"
				local TempEndPoint = Point Name:"ZZZZZTEMPBLAH"
				TempStartPoint.pos.controller = position_constraint()
				TempEndPoint.pos.controller = position_constraint()
				TempStartPoint.pos.controller.appendtarget UpLegBone 25
				TempStartPoint.pos.controller.appendtarget LowLegBone 75
				TempEndPoint.pos.controller.appendtarget LowLegBone 75
				TempEndPoint.pos.controller.appendtarget FootBone 25
				KneeDistExpose =  CommonFns.CreateLookatdistanceExpose TempStartPoint TempEndPoint #((MeasureHelperLookatName),(MeasureHelperLocalName)) AlignTo:undefined reAssignParents:#(UpLegBone,LowLegBone) justdistance:false
				delete TempStartPoint
				delete TempEndPoint
				for n in KneeDistExpose do (append allcreated n)
					
				local KneeDeformController = CommonFns.MakeController (TinySize) "circle" Knee50Null Knee50Null blue KneeDeformControlName "y" "positive"
				KneeDeformController.objectoffsetpos = [0,-(BigSize),0]
				CommonFns.QuickAlign KneeDeformController Knee50Null
				KneeDeformController.parent = Knee50Null
				Knee50Parent.parent = KneeDeformController
				append allcreated KneeDeformController
				append allcontrols KneeDeformController
			 
			
				ca=attributes KneeAdjust
				(
					parameters params rollout:ro_AdjustKnee
					(
					paramblend type:#float ui:spn_kneeBlend
					)
					
					rollout ro_AdjustKnee "Adjust_Knee" 
					(
						edittext edt_moveamount "ExtendAmount"
						spinner spn_kneeblend "Auto Off/On" Range:[0,1,1] scale:0.1
						button btn_apply "Apply" enabled:on

						local KneeType,MoveAmount,FrontBoneName,BackBoneName
						
						on ro_AdjustKnee open do
						(
							kneetype = 	(GetUserProp $ "RigTools_KneeType") as string
							if Kneetype == "Scale" then 
							(
								edt_moveamount.visible = false
								btn_apply.visible = false
							)
							else
							(
								moveamount = (GetUserProp $ "RigTools_MoveAmount")
								frontbonename = (GetUserProp $ "RigTools_FrontBoneName")
								backbonename = (GetUserProp $ "RigTools_BackBoneName")
								edt_moveamount.text = (Moveamount as string)
							)
						)
							
						on btn_apply pressed do
						(
							local NewMoveAmount = edt_moveamount.text
							local frontbone = getnodebyname frontbonename
							local backbone = getnodebyname backbonename
							setUserProp $ "RigTools_MoveAmount"  NewMoveAmount
							backbone.pos.controller[2].Y_Position.controller.setexpression ("(max(Distance*" + (NewMoveAmount as string) + ",0))")
							frontbone.pos.controller[2].Y_Position.controller.setexpression ("-(max(Distance*" + (NewMoveAmount as string) + ",0))")				
						)
					)
				)
				local AttHolder = (EmptyModifier())
				AttHolder.name = "Knee_Controls"
				addmodifier KneeDeformController AttHolder
				custAttributes.add KneeDeformController.modifiers[#Knee_Controls] ca
					
				KneeDeformController.modifiers[#Knee_Controls].KneeAdjust.paramblend = 1
			)
			

			If KneeType == 2 do  -- Scale Bone
			(

			Knee50Bone.scale.controller = Scale_List()
			Knee50Bone.scale.controller.available.controller = Scale_Expression ()
			Knee50Bone.scale.controller[2].controller.addscalartarget  "Distance" KneeDistExpose[1].modifiers[#Stretch_Expose_Controls].StretchControls[#paramstretch]
			Knee50Bone.scale.controller[2].controller.setexpression ("[(max((Distance*0.9),0))+1,1,1]")
			setUserProp KneeDeformController "RigTools_KneeType"  "Scale"
			Knee50Bone.scale.controller.weight[2].controller = Float_Expression()
			Knee50Bone.scale.controller.weight[2].controller.addscalartarget "SliderValue" KneeDeformController.modifiers[#Knee_Controls].KneeAdjust[#paramblend]
			Knee50Bone.scale.controller.weight[2].controller.setexpression "SliderValue"
			
			)
			
			If KneeType == 3 do 
			(
			local Knee50FrontNull = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename (Knee50NullName + "_front") Numdigits:3) size:Tinysize wirecolor:Green
			local Knee50BackNull = Point isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename (Knee50NullName + "_back") Numdigits:3) size:Tinysize wirecolor:Green
			CommonFns.quickalign Knee50FrontNull Knee50Null
			CommonFns.quickalign Knee50BackNull Knee50Null
			Knee50FrontNull.parent = Knee50Null
			Knee50BackNull.parent = Knee50Null
			in coordsys Local Move Knee50FrontNull [0,-(TinySize*0.2),0]
			in coordsys local Move Knee50BackNull [0,TinySize*0.2,0]
			Knee50Bone.parent = Knee50BackNull
			CommonFns.QuickAlign Knee50Bone Knee50BackNull
			
			append allcreated Knee50FrontNull
			append allcreated Knee50BackNull
			
			Knee50Bone.pos.controller = Position_List()
			Knee50Bone.pos.controller.available.controller = Position_XYZ()
				
			Knee50Bone.pos.controller[2].Y_Position.controller = Float_Expression ()	
			Knee50Bone.name = (uniquename (Knee50BoneName + "_back") numdigits:3)
			
			Knee50BoneFront = boneSys.createBone [0,0,0] [(distance UpLegBone LowLegBone *0.15),0,0] [0,0,1]
			Knee50BoneFront.Name = (uniquename (Knee50BoneName+ "_front") Numdigits:3)
			Knee50BoneFront.wirecolor = (color 180 60 60)
			CommonFns.QuickAlign Knee50BoneFront Knee50FrontNull
			Knee50BoneFront.parent = Knee50FrontNull
			append allcreated Knee50BoneFront
			append ExtraBones Knee50bonefront
				
			Knee50BoneFront.pos.controller = position_list()
			Knee50BoneFront.pos.controller.available.controller = position_xyz()
			Knee50BoneFront.pos.controller[2].Y_Position.controller = Float_Expression ()
				
				
			Knee50BoneFront.pos.controller[2].Y_Position.controller.addscalartarget  "Distance" KneeDistExpose[1].modifiers[#Stretch_Expose_Controls].StretchControls[#paramstretch]
			Knee50BoneFront.pos.controller[2].Y_Position.controller.setexpression ("-(max(Distance*" + (Bigsize as string) + ",0))")				
			Knee50Bone.pos.controller[2].Y_Position.controller.addscalartarget  "Distance" KneeDistExpose[1].modifiers[#Stretch_Expose_Controls].StretchControls[#paramstretch]
			Knee50Bone.pos.controller[2].Y_Position.controller.setexpression ("(max(Distance*" + (Bigsize as string) + ",0))")

			Knee50Bone.pos.controller.weight[2].controller = float_expression()
			Knee50Bone.pos.controller.weight[2].controller.addscalartarget "SliderValue" KneeDeformController.modifiers[#Knee_Controls].KneeAdjust[#paramblend]
			Knee50Bone.pos.controller.weight[2].controller.setexpression "SliderValue"
			Knee50BoneFront.pos.controller.weight[2].controller = float_expression()
			Knee50BoneFront.pos.controller.weight[2].controller.addscalartarget "SliderValue" KneeDeformController.modifiers[#Knee_Controls].KneeAdjust[#paramblend]
			Knee50BoneFront.pos.controller.weight[2].controller.setexpression "SliderValue"
			
			setUserProp KneeDeformController "RigTools_MoveAmount"  (BigSize as string)
			setUserProp KneeDeformController "RigTools_KneeType"  "Game"
			setUserProp KneeDeformController "RigTools_FrontBoneName"  (Knee50BoneFront.name)
			setUserProp KneeDeformController "RigTools_BackBoneName"  (Knee50Bone.name)		
			) --Game Compatible
			
			
			
			
			
			append allcreated Knee50Null
			append ExtraBones Knee50Bone
			append allcreated Knee50Bone
			append allcreated Knee50Parent
			
			-----------------------------Add the bendy system-------------------------------------------------------------------------
			


			if Bendy == true do
			(
				

			
				UpLegBendController = CommonFns.MakeController (MediumSize) "triangle" LowLegBone LowLegBone (color 40 200 40) (UpLegBendyControlName)  "x" "negative"			
				CommonFns.quickalign UpLegBendController Knee50Null
				UpLegBendController.objectoffsetpos = [-((Distance UpLegBone LowLegBone)*0.4),0,0]
				UpLegBendController.parent = Knee50Null
				
				LowLegBendController = CommonFns.MakeController (MediumSize) "triangle" LowLegBone LowLegBone (color 40 200 40) (LowLegBendyControlName)  "x" "positive"
				CommonFns.quickalign LowLegBendController Knee50Null
				LowLegBendController.objectoffsetpos = [((Distance UpLegBone LowLegBone)*0.4),0,0]
				LowLegBendController.parent = Knee50Null
				
				append allcreated UpLegBendController
				append allcreated LowLegBendController
				append allcontrols UpLegBendController
				append allcontrols LowLegBendController
				

				UpLegTwistChain[2][3].rotation.controller = orientation_constraint()
				UpLegTwistChain[2][3].rotation.controller.appendtarget UpLegBone 100
				UpLegTwistChain[2][3].rotation.controller.appendtarget UpLegBendController 100
				
				
				LowLegTwistChain[2][2].rotation.controller = orientation_constraint()
				LowLegTwistChain[2][2].rotation.controller.appendtarget LowLegBone 100
				LowLegTwistChain[2][2].rotation.controller.appendtarget LowLegBendController 100
				
				local BaseBendExpression = float_expression()
				BaseBendExpression.AddScalarTarget "SliderValue" SwitchController.modifiers[#LegBlend_Controls].LegBlendControls[#parambendy]
				
				local LegBendExpression = copy BaseBendExpression
				LegBendExpression.setExpression "100-SliderValue"
				LowLegTwistChain[2][2].rotation.controller.weight[1].controller = LegBendExpression
				UpLegTwistChain[2][3].rotation.controller.weight[1].controller = LegBendExpression
				LegBendExpression = copy LegBendExpression
				LegBendExpression.setexpression "SliderValue"
				LowLegTwistChain[2][2].rotation.controller.weight[2].controller = LegBendExpression
				UpLegTwistChain[2][3].rotation.controller.weight[2].controller = LegBendExpression
				local TempTangentPoint = Point Name:"ZZZZZTEMPTANGENT" Pos:[0,0,0]
				TempTangentPoint.pos.controller = position_constraint()
				TempTangentPoint.pos.controller.appendtarget LowLegBone 20
				TempTangentPoint.pos.controller.appendtarget UpLegBone 80
				CommonFns.quickalign UpLegTwistChain[6][1] TempTangentPoint
				TempTangentPoint.pos.controller = position_xyz()
				TempTangentPoint.pos.controller = position_constraint()
				TempTangentPoint.pos.controller.appendtarget LowLegBone 20
				TempTangentPoint.pos.controller.appendtarget Footbone 80
				
				CommonFns.quickalign LowLegTwistChain[6][2] TempTangentPoint
				delete TempTangentPoint
				

			)
			
			for i = 1 to i = allbones.count do
			(
				allbones[i].width = tinysize
				allbones[i].height = tinysize
				allbones[i].sidefins = off
				allbones[i].frontfin = off
				allbones[i].backfin = off
			)
			
					FinalTools.appendobjstolayer Allcreated HiddenComponents:True
					FinalTools.appendobjstolayer MainBonesArray HiddenComponents:False MainBones:True
					FinalTools.appendobjstolayer AllControls HiddenComponents:False Controllers:True
					FinalTools.appendobjstolayer ExtraBones HiddenComponents:False DeformerBones:True
			

				local OutputArray = #()
			
				if bendy == true or stretchy == true do (for i = 1 to i = UpLegTwistChain[1].count do (append allcreated UpLegTwistChain[1][i]))
				if bendy == true or stretchy == true do (for i = 1 to i = LowLegTwistChain[1].count do (append allcreated LowLegTwistChain[1][i]))
			
				append OutputArray Allcreated
				append OutputArray Allbones
				append OutputArray ExtraBones
				append OutputArray AllControls
					

		
	), --This bracket to close make leg
	------------------------------------------------------------------------
	fn MakeheadGuide NeckStart HeadStart HeadEnd =
	( --COME HERE--
		
		
		local AverageScale = ((Distance NeckStart HeadStart) + (Distance HeadStart HeadEnd)/2)
		local TinySize = AverageScale/16
		local MediumSize = AverageScale/10
		local BigSize = AverageScale/6
		local allcreated = #()
		
		local NeckStartReplace = (CommonFns.MakeController MediumSize "sphere" NeckStart NeckStart (color 20 100 200)  "Neck_Start_locator"  "y" "negative")
		delete NeckStart
		NeckStart = NeckStartReplace
		local HeadStartReplace = (CommonFns.MakeController MediumSize "sphere" HeadStart HeadStart (color 20 100 200)  "Head_Start_locator"  "y" "negative")
		delete HeadStart
		HeadStart = HeadStartReplace
		local HeadendReplace = (CommonFns.MakeController MediumSize "sphere" Headend Headend (color 20 100 200)  "Head_end_locator"  "y" "negative")
		delete Headend
		Headend = HeadendReplace
		
		append allcreated NeckStart
		append allcreated HeadStart
		append allcreated Headend
		----MAKE THE GUIDE SPLINES------
			
		local ss,Midpoint,GuideSpline
			
		Midpoint = CommonFns.findIncrementsBetweenTwoPoints NeckStart HeadStart divisions:1 IncludeEnds:False
		
		ss = SplineShape pos:NeckStart.Transform.pos name:(uniquename "NeckSpline" Numdigits:3) color:(color 20 140 70)
		ss.optimize = false
		ss.steps = 12
		addNewSpline ss
		addKnot ss 1 #beziercorner #curve NeckStart.transform.pos NeckStart.Transform.pos Midpoint[1]
		addKnot ss 1 #beziercorner #curve HeadStart.transform.pos Midpoint[1] HeadStart.Transform.pos
		updateShape ss
		ss
		GuideSpline = ss
		
		append allcreated GuideSpline
		
		------------MAKE THE GUIDE SPLINE TANGENT HELPERS AND LANDMARKS-------------------
		
		local NeckTangent, HeadTangent,FloatExpr,TangentToAdd

		local TempWorldPoint = CommonFns.createsystemalignmentpoint NeckStart HeadStart
		
		
		GuideSpline.wirecolor = (color 10 150 80)

		BendControl = (CommonFns.MakeController MediumSize "sphere" TempWorldPoint MidPoint[1] (color 20 200 100)  "Neck_Bend_ZZZ"  "y" "negative")

		settransformlockflags BendControl #{4,5,6,7,8,9}


		local NeckTangent = Point pos:Midpoint[1] isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename "NeckStartTangent" Numdigits:3)  size:0 wirecolor:Green
		
		local HeadTangent = Point pos:Midpoint[1] isSelected:off cross: off centermarker: off axistripod: off box: off name:(uniquename "NeckEndTangent" Numdigits:3) size:0 wirecolor:Green
		NeckTangent.parent = BendControl
		HeadTangent.parent = BendControl
		
		delete TempWorldPoint
		
		append allcreated NeckTangent
		append allcreated HeadTangent
		append allcreated BendControl
		
		
		
		----------------------SKIN THE SPLINES----------------------------------
		local theskin,NeckControlOrigName,HeadControlOrigName,FirstTangentOrigName,SecondTangentOrigName,theskinreverse
		Select GuideSpline
		theskin = Skin()
		addmodifier $ theskin
		max modify mode
		
		NeckStart.Name = "Z1_NeckStart"
		HeadStart.Name = "Z2_HeadStart"

		skinOps.addBone $.modifiers[#Skin] NeckTangent 1
		skinOps.addBone $.modifiers[#Skin] HeadTangent 1
		skinOps.addBone $.modifiers[#Skin] NeckStart 1
		skinOps.addBone $.modifiers[#Skin] HeadStart 1
		$.modifiers[#Skin].filter_vertices = on
		Select GuideSpline
		skinOps.ReplaceVertexWeights $.modifiers[#Skin]  1 3 1
		skinOps.ReplaceVertexWeights $.modifiers[#Skin]  2 3 1
		skinOps.ReplaceVertexWeights $.modifiers[#Skin]  3 1 1
		skinOps.ReplaceVertexWeights $.modifiers[#Skin]  4 2 1
		skinOps.ReplaceVertexWeights $.modifiers[#Skin]  5 4 1
		skinOps.ReplaceVertexWeights $.modifiers[#Skin]  6 4 1
		
		$Z1_NeckStart.Name = "Neck_StartZZZ"
		$Z2_HeadStart.Name = "Head_StartZZZ"
		
		Neckstart = getnodebyname "Neck_StartZZZ"
		HeadStart = getnodebyname "Head_StartZZZ"
		
		local AlignmentPoint = CommonFns.CreateSystemAlignmentPoint HeadStart HeadEnd
		
		AlignmentPoint.size = 0
		

		local HeadGuideBone  = boneSys.createBone HeadStart.transform.pos HeadEnd.transform.pos [0,0,1]
		HeadGuideBone.name = (uniquename "Head_GuideBone" numdigits:3)
		local HeadGuideNub  = boneSys.createBone HeadEnd.transform.pos HeadEnd.transform.pos [0,0,1]
		HeadGuideNub.name = (uniquename "Head_GuideNub" numdigits:3)
		HeadGuideNub.parent = HeadGuideBone
		
		
		
		HeadGuideBone.pos.controller = position_constraint()
		HeadGuideBone.pos.controller.appendtarget HeadStart 100
		
		local LookCons = Lookat_Constraint()
		LookCons.appendtarget HeadEnd 100
		LookCons.target_axis = 0
		LookCons.upnode_ctrl = 1
		LookCons.upnode_world = off
		LookCons.pickUpNode = AlignmentPoint
		LookCons.StoUP_axis = 2
		LookCons.upnode_axis = 2
		LookCons.viewline_length_abs = off
		LookCons.lookat_vector_length = 0
		HeadGuideBone.rotation.controller = LookCons
		HeadGuideBone.boneFreezeLength=false
		HeadGuideBone.boneScaleType=#scale
		HeadGuideNub.pos.controller = position_constraint()
		HeadGuideNub.pos.controller.appendtarget HeadEnd 100
		HeadGuideBone.Height= TinySize*0.5
		HeadGuideBone.Width = TinySize*0.5
		HeadGuideBone.FrontFin = off
		HeadGuideBone.SideFins = off
		HeadGuideBone.BackFin = off
		HeadGuideNub.height = 0
		HeadGuideNub.width = 0
		

		
		local RightTrapGuideBone,RightTrapGuideNub,RightTrapLocator,LeftTrapLocator,TrapLookat
		
		RightTrapLocator = CommonFns.MakeController MediumSize "sphere" NeckStart NeckStart (color 180 40 40)  "TrapRight_locator"  "y" "negative"
		TrapLookat = CommonFns.MakeController TinySize "sphere" NeckStart NeckStart (color 180 40 40)  "TrapLookat"  "y" "negative"
		in coordsys world move RightTrapLocator [(-AverageScale*0.2),0,0]
		TrapLookat.pos.controller = path_constraint()
		TrapLookat.pos.controller.path = GuideSpline
		TrapLookat.pos.controller.percent = 30
		TrapLookat.pos.controller = position_list()
		TrapLookat.pos.controller.available.controller = position_xyz()
		TrapLookat.pos.controller.SetActive 2 		
		
		local RightTrapGuideBone  = boneSys.createBone RightTrapLocator.transform.pos TrapLookat.transform.pos [0,0,1]
		RightTrapGuideBone.name = (uniquename "Trap_GuideBone" numdigits:3)
		local RightTrapGuideNub  = boneSys.createBone TrapLookat.transform.pos TrapLookat.transform.pos [0,0,1]
		RightTrapGuideNub.name = (uniquename "Trap_GuideNub" numdigits:3)
		RightTrapGuideNub.parent = RightTrapGuideBone
		
		RightTrapGuideBone.boneFreezeLength=false
		RightTrapGuideBone.boneScaleType=#scale

		RightTrapGuideBone.Height= TinySize*0.5
		RightTrapGuideBone.Width = TinySize*0.5
		RightTrapGuideBone.FrontFin = off
		RightTrapGuideBone.SideFins = off
		RightTrapGuideBone.BackFin = off
		RightTrapGuideBone.wirecolor = (color 190 60 60)
		RightTrapGuideNub.Height= 0
		RightTrapGuideNub.Width = 0
		
		RightTrapGuideBone.pos.controller = position_constraint()
		RightTrapGuideBone.pos.controller.appendtarget RightTrapLocator 100
		RightTrapGuideNub.pos.controller = position_constraint()
		RightTrapGuideNub.pos.controller.appendtarget TrapLookat 100
		
		local TrapLookCons = Lookat_Constraint()
		TrapLookCons.appendtarget TrapLookat 100
		TrapLookCons.target_axis = 0
		TrapLookCons.upnode_ctrl = 1
		TrapLookCons.upnode_world = off
		TrapLookCons.pickUpNode = NeckStart
		TrapLookCons.StoUP_axis = 2
		TrapLookCons.upnode_axis = 2
		TrapLookCons.viewline_length_abs = off
		TrapLookCons.lookat_vector_length = 0
		RightTrapGuideBone.rotation.controller = TrapLookCons
		
		append allcreated TrapLookat
		append allcreated RightTrapLocator
		append allcreated RightTrapGuideBone
		append allcreated RightTrapGuideNub
		
		local TrapBones = #()
		append TrapBones RightTrapGuideBone
		append TrapBones RightTrapGuideNub
		
		append allcreated AlignmentPoint
		append allcreated HeadGuideBone
		append allcreated HeadGuideNub
		
		local MainBonesArray = #()
		
		Append MainbonesArray NeckStart
		Append MainbonesArray BendControl
		Append MainbonesArray HeadGuideBone
		Append MainbonesArray HeadGuideNub
		
		local OutputArray = #()
		append OutputArray allcreated
		append OutputArray MainBonesArray
		append OutputArray TrapBones
		
		----------------Make the width Guides------------------------------
		
		local NeckStartWidthGuide = CommonFns.MakeController (BigSize*2) "circle" HeadGuideBone HeadGuideBone (color 40 120 80) ("NeckStartWidth")  "z" "positive"
		local NeckEndWidthGuide = CommonFns.MakeController (BigSize*2) "circle" HeadGuideBone HeadGuideBone (color 40 120 80) ("NeckEndWidth")  "z" "positive"
		local HeadWidthGuide = CommonFns.MakeController (BigSize*3) "circle" HeadGuideBone HeadGuideBone (color 40 120 80) ("HeadWidth")  "z" "positive"
		
		CommonFns.QuickAlign NeckStartWidthGuide HeadGuideBone
		CommonFns.QuickAlign NeckEndWidthGuide HeadGuideBone
		
		
		NeckStartWidthGuide.pos.controller = path_constraint()
		NeckStartWidthGuide.pos.controller.path = GuideSpline
		NeckStartWidthGuide.pos.controller.percent = 20
		NeckStartWidthGuide.pos.controller.follow = off



		NeckEndWidthGuide.pos.controller = path_constraint()
		NeckEndWidthGuide.pos.controller.path = GuideSpline
		NeckEndWidthGuide.pos.controller.percent = 80
		NeckEndWidthGuide.pos.controller.follow = off
		NeckEndWidthGuide.Rotation.controller = Orientation_Constraint()

		
		HeadWidthGuide.pos.controller = position_constraint()
		HeadWidthGuide.pos.controller.appendtarget HeadGuideBone 50
		HeadWidthGuide.pos.controller.appendtarget HeadGuideNub 50
		HeadWidthGuide.Rotation.controller = Orientation_constraint()
		HeadWidthGuide.Rotation.controller.appendtarget HeadGuideBone 100
		
		
		
		append allcreated HeadWidthGuide
		append allcreated NeckEndWidthGuide
		append allcreated NeckStartWidthGuide
		
		local NeckStartWidthPoint,NeckStartFrontWidthPoint,NeckEndWidthPoint,NeckEndFrontWidthPoint,HeadWidthPoint,HeadFrontWidthPoint
		
		NeckStartWidthPoint = Point Pos:[0,0,0] Name:"ZZZZZNeckstartWidth" Box:off Cross:off axistripod:off centermarker:off
		NeckStartFrontWidthPoint = Point Pos:[0,0,0] Name:"ZZZZZNeckStartFronWidth" Box:off Cross:off axistripod:off centermarker:off
		NeckEndWidthPoint = Point Pos:[0,0,0] Name:"ZZZZZNeckEndWidth" Box:off Cross:off axistripod:off centermarker:off
		NeckEndFrontWidthPoint = Point Pos:[0,0,0] Name:"ZZZZZNeckEndFrontWidth" Box:off Cross:off axistripod:off centermarker:off
		HeadWidthPoint = Point Pos:[0,0,0] Name:"ZZZZZHeadWidth" Box:off Cross:off axistripod:off centermarker:off
		HeadFrontWidthPoint = Point Pos:[0,0,0] Name:"ZZZZZHeadFrontWidth" Box:off Cross:off axistripod:off centermarker:off
		
		NeckStartWidthPoint.pos.controller = path_constraint()
		NeckStartWidthPoint.pos.controller.path = NeckStartWidthGuide
		NeckStartWidthPoint.pos.controller.percent = 0
		NeckStartFrontWidthPoint.pos.controller = path_constraint()
		NeckStartFrontWidthPoint.pos.controller.path = NeckStartWidthGuide
		NeckStartFrontWidthPoint.pos.controller.percent = 25
		
		NeckEndWidthPoint.pos.controller = path_constraint()
		NeckEndWidthPoint.pos.controller.path = NeckEndWidthGuide
		NeckEndWidthPoint.pos.controller.percent = 0
		NeckEndFrontWidthPoint.pos.controller = path_constraint()
		NeckEndFrontWidthPoint.pos.controller.path = NeckEndWidthGuide
		NeckEndFrontWidthPoint.pos.controller.percent = 25
		
		HeadWidthPoint.pos.controller = path_constraint()
		HeadWidthPoint.pos.controller.path = HeadWidthGuide
		HeadWidthPoint.pos.controller.percent = 0
		HeadFrontWidthPoint.pos.controller = path_constraint()
		HeadFrontWidthPoint.pos.controller.path = HeadWidthGuide
		HeadFrontWidthPoint.pos.controller.percent = 25
		
		append Allcreated NeckStartWidthPoint
		append Allcreated NeckStartFrontWidthPoint
		append Allcreated NeckEndWidthPoint
		append Allcreated NeckEndFrontWidthPoint
		append Allcreated HeadWidthPoint
		append Allcreated HeadFrontWidthPoint
		
		

		local widthsarray = #()
		
		append WidthsArray NeckStartWidthGuide
		append WidthsArray NeckStartWidthPoint
		append WidthsArray NeckStartFrontWidthPoint
		append WidthsArray NeckEndWidthGuide
		append WidthsArray NeckEndWidthPoint
		append WidthsArray NeckEndFrontWidthPoint
		append WidthsArray HeadWidthGuide
		append WidthsArray HeadWidthPoint
		append WidthsArray HeadFrontWidthPoint
		
		append OutputArray WidthsArray

		for n in allcreated do 
		(
			if classof n == bonegeometry do
			(
				n.showFrozenInGray = off
				freeze n
			)
		)
		
		

		OutputArray
		
		--OutputArray contains
		--1: Allcreated
		--2: NeckStart, NeckBend, HeadBone, HeadNub
		--3: Right Trapezius, Right Trapezius nub
		--4:Widths:  NeckBaseWidthGuide, NeckBaseWidth, NeckBaseFrontWidth.    Neck End Guide, Width, Front.   Head Guide Width Front
		

	), -- This bracket to close head guide function
	------------------------------------------------------------------------
	fn MakeHead MainBones TrapBones Prefix:"" Widths:Undefined Squash:1 Scaling:1 Root:undefined LeftShoulder:undefined RightShouldeR:undefined ParentObj:undefined IKFK:1 NumBones:3 ROM:1 TrapBoneStyle:3=
	(
	
		local AverageScale = ((Distance MainBones[1].pos MainBones[2].pos) + (Distance MainBones[3].pos MainBones[4].pos))/2
		local TinySize = AverageScale/16
		local MediumSize = AverageScale/10
		local BigSize = AverageScale/6
		
		
		
		local NeckStartWidth = TinySize
		local NeckStartFrontWidth = TinySize
		local NeckEndWidth = TinySize
		local NeckEndFrontWidth = TinySize
		local HeadWidth = TinySize
		local HeadFrontWidth = TinySize
		
		if Widths != undefined do
		(
			NeckStartFrontWidth = Distance Widths[1] Widths [2]
			NeckStartWidth = Distance Widths[1] Widths[3]
			NeckEndFrontWidth = Distance Widths[4] Widths [5]
			NeckEndWidth = Distance Widths[4] Widths[6]
			HeadFrontWidth = Distance Widths[7] Widths [8]
			HeadWidth = Distance Widths[7] Widths[9]
		)
		
		
		local HeadBoneName = Prefix + "bn_head"
		local NeckBoneName = Prefix + "bn_neck"
		local TrapBoneName = Prefix + "bn_trapezius"
		local HeadControlName = Prefix + "ctrl_head"
		local NeckControlName = Prefix + "ctrl_neck"
		local NeckBendControlName = Prefix + "ctrl_neckbend"
		local NullifierName = Prefix + "null_"
		
		
		if Scaling ==2 do (Scaling = 3)--get it to match up with the IKFKSpine function
		
		local NeckStartPoint = MainBones[1]
		local HeadStartPoint = MainBones[3]
		local NeckBendPoint = MainBones[2]
		local HeadEndPoint = MainBones[4]
			
		local SplineBendInputArray =#(NeckBendPoint.pos,NeckBendPoint.pos)	
		
		local HeadBoneLocation = MainBones[3]
		local HeadNubLocation = MainBones[4]
		
		local RightTrapLocation = TrapBones[1]
		local RightTrapNubLocation = TrapBones[2]
		
		local MirroredTrapArray = CommonFns.MirrorTransformsInArray TrapBones
		
		local LeftTrapLocation = MirroredTrapArray[1]
		local LeftTrapNubLocation = MirroredTrapArray[2]
		
		local Allcreated = #()
		local NeckBonesArray = #()
		local ControlsArray = #()
		

		TrapBones = TrapBoneStyle
		

			local TempNeckPoint = Point Name:"ZZZZTEMPNECKPOINT" Pos:[0,0,0]
			CommonFns.quickalign TempNeckPoint NeckStartPoint
			local TempHeadPoint = Point Name:"ZZZZTEMPHEADPOINT" Pos:[0,0,0]
			CommonFns.quickalign TempHeadPoint HeadStartPoint
			local TempHeadEndPoint = Point Name:"ZZZZTEMPHEADEND" Pos:[0,0,0]
			CommonFns.quickalign TempHeadEndPoint HeadEndPoint
			local HeadController = CommonFns.DrawRectangleControl TempHeadPoint TempHeadPoint TempHeadEndPoint (HeadWidth*1.3) (HeadFrontWidth*1.3) (uniquename HeadControlName numdigits:3)
			CommonFns.ApplycontrollerTypeProperties HeadController ControlType:"Medium" ControlLevel:1
			append Allcreated HeadController
			append ControlsArray HeadController
			
			-----------------------------Create IK/FK Neck-----------------------------------------------------
			
		local HeadBone
		
		local TempFKPointsArray = #()
		local FKNeckBonesArray = #()
		
		local NeckControllersArray = #()
		local NeckNullsArray = #()
		
		local FinalNeckBone
		local FirstNeckBone


		if IKFK == 1 do 
		(
			
			if ROM == 1 then (ROM = false) else (ROM = true)
			if NumBones <3 do (NumBones = 3)
			

			local NeckIKFKChainOutput = FinalTools.MakeIKFKSpine TempNeckPoint TempHeadPoint  0 false Scaling Squash 1 NumBones 0.5 undefined 3 1 SpineBend:SplineBendInputArray  Prefix:ToolSetup.NamePrefix MainBonesName:NeckBoneName UpNodes:ROM SquashBlending:2 
			for i = 1 to i = ((NeckIKFKChainOutput[7].count)-1) do
			(
			CommonFns.ApplyLengthWidthProperties NeckIKFKChainOutput[7][i] (NeckStartWidth*0.8) ObjFrontWidth:(NeckStartFrontWidth*0.8) ObjOffset:undefined
			)	
			
			NeckBonesArray = NeckIKFKChainOutput[7]

			local HeadConnectPoint = NeckIKFKChainOutput[7][(NeckIKFKChainOutput[7].count)]
			in coordsys world HeadController.pos = HeadConnectPoint.transform.pos
			
			HeadBone  = boneSys.createBone HeadConnectPoint.transform.pos TempHeadEndPoint.transform.pos [0,0,1]
			CommonFns.ApplyLengthWidthProperties HeadBone (HeadWidth*0.8) ObjFrontWidth:(HeadFrontWidth*0.8) ObjOffset:undefined
			
			CommonFns.quickAlign HeadBone TempHeadPoint
			in coordsys world HeadBone.transform.pos = HeadConnectPoint.transform.pos
			HeadBone.name = (uniquename HeadBoneName numdigits:3)
			HeadBone.Width = TinySize
			HeadBone.Height = TinySize
			HeadBone.sidefins = off
			HeadBone.frontfin = off
			HeadBone.backfin = off
			
			append Allcreated Headbone
			
			for i = 1 to i = NeckIKFKChainOutput[7].count do
			(
				NeckIKFKChainOutput[7][i].Width = TinySize
				NeckIKFKChainOutput[7][i].Height = TinySize
				NeckIKFKChainOutput[7][i].FrontFin = off
				NeckIKFKChainOutput[7][i].BackFin = off
				NeckIKFKChainOutput[7][i].SideFins = off
			)
			HeadBone.parent = HeadConnectPoint
			
			delete TempNeckPoint
			delete TempHeadPoint
			delete TempHeadEndPoint
			
			local HeadControlNull = Point Name:(uniquename (Nullifiername + "Head_Ctrl") NumDigits:3) Size:Tinysize wireColor: (color 20 80 20)
			CommonFns.quickalign HeadControlNull HeadController
			append Allcreated HeadControlNull
			
			
			local HeadRibcageNoFollowNull = Point Name:(uniquename (Nullifiername + "Head_NoFollowRibcage") NumDigits:3) Size:Tinysize wireColor: (color 20 80 20)
			CommonFns.QuickAlign HeadRibcageNoFollowNull HeadController
			local HeadFollowNull = Point Name:(uniquename (Nullifiername + "Head_FollowRibcage") NumDigits:3) Size:Tinysize wireColor: (color 20 80 20)
			CommonFns.QuickAlign HeadFollowNull HeadController
			append Allcreated HeadRibcageNoFollowNull
			append allcreated HeadFollowNull
			
			
			local MiddleNeckBone = (NumBones/2) as integer
			
			
			
			local NeckController = CommonFns.DrawRectangleControl NeckIKFKChainOutput[7][MiddleNeckBone] NeckIKFKChainOutput[7][MiddleNeckBone] NeckIKFKChainOutput[7][(MiddleNeckBone+1)] (NeckStartWidth*1.3) (NeckStartFrontWidth*1.3) (uniquename NeckControlName numdigits:3)
			CommonFns.ApplycontrollerTypeProperties NeckController ControlType:"Medium" ControlLevel:2
			local NeckControlNull = Point Name:(uniquename (Nullifiername + "Neck_Control") NumDigits:3) Size:Tinysize wireColor: (color 20 80 20)
			CommonFns.QuickAlign NeckControlNull NeckController
			
			append ControlsArray NeckController
			append allcreated NeckControlNull
			
			
			
			
			local BendController = CommonFns.MakeController (BigSize*3) "sphere" NeckIKFKChainOutput[6][2] NeckIKFKChainOutput[6][2] (color 40 120 80) (NeckBendControlName)  "z" "positive"
			local BendControlNull = Point Name:(uniquename (Nullifiername + "Bend_Control") NumDigits:3) Size:(BigSize*2) wireColor: (color 20 80 20) Cross:On
			local BendControlCnsTargetA = Point Name:(uniquename (Nullifiername + "Bend_Control_PosA") NumDigits:3) Size:Tinysize wireColor: (color 20 80 20)
			local BendControlCnsTargetB = Point Name:(uniquename (Nullifiername + "Bend_Control_PosB") NumDigits:3) Size:Tinysize wireColor: (color 20 80 20)

			
			commonFns.QuickAlign BendController NeckIKFKChainOutput[6][2]
			commonFns.QuickAlign BendControlNull BendController
			commonFns.QuickAlign BendControlCnsTargetA BendController
			commonFns.QuickAlign BendControlCnsTargetB BendController
			BendControlCnsTargetA.parent = ParentObj
			BendControlCnsTargetB.parent = HeadController
			
			append allcreated BendControlCnsTargetA
			append allcreated BendControlCnsTargetB
			append allcreated BendControlNull
			
			
			BendControlNull.parent = ParentObj

			--append Allcreated ParentControlNull
			
			NeckIKFKChainOutput[6][2].parent = BendController
			NeckIKFKChainOutput[6][1].parent = BendController
			
			append ControlsArray BendController
			CommonFns.ApplycontrollerTypeProperties BendController ControlType:"Small" ControlLevel:2
			
			BendControlNull.pos.controller = Position_Constraint()
			BendControlNull.pos.controller.appendtarget BendControlCnsTargetA 100
			BendControlNull.pos.controller.appendtarget BendControlCnsTargetB 100
			
			BendController.parent = BendControlNull
			
			
			
			NeckControlNull.parent = parentObj
			NeckController.parent = NeckControlNull
			
			NeckIKFKChainOutput[2][1].parent = parentobj
			HeadFollowNull.parent = NeckController
			HeadBone.rotation.controller = Orientation_constraint()
			HeadBone.rotation.controller.appendtarget HeadController 100
			
			HeadControlNull.rotation.controller = Orientation_Constraint()
			HeadControlNull.rotation.controller.appendtarget HeadFollowNull 100
			HeadControlNull.rotation.controller.appendtarget HeadRibcageNoFollowNull 0
			HeadRibcageNoFollowNull.parent = Root
			NeckIKFKChainOutput[2][3].rotation.controller = orientation_constraint()
			NeckIKFKChainOutput[2][3].rotation.controller.appendtarget HeadController 100
			NeckIKFKChainOutput[2][3].pos.controller = position_constraint()
			NeckIKFKChainOutput[2][3].pos.controller.appendtarget HeadController 100
			
			
			
			
			HeadControlNull.parent = NeckController
			HeadController.parent = HeadControlNull
			
			local ca=attributes NeckBlend
			(
				parameters params rollout:ro_AdjustPivot
				(

				)
				rollout ro_AdjustPivot "Adjust Pivot" 
				(
					button btn_adjustpivot "Adjust Pivot"
					button btn_apply "Apply" enabled:off
					button btn_finalise "Finalise" enabled:off
					
					
					local PivotObject,OrigPivotTransform,boundbox,pivobjectname,trytogetpivot,existingpivot
					
					on btn_adjustpivot pressed do
					(
						boundbox = (in coordsys local nodeLocalBoundingBox $)
						boundbox = (distance boundbox[1] boundbox[2])
						OrigPivotTransform = $.transform
						PivotObject = Point Name:(uniquename "PivotAdjustObject" NumDigits:3) Pos:OrigPivotTransform.pos Size:(boundbox*0.2) Cross:On Box:off CenterMarker:off AxisTripod:off
						PivotObject.transform = $.transform
						setUserProp $ "PivotObjectName" PivotObject.name
						btn_apply.enabled = true
						select PivotObject
					)
					on btn_finalise pressed do
					(
						setUserProp $ "Finalised" "True"
						btn_adjustpivot.enabled = false
						btn_finalise.enabled = false
					)
				on ro_AdjustPivot open do
				(
					trytogetpivot = (getUserProp $ "PivotObjectName")
					try (existingpivot = (getnodebyname trytogetpivot)) catch()
					if existingpivot != undefined do (btn_apply.enabled = true)
					local TestState = (getUserProp $ "Finalised")
						if  (TestState == "True") or (TestState == true) do
						(
							btn_adjustpivot.enabled = false
							btn_apply.enabled = false
							btn_finalise.enabled = false
						)
				)
					on btn_apply pressed do
					(
						local ChildrenOrigPosArray = #()
						local OrigPos
						local origOffset
						for n in $.children do (append ChildrenOrigPosArray n.transform)
						OrigOffset = ($.transform * inverse existingpivot.transform)
						Origpos = $.transform
						in coordsys world $.parent.transform = existingpivot.transform
						for c = 1 to c = ChildrenOrigPosArray.count do (($.children[c]).transform = ChildrenOrigPosArray[c])
						
						
						--$.objectoffsetrot += (origOffset).rotation
						
						setUserProp $ "PivotObjectName" "undefined"
						delete existingpivot
						
					)
				)
			)
			local AttHolder = (EmptyModifier())
			AttHolder.name = "Neck_Pivot"
			addmodifier NeckController AttHolder
			custAttributes.add NeckController.modifiers[#Neck_Pivot] ca	
			
			
			
			
			
			
			local ca=attributes HeadControls
			(
				parameters params rollout:ro_HeadControls
				(
					paramfollow type:#float ui:spn_follow
					parambendfollow type:#float ui:spn_bendfollow
					paramsquash type:#float ui:spn_squash
				)
				rollout ro_HeadControls "Head_Controls" 
				(
					spinner spn_follow "Follow Ribcage" enabled:on  range:[0,100,1] scale:1
					spinner spn_bendfollow "Auto Neckbend" enabled:on  range:[0,100,1] scale:1
					spinner spn_squash "Neck Squash Enabled" enabled:on  range:[0,3,1] scale:1	visible:false
				)
				
				on ro_HeadControls open do 
				(
					if getuserprop $ "RigToolkit_showSquash" == "show" do
					(
						spn_squash.visible = true
					)
				)
				
			)
			AttHolder = (EmptyModifier ())
			AttHolder.name = "Head_Controls"
			addmodifier HeadController AttHolder
			custAttributes.add HeadController.modifiers[#Head_Controls] ca
				
			NeckIKFKChainOutput[2][3].pos.controller.appendtarget HeadController 100
			
			if Squash == 2 do
			(
				for i = 1 to i = ((NeckIKFKChainOutput[7].count)-1) do 
				(
					
					NeckIKFKChainOutput[7][i].scale.controller.weight[NeckIKFKChainOutput[7][i].scale.controller.count].controller.addscalartarget  "SliderValue" HeadController.modifiers[#Head_Controls].HeadControls[#paramsquash]
					
					--NeckIKFKChainOutput[7][i].scale.controller.weight[2].controller.addscalartarget  "SliderValue"HeadController.modifiers[#Head_Controls].HeadControls[#paramsquash]
					--$.scale.controller.weight[1].controller
					--.scale.controller.weight.controller = Float_Expression ()
					
				)
				--COMEHERE now
			)

				
			BendControlNull.pos.controller.weight[2].controller = float_expression()
			BendControlNull.pos.controller.weight[2].controller.AddScalarTarget "Slider" HeadController.modifiers[#Head_Controls].HeadControls[#parambendfollow]
			BendControlNull.pos.controller.weight[2].controller.setexpression ("Slider")
			BendControlNull.pos.controller.weight[1].controller = float_expression()
			BendControlNull.pos.controller.weight[1].controller.AddScalarTarget "Slider" HeadController.modifiers[#Head_Controls].HeadControls[#parambendfollow]
			BendControlNull.pos.controller.weight[1].controller.setexpression ("100-Slider")

			local ScriptToApply = float_expression()
			ScriptToApply.AddScalarTarget "Slider" HeadController.modifiers[#Head_Controls].HeadControls[#paramfollow]
			ScriptToApply.setexpression ("Slider")
				
			HeadControlNull.rotation.controller.weight[1].controller = ScriptToApply
			ScriptToApply = copy ScriptToApply
			ScriptToApply.setexpression "100 - Slider"
			HeadControlNull.rotation.controller.weight[2].controller = ScriptToApply
			
			BendController.objectoffsetpos += [0,Bigsize*4,0]				
				
			FinalNeckBone = NeckIKFKChainOutput[7][(NeckIKFKChainOutput[7].count)]
			FirstNEckBone = NeckIKFKChainOutput[7][1]
		)
		
		-----------------------Make the FK Neck------------------------------------------------
		

		
		
		if IKFK == 2 do
		(

			local ss,Midpoint,GuideSpline
				
			
			ss = SplineShape pos:TempNeckPoint.Transform.pos name:(uniquename "NeckSpline" Numdigits:3) color:(color 20 140 70)
			ss.optimize = false
			ss.steps = 12
			addNewSpline ss
			addKnot ss 1 #beziercorner #curve TempNeckPoint.transform.pos TempNeckPoint.Transform.pos NeckBendPoint.pos
			addKnot ss 1 #beziercorner #curve TempHeadPoint.transform.pos NeckBendPoint.pos TempHeadPoint.Transform.pos
			updateShape ss
			ss
			GuideSpline = ss
			

			
			local TempAlignmentPoint = CommonFns.CreateSystemAlignmentPoint TempNeckPoint TempHeadPoint
			
			if NumBones >1 then (local BaseIncrement = 100/NumBones) else (local BaseIncrement = 100)
			
			for i = 1 to i = (NumBones+1) do
			(
				local PointToAdd = Point Name:(Uniquename "ZZZZZTEMPFKNECKPOINT" Numdigits:3)
				PointToAdd.pos.controller = path_constraint()
				PointToAdd.pos.controller.path = GuideSpline
				PointToAdd.pos.controller.percent = ((i-1)*BaseIncrement)
				append TempFKPointsArray PointToAdd
				
				if i != 1 do
				(
					local BoneToAdd = BoneSys.createbone TempFKPointsArray[(i-1)].transform.pos TempFKPointsArray[(i)].transform.pos [0,0,1]
					BoneToAdd.name = (Uniquename NeckBoneName NumDigits:3)
					CommonFns.ApplyLengthWidthProperties BoneToAdd (NeckStartWidth*0.8) ObjFrontWidth:(NeckStartFrontWidth*0.8) ObjOffset:undefined
					BoneToAdd.Width = TinySize
					BoneToAdd.Height = TinySize
					BoneToAdd.FrontFin = off
					BoneToAdd.BackFin = off
					BoneToAdd.SideFins = off
					append allcreated BoneToAdd
					append FKNeckBonesArray BoneToAdd
					append NeckBonesArray BoneToAdd
				)
			)
				
			for i = 1 to i = ((TempFKPointsArray.count)-1) do
			(
				local PointLookCons = Lookat_Constraint()
				PointLookCons.appendtarget TempFKPointsArray[(i+1)] 100
				PointLookCons.target_axis = 0
				PointLookCons.upnode_ctrl = 1
				PointLookCons.upnode_world = off
				PointLookCons.pickUpNode = TempAlignmentPoint
				PointLookCons.StoUP_axis = 1
				PointLookCons.upnode_axis = 1
				PointLookCons.viewline_length_abs = off
				PointLookCons.lookat_vector_length = 0
				TempFKPointsArray[i].rotation.controller = PointLookCons
				CommonFns.QuickAlign FKNeckBonesArray[i] TempFKPointsArray[i]
				if i != 1 do (FKNeckBonesArray[i].parent = FKNeckBonesArray[(i-1)])
			)
			
				local FinalPointLookCons = Lookat_Constraint()
				FinalPointLookCons.appendtarget TempHeadPoint 100
				FinalPointLookCons.target_axis = 0
				FinalPointLookCons.upnode_ctrl = 1
				FinalPointLookCons.upnode_world = off
				FinalPointLookCons.pickUpNode = TempAlignmentPoint
				FinalPointLookCons.StoUP_axis = 1
				FinalPointLookCons.upnode_axis = 1
				FinalPointLookCons.viewline_length_abs = off
				FinalPointLookCons.lookat_vector_length = 0
				TempFKPointsArray[TempFKPointsArray.count].rotation.controller = FinalPointLookCons
			
			HeadBone  = boneSys.createBone TempHeadPoint.transform.pos TempHeadEndPoint.transform.pos [0,0,1]
			CommonFns.ApplyLengthWidthProperties HeadBone (HeadWidth*0.8) ObjFrontWidth:(HeadFrontWidth*0.8) ObjOffset:undefined
			CommonFns.QuickAlign HeadBone TempFKPointsArray[TempFKPointsArray.count]
			HeadBone.name = (uniquename HeadBoneName numdigits:3)
			append allcreated HeadBone
			in coordsys world HeadController.pos = HeadBone.transform.pos
			HeadBone.Width = TinySize
			HeadBone.Height = TinySize
			HeadBone.sidefins = off
			HeadBone.frontfin = off
			HeadBone.backfin = off
			
			HeadBone.parent = FKNeckBonesArray[(FKNeckBonesArray.count)]
			

			local HeadControlNull = Point Name:(uniquename (Nullifiername + "Head_Ctrl") NumDigits:3) Size:Tinysize wireColor: (color 20 80 20)
			CommonFns.quickalign HeadControlNull HeadController
			append Allcreated HeadControlNull
			
			
			local HeadRibcageNoFollowNull = Point Name:(uniquename (Nullifiername + "Head_NoFollowRibcage") NumDigits:3) Size:Tinysize wireColor: (color 20 80 20)
			CommonFns.QuickAlign HeadRibcageNoFollowNull HeadController
			local HeadFollowNull = Point Name:(uniquename (Nullifiername + "Head_FollowRibcage") NumDigits:3) Size:Tinysize wireColor: (color 20 80 20)
			CommonFns.QuickAlign HeadFollowNull HeadController
			append Allcreated HeadRibcageNoFollowNull
			append allcreated HeadFollowNull
			
			---------------------------------------------
			

			
			for i = 1 to i = NumBones do
			(
				local NeckControllerToAdd = CommonFns.DrawRectangleControl FKNeckBonesArray[i] TempFKPointsArray[i] TempFKPointsArray[(i+1)] (NeckStartWidth*1.3) (NeckStartFrontWidth*1.3) (uniquename NeckControlName numdigits:3)
				CommonFns.ApplycontrollerTypeProperties NeckControllerToAdd ControlType:"Medium" ControlLevel:2
				append NeckControllersArray NeckControllerToAdd
				
				local NeckNullToAdd =  Point Name:(uniquename (Nullifiername + "Neck_Control") NumDigits:3) Size:Tinysize wireColor: (color 20 80 20)
				append NeckNullsArray NeckNullToAdd
				CommonFns.Quickalign NeckNullToAdd NeckControllerToAdd
				NeckControllerToAdd.parent = NeckNullToAdd
				if i != 1 do (NeckNullToAdd.parent = NeckControllersArray[(i-1)])
				if i == 1 do (NeckNullToAdd.parent = parentObj)
					
				FKNeckBonesArray[i].rotation.controller = orientation_constraint()
				FKNeckBonesArray[i].rotation.controller.appendtarget NeckControllerToAdd 100
				append ControlsArray NeckControllerToAdd
				append allcreated NeckControllerToAdd
				append allcreated NeckNullToAdd
			)
			
			local FinalNeckController = NeckControllersArray[(NeckControllersArray.count)]
			
			
			HeadFollowNull.parent = FinalNeckController
			HeadBone.rotation.controller = Orientation_constraint()
			HeadBone.rotation.controller.appendtarget HeadController 100
			
			HeadControlNull.rotation.controller = Orientation_Constraint()
			HeadControlNull.rotation.controller.appendtarget HeadFollowNull 100
			HeadControlNull.rotation.controller.appendtarget HeadRibcageNoFollowNull 0
			HeadRibcageNoFollowNull.parent = Root
			
			HeadControlNull.parent = FinalNeckController
			HeadController.parent = HeadControlNull
			
			
			local ca=attributes HeadControls
			(
				parameters params rollout:ro_HeadControls
				(
					paramfollow type:#float ui:spn_follow
				)
				rollout ro_HeadControls "Head_Controls" 
				(
					spinner spn_follow "Follow Ribcage" enabled:on  range:[0,100,1] scale:1
				)
			)
			AttHolder = (EmptyModifier ())
			AttHolder.name = "Head_Controls"
			addmodifier HeadController AttHolder
			custAttributes.add HeadController.modifiers[#Head_Controls] ca

			local ScriptToApply = float_expression()
			ScriptToApply.AddScalarTarget "Slider" HeadController.modifiers[#Head_Controls].HeadControls[#paramfollow]
			ScriptToApply.setexpression ("Slider")
				
			HeadControlNull.rotation.controller.weight[1].controller = ScriptToApply
			ScriptToApply = copy ScriptToApply
			ScriptToApply.setexpression "100 - Slider"
			HeadControlNull.rotation.controller.weight[2].controller = ScriptToApply
			
			FinalNeckBone = FKNeckBonesArray[FKNeckBonesArray.count]
			firstNeckBone = FKNeckBonesArray[1]
			
			
			
			
			
			
			
			for n in TempFKPointsArray do (delete n)
			delete GuideSpline
			delete TempAlignmentPoint
			
			delete TempNeckPoint
			delete TempHeadPoint
			delete TempHeadEndPoint

			FKNeckBonesArray[1].parent = ParentObj
			
		)---this bracket to close IKFK == 2 loop
			
		------------------------------------Make Trapezius Bones------------------------------	
			
		local TrapBonesArray = #()
			
		if (LeftShoulder != undefined) and (RightShoulder != undefined) and (TrapBones != 1)  do
		(

			local TrapRNull = Point Name:(uniquename (Nullifiername + "Trap_Right") NumDigits:3) Size:Tinysize wireColor: (color 20 80 20)
			local TrapLNull = Point Name:(uniquename (Nullifiername + "Trap_Left") NumDigits:3) Size:Tinysize wireColor: (color 20 80 20)
			local TrapLookat = Point Name:(uniquename (Nullifiername + "Trap_Lookat_R") NumDigits:3) Size:Tinysize wireColor: (color 20 80 20)
			local TrapLookatL = Point Name:(uniquename (Nullifiername + "Trap_Lookat_L") NumDigits:3) Size:Tinysize wireColor: (color 20 80 20)	
			
			append Allcreated TrapRNull
			append Allcreated TrapLNull
			append Allcreated TrapLookat
			append Allcreated TrapLookatL	
				
			CommonFns.QuickAlign TrapRNull RightTrapLocation
			CommonFns.QuickAlign TrapLNull LeftTrapLocation
			CommonFns.QuickAlign TrapLookat RightTrapNubLocation
			CommonFns.QuickAlign TrapLookatL LeftTrapNubLocation	
			
			
				
			TrapRNull.parent = RightShoulder
			TrapLNull.parent = LeftShoulder
			
			TrapLookat.parent = firstNeckBone
			TrapLookatL.parent =firstNeckBone
				
			if TrapBones == 3 then
			(
				
				local TrapRBone = boneSys.createBone TrapRNull.transform.pos TrapLookat.transform.pos [0,0,1]
				TrapRBone.name = (uniquename (TrapBoneName + "R") Numdigits:3)
				local TrapRNub = boneSys.createBone TrapLookat.transform.pos TrapLookat.transform.pos [0,0,1]
				TrapRNub.name = (uniquename (Prefix + "Nub_Trap_R") Numdigits:3	)
				local TrapLBone = boneSys.createBone TrapLNull.transform.pos TrapLookatL.transform.pos [0,0,1]
				TrapLBone.name = (uniquename (TrapBoneName + "L") Numdigits:3	)
				local TrapLNub = boneSys.createBone TrapLookatL.transform.pos TrapLookatL.transform.pos [0,0,1]
				TrapLNub.name = (uniquename (Prefix + "Nub_Trap_L") Numdigits:3	)
				TrapLNub.parent = TrapLBone
				TrapRNub.parent = TrapRBone
				TrapLNub.height = 0
				TrapLNub.width = 0
				TrapRNub.Height = 0
				TrapRNub.Width = 0
				append Allcreated TrapRNub
				append Allcreated TrapLNub
			)
			else
			(
				local TrapSize = ((Distance TrapRNull TrapLookat) *0.3)
				local TrapRBone = boneSys.createBone [0,0,0] [0,0,TrapSize] [0,0,1]
				TrapRBone.name = (uniquename (TrapBoneName + "R") Numdigits:3)
				local TrapLBone = boneSys.createBone [0,0,0] [0,0,TrapSize] [0,0,1]
				TrapLBone.name = (uniquename (TrapBoneName + "L") Numdigits:3	)
			)
			
				
				append Allcreated TrapRBone
				append Allcreated TrapLBone
			
			TrapBonesArray = #(TrapRBone,TrapLBone)
			
			for i = 1 to i = 2 do
			(
			TrapBonesArray[i].height = TinySize
			TrapBonesArray[i].width = TinySize
			TrapBonesArray[i].Wirecolor = color 180 90 90
			TrapBonesArray[i].frontfin = off
			TrapBonesArray[i].backfin = off
			TrapBonesArray[i].sidefins =off
			TrapBonesArray[i].boneFreezeLength=false
			TrapBonesArray[i].boneScaleType=#scale
			)			

			
			
			TrapRBone.Parent = TrapRNull
			TrapLBone.parent = TrapLNull
			
			local TrapLookCons = Lookat_Constraint()
			TrapLookCons.appendtarget TrapLookat 100
			TrapLookCons.target_axis = 0
			TrapLookCons.upnode_ctrl = 1
			TrapLookCons.upnode_world = off
			TrapLookCons.pickUpNode = RightShoulder
			TrapLookCons.StoUP_axis = 2
			TrapLookCons.upnode_axis = 2
			TrapLookCons.viewline_length_abs = off
			TrapLookCons.lookat_vector_length = 0
			TrapRBone.rotation.controller = TrapLookCons
			TrapLookCons = copy TrapLookCons
			local TrapLookCons = Lookat_Constraint()
			TrapLookCons.appendtarget TrapLookatL 100
			TrapLookCons.target_axis = 0
			TrapLookCons.upnode_ctrl = 1
			TrapLookCons.upnode_world = off
			TrapLookCons.pickUpNode = LeftShoulder
			TrapLookCons.StoUP_axis = 2
			TrapLookCons.upnode_axis = 2
			TrapLookCons.viewline_length_abs = off
			TrapLookCons.lookat_vector_length = 0
			TrapLBone.rotation.controller = TrapLookCons
			
			if TrapBones == 3 then
			(
				local posCons = Position_Constraint()
				PosCons.appendtarget TrapLookat 100
				TrapRNub.pos.controller = poscons
				
				local posConsL = position_constraint()
				PosConsL.appendtarget TrapLookatL 100
				TrapLNub.pos.controller = posConsL
			)
			else
			(
				local posCons = Position_Constraint()
				PosCons.appendtarget TrapLookat 100
				PosCons.appendtarget TrapRNull 100
				TrapRBone.pos.controller = poscons
				
				local posConsL = Position_Constraint()
				PosConsL.appendtarget TrapLookatL 100
				PosConsL.appendtarget TrapLNull 100
				TrapLBone.pos.controller = posconsL
			)
			---------------------------------------------------------------------------------------------------------------------
			
			)
			
			
			
			FinalTools.appendobjstolayer Allcreated HiddenComponents:True
			FinalTools.appendobjstolayer NeckBonesArray HiddenComponents:False MainBones:True
			FinalTools.appendobjstolayer HeadBone HiddenComponents:False MainBones:True
			FinalTools.appendobjstolayer ControlsArray HiddenComponents:False Controllers:True
			if TrapBonesArray.count != 0 do (FinalTools.appendobjstolayer TrapBonesArray HiddenComponents:False DeformerBones:True)
			
			
			if IKFK == 1 do (for i = 1 to i = NeckIKFKChainOutput[1].count do (append allcreated NeckIKFKChainOutput[1][i]))
			
			
			local OutputArray = #()
			append OutputArray allcreated
			append OutputArray ControlsArray
			append OutputArray NeckBonesArray
			append OutputArray HeadBone
			append OutputArray TrapBonesArray
			OutputArray
				--OutputArray Containts:
				
				--1: all created
				--2: HeadController,NeckController
				--3: NeckBones
				--4: HeadBone
				--5: Trap Bones
			
			
			
				--1: Allcreated
				--2: GlobalController, Start Controller, EndController
				--3: FK Controllers (or empty array)
				--4:Reverse FK Controllers (or empty array)
				--5: Deformation fin controllers (or empty array )
				--6: StartTangent, EndTangent, StartBendController, EndBendController
				--7:MainBones
		
	),
	
	fn CreateSpaceSwitch ApplyTo ParentListArray NameListArray Prefix:"" ApplySwitchTo:Undefined =
	(
		
	local NullSize = "0.02"
	for i = 1 to i = 4 do (if NameListArray.count < 5 do (append NameListArray "UndefinedName"))
	
	if ApplySwitchto == undefined do (ApplySwitchTo = ApplyTo)
	
	local allcreated = #()
		
	
	local SwitchNullName = Prefix + "null_SpaceSwitchParent_" + ApplyTo.Name
	local DefaultLocName = Prefix + "null_SpaceSwitchParent_Default" 
	
		
	if ParentListArray.count > 0 do (local ParALocName = Prefix + "null_SpaceSwitchLocator" + NameListArray[1])
	if ParentListArray.count > 1 do (local ParBLocName = Prefix + "null_SpaceSwitchLocator" + NameListArray[2])
	if ParentListArray.count > 2 do (local ParCLocName = Prefix + "null_SpaceSwitchLocator" + NameListArray[3])	
	if ParentListArray.count > 3 do (local ParDLocName = Prefix + "null_SpaceSwitchLocator" + NameListArray[4])	
	if ParentListArray.count > 4 do (local ParELocName = Prefix + "null_SpaceSwitchLocator" + NameListArray[5])	
		
	local SwitchNull = Point Pos:[0,0,0] Name:(Uniquename SwitchNullName Numdigits:3) Box:On Cross:Off AxisTripod:Off Centermarker:Off WireColor:(color 250 250 200)
	local DefaultLoc = Point Pos:[0,0,0] Name:(Uniquename DefaultLocName Numdigits:3) Box:On Cross:Off AxisTripod:Off Centermarker:Off WireColor:(color 250 250 200)
	CommonFns.QuickAlign DefaultLoc ApplyTo
	CommonFns.QuickAlign SwitchNull ApplyTo
		
	append Allcreated SwitchNull
	append Allcreated DefaultLoc
		
	DefaultLoc.Parent = ApplyTo.parent
	SwitchNull.parent = ApplyTo.parent
	ApplyTo.Parent = SwitchNull
		
	SwitchNull.rotation.controller = orientation_constraint()
	SwitchNull.pos.controller = position_constraint()
	
	SwitchNull.rotation.controller.appendtarget DefaultLoc 100
	SwitchNull.pos.controller.appendtarget DefaultLoc 100
		
	local ParALoc,ParBLoc,ParCLoc,ParDLoc,ParELoc
		
	if ParentListArray.count > 0 do 
	(
		ParALoc = Point Pos:[0,0,0] Name:(Uniquename ParALocName Numdigits:3) Box:On Cross:Off AxisTripod:Off Centermarker:Off WireColor:(color 250 250 200)
		CommonFns.quickalign ParALoc ApplyTo
		ParALoc.parent = ParentListArray[1]
		SwitchNull.rotation.controller.appendtarget ParALoc 100
		SwitchNull.pos.controller.appendtarget ParALoc 100
		append Allcreated ParALoc
	)
		if ParentListArray.count > 1 do 
	(
		ParBLoc = Point Pos:[0,0,0] Name:(Uniquename ParBLocName Numdigits:3) Box:On Cross:Off AxisTripod:Off Centermarker:Off WireColor:(color 250 250 200)
		CommonFns.quickalign ParBLoc ApplyTo
		ParBLoc.parent = ParentListArray[2]
		SwitchNull.rotation.controller.appendtarget ParBLoc 100
		SwitchNull.pos.controller.appendtarget ParBLoc 100
		append Allcreated ParBLoc
	)
		if ParentListArray.count > 2 do 
	(
		ParCLoc = Point Pos:[0,0,0] Name:(Uniquename ParCLocName Numdigits:3) Box:On Cross:Off AxisTripod:Off Centermarker:Off WireColor:(color 250 250 200)
		CommonFns.quickalign ParCLoc ApplyTo
		ParCLoc.parent = ParentListArray[3]
		SwitchNull.rotation.controller.appendtarget ParCLoc 100
		SwitchNull.pos.controller.appendtarget ParCLoc 100
		append Allcreated ParCLoc
	)
		if ParentListArray.count > 3 do 
	(
		ParDLoc = Point Pos:[0,0,0] Name:(Uniquename ParDLocName Numdigits:3) Box:On Cross:Off AxisTripod:Off Centermarker:Off WireColor:(color 250 250 200)
		CommonFns.quickalign ParDLoc ApplyTo
		ParDLoc.parent = ParentListArray[4]
		SwitchNull.rotation.controller.appendtarget ParDLoc 100
		SwitchNull.pos.controller.appendtarget ParDLoc 100
		append Allcreated ParDLoc
	)
		if ParentListArray.count > 4 do 
	(
		ParELoc = Point Pos:[0,0,0] Name:(Uniquename ParELocName Numdigits:3) Box:On Cross:Off AxisTripod:Off Centermarker:Off WireColor:(color 250 250 200)
		CommonFns.quickalign ParELoc ApplyTo
		ParDLoc.parent = ParentListArray[5]
		SwitchNull.rotation.controller.appendtarget ParELoc 100
		SwitchNull.pos.controller.appendtarget ParELoc 100
		append Allcreated ParELoc
	)
	

	setUserProp applyswitchto "ParAName" NameListArray[1]
	setUserProp applyswitchto "ParBName" NameListArray[2]
	setUserProp applyswitchto "ParCName" NameListArray[3]
	setUserProp applyswitchto "ParDName" NameListArray[4]
	setUserProp applyswitchto "ParEName" NameListArray[5]
	
	
	local ca=attributes SpaceSwitchControls
			(
				parameters params rollout:ro_SpaceSwitchControls
				(
						paramDefault type:#float ui:spn_Default
						paramParA type:#float ui:spn_ParA
						paramParB type:#float ui:spn_ParB
						paramParC type:#float ui:spn_ParC	
						paramParD type:#float ui:spn_ParD		
						paramParE type:#float ui:spn_ParE		
						paramTestString type:#string 
				)
				
				rollout ro_SpaceSwitchControls "SpaceSwitch_Controls" 
				(
					spinner spn_Default "Default" enabled:on  range:[0,100,100] scale:0.5
					spinner spn_ParA "Parent A" enabled:on  range:[0,100,0] scale:0.5	
					spinner spn_ParB "Parent B" enabled:on  range:[0,100,0] scale:0.5 visible:on
					spinner spn_ParC "Parent C" enabled:on  range:[0,100,0] scale:0.5 visible:on
					spinner spn_ParD "Parent D" enabled:on  range:[0,100,0] scale:0.5 visible:on
					spinner spn_ParE "Parent E" enabled:on range:[0,100,0] scale:0.5 visible:on
					button btn_Copy "Copy Obj Pos" enabled:true
					button btn_Paste "PasteObj Pos" enabled:true
					button btn_KeyAll "Key Sliders" enabled:true
					button btn_RemoveKeys "UnKey Sliders" enabled:true
					local CopyMatrix
					
					
					on ro_SpaceSwitchControls open do
					(
					spn_ParA.caption = (getuserprop $ "ParAName")
					spn_ParB.caption = (getuserprop $ "ParBName")	
					spn_ParC.caption = (getuserprop $ "ParCName")	
					spn_ParD.caption = (getuserprop $ "ParDName")	
					spn_ParE.caption = (getuserprop $ "ParEName")		
					if spn_ParB.caption == "UndefinedName" do (spn_ParB.visible = off)
					if spn_ParC.caption == "UndefinedName" do (spn_ParC.visible = off)	
					if spn_ParD.caption == "UndefinedName" do (spn_ParD.visible = off)		
					if spn_ParE.caption == "UndefinedName" do (spn_ParE.visible = off)
					)
					
					
					
					on btn_Copy pressed do 
					(CopyMatrix = $.transform)
					
					on btn_paste pressed do 
					(
						try
						(
						$.rotation *= $.transform.rotation * (inverse CopyMatrix.rotation)
						in coordsys world $.pos = CopyMatrix.pos
						)
						catch
						()
					)
					on btn_KeyAll pressed do 
					(
						try
						(
						--addNewKey $.modifiers[#SpaceSwitch_Controls].ro_SpaceSwitchControls.spn_Default SliderTime 
						with animate on
						(
						$.modifiers[#SpaceSwitch_Controls].ro_SpaceSwitchControls.spn_Default.value +=0
						$.modifiers[#SpaceSwitch_Controls].ro_SpaceSwitchControls.spn_ParA.value +=0
						$.modifiers[#SpaceSwitch_Controls].ro_SpaceSwitchControls.spn_ParB.value +=0	
						$.modifiers[#SpaceSwitch_Controls].ro_SpaceSwitchControls.spn_ParC.value +=0
						$.modifiers[#SpaceSwitch_Controls].ro_SpaceSwitchControls.spn_ParD.value +=0	
						$.modifiers[#SpaceSwitch_Controls].ro_SpaceSwitchControls.spn_ParE.value +=0		
						)

						)
						catch
						()
					)
					on btn_RemoveKeys pressed do 
					(
						try
						(
							local DefaultKeyIndex = getKeyIndex $.modifiers[#SpaceSwitch_Controls].SpaceSwitchControls.paramdefault.controller slidertime
							local ParAKeyIndex = getKeyIndex $.modifiers[#SpaceSwitch_Controls].SpaceSwitchControls.paramParA.controller slidertime
							local ParBKeyIndex = getKeyIndex $.modifiers[#SpaceSwitch_Controls].SpaceSwitchControls.paramParB.controller slidertime
							local ParCKeyIndex = getKeyIndex $.modifiers[#SpaceSwitch_Controls].SpaceSwitchControls.paramParC.controller slidertime
							local ParDKeyIndex = getKeyIndex $.modifiers[#SpaceSwitch_Controls].SpaceSwitchControls.paramParD.controller slidertime
							local ParEKeyIndex = getKeyIndex $.modifiers[#SpaceSwitch_Controls].SpaceSwitchControls.paramParE.controller slidertime
							try (DeleteKey  $.modifiers[#SpaceSwitch_Controls].SpaceSwitchControls.paramdefault.controller DefaultKeyIndex) catch()
							try (DeleteKey  $.modifiers[#SpaceSwitch_Controls].SpaceSwitchControls.paramParA.controller ParAKeyIndex) catch()
							try (DeleteKey  $.modifiers[#SpaceSwitch_Controls].SpaceSwitchControls.paramParB.controller ParBKeyIndex) catch()
							try (DeleteKey  $.modifiers[#SpaceSwitch_Controls].SpaceSwitchControls.paramParC.controller ParCKeyIndex) catch()
							try (DeleteKey  $.modifiers[#SpaceSwitch_Controls].SpaceSwitchControls.paramParD.controller ParDKeyIndex) catch()	
							try (DeleteKey  $.modifiers[#SpaceSwitch_Controls].SpaceSwitchControls.paramParE.controller ParEKeyIndex) catch()
						)
						catch
						()
					)
					

				)
			)
			
			local AttHolder = (EmptyModifier())
			AttHolder.name = "SpaceSwitch_Controls"

			
			
			addmodifier applyswitchto AttHolder
				
			
				
			custAttributes.add ApplySwitchTo.modifiers[#SpaceSwitch_Controls] ca
			ApplySwitchTo.modifiers[#SpaceSwitch_Controls].ro_SpaceSwitchControls.spn_Default.value = 100
			
			SwitchNull.rotation.controller.weight[1].controller = Float_Expression ()
			SwitchNull.rotation.controller.weight[1].controller.AddScalarTarget "SliderValue" ApplySwitchTo.modifiers[#SpaceSwitch_Controls].SpaceSwitchControls[#paramdefault]
			SwitchNull.rotation.controller.weight[1].controller.SetExpression "SliderValue"
			SwitchNull.pos.controller.weight[1].controller = Float_Expression ()
			SwitchNull.pos.controller.weight[1].controller.AddScalarTarget "SliderValue" ApplySwitchTo.modifiers[#SpaceSwitch_Controls].SpaceSwitchControls[#paramdefault]
			SwitchNull.pos.controller.weight[1].controller.SetExpression "SliderValue"
			
			if ParentListArray.count > 0 do 
		(
			SwitchNull.rotation.controller.weight[2].controller = Float_Expression ()
			SwitchNull.rotation.controller.weight[2].controller.AddScalarTarget "SliderValue" ApplySwitchTo.modifiers[#SpaceSwitch_Controls].SpaceSwitchControls[#paramParA]
			SwitchNull.rotation.controller.weight[2].controller.SetExpression "SliderValue"
			SwitchNull.pos.controller.weight[2].controller = Float_Expression ()
			SwitchNull.pos.controller.weight[2].controller.AddScalarTarget "SliderValue" ApplySwitchTo.modifiers[#SpaceSwitch_Controls].SpaceSwitchControls[#paramParA]
			SwitchNull.pos.controller.weight[2].controller.SetExpression "SliderValue"
		)
			if ParentListArray.count > 1 do 
		(
			SwitchNull.rotation.controller.weight[3].controller = Float_Expression ()
			SwitchNull.rotation.controller.weight[3].controller.AddScalarTarget "SliderValue" ApplySwitchTo.modifiers[#SpaceSwitch_Controls].SpaceSwitchControls[#paramParB]
			SwitchNull.rotation.controller.weight[3].controller.SetExpression "SliderValue"
			SwitchNull.pos.controller.weight[3].controller = Float_Expression ()
			SwitchNull.pos.controller.weight[3].controller.AddScalarTarget "SliderValue" ApplySwitchTo.modifiers[#SpaceSwitch_Controls].SpaceSwitchControls[#paramParB]
			SwitchNull.pos.controller.weight[3].controller.SetExpression "SliderValue"
			applyswitchto.modifiers[#SpaceSwitch_Controls].ro_SpaceSwitchControls.spn_parB.visible = true
		)
		
			if ParentListArray.count > 2 do 
		(
			SwitchNull.rotation.controller.weight[4].controller = Float_Expression ()
			SwitchNull.rotation.controller.weight[4].controller.AddScalarTarget "SliderValue" ApplySwitchTo.modifiers[#SpaceSwitch_Controls].SpaceSwitchControls[#paramParC]
			SwitchNull.rotation.controller.weight[4].controller.SetExpression "SliderValue"
			SwitchNull.pos.controller.weight[4].controller = Float_Expression ()
			SwitchNull.pos.controller.weight[4].controller.AddScalarTarget "SliderValue" ApplySwitchTo.modifiers[#SpaceSwitch_Controls].SpaceSwitchControls[#paramParC]
			SwitchNull.pos.controller.weight[4].controller.SetExpression "SliderValue"
			applyswitchto.modifiers[#SpaceSwitch_Controls].ro_SpaceSwitchControls.spn_parC.visible = true
		)
			if ParentListArray.count > 3 do 
		(
			SwitchNull.rotation.controller.weight[5].controller = Float_Expression ()
			SwitchNull.rotation.controller.weight[5].controller.AddScalarTarget "SliderValue" ApplySwitchTo.modifiers[#SpaceSwitch_Controls].SpaceSwitchControls[#paramParD]
			SwitchNull.rotation.controller.weight[5].controller.SetExpression "SliderValue"
			SwitchNull.pos.controller.weight[5].controller = Float_Expression ()
			SwitchNull.pos.controller.weight[5].controller.AddScalarTarget "SliderValue" ApplySwitchTo.modifiers[#SpaceSwitch_Controls].SpaceSwitchControls[#paramParD]
			SwitchNull.pos.controller.weight[5].controller.SetExpression "SliderValue"
			applyswitchto.modifiers[#SpaceSwitch_Controls].ro_SpaceSwitchControls.spn_parD.visible = true
		)
			if ParentListArray.count > 4 do 
		(
			SwitchNull.rotation.controller.weight[6].controller = Float_Expression ()
			SwitchNull.rotation.controller.weight[6].controller.AddScalarTarget "SliderValue" ApplySwitchTo.modifiers[#SpaceSwitch_Controls].SpaceSwitchControls[#paramParE]
			SwitchNull.rotation.controller.weight[6].controller.SetExpression "SliderValue"
			SwitchNull.pos.controller.weight[6].controller = Float_Expression ()
			SwitchNull.pos.controller.weight[6].controller.AddScalarTarget "SliderValue" ApplySwitchTo.modifiers[#SpaceSwitch_Controls].SpaceSwitchControls[#paramParE]
			SwitchNull.pos.controller.weight[6].controller.SetExpression "SliderValue"
			applyswitchto.modifiers[#SpaceSwitch_Controls].ro_SpaceSwitchControls.spn_parE.visible = true
		)
	
	
	
			FinalTools.AppendObjsToLayer Allcreated HiddenComponents:True MainBones:False Controllers:False AnimGeo:False
			
	
	
	
	
	--Create Switch Null. Align it to control
	
	--Create DefaultNull, Align it to Default parent (or controller if there is no parent). parent to default parent or nothing.
		
	--Create ParA Null, align to control
	--ParB, parC etc.
		
		
	),--this bracket to close space switch rollout
	-----------------------------------------------------------------------------------------------------------
	fn AppendObjsToLayer Objs HiddenComponents:True MainBones:False DeformerBones:False Controllers:False AnimGeo:False =
	(
		local Prefix = ToolSetup.NamePrefix
		
		local HiddenComponentsName = Prefix + "Hidden_Rig_Components"
		local MainBonesName = Prefix + "Main_Bones"
		local ControllersName = Prefix + "Controllers"
		local AnimGeoName = Prefix + "AnimGeo"
		local DeformerBonesName = Prefix + "DeformerBones"
			
		if HiddenComponents == false and MainBones == false and Controllers == False and AnimGeo == false do (HiddenComponents == true)
		
		local HiddenComponentsLayer = LayerManager.getLayerFromName HiddenComponentsName
		if HiddenComponentsLayer == undefined do 
		(
			HiddenComponentsLayer = LayerManager.newLayer()
			HiddenComponentsLayer.setname HiddenComponentsName
		)
		local MainBonesLayer = LayerManager.getLayerFromName MainBonesName
		if MainBonesLayer == undefined do 
		(
			MainBonesLayer = LayerManager.newLayer()
			MainBonesLayer.setname MainBonesName
		)
		local ControllersLayer = LayerManager.getLayerFromName ControllersName
		if ControllersLayer == undefined do 
		(
			ControllersLayer = LayerManager.newLayer()
			ControllersLayer.setname ControllersName
		)
		local AnimGeoLayer = LayerManager.getLayerFromName AnimGeoName
		if AnimGeoLayer == undefined do 
		(
			AnimGeoLayer = LayerManager.newLayer()
			AnimGeoLayer.setname AnimGeoName
		)
		local DeformerBonesLayer = LayerManager.getLayerFromName DeformerBonesName
		if DeformerBonesLayer == undefined do 
		(
			DeformerBonesLayer = LayerManager.newLayer()
			DeformerBonesLayer.setname DeformerBonesName
		)
		
		
		
		
		local ApplyToLayer
		

		if HiddenComponents == true do (ApplyToLayer = HiddenComponentsLayer)
		if MainBones == true do (ApplyToLayer = MainBonesLayer)
		if Controllers == true do (ApplyToLayer = ControllersLayer)
		if  AnimGeo == true do (ApplyToLayer =  AnimGeoLayer)
		if  DeformerBones == true do (ApplyToLayer =  DeformerBonesLayer)
		if classof Objs != array do (Objs = #(Objs))
			
		HiddenComponentsLayer.on=false
		
		try
		(	
		for n in (objs) do ApplyToLayer.addnode n
		)
		catch
		()
	),--this bracket to close append objs to layer function
	--------------------------------------------------------------------------
	fn CreateAnimGeo ApplyTo SwitchToggle:undefined ParentSkin:1 SimpleSmooth:1 Prefix:undefined =
	(
		local StartOffset = 0.1
		local EndOffset = 0.1
		local BaseName = Prefix + "AnimGeo"
		--Input: an array of bones. requires the custom user properties to function properly
		--getuserprop $ BoneLength
		--getuserprop $ BoneWidth
		--getuserprop $ BoneFrontWidth
		--getuserprop $ ObjOffset
		
		for i = 1 to i = ApplyTo.count do
		(
			local CurrentBoneWidth = undefined
			local CurrentBone = ApplyTo[i]
			local CurrentBoneWidth = (getuserprop ApplyTo[i] "BoneWidth")
			local CurrentBoneFrontWidth = (getuserprop ApplyTo[i] "BoneFrontWidth")
			local CurrentBoneLength = (getuserprop ApplyTo[i] "BoneLength")
			local CurrentObjOffset = (getuserprop ApplyTo[i] "ObjOffset")
			
			if CurrentBoneWidth != undefined and CurrentBoneWidth !="undefined" do
			(
				local CurrentGeoObj = Cylinder Radius:1 Height:(CurrentBoneLength * (1-(StartOffset + EndOffset))) Name:(Uniquename BaseName NumDigits:3) HeightSegs:1 Sides:10 Wirecolor:(color 220 220 220)
				local CurrentGeoPosOffset = (CurrentBoneLength * StartOffset)
				converttopoly CurrentGeoObj
				CurrentGeoObj.objectoffsetrot = (quat 0 1 0 -1)
				CommonFns.quickalign CurrentGeoObj CurrentBone
				CurrentGeoObj.parent = CurrentBone
				in coordsys local move CurrentGeoObj [CurrentGeoPosOffset,0,0]
				CommonFns.quickalign CurrentGeoObj CurrentBone	
				AppendObjsToLayer CurrentGeoObj HiddenComponents:False MainBones:False DeformerBones:False Controllers:False AnimGeo:True
				in coordsys local scale CurrentGeoObj [1,CurrentBoneWidth,CurrentBoneFrontWidth]
				if currentObjOffset != undefined do (CurrentGeoObj.objectoffsetpos = currentobjoffset)
				ResetXForm CurrentGeoObj
				converttopoly CurrentGeoObj
				
				
				if ParentSkin == 2 do
				(
					local theskin
					Select CurrentGeoObj
					theskin = Skin()
					addmodifier $ theskin
					max modify mode
					skinOps.addBone $.modifiers[#Skin] CurrentBone 1
					$.modifiers[#Skin].filter_vertices = on
					
				)
			)

			
			--Make the object
			--Skin it or parent it
			--Apply it to AnimGeo Layer
		)
	),--this bracket to close make anim geo function
	
	fn CreateMuscleGuide StartPoint EndPoint LookatPoint UpnodeBone BoneDirection:1 MuscleName:"Muscle"  UpNodeAxis:1 MidPointOffset:1 =
	(
		--Bone Direction: 1- Central to Outer, 2- Outer to central, 3:Start to end, 4: End to start
		
		local Allcreated = #()
		
		
		
		
		local AverageScale = ((Distance StartPoint EndPoint) + (Distance StartPoint LookatPoint))/2
		
		local TinySize = (AverageScale *0.1)
		local MediumSize = (AverageScale * 0.2)
		local BigSize = (AverageScale * 0.4)
		
		If classof StartPoint != Matrix3 do (StartPoint = StartPoint.transform)
		if classof EndPoint != Matrix3 do (EndPoint = EndPoint.transform)
		if classof LookatPoint != Matrix3 do (LookatPoint = LookatPoint.transform)
		
		
		local TempStartPoint = Point Name:"ZZZTEMPSTARTPOINT"
		CommonFns.QuickAlign TempStartPoint StartPoint
		local TempEndPoint = Point Name:"ZZZTEMPENDPOINT"
		CommonFns.QuickAlign TempEndPoint EndPoint
		local TempLookatPoint = Point Name:"ZZZTEMPLOOKATPOINT"
		CommonFns.QuickAlign TempLookatPoint LookatPoint
		
		

		
		local StartLocator = CommonFns.MakeController (MediumSize) "sphere" TempStartPoint TempStartPoint (color 160 40 80) (MuscleName + "_Start_Locator") "z" "positive"
		CommonFns.QuickAlign StartLocator TempStartPoint
		local EndLocator = CommonFns.MakeController (MediumSize) "sphere" TempEndPoint TempEndPoint (color 160 40 80) (MuscleName + "_End_Locator") "z" "positive"
		CommonFns.QuickAlign EndLocator TempEndPoint
		local MidPointLocator = CommonFns.MakeController (MediumSize) "sphere" TempLookatPoint TempLookatPoint (color 160 40 80) (MuscleName + "_Mid_Locator") "z" "positive"
		local LookatLocator = CommonFns.MakeController (MediumSize) "triangle" TempLookatPoint TempLookatPoint (color 40 180 160) (MuscleName + "_Lookat_Locator") "x" "positive"
		
		
		append allcreated StartLocator
		append allcreated MidPointLocator
		append allcreated EndLocator
		append allcreated LookatLocator
		
		local TempBaseLookCons = lookat_constraint()
		TempBaseLookCons.target_axis = 0
		TempBaseLookCons.upnode_ctrl = 1
		TempBaseLookCons.upnode_world = off
		TempBaseLookCons.StoUP_axis = UpNodeAxis
		TempBaseLookCons.upnode_axis = UpNodeAxis
		
		TempBaseLookCons.viewline_length_abs = off
		TempBaseLookCons.lookat_vector_length = 0
		TempBaseLookCons.StoUP_axisFlip = off
		
		local LookPointLookCons = copy TempBaseLookCons
		LookPointLookCons.viewline_length_abs = off
		LookPointLookCons.lookat_vector_length = 0	
		LookPointLookCons.appendtarget StartLocator 55
		LookPointLookCons.appendtarget EndLocator 45
		LookPointLookCons.pickUpNode = UpnodeBone	

		
		local MidPointLookCons = copy TempBaseLookCons
		MidPointLookCons.appendtarget LookatLocator 100
		MidPointLookCons.target_axis = 1
		MidPointLookCons.StoUP_axis = 0
		MidPointLookCons.upnode_axis = 0
		MidPointLookCons.pickUpNode = UpnodeBone
		MidPointLocator.rotation.controller = MidPointLookCons


		
		
		LookatLocator.rotation.controller = LookPointLookCons
		CommonFns.QuickAlign MidPointLocator LookatLocator
		MidPointLocator.parent = LookatLocator
		in coordsys LookatLocator move MidPointLocator [((BigSize*2)*MidPointOffset),0,0]
		

		
		local StartBone = BoneSys.CreateBone MidPointLocator.transform.pos StartLocator.transform.pos [0,0,1]
		StartBone.Name = (MuscleName + "_StartBone")
		local StartBoneNub = BoneSys.CreateBone StartLocator.transform.pos StartLocator.transform.pos [0,0,1]
		StartBoneNub.Name = (MuscleName + "_StartBoneNub")
		StartBoneNub.parent = StartBone
		
		local EndBone = BoneSys.CreateBone MidPointLocator.transform.pos EndLocator.transform.pos [0,0,1]
		EndBone.Name = (MuscleName + "_EndBone")
		local EndBoneNub = BoneSys.CreateBone EndLocator.transform.pos EndLocator.transform.pos [0,0,1]
		EndBoneNub.Name = (MuscleName + "_EndBoneNub")
		EndBoneNub.parent = EndBone
		
		append Allcreated StartBone
		append Allcreated EndBone
		append Allcreated StartBoneNub
		append Allcreated EndBoneNub
		
		--Bone Direction:
		
		local StartBoneParent,EndBoneParent,StartBoneLookat,EndBoneLookat
		
		
		if BoneDirection == 1 then
		(
			--1- Central to Outer,
			StartBoneparent = MidPointLocator
			EndBoneparent = MidPointLocator
			
			StartBoneLookat = StartLocator
			EndBoneLookat = EndLocator
		)
		else if BoneDirection == 2 then
		(
			--2- Outer to central, 
			StartBoneparent = StartLocator
			EndBoneparent = EndLocator
			
			StartBoneLookat = MidPointLocator
			EndBoneLookat = MidPointLocator
		)
		else if BoneDirection == 3 then
		(
			--3:Start to end, 
			StartBoneparent = StartLocator
			EndBoneparent = MidPointLocator
			StartBoneLookat = MidPointLocator
			EndBoneLookat = EndLocator
		)
		else
		(
			--4: End to start
			StartBoneparent = MidPointLocator
			EndBoneparent = EndLocator
			
			StartBoneLookat = StartLocator
			EndBoneLookat = MidPointLocator
		)
		
		CommonFns.QuickAlign StartBone StartBoneParent
		StartBone.parent = StartBoneParent
		CommonFns.QuickaLign EndBone EndBoneParent
		EndBone.parent = EndBoneParent
		
		
		local StartLookCons = copy TempBaseLookCons
		StartLookCons.appendtarget StartBoneLookat 100
		StartLookCons.pickUpNode = MidPointLocator

		local EndLookCons = copy TempBaseLookCons
		EndLookCons.appendtarget EndBoneLookat 100
		EndLookCons.pickUpNode = MidPointLocator
		
		StartBone.rotation.controller = StartLookCons
		EndBone.rotation.controller = EndLookCons
		
		StartBoneNub.pos.controller= position_constraint()
		StartBoneNub.pos.controller.appendtarget StartBoneLookat 100
		
		EndBoneNub.pos.controller= position_constraint()
		EndBoneNub.pos.controller.appendtarget EndBoneLookat 100
		
		
		for n in #(StartBone,EndBone,StartBoneNub,EndBoneNub) do
		(
			n.wirecolor = color 180 60 60
			n.sidefins= off
			n.frontfin = off
			n.backfin = off
			n.width = tinysize
			n.height = tinysize
			n.boneFreezeLength=false
			n.boneScaleType= #scale
		)
		StartBoneNub.ishidden = true
		EndBoneNub.ishidden = true
		
		local PosconsPoint = Point Name: "ZZZZPOSCONSGUIDEPOINT"
		PosConsPoint.pos.controller = position_constraint()
		PosConsPoint.pos.controller.appendtarget MidPointLocator 100
		PosConsPoint.rotation.controller = Orientation_constraint()
		PosConsPoint.rotation.controller.appendtarget UpnodeBone 100
		PosConsPoint.ishidden = true
		
		local PosConsPointA = Point Name: "ZZZPOSCONSPOINTA"
		CommonFns.QuickAlign PosConsPointA PosConsPoint
		PosConsPointA.parent = PosConsPoint
		PosConsPointA.ishidden = true
		
		local PosConsPointB = Point Name: "ZZZPOSCONSPOINTB"
		CommonFns.QuickAlign PosConsPointB PosConsPoint
		PosConsPointB.parent = PosConsPoint
		PosConsPointB.ishidden = true
		
		in coordsys Parent Move PosConsPointA [-TinySize,0,0]
		in coordsys Parent Move PosConsPointB [TinySize,0,0]	
			
		delete TempStartPoint
		delete TempEndPoint
		delete TempLookatPoint
		
		append allcreated PosconsPoint
		append allcreated PosConsPointA
		append allcreated PosConsPointB
		
		
		local OutputArray = #()
		
		append OutputArray allcreated
		append OutPutArray #(StartLocator,EndLocator,MidPointLocator,LookatLocator)
		append OutputArray #(StartBone,EndBone)
		append OutputArray #(PosConsPointA,PosConsPointB)
		
		--OutputArray Contains:
		
		--1:Allcreated
		--2: StartLocator,EndLocator,MidPointLocator,LoookatLocator
		--3:StartBone,EndBone
		--4:PosConsPointA,PosConsPointB
		
		--CreateMuscleGuide StartPoint EndPoint LookatPoint UpnodeBone BoneDirection:1 MuscleName:"Muscle"  UpNodeAxis:1
		
		OutputArray
		
	),--This bracket to close createmuscleguide
	
	fn CreateMuscle  StartLocation EndLocation MidPointLocation LookatLocation PosConsPointALocation PosConsPointBLocation StartBoneLocation EndBoneLocation UpNodeBone EndPointConnection Prefix:"" MuscleName:"Muscle" BoneDirection:1 Scaling:true UpNodeAxis:1 =
	(
		--Bone Direction: 1- Central to Outer, 2- Outer to central, 3:Start to end, 4: End to start
		
		local ScaleType
		if Scaling == True then (ScaleType = #scale) else (ScaleType = #none)
		
		
		local Allcreated = #()
		
		local ParentBone = UpNodeBone

		local AverageScale = ((Distance StartLocation.pos EndLocation.pos) + (Distance StartLocation.pos LookatLocation.pos))/2
		
		local TinySize = (AverageScale *0.1)
		local MediumSize = (AverageScale * 0.2)
		local BigSize = (AverageScale * 0.4)
		
		
		local StartLocatorName = (Prefix + "subctrl_start_" + MuscleName)
		local EndLocatorName = (Prefix + "subctrl_end_" + MuscleName)
		local MidPointLocatorName = (Prefix + "subctrl_midpoint_" + MuscleName)
		local PosConsPointAName = (Prefix + "subctrl_midpointA_" + MuscleName)
		local PosConsPointBName = (Prefix + "subctrl_midpointB_" + MuscleName)
		local LookatLocatorName = (Prefix + "subctrl_lookatpoint_" + MuscleName)
		local StartBoneName = (Prefix + "bn_" + MuscleName + "_Start")
		local EndBoneName = (Prefix + "bn_" + MuscleName + "_End")
		local MidPointParentName = (Prefix + "nullifier_" + MuscleName + "_midpoint")
		local StartBoneNubName = (Prefix + "nub_" + MuscleName + "_Start")
		local EndBoneNubName = (Prefix + "nub_" + MuscleName + "_End")
		
		---------------------------------------------------------
		
		local TempPoint = Point Name:"ZZZZZZTEMPPOINT"
		
		local StartLocator = CommonFns.MakeController (TinySize) "sphere" TempPoint TempPoint (color 160 40 80) (StartLocatorName) "z" "positive"
		CommonFns.QuickAlign StartLocator StartLocation
		StartLocator.parent = ParentBone
		
		local EndLocator = CommonFns.MakeController (TinySize) "sphere" TempPoint TempPoint (color 160 40 80) (EndLocatorName) "z" "positive"
		CommonFns.QuickAlign EndLocator EndLocation
		EndLocator.parent = EndPointConnection
		
		
		local MidPointLocator = CommonFns.MakeController (TinySize) "triangle" TempPoint TempPoint (color 160 40 80) (MidPointLocatorName) "y" "positive"
		CommonFns.QuickAlign MidPointLocator MidPointLocation
		
		local LookatLocator = CommonFns.MakeController (TinySize) "triangle" TempPoint TempPoint (color 40 180 160) (LookatLocatorName) "x" "positive"
		CommonFns.QuickAlign LookatLocator LookatLocation
		
		local TempBaseLookCons = lookat_constraint()
		TempBaseLookCons.target_axis = 0
		TempBaseLookCons.upnode_ctrl = 1
		TempBaseLookCons.upnode_world = off
		TempBaseLookCons.StoUP_axis = UpNodeAxis
		TempBaseLookCons.upnode_axis = UpNodeAxis
		TempBaseLookCons.viewline_length_abs = off
		TempBaseLookCons.lookat_vector_length = 0
		TempBaseLookCons.StoUP_axisFlip = off
		
		local LookPointLookCons = copy TempBaseLookCons
		LookPointLookCons.viewline_length_abs = off
		LookPointLookCons.lookat_vector_length = 0	
		LookPointLookCons.appendtarget StartLocator 55
		LookPointLookCons.appendtarget EndLocator 45
		LookPointLookCons.pickUpNode = UpnodeBone	
		
		LookatLocator.rotation.controller = LookPointLookCons
		LookatLocator.parent = ParentBone
		MidPointLocator.parent = LookatLocator
		
		local PosConsPointALocator = CommonFns.MakeController (TinySize) "sphere" TempPoint TempPoint (color 160 40 80) (PosConsPointAName) "z" "positive"
		CommonFns.QuickAlign PosConsPointALocator PosConsPointALocation
		PosConsPointALocator.parent = UpNodeBone
		
		
		local PosConsPointBLocator = CommonFns.MakeController (TinySize) "sphere" TempPoint TempPoint (color 160 40 80) (PosConsPointBName) "z" "positive"
		CommonFns.QuickAlign PosConsPointBLocator PosConsPointBLocation
		PosConsPointBLocator.parent =  LookatLocator
		CommonFns.QuickAlign PosConsPointBLocator MidPointLocator RotOnly:True
		

		local MidPointParent = Point Name:(uniquename MidpointparentName)  Box:off Cross:off Axistripod:off CenterMarker:on 
		CommonFns.Quickalign MidpointParent MidPointLocation
		
		MidPointParent.pos.controller = Position_Constraint()
		MidPointParent.pos.controller.appendtarget PosConsPointALocator 40
		MidPointParent.pos.controller.appendtarget PosConsPointBLocator 60
		MidPointParent.parent = UpodeBone
		MidPointLocator.parent = MidPointParent
		
		
		local StartBone = bonesys.createbone StartLocator.transform.pos MidPointLocator.transform.pos [0,0,1]
		CommonFns.Quickalign StartBone StartBoneLocation
		local StartBoneNub = bonesys.createbone [0,0,0] [TinySize,0,0] [0,0,1]
		StartBoneNub.length = TinySize
		StartBoneNub.Name = (uniquename StartBoneNubName NumDigits:3)
		StartBone.Name = (uniquename StartBoneName NumDigits:3)
		
		local EndBone = bonesys.createbone EndLocator.transform.pos MidPointLocator.transform.pos [0,0,1]
		CommonFns.Quickalign EndBone EndBoneLocation
		local EndBoneNub = bonesys.createbone [0,0,0] [TinySize,0,0] [0,0,1]
		EndBoneNub.length = TinySize
		EndBoneNub.Name = (uniquename EndBoneNubName NumDigits:3)
		
		
		EndBone.Name = (uniquename EndBoneName NumDigits:3)
		
		delete TempPoint

		append allcreated StartLocator
		append allcreated MidPointLocator
		append allcreated EndLocator
		append allcreated PosConsPointALocator
		append allcreated PosConsPointBLocator
		append allcreated LookatLocator
		append allcreated MidPointParent
		append allcreated StartBone
		append allcreated EndBone
		append Allcreated StartBoneNub
		append Allcreated EndBoneNub
		--------------------------------------------------------------------------------------------------------------
		--Bone Direction:
		if BoneDirection == 1 then
		(
			--1- Central to Outer,
			StartBoneparent = MidPointLocator
			EndBoneparent = MidPointLocator
			
			StartBoneLookat = StartLocator
			EndBoneLookat = EndLocator
		)
		else if BoneDirection == 2 then
		(
			--2- Outer to central, 
			StartBoneparent = StartLocator
			EndBoneparent = EndLocator
			
			StartBoneLookat = MidPointLocator
			EndBoneLookat = MidPointLocator
		)
		else if BoneDirection == 3 then
		(
			--3:Start to end, 
			StartBoneparent = StartLocator
			EndBoneparent = MidPointLocator
			StartBoneLookat = MidPointLocator
			EndBoneLookat = EndLocator
		)
		else
		(
			--4: End to start
			StartBoneparent = MidPointLocator
			EndBoneparent = EndLocator
			
			StartBoneLookat = StartLocator
			EndBoneLookat = MidPointLocator
		)
		
		StartBone.parent = StartBoneParent
		EndBone.parent = EndBoneParent
		
		
		local StartLookCons = copy TempBaseLookCons
		StartLookCons.appendtarget StartBoneLookat 100
		StartLookCons.pickUpNode = PosConsPointBLocator

		local EndLookCons = copy TempBaseLookCons
		EndLookCons.appendtarget EndBoneLookat 100
		EndLookCons.pickUpNode = PosConsPointBLocator
		
		StartBone.rotation.controller = StartLookCons
		EndBone.rotation.controller = EndLookCons
		
		StartBoneNub.pos.controller= position_constraint()
		StartBoneNub.pos.controller.appendtarget StartBoneLookat 100
		
		EndBoneNub.pos.controller= position_constraint()
		EndBoneNub.pos.controller.appendtarget EndBoneLookat 100
		
		StartBoneNub.parent = StartBone
		EndBoneNub.parent = EndBone
		
		for n in #(StartBone,EndBone,StartBoneNub,EndBoneNub) do
		(
			n.wirecolor = color 180 60 60
			n.sidefins= off
			n.frontfin = off
			n.backfin = off
			n.width = tinysize
			n.height = tinysize
			n.boneFreezeLength=false
			n.boneScaleType= ScaleType
			n.resetbonestretch
			
		)
		

		
		
		
		for n in #(StartBoneNub,EndBoneNub) do
		(
			n.width = tinysize*0.2
			n.height = tinysize*0.2
			n.boneFreezeLength=true
			n.boneScaleType= #none
			
		)
		
		local OutputArray = #()
		
		append OutputArray allcreated
		
		append OutputArray #(StartLocator,EndLocator,MidPointLocator,LookatLocator)
		append OutputArray #(StartBone,StartBoneNub,EndBone,EndBoneNub)
		append OutputArray #(PosConsPointA,PosConsPointB,MidPointParent)
		
		--Output Array Contains:
		
		--1: Allcreated
		--2: (StartLocator,EndLocator,MidPointLocator,LookatLocator)
		--3: (StartBone,StartBoneNub,EndBone,EndBoneNub)
		--4: (PosConsPointA,PosConsPointB,MidPointParent)
		
		
		--fn CreateMuscle  StartLocation EndLocation MidPointLocation LookatLocation PosConsPointALocation PosConsPointBLocation StartBoneLocation EndBoneLocation UpNodeBone EndPointConnection Prefix:"" MuscleName:"Muscle" BoneDirection:1 Scaling:true UpNodeAxis:1 =

		
		
		
		OutputArray
	)--This bracket to close create muscle
	
	) --THIS BRACKET TO CLOSE FINAL TOOLS ROLLOUT

	
	CommonFns = CommonFns()
	ToolSetup = ToolSetup()
	FinalTools = FinalTools()

	---------------------------------------------------------------------------------------------------
	-----------------ROLLOUT DEFINITIONS AND EVENT HANDLERS----------------------
	
		
	rollout IKFKSpineRollout "Make IK/FK Spine" width:197 height:724
	(
		groupBox grp1 "Start/End Points" pos:[10,16] width:175 height:88
		button btn_PickStart "Pick" pos:[87,36] width:90 height:27 enabled:true
		groupBox grp6 "Joint Scaling" pos:[9,264] width:171 height:81
		radioButtons rdo_jointScaling "" pos:[17,286] width:134 height:48 labels:#("Always On (faster)", "On/Off Blend (versatile)", "Always Off (faster)") default:2 columns:1
		checkbox chk_squashStretch "Include Squash/Stretch" pos:[25,461] width:150 height:21 checked:true
		button btn_create "Create" pos:[13,675] width:75 height:28
		button btn_undo "Undo" pos:[96,676] width:78 height:27
		groupBox grp10 "FK Controllers" pos:[11,185] width:171 height:73
		spinner spn_amountControls "" pos:[93,207] width:50 height:16 range:[0,100,1] scale:1
		label lbl2 "Label" pos:[146,232] width:0 height:0
		label lbl3 "FK controls:" pos:[26,207] width:64 height:18
		button btn_PickEnd "Pick" pos:[88,70] width:90 height:27 enabled:true
		label lbl4 "Start Point:" pos:[20,41] width:69 height:18
		label lbl5 "End Point:" pos:[26,75] width:63 height:18
		checkbox chk_addreverse "Add Reverse FK Controls" pos:[20,229] width:156 height:22
		groupBox grp19 "Bone Size Ratio" pos:[10,525] width:174 height:96
		spinner spn_numbones "" pos:[96,640] width:50 height:16 range:[3,50,3] scale:1
		label lbl13 "Number of bones:" pos:[17,634] width:61 height:32
		radioButtons rdo_boneEase "" pos:[21,540] width:67 height:48 labels:#("Linear", "Ease in", "Ease Out") columns:1
		spinner spn_ease "" pos:[54,594] width:50 height:16 range:[0.1,0.9,0.1] scale:0.1
		label lbl14 "Ease:" pos:[21,596] width:25 height:11
		groupBox grp26 "No-Scale Position" pos:[10,351] width:169 height:82
		radioButtons rdo_noscaleposition "" pos:[18,374] width:158 height:48 labels:#("Always Top (faster)", "Top/Bottom Blend (versatile)", "Always Bottom (faster)") default:2 columns:1
		groupBox grp27 "Squash/Stretch" pos:[10,439] width:170 height:82
		radioButtons rdo_squashtype "" pos:[36,483] width:85 height:32 labels:#("Fixed Value", "Slider Control") default:2 columns:1
		groupBox grp34 "Orientation" pos:[10,109] width:176 height:71
		radioButtons rdo_orientation "" pos:[19,135] width:58 height:32 labels:#("Auto", "Custom") default:1 columns:1
		button btn_pickOri "Pick" pos:[93,136] width:82 height:27 enabled:false
				
		local PickedStart,PickedEnd,FKControlsAmount,NewState,PickedOri
		
		-----------------------------------------------------------------------PICKING THE TWO OBJECTS--------------------------------------------------------------------------
		
		
		
		
		on rdo_jointscaling changed NewState do
		(
			if NewState == 1 then
			(
				rdo_noscaleposition.enabled = false


			)
			else if NewState == 2 then
			(
				rdo_noscaleposition.enabled = true
				chk_squashstretch.enabled = true
				chk_squashstretch.checked = true
				rdo_squashtype.enabled = true
			)
			else
			(
				rdo_noscaleposition.enabled = true
				chk_squashstretch.enabled = false
				rdo_squashtype.enabled = false
			)
		)
		
		on chk_squashstretch changed NewState do
		(
			if NewState == off then
			(
				rdo_squashtype.enabled = false
			)
			else
			(
				rdo_squashtype.enabled = true
			)
		)
		
		
		on rdo_orientation changed NewState do
		(
			if NewState == 1 then
			(
				btn_pickOri.enabled = false
				btn_pickOri.caption = "Pick"
				PickedOri = undefined
				
			)
			else
			(
			btn_pickOri.enabled = true
			)
		)
		
		on btn_pickOri pressed do
		(
			local TryToPickOri
			PickedOri = undefined
			TryToPickOri = undefined
			TryToPickOri = pickobject prompt:"PickOri" rubberBand:[0,0,0] forceListenerFocus:false count:1
			if classof TrytoPickOri != Undefined and TryToPickOri != #() and classof TrytoPickOri != UndefinedClass then
			(
				btn_PickOri.Caption = TryToPickOri.name
				PickedOri = TryToPickOri
			)
			else
			(
				btn_PickOri.Caption = "Pick"
				PickedOri = undefined
			)
		)
		
		
		
		on btn_PickStart pressed do
		(
			local TryToPickStart
			TryToPickStart = undefined
			TryToPickStart = pickobject prompt:"PickStart" rubberBand:[0,0,0] forceListenerFocus:false count:1
			if classof TrytoPickStart != Undefined and TryToPickStart != #() and classof TrytoPickStart != UndefinedClass then
			(
				btn_PickStart.Caption = TryToPickStart.name
				PickedStart = TryToPickStart
			)
			else
			(
				btn_PickStart.Caption = "Pick"
				PickedStart = undefined
			)
		)
		
		on btn_PickEnd pressed do
		(
			local TryToPickEnd
			TryToPickEnd = undefined
			TryToPickend = pickobject prompt:"PickStart" rubberBand:[0,0,0] forceListenerFocus:false count:1
			if classof TrytoPickEnd != Undefined and TryToPickEnd != #() and classof TrytoPickEnd != UndefinedClass then
			(
				btn_PickEnd.Caption = TryToPickEnd.name
				PickedEnd = TryToPickEnd
			)
			else
			(
				btn_PickEnd.Caption = "Pick"
				PickedEnd = undefined
			)
		)
		-----------------------------------------------------------MAIN BUTTON PRESS----------------------------------------------------------------------------------
		on btn_create pressed do
		(
		FinalTools.MakeIKFKSpine PickedStart PickedEnd spn_amountControls.value chk_addreverse.checked rdo_jointScaling.state chk_squashStretch.checked rdo_boneEase.state spn_Numbones.value spn_ease.value PickedOri rdo_Noscaleposition.state rdo_squashtype.state
		)
		
	)--this bracket to close IKFKRollout

	--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
	------------------------------------------------------------ CREATE SPINE GUIDE ROLLOUT----------------------------------------------------------------------------
		rollout CreateSpineRollout "Create Spine" width:229 width:229 height:588
		(
			bitmap bmp2 "Bitmap" pos:[0,0] width:229 height:588 fileName:"RigToolKit_CreateSpine.png"
			button btn_pickstart "Pick" pos:[119,85] width:78 height:21
			button btn_pickend "Pick" pos:[118,112] width:78 height:21
			label lbl2 "Bottom of pelvis:" pos:[27,88] width:81 height:20
			label lbl3 "Top of ribcage:" pos:[31,115] width:81 height:20
			groupBox grp1 "Start/End Points" pos:[10,66] width:201 height:108
			button btn_createguide "Create Layout Guide" pos:[49,142] width:116 height:22
			radioButtons rdo_jointscaling "" pos:[22,203] width:134 height:48 labels:#("Always on (faster)", "On/Off Blend (versatile)", "Always Off (faster)") Columns:1
			groupBox grp2 "Joint Scaling" pos:[12,182] width:199 height:79
			groupBox grp46 "No-Scale Position" pos:[11,265] width:199 height:77
			radioButtons rdo_noscalepos "" pos:[21,286] width:158 height:48 labels:#("Always Top (faster)", "Top/Bottom Blend (versatile)", "Always Bottom (faster)") default:1 Columns:1
			checkbox chk_advdeform "Create Abdomen Fins" pos:[31,483] width:154 height:20 checked:true
			button btn_create "Create" pos:[58,554] width:82 height:25 enabled:off
			button btn_pickConnect "Pick" pos:[105,527] width:78 height:21
			label lbl44c "Connect To:" pos:[31,528] width:68 height:20
			groupBox grp4 "Range of Motion" pos:[10,344] width:199 height:56
			radioButtons rdo_rom "" pos:[19,363] width:86 height:32 labels:#("Low (Faster)", "High (Slower)") default:1 Columns:1
			groupBox grp5 "Squash and stretch" pos:[10,404] width:199 height:75
			radioButtons rdo_squash "" pos:[19,423] width:134 height:48 labels:#("None", "Always On (Faster)", "On/Off Blend (versatile)") default:1 Columns:1 enabled:on
			spinner spn_numbones "Num Bones:" pos:[48,504] width:106 height:16 range:[3,30,5] type:#integer


		local PickedStart,PickedEnd,spineGuideOutput,PickedConnect,NewState

		on CreateSpineRollout close do
		(
			try
			(
			local WM_SYSCOMMAND = 0x112
			local SC_RESTORE = 0xf120

			--http://forums.cgsociety.org/archive/index.php/t-644118.html --Minimise/Maximise rollout code snippet from this post.

			local OpenDialogs = UIAccessor.GetPopupDialogs()
			local DialogTitles = for d in OpenDialogs collect (UIAccessor.GetWindowText d)
			local DialogIndex = findItem DialogTitles "Rigging Toolkit"
			local MyHandle = OpenDialogs[DialogIndex]

			local MaxWindows = windows.getChildrenHWND 0 parent:#max
			local WindowTitles = for w in MaxWindows collect w[5]
			local WindowIndex = findItem WindowTitles "Rigging Toolkit"
			MyHandle = MaxWindows[WindowIndex][1]
			windows.sendMessage MyHandle WM_SYSCOMMAND SC_RESTORE 0
			)
			catch()
		)
		
		on CreateSpineRollout open do
		(
			if ToolSetup.Turbomode == on do
			(
				
				if ToolSetup.LastPickedRoot != undefined do 
				(
					btn_PickConnect.caption = ToolSetup.lastpickedRoot.name
					PickedConnect = ToolSetup.lastpickedRoot
				)
			)
			try
			(
				local TryToPickStart
				TryToPickStart = undefined
				TryToPickStart = pickobject prompt:"PickStart" rubberBand:[0,0,0] forceListenerFocus:false count:1
				if classof TrytoPickStart != Undefined and TryToPickStart != #() and classof TrytoPickStart != UndefinedClass then
				(
					btn_PickStart.Caption = TryToPickStart.name
					PickedStart = TryToPickStart
				)
				else
				(
					btn_PickStart.Caption = "Pick"
					PickedStart = undefined
				)
				if TryToPickStart != undefined do
				(
					local TryToPickEnd
					TryToPickEnd = undefined
					TryToPickend = pickobject prompt:"PickStart" rubberBand:[0,0,0] forceListenerFocus:false count:1
					if classof TrytoPickEnd != Undefined and TryToPickEnd != #() and classof TrytoPickEnd != UndefinedClass then
					(
						btn_PickEnd.Caption = TryToPickEnd.name
						PickedEnd = TryToPickEnd
					)
					else
					(
						btn_PickEnd.Caption = "Pick"
						PickedEnd = undefined
					)
				)
			)
			catch()
			

			
		)
		
		on rdo_jointscaling changed NewState do
		(
			if NewState == 1 then
			(
				rdo_noscalepos.enabled = false
				rdo_squash.enabled = true
			)
			else if NewState == 2 then
			(
				rdo_noscalepos.enabled = true
				rdo_squash.enabled = true
			)
			else
			(
				rdo_noscalepos.enabled = true
				rdo_squash.enabled = false
			)
		)
		
		
		
		
		on btn_PickConnect pressed do
		(
			local TryToPickConnect
			TryToPickConnect = undefined
			TryToPickConnect = pickobject prompt:"PickConnect" rubberBand:[0,0,0] forceListenerFocus:false count:1
			if classof TrytoPickConnect != Undefined and TryToPickConnect != #() and classof TrytoPickConnect != UndefinedClass then
			(
				btn_PickConnect.Caption = TryToPickConnect.name
				PickedConnect = TryToPickConnect
			)
			else
			(
				btn_PickConnect.Caption = "Pick"
				PickedConnect = undefined
			)
		)
		
		on btn_PickStart pressed do
		(
			local TryToPickStart
			TryToPickStart = undefined
			TryToPickStart = pickobject prompt:"PickStart" rubberBand:[0,0,0] forceListenerFocus:false count:1
			if classof TrytoPickStart != Undefined and TryToPickStart != #() and classof TrytoPickStart != UndefinedClass then
			(
				btn_PickStart.Caption = TryToPickStart.name
				PickedStart = TryToPickStart
			)
			else
			(
				btn_PickStart.Caption = "Pick"
				PickedStart = undefined
			)
		)
		
		on btn_PickEnd pressed do
		(
			local TryToPickEnd
			TryToPickEnd = undefined
			TryToPickend = pickobject prompt:"PickStart" rubberBand:[0,0,0] forceListenerFocus:false count:1
			if classof TrytoPickEnd != Undefined and TryToPickEnd != #() and classof TrytoPickEnd != UndefinedClass then
			(
				btn_PickEnd.Caption = TryToPickEnd.name
				PickedEnd = TryToPickEnd
			)
			else
			(
				btn_PickEnd.Caption = "Pick"
				PickedEnd = undefined
			)
		)
		
		on btn_createguide pressed do
		(
		if spineGuideOutput != undefined then 
		(
			delete spineGuideOutput[1]
			spineGuideOutput = undefined
			PickedStart = undefined
			PickedEnd = undefined
			btn_PickEnd.Caption = "Pick"
			btn_PickStart.Caption = "Pick"
			btn_Createguide.caption = "Create Layout Guide"
			btn_create.enabled = off
		)
		else
		(
		if PickedStart != undefined and PickedEnd != undefined do
		(
		spineGuideOutput = FinalTools.MakeSpineGuide PickedStart PickedEnd
		btn_createguide.caption = "Delete Guide"
		btn_create.enabled = on
		)
		)

		)
		
		on btn_create pressed do
		(
		
		--Here: the information output from the spineguide function is reassembled into variables. kinda messy... meh... *shrugs*
			
		local allSpineGuideComponents = spineGuideOutput[1]
		local measurementComponents = spineGuideOutput[2]
		local bendcontrols = #()
		append bendcontrols spineGuideOutput[3][1].transform.pos
		append bendcontrols spineGuideOutput[3][2].transform.pos	
		local PelvisWidthGuide = measurementComponents[1]
		local MidlineWidthGuide = measurementComponents[4]
		local RibcageWidthGuide = measurementComponents[7]
		local PelvisWidthPoint = measurementComponents[2]
		local PelvisFrontWidthPoint = measurementComponents[3]
		local MidlineWidthPoint = measurementComponents[5]
		local MidlineFrontWidthPoint = measurementComponents[6]
		local RibcageWidthPoint = measurementComponents[8]
		local RibcageFrontWidthPoint = measurementComponents[9]
			
		local PelvisPercent = (measurementComponents[1].pos.controller.PERCENT as float)
		local MidlinePercent = (measurementComponents[4].pos.controller.PERCENT as float)	
		local RibcagePercent = (measurementComponents[7].pos.controller.PERCENT as float)
			
		local measurementsArray = #()
		local PercentsArray = #() 
		
		append measurementsArray (distance PelvisWidthPoint PelvisWidthGuide)
		append measurementsArray (distance PelvisFrontWidthPoint PelvisWidthGuide)
		append measurementsArray (distance MidlineWidthPoint MidlineWidthGuide)
		append measurementsArray (distance MidlineFrontWidthPoint MidlineWidthGuide)
		append measurementsArray (distance RibcageWidthPoint RibcageWidthGuide)
		append measurementsArray (distance RibcageFrontWidthPoint RibcageWidthGuide)
		
		-------Next: The width values are updated to match the bones-----------------------------------------------
		
		local Bone1Width = measurementsArray[1]
		local Bone1FrontWidth = measurementsArray[2]
		local Bone1Percent = PelvisPercent
		local Bone2Percent = (PelvisPercent-((PelvisPercent - RibcagePercent)*0.33))
		local Bone2Width = ((measurementsArray[1])-((measurementsArray[1]-measurementsArray[3])*0.66))
		local Bone2FrontWidth = ((measurementsArray[2])-((measurementsArray[2]-measurementsArray[4])*0.66))
		local Bone3Width = ((measurementsArray[3])-((measurementsArray[3]-measurementsArray[5])*0.33))
		local Bone3FrontWidth =((measurementsArray[4])-((measurementsArray[4]-measurementsArray[6])*0.33))
		local Bone3Percent = (PelvisPercent-((PelvisPercent - RibcagePercent)*0.66))
		local RibcageBend = bendcontrols[2]
		local PelvisBend = bendcontrols[1]
		

		local PercentIncrement = (100-(Bone1Percent)-(100-RibcagePercent))/(spn_numbones.value-2)
		local PercentList = #(0,Bone1Percent)
		
		for i = 1 to i = (spn_numbones.value-3) do
		(
			append PercentList ((PercentList[(i+1)]) + PercentIncrement)
		)
		append PercentList RibcagePercent
		append Percentlist 100
		local WidthList = #(Bone1Width,Bone1FrontWidth,Bone2Width,Bone2FrontWidth,Bone3Width,Bone3FrontWidth)
		
		---------Finally: Assembled into arrays which match the create spine function---------------------------------
		
		
		local UseUpNodes
		if rdo_rom.state == 1 then (UseUpNodes = true) else (UseUpNodes = false)
		local MainControlAlignPoint = CommonFns.createsystemalignmentpoint PickedStart PickedEnd  NoLookat:true 
		in coordsys local rotate MainControlAlignPoint (angleaxis 90 [0,1,0])
		local SpineMainControl = CommonFns.MakeController (Bone3Width * 3.5) "rectangle" PelvisWidthGuide PelvisWidthGuide (color 40 120 80) (ToolSetup.NamePrefix + "ctrl_body_main")  "x" "positive"
		CommonFns.ApplycontrollerTypeProperties SpineMainControl ControlType:"Big" ControlLevel:1
		FinalTools.AppendObjsToLayer MainControlAlignPoint HiddenComponents:True MainBones:False DeformerBones:False Controllers:True AnimGeo:False
		FinalTools.AppendObjsToLayer SpineMainControl HiddenComponents:False MainBones:False DeformerBones:False Controllers:True AnimGeo:False
		
		CommonFns.quickalign SpineMainControl MainControlAlignPoint RotOnly:true
		delete MainControlAlignPoint
		local MainControlNull = Point Pos:[0,0,0] Name:(uniquename (ToolSetup.NamePrefix + "Null_MainBodyControl") Numdigits:3) Box:Off Cross:On AxisTripod:off CenterMarker:off Size:(Bone3Width*0.05)
		CommonFns.quickAlign MainControlNull SpineMainControl
		
		FinalTools.AppendObjsToLayer MainControlNull HiddenComponents:True MainBones:False DeformerBones:False Controllers:False AnimGeo:False
		MainControlNull.parent = PickedConnect
		SpineMainControl.parent = MainControlNull
		
		
		if chk_advdeform.checked == false do (WidthList = undefined)
		
		
		
		
		local SquashState = rdo_squash.state--fix to match ikfk spine function
		local SquashAlwaysBlend = 1 -- 1 is always on , 2 is blend
		
		if rdo_squash.state == 3 do 
		(
			SquashState = 2
			SquashAlwaysBlend = 2
		)
		

		
		

		local RibcageControlNull =  Point Pos:[0,0,0] Name:(uniquename (ToolSetup.NamePrefix + "Null_RibcageControl") Numdigits:3) Box:Off Cross:On AxisTripod:off CenterMarker:off Size:(Bone3Width*0.05)
		local PelvisControlNull =  Point Pos:[0,0,0] Name:(uniquename (ToolSetup.NamePrefix + "Null_PelvisControl") Numdigits:3) Box:Off Cross:On AxisTripod:off CenterMarker:off Size:(Bone3Width*0.05)
		FinalTools.AppendObjsToLayer RibcageControlNull HiddenComponents:True MainBones:False DeformerBones:False Controllers:False AnimGeo:False
		FinalTools.AppendObjsToLayer PelvisControlNull HiddenComponents:True MainBones:False DeformerBones:False Controllers:False AnimGeo:False
		
		local MakeSpineOutput = FinalTools.MakeIKFKSpine PickedStart PickedEnd 0 false rdo_Jointscaling.state SquashState 1 spn_numbones.value 0.5 undefined rdo_noscalepos.state 2  squashblending:SquashAlwaysBlend spinebend:bendcontrols exactpercents:PercentList includefins:WidthList TangentRotNullifiers:true DisplayScale:0.6 MainControlObject:SpineMainControl Prefix:ToolSetup.NamePrefix LiveTwist:True UpNodes:UseUpNodes Bendcontrollers:True 
		try (SpineMainControl.modifiers[#Spine_Controls].SpineControls.paramFinAuto = 1) catch()
		try (SpineMainControl.modifiers[#Spine_Controls].SpineControls.paramTwistDisperse = -0.8) catch()
		
		MakeSpineOutput[2][1].parent = SpineMainControl
		
		
			--------CUT OFF THE START/END BONES AND REPLACE WITH NEW BONES FOR PELVIS/RIBCAGE---------
		
		local FinalRibcageBone,FinalPelvisBone,PelvisReverseNull,PelvisReverseControlNull,RibcageControlTargetNull,RibcageBoneTargetNull
		
		FinalPelvisBone = boneSys.createBone MakeSpineOutput[7][2].transform.pos MakeSpineOutput[7][1].transform.pos [0,0,1]
		FinalPelvisBone.width = MakeSpineOutput[7][1].width
		FinalPelvisBone.height = MakeSpineOutput[7][1].height
		FinalPelvisBone.taper = 70
		FinalPelvisBone.sideFins = off
		FinalPelvisBone.frontFin = off
		FinalPelvisBone.backFin = off
		FinalPelvisBone.name = (uniquename (ToolSetup.NamePrefix + "bn_pelvis") NumDigits:3)
		
		FinalRibcageBone = boneSys.createBone MakeSpineOutput[7][((MakeSpineOutput[7].count)-1)].transform.pos MakeSpineOutput[7][((MakeSpineOutput[7].count))].transform.pos [0,0,1]
		FinalRibcageBone.width = MakeSpineOutput[7][1].width
		FinalRibcageBone.height = MakeSpineOutput[7][1].height
		FinalRibcageBone.taper = 70
		FinalRibcageBone.sideFins = off
		FinalRibcageBone.frontFin = off
		FinalRibcageBone.backFin = off
		FinalRibcageBone.name = (uniquename (ToolSetup.NamePrefix + "bn_ribcage") NumDigits:3)
		
		CommonFns.QuickAlign FinalRibcageBone MakeSpineOutput[7][((MakeSpineOutput[7].count)-1)]
		local TempPelvisPoint = Point Name:"ZZZZZZTEMPPELVISPOINT"
		CommonFns.quickalign TempPelvisPoint FinalPelvisBone


		local TempBaseLookCons = lookat_constraint()
		TempBaseLookCons.target_axis = 0
		TempBaseLookCons.upnode_ctrl = 1
		TempBaseLookCons.upnode_world = off
		TempBaseLookCons.StoUP_axis = 2
		TempBaseLookCons.viewline_length_abs = off
		TempBaseLookCons.lookat_vector_length = 0
		TempBaseLookCons.StoUP_axisFlip = on

		local PelvisLookat = copy TempBaseLookCons

		
		PelvisLookat.appendtarget MakeSpineOutput[7][1] 100
		PelvisLookat.pickUpNode = MakeSpineOutput[7][1]
		PelvisLookat.upnode_world = off
		PelvisLookat.StoUP_axis = 2
		PelvisLookat.upnode_axis = 2
		TempPelvisPoint.rotation.controller = PelvisLookat
		CommonFns.quickalign FinalPelvisBone TempPelvisPoint
		delete TempPelvisPoint
		

		
		
		--

		
		
		
		
		
		-----------------------
		for i = 2 to i = ((MakeSpineOutput[7].count)-2) do
		(CommonFns.ApplyLengthWidthProperties #(MakeSpineOutput[7][i]) Bone2FrontWidth ObjFrontWidth:Bone2Width ObjOffset:undefined)
		CommonFns.ApplyLengthWidthProperties #(FinalPelvisBone) Bone1FrontWidth ObjFrontWidth:Bone1Width ObjOffset:undefined
		CommonFns.ApplyLengthWidthProperties #(FinalRibcageBone) Bone3FrontWidth ObjFrontWidth:Bone3Width ObjOffset:undefined
		
		local OldBonesArray = #(MakeSpineOutput[7][1],MakeSpineOutput[7][((MakeSpineOutput[7].count)-1)],MakeSpineOutput[7][((MakeSpineOutput[7].count))])
		
		FinalTools.AppendObjsToLayer OldBonesArray HiddenComponents:True MainBones:False DeformerBones:False Controllers:False AnimGeo:False
		FinalTools.AppendObjsToLayer #(FinalPelvisBone,FinalRibcageBone) HiddenComponents:False MainBones:True DeformerBones:False Controllers:False AnimGeo:False
		
		for n in oldBonesArray do (n.wirecolor = color 0 2 16)	
		MakeSpineOutput[7][1].name = (uniquename (ToolSetup.nameprefix + "cnstarget_pelvisbone") Numdigits:3)
		MakeSpineOutput[7][((MakeSpineOutput[7].count)-1)].name = (uniquename (ToolSetup.nameprefix + "cnstarget_ribcagebone") Numdigits:3)
		MakeSpineOutput[7][((MakeSpineOutput[7].count))].name = (uniquename (ToolSetup.nameprefix + "cnstarget_ribcagenub") Numdigits:3)
		
		ToolSetup.lastcreatedribcage = FinalRibcageBone
		ToolSetup.lastcreatedpelvis = FinalPelvisBone
		
		CommonFns.quickAlign PelvisControlNull SpineMainControl
		PelvisControlNull.pos = MakeSpineOutput[7][2].transform.pos
		CommonFns.quickAlign RibcageControlNull SpineMainControl
		
		local RibcagePos = ((spn_numbones.value)-1)
		RibcageControlNull.pos = MakeSpineOutput[7][RibcagePos].transform.pos
		in coordsys local rotate RibcageControlNull (angleaxis -90 [0,1,0])
		in coordsys local rotate PelvisControlNull (angleaxis -90 [0,1,0])
		
		local PelvisControlShape = CommonFns.DrawRectangleControl PelvisControlNull MakeSpineOutput[7][2] MakeSpineOutput[7][1] (Bone3Width * 2.0) (Bone3Width * 2.0) ("BLAH" )
		local RibcageControlShape = CommonFns.DrawRectangleControl RibcageControlNull MakeSpineOutput[7][RibcagePos] MakeSpineOutput[7][(RibcagePos+2)] (Bone3Width * 2.0) (Bone3Width * 2.0) ("BLAH" )
		in coordsys local rotate RibcageControlNull (angleaxis 90 [0,1,0])
		in coordsys local rotate PelvisControlNull (angleaxis 90 [0,1,0])
		local PelvisControl = splineshape Name:(uniquename (ToolSetup.NamePrefix + "ctrl_pelvis"))
		local RibcageControl = splineshape Name:(uniquename (ToolSetup.NamePrefix + "ctrl_ribcage"))	
		CommonFns.QuickAlign PelvisControl PelvisControlNull
		CommonFns.QuickAlign RibcageControl RibcageControlNull
		addandweld PelvisControl PelvisControlShape 0
		addandweld RibcageControl RibcageControlShape 0
			
		CommonFns.ApplycontrollerTypeProperties PelvisControl ControlType:"Big" ControlLevel:2
		CommonFns.ApplycontrollerTypeProperties RibcageControl ControlType:"Big" ControlLevel:2
		FinalTools.AppendObjsToLayer PelvisControl HiddenComponents:False MainBones:False DeformerBones:False Controllers:True AnimGeo:False
		FinalTools.AppendObjsToLayer RibcageControl HiddenComponents:False MainBones:False DeformerBones:False Controllers:True AnimGeo:False
		PelvisControl.objectoffsetpos = [0,0,-(Distance MakeSpineOutput[7][2] MakeSpineOutput[7][1])]
		PelvisControl.parent = PelvisControlNull
		RibcageControl.parent = RibcageControlNull
		
		
		local CnsTargetRibcageControl = Point Pos:[0,0,0] Name:(uniquename (Toolsetup.nameprefix + "CnsTarget_RibcageControl") Numdigits:3) Box:Off Cross:On AxisTripod:off CenterMarker:off Size:(Bone3Width*0.05)
		local CnsTargetPelvisControl = Point Pos:[0,0,0] Name:(uniquename (ToolSetup.nameprefix + "CnsTarget_PelvisControl") Numdigits:3) Box:Off Cross:On AxisTripod:off CenterMarker:off Size:(Bone3Width*0.05)
		FinalTools.AppendObjsToLayer CnsTargetPelvisControl HiddenComponents:True MainBones:False DeformerBones:False Controllers:False AnimGeo:False
		FinalTools.AppendObjsToLayer CnsTargetRibcageControl HiddenComponents:True MainBones:False DeformerBones:False Controllers:False AnimGeo:False
		CommonFns.quickalign CnsTargetRibcageControl MakeSpineOutput[2][3]
		CommonFns.quickalign CnsTargetPelvisControl MakeSpineOutput[2][2]
		CnsTargetPelvisControl.parent = PelvisControl
		CnsTargetRibcageControl.parent = RibcageControl
		MakeSpineOutput[2][2].rotation.controller = orientation_constraint()
		MakeSpineOutput[2][3].rotation.controller = orientation_constraint()
		MakeSpineOutput[2][2].pos.controller = position_constraint()
		MakeSpineOutput[2][3].pos.controller = position_constraint()
		
		MakeSpineOutput[2][2].rotation.controller.appendtarget CnsTargetPelvisControl 100
		MakeSpineOutput[2][3].rotation.controller.appendtarget CnsTargetRibcageControl 100
		MakeSpineOutput[2][2].pos.controller.appendtarget CnsTargetPelvisControl 100
		MakeSpineOutput[2][3].pos.controller.appendtarget CnsTargetRibcageControl 100
		
		RibcageControlNull.parent = SpineMainControl
		PelvisControlNull.parent = SpineMainControl
		
	
		if PickedConnect != undefined do (ToolSetup.lastpickedRoot = PickedConnect)
		----------------------------------------------------------------
		
		PelvisReverseNull = point Name:(uniquename (ToolSetup.NamePrefix + "Null_PelvisReverse") numdigits:3) Size:((FinalPelvisBone.width)*0.2)
		CommonFns.QuickAlign PelvisReverseNull FinalPelvisBone
		PelvisReverseNull.parent = MakeSpineOutput[7][1]
		
		PelvisReverseControlNull = point Name:(uniquename (ToolSetup.NamePrefix + "Null_PelvisReverse") numdigits:3) Size:((FinalPelvisBone.width)*0.2)
		CommonFns.QuickAlign PelvisReverseControlNull FinalPelvisBone
		PelvisReverseControlNull.parent = PelvisControl
		
		RibcageControlTargetNull = point Name:(uniquename (ToolSetup.NamePrefix + "Null_RibcageBoneTarget") numdigits:3) Size:((FinalPelvisBone.width)*0.2)
		CommonFns.QuickAlign RibcageControlTargetNull FinalRibcageBone
		RibcageControlTargetNull.parent = RibcageControl
		
		RibcageBoneTargetNull = point Name:(uniquename (ToolSetup.NamePrefix + "Null_RibcageBoneControlTarget") numdigits:3) Size:((FinalPelvisBone.width)*0.2)
		CommonFns.QuickAlign RibcageBoneTargetNull FinalRibcageBone
		RibcageBoneTargetNull.parent = MakeSpineOutput[7][((MakeSpineOutput[7].count)-1)]
		
		FinalRibcageBone.parent = RibcageBoneTargetNull
		FinalPelvisBone.parent = PelvisReverseNull
		FinalPelvisBone.pos.controller = position_constraint()
		FinalPelvisBone.pos.controller.appendtarget MakeSpineOutput[7][2] 100
		
		local ca=attributes RibcageBlend
		(
			parameters params rollout:ro_RibcageBlend
			(
				paramblend type:#float ui:spn_SpineController
			)
			rollout ro_RibcageBlend "Ribcage_Blend" 
			(
				spinner spn_SpineController "Constrain Bone" enabled:on  range:[0,1,0] scale:0.02
			)
			rollout ro_AdjustPivot "Adjust Pivot" 
			(
				button btn_adjustpivot "Adjust Pivot"
				button btn_apply "Apply" enabled:off
				button btn_finalise "Finalise" enabled:on
				
				
			local PivotObject,OrigPivotTransform,boundbox,pivobjectname,trytogetpivot,existingpivot
				
			on btn_adjustpivot pressed do
			(
				boundbox = (in coordsys local nodeLocalBoundingBox $)
				boundbox = (distance boundbox[1] boundbox[2])
				OrigPivotTransform = $.transform
				PivotObject = Point Name:(uniquename "PivotAdjustObject" NumDigits:3) Pos:OrigPivotTransform.pos Size:(boundbox*0.2) Cross:On Box:off CenterMarker:off AxisTripod:off
				PivotObject.transform = $.transform
				setUserProp $ "PivotObjectName" PivotObject.name
				btn_apply.enabled = true
				select PivotObject
			)
			on btn_finalise pressed do
			(
				setUserProp $ "Finalised" "True"
				btn_adjustpivot.enabled = false
				btn_finalise.enabled = false
			)
			on ro_AdjustPivot open do
			(
				trytogetpivot = (getUserProp $ "PivotObjectName")
				try (existingpivot = (getnodebyname trytogetpivot)) catch()
				if existingpivot != undefined do (btn_apply.enabled = true)
				local TestState = (getUserProp $ "Finalised")
					if  (TestState == "True") or (TestState == true) do
					(
						btn_adjustpivot.enabled = false
						btn_apply.enabled = false
						btn_finalise.enabled = false
					)
			)
			on btn_apply pressed do
			(

				local ChildrenOrigPosArray = #()
				local OrigPos
				local origOffset
				for n in $.children do (append ChildrenOrigPosArray n.transform)
				OrigOffset = ($.transform * inverse existingpivot.transform)
				Origpos = $.transform
				in coordsys world $.parent.transform = existingpivot.transform
				for c = 1 to c = ChildrenOrigPosArray.count do (($.children[c]).transform = ChildrenOrigPosArray[c])
				
				
				$.objectoffsetpos += (origOffset).pos
				--$.objectoffsetrot += (origOffset).rotation
				
				setUserProp $ "PivotObjectName" "undefined"
				delete existingpivot
				
			)
			
			)
		)
		AttHolder = (EmptyModifier())
		AttHolder.name = "Ribcage_Blend"
		addmodifier RibcageControl AttHolder
		custAttributes.add RibcageControl.modifiers[#Ribcage_Blend] ca
			
		local ca=attributes PelvisBlend
		(
			parameters params rollout:ro_PelvisBlend
			(
				paramblend type:#float ui:spn_SpineController
			)
			rollout ro_PelvisBlend "Pelvis_Blend" 
			(
				spinner spn_SpineController "Constrain bone" enabled:on  range:[0,1,0] scale:0.02
			)
			rollout ro_AdjustPivot "Adjust Pivot" 
			(
				button btn_adjustpivot "Adjust Pivot"
				button btn_apply "Apply" enabled:off
				button btn_finalise "Finalise" enabled:on
				
				
				
				local PivotObject,OrigPivotTransform,boundbox,pivobjectname,trytogetpivot,existingpivot
				
				on btn_adjustpivot pressed do
				(
					boundbox = (in coordsys local nodeLocalBoundingBox $)
					boundbox = (distance boundbox[1] boundbox[2])
					OrigPivotTransform = $.transform
					PivotObject = Point Name:(uniquename "PivotAdjustObject" NumDigits:3) Pos:OrigPivotTransform.pos Size:(boundbox*0.2) Cross:On Box:off CenterMarker:off AxisTripod:off
					PivotObject.transform = $.transform
					setUserProp $ "PivotObjectName" PivotObject.name
					btn_apply.enabled = true
					select PivotObject
				)
				on btn_finalise pressed do
				(
					setUserProp $ "Finalised" "True"
					btn_adjustpivot.enabled = false
					btn_finalise.enabled = false
				)
				
				
				on ro_AdjustPivot open do
				(
					trytogetpivot = (getUserProp $ "PivotObjectName")
					try (existingpivot = (getnodebyname trytogetpivot)) catch()
					if existingpivot != undefined do (btn_apply.enabled = true)
					local TestState = (getUserProp $ "Finalised")
						if  (TestState == "True") or (TestState == true) do
						(
							btn_adjustpivot.enabled = false
							btn_apply.enabled = false
							btn_finalise.enabled = false
						)
				)
				on btn_apply pressed do
				(
					local ChildrenOrigPosArray = #()
					local OrigPos
					local origOffset
					for n in $.children do (append ChildrenOrigPosArray n.transform)
					OrigOffset = ($.transform * inverse existingpivot.transform)
					Origpos = $.transform
					in coordsys world $.parent.transform = existingpivot.transform
					for c = 1 to c = ChildrenOrigPosArray.count do (($.children[c]).transform = ChildrenOrigPosArray[c])
					
					
					$.objectoffsetpos += (origOffset).pos
					--$.objectoffsetrot += (origOffset).rotation
					
					setUserProp $ "PivotObjectName" "undefined"
					delete existingpivot
					
				)
			)
		)
		AttHolder = (EmptyModifier ())
		AttHolder.name = "Pelvis_Blend"
		addmodifier PelvisControl AttHolder
		custAttributes.add PelvisControl.modifiers[#Pelvis_Blend] ca	
			
		FinalRibcageBone.rotation.controller = rotation_list()
			
			
		local RibcageBlendController = float_expression ()
		RibcageBlendController.AddScalarTarget "Blend" RibcageControl.modifiers[#Ribcage_Blend].RibcageBlend[#paramblend]
		RibcageBlendController.setexpression ("Blend")
			
		local RibcageOriConstraint = orientation_constraint()
		RibcageOriConstraint.appendtarget RibcageControlTargetNull 100
		FinalRibcageBone.rotation.controller.Available.controller = RibcageOriConstraint
		FinalRibcageBone.rotation.controller.weight[2].controller = RibcageBlendController
		

		FinalPelvisBone.rotation.controller = rotation_list()

		local pelvisBlendController = float_expression ()
		pelvisBlendController.AddScalarTarget "Blend" pelvisControl.modifiers[#pelvis_Blend].pelvisBlend[#paramblend]
		pelvisBlendController.setexpression ("Blend")
		
		local PelvisOriConstraint = orientation_constraint()
		PelvisOriConstraint.appendtarget PelvisReverseControlNull 100
		FinalPelvisBone.rotation.controller.Available.controller = PelvisOriConstraint
		FinalPelvisBone.rotation.controller.weight[2].controller = pelvisBlendController
		
		FinalTools.AppendObjsToLayer #(PelvisReverseNull,PelvisReverseControlNull,RibcageControlTargetNull,RibcageBoneTargetNull) HiddenComponents:True MainBones:False DeformerBones:False Controllers:False AnimGeo:False
		
--FinalOutputArray:

			--1: Allcreated
			--2: GlobalController, Start Controller, EndController
			--3: FK Controllers (or empty array)
			--4:Reverse FK Controllers (or empty array)
			--5: Deformation fin controllers (or empty array )
			--6: StartTangent, EndTangent, StartBendController, EndBendController
			--7:MainBones
		
		
		delete spineGuideOutput[1]
		try(destroyDialog CreateSpineRollout)catch("")
		
		try
		(
		local WM_SYSCOMMAND = 0x112
		local SC_RESTORE = 0xf120

		--http://forums.cgsociety.org/archive/index.php/t-644118.html --Minimise/Maximise rollout code snippet from this post.

		local OpenDialogs = UIAccessor.GetPopupDialogs()
		local DialogTitles = for d in OpenDialogs collect (UIAccessor.GetWindowText d)
		local DialogIndex = findItem DialogTitles "Rigging Toolkit"
		local MyHandle = OpenDialogs[DialogIndex]

		local MaxWindows = windows.getChildrenHWND 0 parent:#max
		local WindowTitles = for w in MaxWindows collect w[5]
		local WindowIndex = findItem WindowTitles "Rigging Toolkit"
		MyHandle = MaxWindows[WindowIndex][1]
		windows.sendMessage MyHandle WM_SYSCOMMAND SC_RESTORE 0
		)
		catch()


-- restore previous dialog title

		
		
		
		
		--Pelvis Width = 0
		--Midline Width  = 0.5
		--RibcageWidth = 1.
		--Spine1 = 0
		--Spine 2 = 
		
		
			-------------------------------------------------------------
			--OutputArray contains:
			--1: All created
			--2: Measurement Pairs (Pelvis Guide, Pelvis Width, Pelvis Front),(Midline Guide, Width, Front), (Ribcage Guide, Width, Front)
			-------------------------------------------------------------
			
			
			---------------------------------------------------------------------------------
			--OutputArray Contains:
			--1: All created, 
			--2: Pelvis Width, Pelvis Front Width, Midline Width, Midline Front Width, Ribcage Width, Ribcage Front Width, 
			--3: Exact Percents: 0, Spine 1 Start, Spine 2 Start, Spine 3 Start,RibcageStart,Ribcage Start, 100


			
			----------------------------------------------------------------------------------

		)
		
		
		
	)--this bracket to close spine guide rollout
	-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
		rollout MakeArmGuideRollout "Make Arm Setup" width:229 height:794
	(
		bitmap bmp2 "Bitmap" pos:[0,0] width:229 height:794 fileName:"RigToolKit_CreateArm.png"
		button btn_pickUpArm "Pick" pos:[125,64] width:78 height:24
		button btn_pickElbow "Pick" pos:[124,94] width:78 height:24
		label lbl2 "Upper Arm point:" pos:[33,69] width:87 height:20
		label lbl3 "Elbow point:" pos:[56,96] width:59 height:20
		groupBox grp1 "Layout Points" pos:[12,46] width:201 height:144
		button btn_createguide "Create Layout Guide" pos:[52,159] width:116 height:22
		button btn_create "Create" pos:[15,762] width:82 height:25 enabled:false
		button btn_PickHand "Pick" pos:[125,127] width:78 height:24
		label lbl4 "Hand point:" pos:[57,131] width:58 height:20
		spinner spn_numfingers "" pos:[136,218] width:47 height:16 range:[0,4,4] type:#integer scale:1
		label lbl5 "Number of fingers:" pos:[34,217] width:100 height:21
		groupBox grp4 "Fingers" pos:[12,193] width:202 height:94
		checkbox chk_thumb "Create Thumb" pos:[54,263] width:91 height:19 checked:true
		spinner spn_fingerjoints "" pos:[136,240] width:47 height:16 range:[0,3,3] type:#integer scale:1
		label lbl6 "Joints per finger:" pos:[34,242] width:100 height:21
		groupBox grp7g "Kinematics" pos:[13,444] width:202 height:73
		radioButtons rdo_kinematics "" pos:[25,464] width:81 height:48 labels:#("IK Only", "FK Only", "IK/FK Blend") columns:1
		checkbox chk_stretchy "Stretchy Arm Blend" pos:[48,539] width:137 height:20
		checkbox chk_bendy "Bendy Arm Blend" pos:[49,558] width:123 height:20
		checkbox chk_mirror "Create Mirror" pos:[112,766] width:88 height:14
		checkbox chk_extrashoulder "Extra Shoulder Bones" pos:[37,310] width:150 height:19
		checkbox chk_split "Split Shoulder" pos:[38,332] width:157 height:21
		groupBox grp4gu "Shoulders" pos:[13,291] width:203 height:64
		spinner spn_twist "" pos:[107,654] width:47 height:16 range:[0,10,3] type:#integer scale:1
		label lbl6g "Twist Bones:" pos:[36,654] width:68 height:21
		button btn_pickarmparent "Pick" pos:[122,700] width:78 height:24
		label lbl7 "Connect To:" pos:[52,708] width:70 height:20
		groupBox grp5 "Connections" pos:[13,680] width:205 height:79
		button btn_pickroot "Pick" pos:[121,730] width:78 height:24
		label lbl8 "Character Root:" pos:[32,731] width:91 height:22
		checkbox chk_squashstretch "Include Squash/Stretch" pos:[50,579] width:150 height:20 enabled:false
		groupBox grp_elbow "Elbow Deformers" pos:[13,364] width:203 height:76
		radioButtons rdo_elbow "" pos:[28,383] width:106 height:48 labels:#("Basic", "Scale Bone", "Game Compatible") columns:1
		groupBox grp_Extra "Extra Features" pos:[13,524] width:202 height:150
		radioButtons rdo_bendyROM "Bendy Range of Motion" pos:[51,602] width:94 height:46 labels:#("Low (faster)", "High (Versatile)") columns:1

		on MakeArmGuideRollout close do
		(
				try
		(
		local WM_SYSCOMMAND = 0x112
		local SC_RESTORE = 0xf120

		--http://forums.cgsociety.org/archive/index.php/t-644118.html --Minimise/Maximise rollout code snippet from this post.

		local OpenDialogs = UIAccessor.GetPopupDialogs()
		local DialogTitles = for d in OpenDialogs collect (UIAccessor.GetWindowText d)
		local DialogIndex = findItem DialogTitles "Rigging Toolkit"
		local MyHandle = OpenDialogs[DialogIndex]

		local MaxWindows = windows.getChildrenHWND 0 parent:#max
		local WindowTitles = for w in MaxWindows collect w[5]
		local WindowIndex = findItem WindowTitles "Rigging Toolkit"
		MyHandle = MaxWindows[WindowIndex][1]
		windows.sendMessage MyHandle WM_SYSCOMMAND SC_RESTORE 0
		)
		catch()
		)
		
			
		local PickedUpArm,PickedElbow,PickedHand,ArmguideOutput,PickedArmParent,PickedRoot
		local ChangedTo
		
		
		on chk_bendy changed ChangedTo do 
		(
			if ChangedTo == true then (Chk_Squashstretch.enabled = true)
			else if (ChangedTo == false) and (Chk_stretchy.checked == false) then 
			(
				Chk_Squashstretch.enabled = false
				Chk_Squashstretch.checked = false
				)
		)
		
		on chk_stretchy changed ChangedTo do 
		(
			if ChangedTo == true then (Chk_Squashstretch.enabled = true)
			else if (ChangedTo == false) and (chk_bendy.checked == false) then 
			(
				Chk_Squashstretch.enabled = false
				Chk_Squashstretch.checked = false
			)
		)
		
		
		
		
		on MakeArmGuideRollout open do
		(
			rdo_kinematics.state = 2
			if ToolSetup.Turbomode == on do
			(
				
				if ToolSetup.LastCreatedRibcage != undefined do 
				(
					try
					(
					select ToolSetup.lastcreatedribcage
					btn_PickArmParent.caption = ToolSetup.lastcreatedribcage.name
					PickedArmParent = ToolSetup.lastcreatedribcage
					)catch()
				)
				if ToolSetup.LastPickedRoot != undefined do 
				(
					try
					(
					select ToolSetup.lastpickedroot
					btn_Pickroot.caption = ToolSetup.lastpickedRoot.name
					Pickedroot = ToolSetup.lastpickedRoot
					)catch()
				)
			)
			
			
			
			try
			(
			local TryToPickUpArm
			TryToPickUpArm = undefined
			PickedUpArm = undefined
			TryToPickUpArm = pickobject prompt:"PickUpArm" rubberBand:[0,0,0] forceListenerFocus:false count:1
			if classof TrytoPickUpArm != Undefined and TryToPickUpArm != #() and classof TrytoPickUpArm != UndefinedClass then
			(
				btn_PickUpArm.Caption = TryToPickUparm.name
				PickedUpArm = TryToPickUpArm
			)
			else
			(
				btn_PickUpArm.Caption = "Pick"
				PickedUpArm = undefined
			)
			if PickedUpArm != undefined do
			(
				local TryToPickElbow
				TryToPickElbow = undefined
				PickedElbow = undefined
				TryToPickElbow = pickobject prompt:"PickElbow" rubberBand:[0,0,0] forceListenerFocus:false count:1
				if classof TrytoPickElbow != Undefined and TryToPickElbow != #() and classof TrytoPickElbow != UndefinedClass then
				(
					btn_PickElbow.Caption = TryToPickElbow.name
					PickedElbow = TryToPickElbow
				)
				else
				(
					btn_PickElbow.Caption = "Pick"
					PickedElbow = undefined
				)
							local TryToPickHand
				TryToPickHand = undefined
				PickedHand = undefined
				TryToPickHand = pickobject prompt:"PickHand" rubberBand:[0,0,0] forceListenerFocus:false count:1
				if classof TrytoPickHand != Undefined and TryToPickHand != #() and classof TrytoPickHand != UndefinedClass then
				(
					btn_PickHand.Caption = TryToPickHand.name
					PickedHand = TryToPickHand
				)
				else
				(
					btn_PickHand.Caption = "Pick"
					PickedHand = undefined
				)
			)
		)
		catch()
		)
		
		on btn_pickUpArm pressed do
		(
			local TryToPickUpArm
			TryToPickUpArm = undefined
			PickedUpArm = undefined
			TryToPickUpArm = pickobject prompt:"PickUpArm" rubberBand:[0,0,0] forceListenerFocus:false count:1
			if classof TrytoPickUpArm != Undefined and TryToPickUpArm != #() and classof TrytoPickUpArm != UndefinedClass then
			(
				btn_PickUpArm.Caption = TryToPickUparm.name
				PickedUpArm = TryToPickUpArm
			)
			else
			(
				btn_PickUpArm.Caption = "Pick"
				PickedUpArm = undefined
			)
		)
		
		on btn_pickElbow pressed do
		(
			local TryToPickElbow
			TryToPickElbow = undefined
			PickedElbow = undefined
			TryToPickElbow = pickobject prompt:"PickElbow" rubberBand:[0,0,0] forceListenerFocus:false count:1
			if classof TrytoPickElbow != Undefined and TryToPickElbow != #() and classof TrytoPickElbow != UndefinedClass then
			(
				btn_PickElbow.Caption = TryToPickElbow.name
				PickedElbow = TryToPickElbow
			)
			else
			(
				btn_PickElbow.Caption = "Pick"
				PickedElbow = undefined
			)
		)
		
				on btn_pickHand pressed do
		(
			local TryToPickHand
			TryToPickHand = undefined
			PickedHand = undefined
			TryToPickHand = pickobject prompt:"PickHand" rubberBand:[0,0,0] forceListenerFocus:false count:1
			if classof TrytoPickHand != Undefined and TryToPickHand != #() and classof TrytoPickHand != UndefinedClass then
			(
				btn_PickHand.Caption = TryToPickHand.name
				PickedHand = TryToPickHand
			)
			else
			(
				btn_PickHand.Caption = "Pick"
				PickedHand = undefined
			)
		)
		
		on btn_pickArmParent pressed do
		(
			local TryToPickArmParent
			TryToPickArmParent = undefined
			PickedArmParent = undefined
			TryToPickArmParent = pickobject prompt:"PickUpArm" rubberBand:[0,0,0] forceListenerFocus:false count:1
			if classof TryToPickArmParent != Undefined and TryToPickArmParent != #() and classof TryToPickArmParent != UndefinedClass then
			(
				btn_PickArmParent.Caption = TryToPickArmParent.name
				PickedArmParent = TryToPickArmParent
			)
			else
			(
				btn_pickArmParent.Caption = "Pick"
				PickedArmParent = undefined
			)
		)
		
		on btn_pickRoot pressed do
		(
			local TryToPickRoot
			TryToPickRoot = undefined
			PickedRoot = undefined
			TryToPickRoot = pickobject prompt:"PickUpArm" rubberBand:[0,0,0] forceListenerFocus:false count:1
			if classof TryToPickRoot != Undefined and TryToPickRoot != #() and classof TryToPickRoot != UndefinedClass then
			(
				btn_pickRoot.Caption = TryToPickRoot.name
				PickedRoot = TryToPickRoot
			)
			else
			(
				btn_pickRoot.Caption = "Pick"
				PickedRoot = undefined
			)
		)

		
		on btn_createguide pressed do
		(
			if ArmGuideOutput != undefined then 
			(
				try (delete ArmGuideOutput[1]) catch()
				ArmGuideOutput = undefined
				btn_PickHand.Caption = "Pick"
				PickedHand = undefined
				btn_PickElbow.Caption = "Pick"
				PickedElbow = undefined
				btn_PickUpArm.Caption = "Pick"
				PickedUpArm = undefined
				btn_createguide.caption = "Create Layout Guide"
				btn_create.enabled = false
			)
			else
			(
				if PickedElbow != undefined and PickedUpArm != undefined and PickedHand != undefined do
				(
					ArmGuideoutput = FinalTools.MakeArmGuide PickedUpArm PickedElbow PickedHand spn_Numfingers.value spn_fingerjoints.value chk_thumb.checked chk_split.checked chk_extrashoulder.checked
					btn_createguide.caption = "Delete Guide"
					btn_create.enabled = true
				)
			)
		)
		
		on btn_create pressed do
		(
		
		local MainArmbones = #()
		local Finger1 = #()
		local Finger2 =#()
		local Finger3 =#()
		local Finger4 =#()
		local Thumb =#()
		local ShoulderBones = #()
		local OutputNumFingers
		local OutputIncludeThumb
		local ThumbDigitNumber
		local OutputAlignment
		local RibcageAlignment
		local ArmWidths = #()
			
		if (Chk_Bendy.checked == true) or (Chk_Stretchy.checked == true) do
		(
			if Spn_Twist.value <3 do (Spn_Twist.value = 3)
		)
		
		MainArmBones = ArmGuideOutput[2]
		for i = 1 to i = MainArmBones.count do 
		(
			MainArmBones[i] = MainArmBones[i].transform
		)
		OutputNumFingers = ArmGuideOutput[5][1]
		OutputIncludeThumb = ArmGuideOutput[5][3]
		if OutputIncludeThumb == true then (ThumbDigitNumber = OutputNumFingers+1) else (ThumbDigitNumber = undefined)
		if OutputIncludeThumb == true then 
		(
			Thumb = ArmGuideOutput[4][ThumbDigitNumber]
			for i = 1 to i = Thumb.count do (Thumb[i] = Thumb[i].transform)
		)
		if OutputNumFingers >0 do 
		(
			Finger1 = ArmGuideOutput[4][1]
			for i = 1 to i = Finger1.count do (Finger1[i] = Finger1[i].transform)
		)
		if OutputNumFingers >1 do 
		(
			Finger2 = ArmGuideOutput[4][2]
			for i = 1 to i = Finger2.count do (Finger2[i] = Finger2[i].transform)
		)
		if OutputNumFingers >2 do 
		(
			Finger3 = ArmGuideOutput[4][3]
			for i = 1 to i = Finger3.count do (Finger3[i] = Finger3[i].transform)
		)
		if OutputNumFingers >3 do 
		(
			Finger4 = ArmGuideOutput[4][4]
			for i = 1 to i = Finger4.count do (Finger4[i] = Finger4[i].transform)
		)
		ShoulderBones = ArmGuideOutput[3]
		for i = 1 to i = ShoulderBones.count do (Shoulderbones[i] = ShoulderBones[i].transform)
		OutputAlignment = ArmGuideOutput[5][5]
		RibcageAlignment = ArmGuideOutput[5][6].transform
		
		
		local UpArmStartFrontWidth = Distance ArmGuideOutput[6][1][1] ArmGuideOutput [6][1][2]
		local UpArmStartWidth = Distance ArmGuideOutput[6][1][1] ArmGuideOutput [6][1][3]
		local UpArmEndFrontWidth = Distance ArmGuideOutput[6][2][1] ArmGuideOutput [6][2][2]
		local UpArmEndWidth = Distance ArmGuideOutput[6][2][1] ArmGuideOutput [6][2][3]
		local WristFrontWidth = Distance ArmGuideOutput[6][3][1] ArmGuideOutput [6][3][2]
		local WristWidth = Distance ArmGuideOutput[6][3][1] ArmGuideOutput [6][3][3]
		local HandFrontWidth = Distance ArmGuideOutput[6][4][1] ArmGuideOutput [6][4][2]
		local HandWidth = Distance ArmGuideOutput[6][4][1] ArmGuideOutput [6][4][3]
		local RibcageFrontWidth = Distance ArmGuideOutput[6][5][1] ArmGuideOutput [6][5][2]
		local RibcageWidth = Distance ArmGuideOutput[6][5][1] ArmGuideOutput [6][5][3]
		append ArmWidths UpArmStartFrontWidth
		append ArmWidths UpArmStartWidth
		append ArmWidths UpArmEndFrontWidth
		append ArmWidths UpArmEndWidth
		append ArmWidths WristFrontWidth
		append ArmWidths WristWidth
		append ArmWidths HandFrontWidth
		append ArmWidths HandWidth
		append ArmWidths RibcageFrontWidth
		append ArmWidths RibcageWidth
		
		local ScapulaComponents = ArmGuideOutput[7]
		local PectoralComponents = ArmGuideOutput[8]
		local TeresComponents = ArmGuideOutput[9]
		
		if ScapulaComponents.count != 0 do (for i = 1 to i = ScapulaComponents.count do (ScapulaComponents[i] = ScapulaComponents[i].transform))
		if PectoralComponents.count != 0 do (for i = 1 to i = PectoralComponents.count do (PectoralComponents[i] = PectoralComponents[i].transform))
		if TeresComponents.count != 0 do (for i = 1 to i = TeresComponents.count do (TeresComponents[i] = TeresComponents[i].transform))
			
			
		local ExtraShoulderBonesArray = #(ScapulaComponents,PectoralComponents,TeresComponents)
		--OutputArray:
		--1:Allcreated
		--2: Main Bones (UpArmBone,LowArmBone,HandBone,HandNub)
		--3: Shoulder Bones: Clavicle, Clavicle Nub,  --Possibly: ClavSplitBone, Nub
		--4: Finger Bones: FINGER 1: base mid end nub, FINGER 2: etc final is THUMB
		--5: Settings Array
		--6: Width Helpers (UpArmStart Guide, UparmStartFrontWidth, UpArmStartWidth) (UpaRm end) (Wrist) (Hand)
		--7: Scapula GuideBone, ScapulaGuideNub
		--8:Pectoral Components --PecStartLocator,PecEndLocator,PecMidPointLocator,PecLookatLocator,PecPosConsPointA,PecPosConsPointB,PecStartBone,PecEndBone
		--9: Teres Components
			
			
			


		----REASSEMBLE THE ARM GUIDE OUTPUT ArmGuideoutput
		delete ArmGuideOutput[1]
		
		FinalTools.MakeArm MainArmBones Finger1 Finger2 Finger3 Finger4 Thumb ShoulderBones OutputAlignment RibcageAlignment rdo_kinematics.state chk_stretchy.checked chk_bendy.checked ArmWidths:ArmWidths ArmParent:PickedArmParent Root:PickedRoot Prefix:ToolSetup.NamePrefix TwistBones:spn_twist.value ExtraShoulderBones:chk_extrashoulder.checked Split:Chk_split.checked ExtraBonesInput:ExtraShoulderBonesArray Squash:chk_squashstretch.checked ElbowType:rdo_elbow.state BendyRom:rdo_bendyROM.state
		if Chk_Mirror.checked == true do
		(
		if Finger1.count != 0 do (Finger1 = CommonFns.MirrorTransformsInArray finger1)
		if Finger2.count != 0 do (Finger2 = CommonFns.MirrorTransformsInArray finger2)
		if Finger3.count != 0 do (Finger3 = CommonFns.MirrorTransformsInArray finger3)	
		if Finger4.count != 0 do (Finger4 = CommonFns.MirrorTransformsInArray finger4)	
		if Thumb.count != 0 do (Thumb = CommonFns.MirrorTransformsInArray Thumb)	
		MainArmBones = CommonFns.MirrorTransformsInArray MainArmBones
		ShoulderBones = CommonFns.MirrorTransformsInArray ShoulderBones	
		if OutputAlignment == "r" or OutputAlignment == "R" then (OutputAlignment = "L") else (OutputAlignment = "R")
		
		if ScapulaComponents.count != 0 do (ScapulaComponents = CommonFns.MirrorTransformsInArray ScapulaComponents)
		if PectoralComponents.count != 0 do (PectoralComponents = CommonFns.MirrorTransformsInArray PectoralComponents)
		if TeresComponents.count != 0 do (TeresComponents = CommonFns.MirrorTransformsInArray TeresComponents)
		ExtraShoulderBonesArray = #(ScapulaComponents,PectoralComponents,TeresComponents)	
			
		
		FinalTools.MakeArm MainArmBones Finger1 Finger2 Finger3 Finger4 Thumb ShoulderBones OutputAlignment RibcageAlignment rdo_kinematics.state chk_stretchy.checked chk_bendy.checked ArmWidths:ArmWidths ArmParent:PickedArmParent Root:PickedRoot Prefix:ToolSetup.NamePrefix TwistBones:spn_twist.value ExtraShoulderBones:chk_extrashoulder.checked Split:Chk_split.checked ExtraBonesInput:ExtraShoulderBonesArray Squash:chk_squashstretch.checked ElbowType:rdo_elbow.state BendyRom:rdo_bendyROM.state
		)
		destroydialog MakeArmGuideRollout
				
		)
	) --This bracket to close arm guide rollout
	
	rollout LegGuideRollout "Leg Guide" width:229 height:765
	(
		bitmap bmp2 "Bitmap" pos:[0,0] width:229 height:765 fileName:"RigToolKit_CreateLeg.png"
		button btn_pickUpLeg "Pick" pos:[124,71] width:78 height:24
		button btn_pickKnee "Pick" pos:[123,104] width:78 height:24
		label lbl2 "Upper Leg Point:" pos:[32,76] width:87 height:20
		label lbl3 "Knee Point:" pos:[55,108] width:59 height:20
		groupBox grp1 "Layout Points" pos:[11,53] width:201 height:202
		button btn_createguide "Create Layout Guide" pos:[50,222] width:116 height:22
		button btn_create "Create" pos:[19,729] width:82 height:25 enabled:off
		button btn_PickAnkle "Pick" pos:[124,139] width:78 height:24
		label lbl4 "Ankle Point:" pos:[56,143] width:58 height:20
		groupBox grp_extra "Kinematics" pos:[10,342] width:202 height:78
		radioButtons rdo_kinematics "" pos:[28,362] width:81 height:48 labels:#("IK Only", "FK Only", "IK/FK Blend") columns:1
		checkbox chk_stretchy "Stretchy Leg Blend" pos:[46,447] width:137 height:20
		checkbox chk_bendy "Bendy Leg Blend" pos:[46,470] width:123 height:20
		checkbox chk_mirror "Create Mirror" pos:[119,734] width:88 height:14
		checkbox chk_footRig "Create Foot Rig" pos:[46,495] width:123 height:20 checked:true
		button btn_PickConnect "Pick" pos:[124,659] width:78 height:24
		button btn_PickRoot "Pick" pos:[123,689] width:78 height:24
		label lbl22 "Connect to:" pos:[38,664] width:64 height:20
		label lbl23 "Character Root:" pos:[19,693] width:80 height:20
		groupBox grp14 "Connections" pos:[11,640] width:201 height:85
		spinner spn_twistbones "" pos:[111,610] width:54 height:16 range:[0,10,0] type:#integer
		label lbl50b "Twist Bones:" pos:[43,610] width:61 height:20
		groupBox grp4 "Extra Features" pos:[11,422] width:202 height:215
		checkbox chk_squash "Include Squash" pos:[48,520] width:123 height:20 enabled:false checked:false
		radioButtons rdo_bendyrom "Bendy ROM" pos:[53,548] width:86 height:46 enabled:false labels:#("Low (Faster)", "High (Slower)") columns:1
		radioButtons rdo_legtype "" pos:[55,167] width:104 height:48 labels:#("Human Leg", "Quadruped Front", "Quadruped Back") columns:1
		groupBox grp25h "Knee/Hip Deformers" pos:[12,260] width:202 height:78
		radioButtons rdo_kneedeform "" pos:[30,280] width:106 height:48 labels:#("Basic","Scale Bones","Game Compatible") columns:1
		
		local PickedUpLeg,PickedKnee,PickedAnkle,LegGuideOutput,PickedLegParent,PickedRoot
		
		local ChangedTo
		
		on LegGuideRollout close do
		(
				try
		(
		local WM_SYSCOMMAND = 0x112
		local SC_RESTORE = 0xf120

		--http://forums.cgsociety.org/archive/index.php/t-644118.html --Minimise/Maximise rollout code snippet from this post.

		local OpenDialogs = UIAccessor.GetPopupDialogs()
		local DialogTitles = for d in OpenDialogs collect (UIAccessor.GetWindowText d)
		local DialogIndex = findItem DialogTitles "Rigging Toolkit"
		local MyHandle = OpenDialogs[DialogIndex]

		local MaxWindows = windows.getChildrenHWND 0 parent:#max
		local WindowTitles = for w in MaxWindows collect w[5]
		local WindowIndex = findItem WindowTitles "Rigging Toolkit"
		MyHandle = MaxWindows[WindowIndex][1]
		windows.sendMessage MyHandle WM_SYSCOMMAND SC_RESTORE 0
		)
		catch()
		)
		
		
		on LegGuideRollout open do
		(
			if ToolSetup.Turbomode == on do
			(
				
				if ToolSetup.LastCreatedPelvis != undefined do 
				(
					try
					(
					select ToolSetup.lastcreatedpelvis
					btn_PickConnect.caption = ToolSetup.lastcreatedpelvis.name
					PickedLegParent = ToolSetup.lastcreatedpelvis
					)catch()
				)
				if ToolSetup.LastPickedRoot != undefined do 
				(
					try
					(
					select ToolSetup.lastpickedroot
					btn_Pickroot.caption = ToolSetup.lastpickedRoot.name
					Pickedroot = ToolSetup.lastpickedRoot
					)catch()
				)
			)



			try
			(
				local TryToPickUpLeg
				TryToPickUpLeg = undefined
				PickedUpLeg = undefined
				TryToPickUpLeg = pickobject prompt:"PickUpArm" rubberBand:[0,0,0] forceListenerFocus:false count:1
				if classof TryToPickUpLeg != Undefined and TryToPickUpLeg != #() and classof TryToPickUpLeg != UndefinedClass then
				(
					btn_pickUpLeg.Caption = TryToPickUpLeg.name
					PickedUpLeg = TryToPickUpLeg
				)
				else
				(
					btn_pickUpLeg.Caption = "Pick"
					PickedUpLeg = undefined
				)
			if PickedUpLeg != undefined do
			(
				local TryToPickKnee
				TryToPickKnee = undefined
				PickedKnee = undefined
				TryToPickKnee = pickobject prompt:"PickElbow" rubberBand:[0,0,0] forceListenerFocus:false count:1
				if classof TryToPickKnee != Undefined and TryToPickKnee != #() and classof TryToPickKnee != UndefinedClass then
				(
					btn_PickKnee.Caption = TryToPickKnee.name
					PickedKnee = TryToPickKnee
				)
				else
				(
					btn_PickElbow.Caption = "Pick"
					PickedKnee = undefined
				)
				local TryToPickAnkle
				TryToPickAnkle = undefined
				PickedAnkle = undefined
				TryToPickAnkle = pickobject prompt:"PickHand" rubberBand:[0,0,0] forceListenerFocus:false count:1
				if classof TryToPickAnkle != Undefined and TryToPickAnkle != #() and classof TryToPickAnkle != UndefinedClass then
				(
					btn_PickAnkle.Caption = TryToPickAnkle.name
					PickedAnkle = TryToPickAnkle
				)
				else
				(
					btn_PickHand.Caption = "Pick"
					PickedAnkle = undefined
				)
			)
		)
		catch()
		)
		
		on chk_bendy changed changedto do 
		(
			if changedto == true then 
			(
				rdo_bendyrom.enabled = true
				chk_squash.enabled = true
			) 
			else if (changedto == false) and (chk_stretchy.checked) == false then
			(
				rdo_bendyrom.enabled = false
				chk_squash.enabled = false
				chk_squash.checked = false
			)
			else
			(
			rdo_bendyrom.enabled = false
			)
		)
		
		on chk_stretchy changed changedto do 
		(
			if changedto == true then 
			(
				chk_squash.enabled = true
			) 
			else if (changedto == false) and (chk_bendy.checked) == false then
			(
				chk_squash.enabled = false
				chk_squash.checked = false
			)
		)
		
		
		on btn_pickUpLeg pressed do
		(
			local TryToPickUpLeg
			TryToPickUpLeg = undefined
			PickedUpLeg = undefined
			TryToPickUpLeg = pickobject prompt:"PickUpArm" rubberBand:[0,0,0] forceListenerFocus:false count:1
			if classof TryToPickUpLeg != Undefined and TryToPickUpLeg != #() and classof TryToPickUpLeg != UndefinedClass then
			(
				btn_pickUpLeg.Caption = TryToPickUpLeg.name
				PickedUpLeg = TryToPickUpLeg
			)
			else
			(
				btn_pickUpLeg.Caption = "Pick"
				PickedUpLeg = undefined
			)
		)
		
		on btn_pickKnee pressed do
		(
			local TryToPickKnee
			TryToPickKnee = undefined
			PickedKnee = undefined
			TryToPickKnee = pickobject prompt:"PickElbow" rubberBand:[0,0,0] forceListenerFocus:false count:1
			if classof TryToPickKnee != Undefined and TryToPickKnee != #() and classof TryToPickKnee != UndefinedClass then
			(
				btn_PickKnee.Caption = TryToPickKnee.name
				PickedKnee = TryToPickKnee
			)
			else
			(
				btn_PickKnee.Caption = "Pick"
				PickedKnee = undefined
			)
		)
		
		on btn_PickAnkle pressed do
		(
			local TryToPickAnkle
			TryToPickAnkle = undefined
			PickedAnkle = undefined
			TryToPickAnkle = pickobject prompt:"PickHand" rubberBand:[0,0,0] forceListenerFocus:false count:1
			if classof TryToPickAnkle != Undefined and TryToPickAnkle != #() and classof TryToPickAnkle != UndefinedClass then
			(
				btn_PickAnkle.Caption = TryToPickAnkle.name
				PickedAnkle = TryToPickAnkle
			)
			else
			(
				btn_PickAnkle.Caption = "Pick"
				PickedAnkle = undefined
			)
		)
		
		on btn_PickConnect pressed do
		(
			local TryToPickLegParent
			TryToPickLegParent = undefined
			PickedLegParent = undefined
			TryToPickLegParent = pickobject prompt:"PickUpArm" rubberBand:[0,0,0] forceListenerFocus:false count:1
			if classof TryToPickLegParent != Undefined and TryToPickLegParent != #() and classof TryToPickLegParent != UndefinedClass then
			(
				btn_PickConnect.Caption = TryToPickLegParent.name
				PickedLegParent = TryToPickLegParent
			)
			else
			(
				btn_PickConnect.Caption = "Pick"
				PickedLegParent = undefined
			)
		)
		
		on btn_pickRoot pressed do
		(
			local TryToPickRoot
			TryToPickRoot = undefined
			PickedRoot = undefined
			TryToPickRoot = pickobject prompt:"PickUpArm" rubberBand:[0,0,0] forceListenerFocus:false count:1
			if classof TryToPickRoot != Undefined and TryToPickRoot != #() and classof TryToPickRoot != UndefinedClass then
			(
				btn_pickRoot.Caption = TryToPickRoot.name
				PickedRoot = TryToPickRoot
			)
			else
			(
				btn_pickRoot.Caption = "Pick"
				PickedRoot = undefined
			)
		)

		
		on btn_createguide pressed do
		(
			if LegGuideOutput != undefined then 
			(
				delete LegGuideOutput[1]
				LegGuideOutput = undefined
				btn_PickAnkle.Caption = "Pick"
				PickedAnkle = undefined
				btn_PickKnee.Caption = "Pick"
				PickedKnee = undefined
				btn_PickUpLeg.Caption = "Pick"
				PickedUpLeg = undefined
				btn_createguide.caption = "Create Layout Guide"
				btn_create.enabled = off
			)
			else
			(
				if PickedKnee != undefined and PickedUpLeg != undefined and PickedAnkle != undefined do
				(
					
					LegGuideoutput = FinalTools.MakeLegGuide PickedUpLeg PickedKnee PickedAnkle Legtype:rdo_legtype.state
					btn_createguide.caption = "Delete Guide"
					btn_create.enabled = on
				)
			)
		)
		
		on btn_create pressed do
		(

				if (Chk_Bendy.checked == true) or (Chk_Stretchy.checked == true) do (if spn_twistbones.value < 3 do (spn_twistbones.value = 3))
				local MainLegBones = LegGuideOutput[2]
					
				for i = 1 to i = MainLegBones.count do
				(MainLegBones[i] = MainLegBones[i].transform)
				
				local FootRigLocators = LegGuideOutput[3]
				for i = 1 to i = FootRigLocators.count do
				(FootRigLocators[i] = FootRigLocators[i].transform)
				
				local ThighWidth = (Distance LegGuideOutput[4][1] LegGuideOutput[4][2])
				local ThighFrontWidth = (Distance LegGuideOutput[4][1] LegGuideOutput[4][3])
				local KneeWidth = (Distance LegGuideOutput[4][4] LegGuideOutput[4][5])
				local KneeFrontWidth = (Distance LegGuideOutput[4][4] LegGuideOutput[4][6])
				local AnkleWidth = (Distance LegGuideOutput[4][7] LegGuideOutput[4][8])
				local AnkleFrontWidth = (Distance LegGuideOutput[4][7] LegGuideOutput[4][9])
				local ToeWidth = (Distance LegGuideOutput[4][10] LegGuideOutput[4][11])
				local ToeFrontWidth = (Distance LegGuideOutput[4][10] LegGuideOutput[4][12])
				local WidthsArray = #(ThighWidth,ThighFrontWidth,KneeWidth,KneeFrontWidth,AnkleWidth,AnkleFrontWidth,ToeWidth,ToeFrontWidth)
				if Chk_FootRig.checked == false do (FootRigLocators = undefined)
				local DetectedAlignment = LegGuideOutput[5]
				
				local ScapulaInput = false
				if rdo_legtype.state == 2 do (ScapulaInput = #(LegGuideOutput[6][1].transform))
					
				local MakeLegOutput = FinalTools.MakeLeg MainLegBones FootRigLocators Prefix:ToolSetup.NamePrefix InputWidths:WidthsArray Kinematics:Rdo_Kinematics.state Stretchy:Chk_Stretchy.checked bendy:Chk_bendy.checked ConnectTo:PickedLegParent Root:PickedRoot DetectedAlignment:DetectedAlignment TwistBones:Spn_TwistBones.value FootRig:Chk_FootRig.checked IncludeSquash:Chk_Squash.checked BendyRom:rdo_bendyRom.state KneeType:rdo_kneedeform.state LegType:rdo_legtype.state Scapula:ScapulaInput
				
				
				if chk_Mirror.checked == true do
				(
					if rdo_legtype.state == 2 do (ScapulaInput = CommonFns.MirrorTransformsInArray ScapulaInput)
					MainLegBones = CommonFns.MirrorTransformsInArray MainLegBones
					if Chk_FootRig.checked != false do (FootRigLocators = CommonFns.MirrorTransformsInArray FootRigLocators)
					if DetectedAlignment == "r" or DetectedAlignment == "R" then (DetectedAlignment = "L") else (DetectedAlignment = "R")
					local MakeLegOutputMirrored = FinalTools.MakeLeg MainLegBones FootRigLocators Prefix: ToolSetup.nameprefix InputWidths:WidthsArray Kinematics:Rdo_Kinematics.state Stretchy:Chk_Stretchy.checked bendy:Chk_bendy.checked ConnectTo:PickedLegParent Root:PickedRoot DetectedAlignment:DetectedAlignment TwistBones:Spn_TwistBones.value FootRig:Chk_FootRig.checked IncludeSquash:Chk_Squash.checked BendyRom:rdo_bendyRom.state KneeType:rdo_kneedeform.state LegType:rdo_legtype.state Scapula:ScapulaInput
			)
				
				--select MakeLegOutput[1]

				destroydialog LegGuideRollout
			
				
				
				delete LegGuideOutput[1]
						----------OutputArray Contains---------------
			--1: all created
			--2: Main Bones: Upper Leg, Lower Leg, Foot, Toe
			--3: Foot Rig Locators: Heel, Foot Tip, Foot Inner, Foot Outer
			--4: Widths: ThighGuide, Thigh Width, Thigh Front >>>> Knee, Ankle, Toe
			--5: DetectedAlignment
				
			
		)
		

		
		
	) --this bracket to close leg guide rollout
	
	rollout HeadGuideRollout "Make Head" width:229 height:648
	(
		bitmap bmp2 "Bitmap" pos:[0,0] width:229 height:648 fileName:"RigToolKit_CreateHead.png"
		button btn_pickheadend "Pick" pos:[118,66] width:78 height:24
		button btn_pickheadstart "Pick" pos:[117,96] width:78 height:24
		label lbl2 "Neck Start:" pos:[35,130] width:59 height:20
		label lbl3 "Head Start:" pos:[33,98] width:81 height:20
		groupBox grp1 "Start/End Points" pos:[9,49] width:201 height:142
		button btn_createguide "Create Layout Guide" pos:[47,160] width:116 height:22
		radioButtons rdo_scaling "" pos:[25,303] width:114 height:32 labels:#("Stretchy neck", "Non-Stretchy Neck") default:1 Columns:1
		button btn_create "Create" pos:[66,617] width:82 height:25 enabled:off
		label lbl11 "Top of Head:" pos:[27,70] width:81 height:20
		button btn_pickneckstart "Pick" pos:[117,128] width:78 height:24
		groupBox grp9 "Connections" pos:[10,526] width:201 height:86
		button btn_pickparent "Pick" pos:[121,547] width:78 height:24
		button btn_pickRoot "Pick" pos:[120,577] width:78 height:24
		label lbl12 "Connect To:" pos:[46,551] width:63 height:20
		label lbl13 "Ribcage NoFollow:" pos:[20,579] width:98 height:20
		button btn_pickRightShoulder "Pick" pos:[118,460] width:78 height:24
		label lbl14 "Right Shoulder:" pos:[26,463] width:78 height:20
		button btn_PickLeftShoulder "Pick" pos:[119,489] width:78 height:24
		label lbl15 "Left Shoulder:" pos:[29,494] width:78 height:20
		groupBox grp_necktype "Neck Type" pos:[9,197] width:201 height:185
		radioButtons rdo_necktype "" pos:[26,224] width:65 height:32 labels:#("IK Neck", "FK Neck") default:1 Columns:1
		spinner spn_numbones "Num. Bones:" pos:[50,261] width:101 height:16 range:[0,100,0] type:#integer scale:1
		groupBox grp_Iksettings "IK Settings" pos:[9,286] width:201 height:96
		radioButtons rdo_trapbones "" pos:[28,408] width:106 height:48 labels:#("None", "Game Compatible", "Stretchy bones") default:1
		groupBox grp_trapbones "Trapezius bones" pos:[9,387] width:201 height:133
		radioButtons rdo_rom "" pos:[26,342] width:121 height:32 labels:#("High range of motion", "Low range of motion") Columns:1
		
		local PickedNeckStart,PickedHeadStart,PickedHeadEnd,HeadGuideOutput,PickedParent,PickedRoot,PickedRightShoulder,PickedLeftShoulder
		
		
		
		on HeadGuideRollout close do
		(
				try
		(
		local WM_SYSCOMMAND = 0x112
		local SC_RESTORE = 0xf120

		--http://forums.cgsociety.org/archive/index.php/t-644118.html --Minimise/Maximise rollout code snippet from this post.

		local OpenDialogs = UIAccessor.GetPopupDialogs()
		local DialogTitles = for d in OpenDialogs collect (UIAccessor.GetWindowText d)
		local DialogIndex = findItem DialogTitles "Rigging Toolkit"
		local MyHandle = OpenDialogs[DialogIndex]

		local MaxWindows = windows.getChildrenHWND 0 parent:#max
		local WindowTitles = for w in MaxWindows collect w[5]
		local WindowIndex = findItem WindowTitles "Rigging Toolkit"
		MyHandle = MaxWindows[WindowIndex][1]
		windows.sendMessage MyHandle WM_SYSCOMMAND SC_RESTORE 0
		)
		catch()
		)
		
		
		
		on HeadGuideRollout Open do
		(
			if ToolSetup.Turbomode == on do
			(
				
				if ToolSetup.LastCreatedRibcage != undefined do 
				(
					try
					(
					select ToolSetup.lastcreatedribcage
					btn_PickParent.caption = ToolSetup.lastcreatedribcage.name
					PickedParent = ToolSetup.lastcreatedribcage
					)catch()
				)
				if ToolSetup.LastPickedRoot != undefined do 
				(
					try
					(
					select ToolSetup.lastpickedroot
					btn_Pickroot.caption = ToolSetup.lastpickedRoot.name
					Pickedroot = ToolSetup.lastpickedRoot
					)catch()
				)
			)
			

			try
			(
			local TryToPickNeckStart
			TryToPickNeckStart = undefined
			PickedNeckStart = undefined
			TryToPickNeckStart = pickobject prompt:"PickUpArm" rubberBand:[0,0,0] forceListenerFocus:false count:1
			if classof TryToPickNeckStart != Undefined and TryToPickNeckStart != #() and classof TryToPickNeckStart != UndefinedClass then
			(
				btn_pickNeckStart.Caption = TryToPickNeckStart.name
				PickedNeckStart = TryToPickNeckStart
			)
			else
			(
				btn_pickNeckStart.Caption = "Pick"
				PickedNeckStart = undefined
			)
			if PickedNeckStart != undefined do
			(
				local TryToPickHeadStart
				TryToPickHeadStart = undefined
				PickedHeadStart = undefined
				TryToPickHeadStart = pickobject prompt:"PickElbow" rubberBand:[0,0,0] forceListenerFocus:false count:1
				if classof TryToPickHeadStart != Undefined and TryToPickHeadStart != #() and classof TryToPickHeadStart != UndefinedClass then
				(
					btn_PickHeadStart.Caption = TryToPickHeadStart.name
					PickedHeadStart = TryToPickHeadStart
				)
				else
				(
					btn_PickHeadStart.Caption = "Pick"
					PickedHeadStart = undefined
				)
				local TryToPickHeadEnd
				TryToPickHeadEnd = undefined
				PickedHeadEnd = undefined
				TryToPickHeadEnd = pickobject prompt:"PickHand" rubberBand:[0,0,0] forceListenerFocus:false count:1
				if classof TryToPickHeadEnd != Undefined and TryToPickHeadEnd != #() and classof TryToPickHeadEnd != UndefinedClass then
				(
					btn_PickHeadEnd.Caption = TryToPickHeadEnd.name
					PickedHeadEnd = TryToPickHeadEnd
				)
				else
				(
					btn_PickHeadEnd.Caption = "Pick"
					PickedHeadEnd = undefined
				)
			)
		)
		catch()
		)
	
		
		on btn_pickNeckStart pressed do
		(
			local TryToPickNeckStart
			TryToPickNeckStart = undefined
			PickedNeckStart = undefined
			TryToPickNeckStart = pickobject prompt:"PickUpArm" rubberBand:[0,0,0] forceListenerFocus:false count:1
			if classof TryToPickNeckStart != Undefined and TryToPickNeckStart != #() and classof TryToPickNeckStart != UndefinedClass then
			(
				btn_pickNeckStart.Caption = TryToPickNeckStart.name
				PickedNeckStart = TryToPickNeckStart
			)
			else
			(
				btn_pickNeckStart.Caption = "Pick"
				PickedNeckStart = undefined
			)
		)
		
		on btn_pickHeadStart pressed do
		(
			local TryToPickHeadStart
			TryToPickHeadStart = undefined
			PickedHeadStart = undefined
			TryToPickHeadStart = pickobject prompt:"PickElbow" rubberBand:[0,0,0] forceListenerFocus:false count:1
			if classof TryToPickHeadStart != Undefined and TryToPickHeadStart != #() and classof TryToPickHeadStart != UndefinedClass then
			(
				btn_PickHeadStart.Caption = TryToPickHeadStart.name
				PickedHeadStart = TryToPickHeadStart
			)
			else
			(
				btn_PickHeadStart.Caption = "Pick"
				PickedHeadStart = undefined
			)
		)
		
		on btn_PickHeadEnd pressed do
		(
			local TryToPickHeadEnd
			TryToPickHeadEnd = undefined
			PickedHeadEnd = undefined
			TryToPickHeadEnd = pickobject prompt:"PickHand" rubberBand:[0,0,0] forceListenerFocus:false count:1
			if classof TryToPickHeadEnd != Undefined and TryToPickHeadEnd != #() and classof TryToPickHeadEnd != UndefinedClass then
			(
				btn_PickHeadEnd.Caption = TryToPickHeadEnd.name
				PickedHeadEnd = TryToPickHeadEnd
			)
			else
			(
				btn_PickHeadEnd.Caption = "Pick"
				PickedHeadEnd = undefined
			)
		)
		
		on btn_PickParent pressed do
		(
			local TryToPickParent
			TryToPickParent = undefined
			PickedParent = undefined
			TryToPickParent = pickobject prompt:"PickUpArm" rubberBand:[0,0,0] forceListenerFocus:false count:1
			if classof TryToPickParent != Undefined and TryToPickParent != #() and classof TryToPickParent != UndefinedClass then
			(
				btn_PickParent.Caption = TryToPickParent.name
				PickedParent = TryToPickParent
			)
			else
			(
				btn_PickParent.Caption = "Pick"
				PickedParent = undefined
			)
		)
		
		on btn_pickRoot pressed do
		(
			local TryToPickRoot
			TryToPickRoot = undefined
			PickedRoot = undefined
			TryToPickRoot = pickobject prompt:"PickUpArm" rubberBand:[0,0,0] forceListenerFocus:false count:1
			if classof TryToPickRoot != Undefined and TryToPickRoot != #() and classof TryToPickRoot != UndefinedClass then
			(
				btn_pickRoot.Caption = TryToPickRoot.name
				PickedRoot = TryToPickRoot
			)
			else
			(
				btn_pickRoot.Caption = "Pick"
				PickedRoot = undefined
			)
		)
		
		on btn_createguide pressed do
		(
			if HeadGuideOutput != undefined then 
			(
				try (delete HeadGuideOutput[1]) catch()
				HeadGuideOutput = undefined
				btn_PickNeckStart.Caption = "Pick"
				PickedNeckStart = undefined
				btn_PickHeadStart.Caption = "Pick"
				PickedHeadStart = undefined
				btn_PickHeadEnd.Caption = "Pick"
				PickedHeadEnd = undefined
				btn_createguide.caption = "Create Layout Guide"
				btn_create.enabled = off
			)
			else
			(
				if PickedNeckStart != undefined and PickedHeadStart != undefined and PickedHeadEnd != undefined do
				(
					HeadGuideoutput = FinalTools.MakeHeadGuide PickedNeckStart PickedHeadStart PickedHeadEnd
					btn_createguide.caption = "Delete Guide"
					btn_create.enabled = on
				)
			)
		)
			
		on btn_pickRightShoulder pressed do
		(
			local TryToPickRightShoulder
			TryToPickRightShoulder = undefined
			PickedRightShoulder = undefined
			TryToPickRightShoulder = pickobject prompt:"PickElbow" rubberBand:[0,0,0] forceListenerFocus:false count:1
			if classof TryToPickRightShoulder != Undefined and TryToPickRightShoulder != #() and classof TryToPickRightShoulder != UndefinedClass then
			(
				btn_PickRightShoulder.Caption = TryToPickRightShoulder.name
				PickedRightShoulder = TryToPickRightShoulder
			)
			else
			(
				btn_pickRightShoulder.Caption = "Pick"
				PickedRightShoulder = undefined
			)
		)
		
		on btn_PickLeftShoulder pressed do
		(
			local TryToPickLeftShoulder
			TryToPickLeftShoulder = undefined
			PickedLeftShoulder = undefined
			TryToPickLeftShoulder = pickobject prompt:"PickHand" rubberBand:[0,0,0] forceListenerFocus:false count:1
			if classof TryToPickLeftShoulder != Undefined and TryToPickLeftShoulder != #() and classof TryToPickLeftShoulder != UndefinedClass then
			(
				btn_PickLeftShoulder.Caption = TryToPickLeftShoulder.name
				PickedLeftShoulder = TryToPickLeftShoulder
			)
			else
			(
				btn_PickLeftShoulder.Caption = "Pick"
				PickedLeftShoulder = undefined
			)
		)
		
		
		on btn_create pressed do
		(
			for i = 1 to i = HeadGuideOutput[2].count do 
			(
				HeadGuideOutput[2][i] = HeadGuideOutput[2][i].transform
			)
			for i = 1 to i = HeadGuideOutput[3].count do 
			(
				HeadGuideOutput[3][i] = HeadGuideOutput[3][i].transform
			)
			local JointScaling = rdo_scaling.state
			
			local SquashInput
			if JointScaling == 1 then (SquashInput = 2) else (SquashInput = 1)
			
			try (local TrapBonesInput = rdo_trapbones.state) catch
			(
				messagebox "Trap bones radiobutton bugged out again... Setting to off as default"
				local TrapBonesInput = 1
			)
			local ROMInput = rdo_Rom.state
			local NecktypeInput = rdo_necktype.state
			FinalTools.MakeHead HeadGuideOutput[2] HeadGuideOutput[3] Widths:HeadGuideOutput[4] Scaling:JointScaling Squash:SquashInput Root:PickedRoot LeftShoulder:PickedLeftShoulder RightShouldeR:PickedRightShoulder ParentObj:PickedParent Prefix:ToolSetup.NamePrefix IKFK:NecktypeInput NumBones:spn_numbones.value ROM:ROMInput TrapBoneStyle:TrapBonesInput
			delete HeadGuideOutput[1]
			destroydialog HeadGuideRollout
		)
		
				--OutputArray contains
		--1: Allcreated
		--2: NeckStart, NeckBend, HeadBone, HeadNub
		--3: Right Trapezius, Right Trapezius nub
		--4:Widths:  NeckBaseWidthGuide, NeckBaseWidth, NeckBaseFrontWidth.    Neck End Guide, Width, Front.   Head Guide Width Front
		

		
			
		
	)--this bracket to close make head rollout
	rollout SpaceSwitchRollout "Space Switch" width:280 height:388
	(
	bitmap bmp2 "Bitmap" pos:[0,0] width:280 height:388 fileName:"RigToolKit_SpaceSwitch.png"
	button btn_Pickapplyto "Pick" pos:[134,47] width:75 height:26
	label lbl1 "Apply switch to:" pos:[40,53] width:83 height:16
	button btn_pickparA "Pick" pos:[72,154] width:75 height:26
	button btn_pickparB "Pick" pos:[72,191] width:75 height:26
	button btn_pickparC "Pick" pos:[71,231] width:75 height:26
	button btn_pickparD "Pick" pos:[71,271] width:75 height:26
	button btn_pickparE "Pick" pos:[70,307] width:75 height:26
	label lbl2 "Parent A:" pos:[16,158] width:50 height:16
	label lbl3 "Parent B:" pos:[15,197] width:48 height:16
	label lbl4 "Parent C:" pos:[16,237] width:50 height:16
	label lbl5 "Parent D:" pos:[18,275] width:48 height:16
	label lbl6 "Parent E:" pos:[18,312] width:51 height:16
	groupBox grp1 "Parent Object(s)" pos:[5,127] width:259 height:221
	button btn_create "Create" pos:[84,357] width:81 height:24
	editText edtA "" pos:[161,158] width:73 height:18 Text:"Enter Name"
	editText edtB "" pos:[162,195] width:73 height:18 Text:"Enter Name"
	editText edtC "" pos:[162,236] width:73 height:18 Text:"Enter Name"
	editText edtD "" pos:[161,277] width:73 height:18 Text:"Enter Name"
	editText edtE "" pos:[161,313] width:73 height:18 Text:"Enter Name"
	button btn_controls "Pick" pos:[136,85] width:75 height:26
	label lbl7 "Apply controls to: (Optional)" pos:[34,87] width:102 height:30
		
		local PickedApplyTo,PickedParA,PickedParB,PickedParC,PickedParD,PickedParE,PickedControlObj
		
		
		on SpaceSwitchRollout close do
		(
				try
		(
		local WM_SYSCOMMAND = 0x112
		local SC_RESTORE = 0xf120

		--http://forums.cgsociety.org/archive/index.php/t-644118.html --Minimise/Maximise rollout code snippet from this post.

		local OpenDialogs = UIAccessor.GetPopupDialogs()
		local DialogTitles = for d in OpenDialogs collect (UIAccessor.GetWindowText d)
		local DialogIndex = findItem DialogTitles "Rigging Toolkit"
		local MyHandle = OpenDialogs[DialogIndex]

		local MaxWindows = windows.getChildrenHWND 0 parent:#max
		local WindowTitles = for w in MaxWindows collect w[5]
		local WindowIndex = findItem WindowTitles "Rigging Toolkit"
		MyHandle = MaxWindows[WindowIndex][1]
		windows.sendMessage MyHandle WM_SYSCOMMAND SC_RESTORE 0
		)
		catch()
		)
		
		
		on btn_PickApplyTo pressed do
		(
			local TryToPickApplyTo
			TryToPickApplyTo = undefined
			PickedApplyTo = undefined
			TryToPickApplyTo = pickobject prompt:"PickHand" rubberBand:[0,0,0] forceListenerFocus:false count:1
			if classof TryToPickApplyTo != Undefined and TryToPickApplyTo != #() and classof TryToPickApplyTo != UndefinedClass then
			(
				btn_PickApplyTo.Caption = TryToPickApplyTo.name
				PickedApplyTo = TryToPickApplyTo
			)
			else
			(
				btn_PickApplyTo.Caption = "Pick"
				PickedApplyTo = undefined
			)
		)
			
		on btn_Controls pressed do
		(
			local TryToPickControlObj
			TryToPickControlObj = undefined
			PickedControlObj = undefined
			TryToPickControlObj = pickobject prompt:"PickHand" rubberBand:[0,0,0] forceListenerFocus:false count:1
			if classof TryToPickControlObj != Undefined and TryToPickControlObj != #() and classof TryToPickControlObj != UndefinedClass then
			(
				btn_Controls.Caption = TryToPickControlObj.name
				PickedControlObj = TryToPickControlObj
			)
			else
			(
				btn_Controls.Caption = "Pick"
				PickedControlObj = undefined
			)
		)
			
		
				on btn_PickParA pressed do
		(
			local TryToPickParA
			TryToPickParA = undefined
			PickedParA = undefined
			TryToPickParA = pickobject prompt:"PickHand" rubberBand:[0,0,0] forceListenerFocus:false count:1
			if classof TryToPickParA != Undefined and TryToPickParA != #() and classof TryToPickParA != UndefinedClass then
			(
				btn_PickParA.Caption = TryToPickParA.name
				PickedParA = TryToPickParA
			)
			else
			(
				btn_PickParA.Caption = "Pick"
				PickedParA = undefined
			)
		)
				on btn_PickParB pressed do
		(
			local TryToPickParB
			TryToPickParB = undefined
			PickedParB = undefined
			TryToPickParB = pickobject prompt:"PickHand" rubberBand:[0,0,0] forceListenerFocus:false count:1
			if classof TryToPickParB != Undefined and TryToPickParB != #() and classof TryToPickParB != UndefinedClass then
			(
				btn_PickParB.Caption = TryToPickParB.name
				PickedParB = TryToPickParB
			)
			else
			(
				btn_PickParB.Caption = "Pick"
				PickedParB = undefined
			)
		)
				on btn_PickParC pressed do
		(
			local TryToPickParC
			TryToPickParC = undefined
			PickedParC = undefined
			TryToPickParC = pickobject prompt:"PickHand" rubberBand:[0,0,0] forceListenerFocus:false count:1
			if classof TryToPickParC != Undefined and TryToPickParC != #() and classof TryToPickParC != UndefinedClass then
			(
				btn_PickParC.Caption = TryToPickParC.name
				PickedParC = TryToPickParC
			)
			else
			(
				btn_PickParC.Caption = "Pick"
				PickedParC = undefined
			)
		)
		on btn_PickParD pressed do
		(
			local TryToPickParD
			TryToPickParD = undefined
			PickedParD = undefined
			TryToPickParD = pickobject prompt:"PickHand" rubberBand:[0,0,0] forceListenerFocus:false count:1
			if classof TryToPickParD != Undefined and TryToPickParD != #() and classof TryToPickParD != UndefinedClass then
			(
				btn_PickParD.Caption = TryToPickParD.name
				PickedParD = TryToPickParD
			)
			else
			(
				btn_PickParD.Caption = "Pick"
				PickedParD = undefined
			)
		)
		on btn_PickParE pressed do
		(
			local TryToPickParE
			TryToPickParE = undefined
			PickedParE = undefined
			TryToPickParE = pickobject prompt:"PickHand" rubberBand:[0,0,0] forceListenerFocus:false count:1
			if classof TryToPickParE != Undefined and TryToPickParE != #() and classof TryToPickParE != UndefinedClass then
			(
				btn_PickParE.Caption = TryToPickParE.name
				PickedParE = TryToPickParE
			)
			else
			(
				btn_PickParE.Caption = "Pick"
				PickedParE = undefined
			)
		)
		
		on btn_create pressed do
		(
			local ParentListArray = #()
			local NamelistArray = #()
			
			
			
			if edtA.text != "" and edtA.text != "Enter Name" do (append NamelistArray edtA.text)
			if edtB.text != "" and edtB.text != "Enter Name" do (append NamelistArray edtB.text)
			if edtC.text != "" and edtC.text != "Enter Name" do (append NamelistArray edtC.text)
			if edtD.text != "" and edtD.text != "Enter Name" do (append NamelistArray edtD.text)
			if edtE.text != "" and edtE.text != "Enter Name" do (append NamelistArray edtE.text)
			
			
			
			if PickedParA != undefined do (appendifunique ParentListArray PickedParA)
			if PickedParB != undefined do (appendifunique ParentListArray PickedParB)
			if PickedParC != undefined do (appendifunique ParentListArray PickedParC)
			if PickedParD != undefined do (appendifunique ParentListArray PickedParD)
			if PickedParE != undefined do (appendifunique ParentListArray PickedParE)
			
			if ParentListArray.count > 0 and NameListArray.count >0 then 
			(
				FinalTools.CreateSpaceSwitch PickedApplyTo ParentListArray NamelistArray ApplySwitchTo:PickedControlObj Prefix:ToolSetup.NamePrefix
				Messagebox "Space switch has been applied."
			) 
			else 
			(
				if ParentListArray.count <1 do (Messagebox "Pick at least one object") 
				if NameListArray.count <1 do (Messagebox "Please name the space switches.")
			)
			
		)
	)--this bracket to close space switch rollout

	rollout AnimGeoRollout "Make Anim Geo" width:320 height:271
	(
		bitmap bmp2 "Bitmap" pos:[0,0] width:320 height:271 fileName:"RigToolKit_AnimGeo.png"
		button btn_apply "Apply to selection" pos:[119,76] width:142 height:25
		radioButtons rdo_parentskin "" pos:[28,70] width:66 height:32 labels:#("Parented", "Skinned") Columns:1
		groupBox grp3 "Create Geo" pos:[6,53] width:303 height:60
		groupBox grp38 "Tweak Geo/Controllers" pos:[7,126] width:302 height:135
		button btn_posx "+" pos:[82,161] width:22 height:22
		button btn_posxmin "-" pos:[107,161] width:22 height:22
		button btn_posy "+" pos:[83,188] width:22 height:22
		button btn_posymin "-" pos:[108,188] width:22 height:22
		button btn_posz "+" pos:[83,214] width:22 height:22
		button btn_posZmin "-" pos:[108,214] width:22 height:22
		button btn_scalex "+" pos:[156,162] width:22 height:22
		button btn_scalexmin "-" pos:[181,162] width:22 height:22
		button btn_scaley "+" pos:[157,189] width:22 height:22
		button btn_scaleymin "-" pos:[182,189] width:22 height:22
		button btn_scalez "+" pos:[157,215] width:22 height:22
		button btn_scalezmin "-" pos:[182,215] width:22 height:22
		button btn_rotx "+" pos:[225,163] width:22 height:22
		button btn_rotxmin "-" pos:[250,163] width:22 height:22
		button btn_roty "+" pos:[226,190] width:22 height:22
		button btn_rotymin "-" pos:[251,190] width:22 height:22
		button btn_rotz "+" pos:[226,216] width:22 height:22
		button btn_rotzmin "-" pos:[251,216] width:22 height:22
		groupBox grp39 "Position" pos:[72,144] width:72 height:101
		groupBox grp40 "Scale" pos:[143,144] width:72 height:101
		groupBox grp41 "Rotation" pos:[214,144] width:72 height:101
		groupBox grp43 "" pos:[21,144] width:52 height:101
		label lbl65 "X" pos:[44,161] width:10 height:17
		label lbl66 "Y" pos:[44,189] width:10 height:17
		label lbl67 "Z" pos:[44,219] width:10 height:17
		
		local PickedSwitch
		
		on AnimGeoRollout close do
		(
			try
			(
			local WM_SYSCOMMAND = 0x112
			local SC_RESTORE = 0xf120

			--http://forums.cgsociety.org/archive/index.php/t-644118.html --Minimise/Maximise rollout code snippet from this post.

			local OpenDialogs = UIAccessor.GetPopupDialogs()
			local DialogTitles = for d in OpenDialogs collect (UIAccessor.GetWindowText d)
			local DialogIndex = findItem DialogTitles "Rigging Toolkit"
			local MyHandle = OpenDialogs[DialogIndex]

			local MaxWindows = windows.getChildrenHWND 0 parent:#max
			local WindowTitles = for w in MaxWindows collect w[5]
			local WindowIndex = findItem WindowTitles "Rigging Toolkit"
			MyHandle = MaxWindows[WindowIndex][1]
			windows.sendMessage MyHandle WM_SYSCOMMAND SC_RESTORE 0
			)
			catch()
		)
		
		on btn_apply pressed do
		(
			local BoneSelection =#()
			
			for obj in selection do
			(
				if classof obj == BoneGeometry do (append BoneSelection obj)
			)
			
			if BoneSelection.count != 0 do
			(
			FinalTools.CreateAnimGeo BoneSelection ParentSkin:rdo_parentskin.state Prefix:ToolSetup.NamePrefix
			)
		)
		
		on btn_posx pressed do (CommonFns.TweakObjectOffset TweakType:"Pos" Positive:True Axis:"X")
		on btn_posxmin pressed do (CommonFns.TweakObjectOffset TweakType:"Pos" Positive:false Axis:"X")	
		on btn_posy pressed do (CommonFns.TweakObjectOffset TweakType:"Pos" Positive:True Axis:"Y")
		on btn_posymin pressed do (CommonFns.TweakObjectOffset TweakType:"Pos" Positive:false Axis:"Y")
		on btn_posz pressed do (CommonFns.TweakObjectOffset TweakType:"Pos" Positive:True Axis:"Z")
		on btn_poszmin pressed do (CommonFns.TweakObjectOffset TweakType:"Pos" Positive:false Axis:"Z")
		on btn_rotx pressed do (CommonFns.TweakObjectOffset TweakType:"Rot" Positive:True Axis:"X")
		on btn_rotxmin pressed do (CommonFns.TweakObjectOffset TweakType:"Rot" Positive:false Axis:"X")	
		on btn_roty pressed do (CommonFns.TweakObjectOffset TweakType:"Rot" Positive:True Axis:"Y")
		on btn_rotymin pressed do (CommonFns.TweakObjectOffset TweakType:"Rot" Positive:false Axis:"Y")	
		on btn_rotz pressed do (CommonFns.TweakObjectOffset TweakType:"Rot" Positive:True Axis:"Z")
		on btn_rotzmin pressed do (CommonFns.TweakObjectOffset TweakType:"Rot" Positive:false Axis:"Z")	
		on btn_scalex pressed do (CommonFns.TweakObjectOffset TweakType:"Scale" Positive:True Axis:"X")
		on btn_scalexmin pressed do (CommonFns.TweakObjectOffset TweakType:"Scale" Positive:false Axis:"X")	
		on btn_scaley pressed do (CommonFns.TweakObjectOffset TweakType:"Scale" Positive:True Axis:"Y")
		on btn_scaleymin pressed do (CommonFns.TweakObjectOffset TweakType:"Scale" Positive:false Axis:"Y")	
		on btn_scalez pressed do (CommonFns.TweakObjectOffset TweakType:"Scale" Positive:True Axis:"Z")
		on btn_scalezmin pressed do (CommonFns.TweakObjectOffset TweakType:"Scale" Positive:false Axis:"Z")	
	
	) --this bracket to close anim geo rollout

	rollout MainPageRollout "Rigging Toolkit" width:730 height:200 
	(
		bitmap bmp2 "Bitmap" pos:[0,0] width:730 height:200 fileName:"RigToolKit_TitleLogo.png"
		editText edt_prefix "" pos:[586,16] width:114 height:18 text:"MyRig"
		label lbl2 "Rig Prefix:" pos:[530,18] width:56 height:21
		groupBox grp2 "Rig Modules" pos:[10,64] width:410 height:130
		groupBox grp5 "Extra Tools" pos:[428,64] width:290 height:130
		button btn_spine "Spine" pos:[28,166] width:69 height:21
		button btn_arm "Arm" pos:[130,167] width:69 height:21
		button btn_leg "Leg" pos:[230,167] width:69 height:21
		button btn_head "Head" pos:[329,168] width:69 height:21
		button btn_spaceswitch "Space Switch" pos:[442,167] width:79 height:21
		button btn_animgeo "Anim Geo" pos:[547,167] width:79 height:21
		
		local PrefixChangedTo
		
		
		on btn_spine pressed do
		(
			try(destroyDialog CreateSpineRollout)catch("")
			MainPageRollout.placement = #minimized
			createDialog CreateSpineRollout pos:[50,350]
		)
		
		on btn_arm pressed do
		(
			try(destroyDialog MakeArmGuideRollout)catch("")
			MainPageRollout.placement = #minimized
			createDialog MakeArmGuideRollout pos:[50,130]
		)
		
		on btn_leg pressed do
		(
			try(destroyDialog LegGuideRollout)catch("")
			MainPageRollout.placement = #minimized
			createDialog LegGuideRollout pos:[50,130]
		)
		
		on btn_head pressed do
		(
			try(destroyDialog HeadGuideRollout)catch("")
			MainPageRollout.placement = #minimized
			createDialog HeadGuideRollout pos:[50,280]
		)
		
		on btn_spaceswitch pressed do
		(
			try(destroyDialog SpaceSwitchRollout)catch("")
			MainPageRollout.placement = #minimized
			createDialog SpaceSwitchRollout pos:[50,350]
		)
		
		on btn_animgeo pressed do
		(
			try(destroyDialog AnimGeoRollout)catch("")
			MainPageRollout.placement = #minimized
			createDialog AnimGeoRollout pos:[50,350]
		)
		
		
		on edt_prefix changed PrefixChangedTo do
		(
			if PrefixChangedTo != "" do (ToolSetup.nameprefix = (PrefixChangedTo+ "_"))
		)
		
		
	)-- this bracket to close main page rollout
	;
	

		
		-------------------------------------------------------------------------------------
	------------------------------BODY OF CODE -----------------------------------
	--try(destroyDialog FlexiSpineRollout)catch("") --Attempt to close the main dialog if it is already open
	--createDialog CreateSpineRollout
	--createDialog IKFKSpineRollout
	--createDialog MakeArmGuideRollout
	--createDialog LegGuideRollout
	createdialog MainPageRollout style:#(#style_titlebar, #style_border, #style_sysmenu,#style_minimizebox) pos:[50,150]
	
	--createDialog HeadGuideRollout
)

--Rigging Toolkit
--JackBowyer545@gmail.com
